/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.StringReader;
import java.io.PrintStream;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ErrorHandler;
import xmto2xmta.ErrorEvent;
import xmto2xmta.ErrorListener;
import xmto2xmta.Xmto2Xmta;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTO;

//=============================================================================
//  XmtO2A.java
//
//  Overview  :
//    This program validates a XMT-O file and/or converts it into a XMT-A file.
//=============================================================================

class XmtO2A {
	private static int XMTO_INDEX = 0;
	private static int XMTA_INDEX = 1;
	private static int LOGFILE_INDEX = 2;
	private static int CONVERSION_FLAG_INDEX = 3;
	private static int VALIDATION_FLAG_INDEX = 4;
	private static int INDEX_COUNT = 5;
	
	//private StringBuffer logOutput = new StringBuffer();
/***********************************************************************
 **********************************************************************/
public void convert(String[] parameters) {
	System.out.println("Generating with XMT-O file - " + parameters[XMTO_INDEX]);

	//
	Document xmtODoc = parseAndValidateDocument(parameters[XMTO_INDEX], parameters[VALIDATION_FLAG_INDEX]);
	if (null == xmtODoc)
		return;
	Document xmtADoc = null;

	// Test to see if we have an XMT-O document.
	String xmtoRootName = xmtODoc.getDocumentElement().getNodeName();
	if (null == xmtoRootName) {
		System.err.println("XmtO2A.convert() - Error parsing XMT document - invalid Root Name (should be XMT-O)");
		return;
	}
	if (!xmtoRootName.equals(DOMXMTO.XMT_dash_O)) {
		System.err.println("XmtO2A.convert() - Error parsing XMT document - invalid Root Name of \"" + xmtoRootName + "\" should be XMT-O");
		return;
	}

	// We definitely have an XMT-O document.
	// Validation has been done, let's see if conversion is needed.
	if (!parameters[CONVERSION_FLAG_INDEX].equals("Y")) {
	    System.out.println("XmtO2A.convert() - No need for conversion - exiting.");
	    return;
	} else
	    System.out.println("XmtO2A - Converting Xmt-O file into Xmt-A");
	    
	// Generate the Document for the XMT-A.
	xmtADoc = generateNewXmtaDocument();

	// Convert from the XMT-O to XMT-A
	try {
		Xmto2Xmta converter = new Xmto2Xmta();
		converter.setXmtODocPath(parameters[XMTO_INDEX]);
		converter.addErrorListener(new ErrorListener() {
			public void trace(ErrorEvent errorEvent) {
				//					System.out.println("Xmto2Xmta_converter.ErrorListener.trace() - errorId=" + errorEvent.getErrorId() + " message=" + errorEvent.getMessage());
			}
			public void oK(ErrorEvent errorEvent) {
				//					System.out.println("Xmto2Xmta_converter.ErrorListener.oK() - errorId=" + errorEvent.getErrorId() + " message=" + errorEvent.getMessage());
			}
			public void warning(ErrorEvent errorEvent) {
				//                  System.out.println("Xmto2Xmta_converter.ErrorListener.warning() - errorId=" + errorEvent.getErrorId() + " message=" + errorEvent.getMessage());
			}
			public void error(ErrorEvent errorEvent) {
				System.out.println("Xmto2Xmta_converter.ErrorListener.error() - errorId=" + errorEvent.getErrorId() + " message=" + errorEvent.getMessage());
			}
			public void fatalError(ErrorEvent errorEvent) {
				System.out.println("Xmto2Xmta_converter.ErrorListener.fatalError() - errorId=" + errorEvent.getErrorId() + " message=" + errorEvent.getMessage());
			}
		});
		converter.convert(xmtODoc, xmtADoc);
	} catch (Exception e) {
		System.err.println("XmtO2A.convert() - Error converting from XMT-O to XMT-A exception=" + e);
		e.printStackTrace();
		return;
	}

	// Generate the XMT-A file.
	try {
		OutputFormat outputFormat = new OutputFormat();
		outputFormat.setIndent(2);
		outputFormat.setLineWidth(0);
		outputFormat.setEncoding("UTF-8");
		outputFormat.setLineSeparator(System.getProperty("line.separator"));
		XMLSerializer serializer = new XMLSerializer(new FileOutputStream(parameters[XMTA_INDEX]), outputFormat);
		serializer.serialize(xmtADoc);
		System.out.println("    Generated XMT-A file - " + parameters[XMTA_INDEX]);
	} catch (Exception e) {
		System.err.println("XmtO2A.convert() - Error in writing XMT-A file \"" + parameters[XMTA_INDEX] + "\" exception=" + e);
		e.printStackTrace();
		return;
	}
	
	// Print out log in a file or on the screen
		
	
}

/***********************************************************************
 **********************************************************************/
public Document generateNewXmtaDocument() {
	// NOTE: create a dom parser for the XMT-A dom tree, this is so that the default attributes 
	// are put in the tree.  If you don't use a dom parser then the dtd is not parsed to determine
	// the attributes.
	DOMParser domAParser = new DOMParser();
	try {
		domAParser.setFeature("http://apache.org/xml/features/continue-after-fatal-error", true);
		domAParser.setFeature("http://xml.org/sax/features/namespaces", true);
		domAParser.setErrorHandler(new ErrorHandler() {
			public void warning(SAXParseException exception) throws SAXException {
				System.out.println("domAParser.ErrorHandler.warning() - PublicId=" + exception.getPublicId() + " SystemId=" + exception.getSystemId() + " line=" + exception.getLineNumber() + " column=" + exception.getColumnNumber() + " exception=" + exception);
			}
			public void error(SAXParseException exception) throws SAXException {
				System.out.println("domAParser.ErrorHandler.error() - PublicId=" + exception.getPublicId() + " SystemId=" + exception.getSystemId() + " line=" + exception.getLineNumber() + " column=" + exception.getColumnNumber() + " exception=" + exception);
			}
			public void fatalError(SAXParseException exception) throws SAXException {
				System.out.println("domAParser.ErrorHandler.fatalError() - PublicId=" + exception.getPublicId() + " SystemId=" + exception.getSystemId() + " line=" + exception.getLineNumber() + " column=" + exception.getColumnNumber() + " exception=" + exception);
			}
		});
	} catch (SAXException e) {
		System.err.println("XmtO2A.generateNewXmtaDocument() - setting parser features - exception - " + e);
		e.printStackTrace();
		return null;
	}


	// Now create a new XMT-A document  
	String rootName = DOMXMTA.XMT_dash_A;
	StringBuffer newFileData = new StringBuffer();
	newFileData.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	newFileData.append("<").append(DOMXMTA.XMT_dash_A).append(">");
	//		newFileData.append("<").append(DOMXMTA.Header).append("/>");
	newFileData.append("</").append(DOMXMTA.XMT_dash_A).append(">");

	// Parse the temporary XMT-A document, necessary for conversion.
	try {
		domAParser.parse(new InputSource(new StringReader(newFileData.toString())));
	} catch (Exception e) {
		System.err.println("XmtO2A.generateNewXmtaDocument() - Error parsing new XMT-A model exception - " + e);
		e.printStackTrace();
		return null;
	}

	//
	Document xmtADoc = domAParser.getDocument();
	return xmtADoc;
}
/***********************************************************************
 **********************************************************************/
public static void main(String[] args) {
	XmtO2A xmtO2A = new XmtO2A();

	// look for help
	if (0 < args.length) {
		String arg0 = args[0];
		if (arg0.equals("?") || arg0.equals("-?")) {
			xmtO2A.usage();
			return;
		}
	}

	// get the file names
	String[][] parameters = xmtO2A.parseArgs(args);
	if (null == parameters) {
		xmtO2A.usage();
		return;
	}

	//
	for (int i = 0; i < parameters.length; i++) {
	    
        // Redefine System.out in case we want the messages in a log file.
        if (null != parameters[i][LOGFILE_INDEX]) {
            try {
                FileOutputStream fos = new FileOutputStream(parameters[i][LOGFILE_INDEX]);
                System.setOut(new PrintStream(fos));
            } catch (Exception e) {
                // we didn't manage to direct message to that file.
                System.err.println("XmtO2A - Failure to open log file " + parameters[i][LOGFILE_INDEX]);
            }
        }
		xmtO2A.convert(parameters[i]);
	}
	System.exit(0);
}
/***********************************************************************
 **********************************************************************/
public Document parseAndValidateDocument(String fileName, String validation) {
	// NOTE: create a dom parser for the XMT-A dom tree, this is so that the default attributes 
	// are put in the tree.  If you don't use a dom parser then the dtd is not parsed to determine
	// the attributes.
	DOMParser domParser = new DOMParser();
	try {
		domParser.setFeature("http://xml.org/sax/features/validation", validation.equals("Y"));
		domParser.setFeature("http://apache.org/xml/features/continue-after-fatal-error", true);
		domParser.setFeature("http://xml.org/sax/features/namespaces", true);
		domParser.setFeature("http://apache.org/xml/features/validation/schema", true);
		final String finalFileName = fileName;
		domParser.setErrorHandler(new ErrorHandler() {
			public void warning(SAXParseException exception) throws SAXException {
				System.out.println("DOM Parser warning - file=" + finalFileName + " PublicId=" + exception.getPublicId() + " SystemId=" + exception.getSystemId() + " line=" + exception.getLineNumber() + " column=" + exception.getColumnNumber() + " exception=" + exception);
			}
			public void error(SAXParseException exception) throws SAXException {
			    if (exception.toString().indexOf("ANY---")==-1)
				System.out.println("DOM Parser error - file=" + finalFileName + " PublicId=" + exception.getPublicId() + " SystemId=" + exception.getSystemId() + " line=" + exception.getLineNumber() + " column=" + exception.getColumnNumber() + " exception=" + exception);
			}
			public void fatalError(SAXParseException exception) throws SAXException {
				System.out.println("DOM Parser fatalError - file=" + finalFileName + " PublicId=" + exception.getPublicId() + " SystemId=" + exception.getSystemId() + " line=" + exception.getLineNumber() + " column=" + exception.getColumnNumber() + " exception=" + exception);
			}
		});
	} catch (SAXException e) {
		System.err.println("XmtO2A.parseAndValidateDocument() - setting parser features - exception - " + e);
		e.printStackTrace();
		return null;
	}


	// First parse the document
	try {
		domParser.parse((new InputSource(new FileInputStream(fileName))));
	} catch (Exception e) {
		System.err.println("XmtO2A.parseAndValidate() - Error parsing XMT document exception - " + e);
		e.printStackTrace();
		return null;
	}

	//
	Document xmtDoc = domParser.getDocument();
	return xmtDoc;
}
/***********************************************************************
 **********************************************************************/
public String[][] parseArgs(String[] args) {

	// No arguments
	if (0 == args.length) {
		return null;
	}

	//  we have at least one argument
	//  use the (first) argument to identify the xml input file
	String xmtoFileName = args[0];
	String xmtaFileName = null;
	String xmlLogFileName = null;
	String validationFlag = "Y";
	String conversionFlag = "Y";

	// Create the xmta file name.
	String xmlFileExtension = ".xml";
	// if the xmto file ends with .xml then insert 'A' before .xml else append A.mp4 to the xmto file name.
	if (xmtoFileName.endsWith(xmlFileExtension)) {
		int endIndex = xmtoFileName.length() - xmlFileExtension.length();
		xmtaFileName = xmtoFileName.substring(0, endIndex) + 'A' + xmlFileExtension;
	} else {
		xmtaFileName = xmtoFileName + 'A' + xmlFileExtension;
	}
	
	// Now we need to loop through the rest of the arguments to determine what we have
	if (1 < args.length) {
		int nextArg = 1;
		while (nextArg < args.length) {
			String upperCaseArg = args[nextArg].toUpperCase();
			// Check to make sure we have a parameter type indicator
			if (2 != upperCaseArg.length() || '-' != upperCaseArg.charAt(0))
				return null;
			switch (upperCaseArg.charAt(1)) {
				case 'A' :
					if (nextArg + 1 >= args.length)
						return null;
					xmtaFileName = args[nextArg + 1];
					nextArg += 2;
					break;
				case 'L' :
					if (nextArg + 1 >= args.length)
						return null;
					xmlLogFileName = args[nextArg + 1];
					nextArg += 2;
					break;
				case 'X' :
					conversionFlag = "N";
					nextArg++;
					break;
				case 'V' :
					validationFlag = "N";
					nextArg++;
					break;
				default :
					return null;
			}
		}
	}

	//
	String[][] parameters = new String[1][INDEX_COUNT];
	parameters[0][XMTO_INDEX] = xmtoFileName;
	parameters[0][XMTA_INDEX] = xmtaFileName;
	parameters[0][LOGFILE_INDEX] = xmlLogFileName;
	parameters[0][CONVERSION_FLAG_INDEX] = conversionFlag;
	parameters[0][VALIDATION_FLAG_INDEX] = validationFlag;

	//
	return parameters;
}
/***********************************************************************
 * Prints proper usage of this program.
 **********************************************************************/
public void usage() {
	//                  0        1         2         3         4         5         6         7         8
	//                  12345678901234567890123456789012345678901234567890123456789012345678901234567890
	System.out.println("This program validates a XMT-O file and/or converts it into a XMT-A file.");
	System.out.println("");
	System.out.println("  Usage: XmtO2A  <XMT.xml> [-a <XMT-A.xml>]");
	System.out.println("                           [-l <logfile.xml>]");
	System.out.println("                           [-x] [-v] [-?]");
	System.out.println("");
	System.out.println("  <XMT.xml>      - input XMT-O file name.");
	System.out.println("  <XMT-A.xml>    - optional XMT-A file name.  If one is not provided then the");
	System.out.println("                   XMT-A file will have the XMT-O file name with 'A' appended.");
	System.out.println("  <logfile.xml>  - optional xml file name where messges will be logged. If one");
	System.out.println("                   is not provided then any message will be printed on the");
	System.out.println("                   console.");
	System.out.println("  -x             - optional flag for validation only. If present, the XMT-O");
	System.out.println("                   file will be validated but not converted into XMT-A.");
	System.out.println("  -v             - optional flag for no validation.If present, no validation");
	System.out.println("                   of the XMT-O file will be performed before conversion.");
	System.out.println("  -?             - this help information.");
	System.out.println("");
}
}
