/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  ErrorEvent.java
//
//=============================================================================
package xmto2xmta;

import java.util.*;
import org.w3c.dom.Element;

/**********************************************************************
 * Events used to signal what has errors in the XMT-O to XMT-A converter.
 *
 * <p><b>History:</b><ul>
 * <li>04 Oct 2000 - Added class
 * </ul>
 *********************************************************************/
public class ErrorEvent extends EventObject {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 * Error level value. 
	 * 0 = OK.
	 * 1 = Warning.
	 * 2 = Error.
	 * 3 = Fatal Error.
	 **********************************************************************/
	private int errorLevel_;
	public static final int OK = 0;
	public static final int TRACE = 1;
	public static final int WARNING = 2;
	public static final int ERROR = 3;
	public static final int FATAL_ERROR = 4;

	/*********************************************************************** 
	 * Unique identifier for error messsage.
	 **********************************************************************/
	private int errorId_;
	public static final int UNDEFINED = 0;

	/*********************************************************************** 
	 * User string.  String which can be displayed to the user.  
	 * This should be a NLS translated string.
	 **********************************************************************/
	private String message_;

	/*********************************************************************** 
	 * User string.  String which can be displayed to the user.  
	 * This should be a NLS translated string.
	 **********************************************************************/
	private Element xmtoElement_;

	/*********************************************************************** 
	 * User string.  String which can be displayed to the user.  
	 * This should be a NLS translated string.
	 **********************************************************************/
	private String xmtoAttribute_;
/**********************************************************************
 * ConnectionModelEvent constructor comment.
 *
 * @param source ConnectionModel
 *********************************************************************/
public ErrorEvent(Xmto2Xmta source, int errorLevel, int errorId, String message) {
	this(source, errorLevel, errorId, message, null, null);
}
/**********************************************************************
 * ConnectionModelEvent constructor comment.
 *
 * @param source ConnectionModel
 *********************************************************************/
public ErrorEvent(Xmto2Xmta source, int errorLevel, int errorId, String message, Element xmtoElement) {
	this(source, errorLevel, errorId, message, xmtoElement, null);
}
/**********************************************************************
 * ConnectionModelEvent constructor comment.
 *
 * @param source ConnectionModel
 *********************************************************************/
public ErrorEvent(Xmto2Xmta source, int errorLevel, int errorId, String message, Element xmtoElement, String xmtoAttribute) {
	super(source);
	errorLevel_ = errorLevel;
	errorId_ = errorId;
	message_ = (null == message) ? "" : message;
	xmtoElement_ = xmtoElement;
	xmtoAttribute_ = xmtoAttribute;
}
/**********************************************************************
 *********************************************************************/
public int getErrorId() {
	return errorId_;
}
/**********************************************************************
 *********************************************************************/
public int getErrorLevel() {
	return errorLevel_;
}
/**********************************************************************
 *********************************************************************/
public String getMessage() {
	return message_;
}
/**********************************************************************
 *********************************************************************/
public String getProblemAttribute() {
	return xmtoAttribute_;
}
/**********************************************************************
 *********************************************************************/
public Element getProblemElement() {
	return xmtoElement_;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
