/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  EventActiveInterval.java
//
//=============================================================================
package xmto2xmta;

import java.util.Vector;
import xmtutil.*;
import xmto2xmta.timegraph.*;

/*********************************************************************** 
 * Class containing a specific life cycle information for an object.
 *
 * <p><b>History:</b><ul>
 * <li>27 Mar 2001  - Created file.
 * </ul><p>
 *********************************************************************/
public class EventActiveInterval {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	// These are the only meaningful fill type values in ActiveInterval.
	public static final int FILL_FREEZE = 2;
	public static final int FILL_TRANSITION = 4;

	/***********************************************************************
	 * Input structures
	 **********************************************************************/
	private TimingAttribute observedBeginTime_ = null;
	private TimingAttribute fillEndTime_ = null;
	private int fillType_ = 0;
	private ELASid elasid = null;
	private ELASid parentELASid = null;
	private TimingAttribute beginEvent = null;
	private TimingAttribute endEvent = null;
	private TimingAttribute simpleDuration = null;

/***********************************************************************
 **********************************************************************/
	public EventActiveInterval() {
	}

/***********************************************************************
 **********************************************************************/
public TimingAttribute getFillEndTime() {
	return fillEndTime_;
}

/***********************************************************************
 **********************************************************************/
public int getFillType() {
	return fillType_;
}

/***********************************************************************
 * observed begin time
 **********************************************************************/
public TimingAttribute getObservedBeginTime() {
	return observedBeginTime_;
}

/***********************************************************************
 * simple duration
 **********************************************************************/
public TimingAttribute getSimpleDuration() {
	return simpleDuration;
}

/***********************************************************************
 * begin event information
 **********************************************************************/
public TimingAttribute getBeginEvent() {
	return beginEvent;
}

/***********************************************************************
 * end event information
 **********************************************************************/
public TimingAttribute getEndEvent() {
	return endEvent;
}

public ELASid getELASid() {  // ELASid
	return elasid;
}

public ELASid getParentELASid() {  // parent ELASid
	return parentELASid;
}

/***********************************************************************
 **********************************************************************/
public void setObservedBeginTime(TimingAttribute observedBeginTime) {
	observedBeginTime_ = observedBeginTime;
}

/***********************************************************************
 **********************************************************************/
public void setFillEndTime(TimingAttribute fillEndTime) {
	fillEndTime_ = fillEndTime;
}

public void setBeginEvent(TimingAttribute beginEvent) {
	this.beginEvent = beginEvent;
}

public void setEndEvent(TimingAttribute endEvent) {
	this.endEvent = endEvent;
}
/***********************************************************************
 **********************************************************************/
public void setSimpleDuration(TimingAttribute simpleDuration) {
	this.simpleDuration = simpleDuration;
}

/***********************************************************************
 **********************************************************************/
public void setELASid(ELASid elasid) { // ELASid
	this.elasid = elasid;
}

public void setParentELASid(ELASid elasid) { // parent ELASid
	this.parentELASid = elasid;
}

/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}

}
