/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  IdManager.java
//
//=============================================================================
package xmto2xmta;

/*********************************************************************** 
 * This is the Java source code for managing the ids for the new XMT-A nodes.
 *
 * <p><b>History:</b><ul>
 * <li>12 Jan 2001 - Created.
 *
 *********************************************************************/
public class IdManager {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	//
	static public final int IodID = 0;
	static public final int IodBifsESID = 1;
	static public final int IodOdESID = 2;
	static public final int ESID = 3;
	static public final int ODID = 4;
	static public final int RootBackgroundGroupID = 5;
	static public final int RootOrderedGroupID = 6;
	static public final int SwitchID = 7;
	static public final int Transform2dID = 8;
	static public final int Material2dID = 9;
	static public final int TimeSensorID = 10;
	static public final int TouchSensorID = 11;
	static public final int ConditionalID = 12;
	static public final int ValuatorID = 13;
	static public final int ColorInterpolatorID = 14;
	static public final int PositionInterpolator2dID = 15;
	static public final int RectangleID = 16;
	static public final int CircleID = 17;
	static public final int TextID = 18;
	static public final int ImageTextureID = 19;
	static public final int MovieTextureID = 20;
	static public final int AudioSourceID = 21;
	static public final int OrderedGroupID = 22;
	static public final int FontStyleID = 23;
	static public final int Background2dID = 24;
	static public final int GroupID = 25;
	static public final int ConditionalInsertID = 26;
	static public final int ConditionalInsertTimerID = 27;
	static public final int ConditionalDeleteID = 28;
	static public final int ConditionalDeleteTimerID = 29;
	static public final int TimeSensorBeginOffsetID = 30;
	static public final int TimeSensorEndOffsetID = 31;
	static public final int ShapeID = 32;
	static public final int Sound2dID = 33;
	static public final int RootDefsGroupID = 34;
	static public final int RootLayoutGroupID = 35;
	static public final int Layer2dID = 36;
	static public final int RegionLayoutGroupID = 37;
	static public final int RegionOrderedGroupID = 38;
	static public final int AppearanceID = 39;
	static public final int DefsID = 40;
	static public final int PointsID = 41;
	static public final int LinesID = 42;
	static public final int PolygonsID = 43;
	static public final int CurveID = 44;
	static public final int LinePropertiesID = 45;
	static public final int MaterialKeyID = 46;
	static public final int TextureTransformID = 47;
	static public final int TextureID = 48;
	static public final int DelayID = 49;
	static public final int InterpolatorID = 50;
	static public final int RootAnimationGroupID = 51;
	static public final int ColorID = 52;
	static public final int Coordinate2DID = 53;
	static public final int COUNT_TEST = 54;

	//
	static private final String[] prefixes_ = {/* */
	/* 00 */ "IODID:",
	/* 01 */ "IODBIFSESID:",
	/* 02 */ "IODODESID:",
	/* 03 */ "ESID:",
	/* 04 */ "ODID:",
	/* 05 */ "ROOT_BACKGROUND_GROUP:",
	/* 06 */ "ROOT_ORDEREDGROUP:",
	/* 07 */ "SWITCH:",
	/* 08 */ "TRANSFORM2D:",
	/* 09 */ "MATERIAL2D:",
	/* 10 */ "TIMESENSOR:",
	/* 11 */ "TOUCHSENSOR:",
	/* 12 */ "CONDITIONAL:",
	/* 13 */ "VALUATOR:",
	/* 14 */ "COLORINTERPOLATOR:",
	/* 15 */ "POSITIONINTERPOLATOR2D:",
	/* 16 */ "RECTANGLE:",
	/* 17 */ "CIRCLE:",
	/* 18 */ "TEXT:",
	/* 19 */ "IMAGETEXTURE:",
	/* 20 */ "MOVIETEXTURE:",
	/* 21 */ "AUDIOSOURCE:",
	/* 22 */ "ORDEREDGROUP:",
	/* 23 */ "FONTSTYLE:",
	/* 24 */ "BACKGROUND2D:",
	/* 25 */ "GROUP:",
	/* 26 */ "CONDITIONAL_INSERT:",
	/* 27 */ "CONDITIONAL_INSERT_TIMER:",
	/* 28 */ "CONDITIONAL_DELETE:",
	/* 29 */ "CONDITIONAL_DELETE_TIMER:",
	/* 30 */ "TIMESENSOR_BEGIN_OFFSET:",
	/* 31 */ "TIMESENSOR_END_OFFSET:",
	/* 32 */ "SHAPE:",
	/* 33 */ "SOUND2D:",
	/* 34 */ "ROOT_DEFS_GROUP:",
	/* 35 */ "ROOT_LAYOUT_GROUP:",
	/* 36 */ "LAYER2D:",
	/* 37 */ "REGION_LAYOUT_GROUP:",
	/* 38 */ "REGION_ORDEREDGROUP:",
	/* 39 */ "APPEARANCE:",
	/* 40 */ "DEFS:",
	/* 41 */ "POINTS:",
	/* 42 */ "LINES:",
	/* 43 */ "POLYGONS:",
	/* 44 */ "CURVE:",
	/* 45 */ "LINEPROPERTIES:",
	/* 46 */ "MATERIALKEY",
	/* 47 */ "TEXTURETRANSFORM:",
	/* 48 */ "TEXTURE:",
	/* 49 */ "DELAY:",
	/* 50 */ "INTERPOLATOR:",
	/* 51 */ "ROOT_ANIMATION_GROUP:",
	/* 52 */ "COLOR:",
	/* 53 */ "COORDINATE2D:",
	/* */};

	//
	static private final String unknownPrefix_ = "UNSUPPORTEDID:";
	static private int unknownIndex_ = 1;
	static private int[] nextIndex = null;
	static private int nextElementIndex_ = 1;
/*********************************************************************** 
 **********************************************************************/
static public Integer createNewElementIndex() {
	if (null == nextIndex)
		init();

	//
	Integer index = new Integer(nextElementIndex_);
	nextElementIndex_++;

	//
	return index;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public String getNextId(int idType) {
	if (null == nextIndex)
		init();

	//
	String id;
	if (idType < 0 || idType > prefixes_.length) {
		id = unknownPrefix_ + unknownIndex_;
		unknownIndex_++;
	} else {
		id = prefixes_[idType] + nextIndex[idType];
		nextIndex[idType] ++;
	}

	//
	return id;
}
/*********************************************************************** 
 **********************************************************************/
static public String getPrefix(int idType) {
	if (null == nextIndex)
		init();

	//
	String id;
	if (idType < 0 || idType > prefixes_.length) {
		id = unknownPrefix_;
	} else {
		id = prefixes_[idType];
	}

	//
	return id;
}
/*********************************************************************** 
 **********************************************************************/
static private void init() {
	if (null != nextIndex)
		return;

	//
	nextIndex = new int[prefixes_.length];
	resetIds();
	if (prefixes_.length != COUNT_TEST)
		System.out.println("IdManager.init() - Length of prefixes (" + prefixes_.length + ") is different than the number of id types (" + COUNT_TEST + ").");
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void resetIds() {
	if (null == nextIndex)
		init();

	//
	for (int i = 0; i < nextIndex.length; i++) {
		nextIndex[i] = 1;
	}
	unknownIndex_ = 1;
	nextElementIndex_ = 1;	
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
