/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MapRoot.java
//
//=============================================================================
package xmto2xmta;

import org.w3c.dom.Element;
import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
/*********************************************************************** 
 * Class to determine resolve the timing for the XMT-O tree.
 *
 * <p><b>History:</b><ul>
 * <li>27 Mar 2001 - Created file.
 * </ul><p>
 *********************************************************************/
abstract public class MapRoot implements MapInterface {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static public boolean debug_ = false;

	/***********************************************************************
	 * Input structures
	 **********************************************************************/
	private Xmto2Xmta xmto2xmta_ = null;
	private xmto2xmta.tree.ConvertElement treeElement_ = null;
/**********************************************************************
 *********************************************************************/
public void fireErrorEvent(int errorLevel, int errorId, String message) {
	fireErrorEvent(errorLevel, errorId, message, null, null);
}
/**********************************************************************
 *********************************************************************/
public void fireErrorEvent(int errorLevel, int errorId, String message, Element xmtoElement) {
	fireErrorEvent(errorLevel, errorId, message, xmtoElement, null);
}
/**********************************************************************
 *********************************************************************/
public void fireErrorEvent(int errorLevel, int errorId, String message, Element xmtoElement, String xmtoAttribute) {
	if (null != xmto2xmta_)
		xmto2xmta_.fireErrorEvent(new ErrorEvent(xmto2xmta_, errorLevel, errorId, message, xmtoElement, xmtoAttribute));
}
/**********************************************************************
 *********************************************************************/
public xmto2xmta.tree.ConvertElement getConvertElement() {
	return treeElement_;
}
/**********************************************************************
 *********************************************************************/
public void init(Xmto2Xmta xmto2xmta, xmto2xmta.tree.ConvertElement element) throws Exception {
	xmto2xmta_ = xmto2xmta;
	treeElement_ = element;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
