/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  Xmto2Xmta.java
//
//=============================================================================
package xmto2xmta;

import java.awt.Color;
import java.awt.Dimension;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import javax.swing.event.EventListenerList;
import org.apache.xerces.dom.TextImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import xmto2xmta.mapanimation.AnimationMapper;
import xmto2xmta.mapaugmentation.AugmentationMapper;
import xmto2xmta.mapgroup.GroupMapper;
import xmto2xmta.mapmedia.MediaMapper;
import xmto2xmta.mapstructure.StructureMapper;
import xmto2xmta.mapxmediachildren.XmediaChildrenMapper;
import xmto2xmta.mapxmtamedia.XmtaMediaMapper;
import xmto2xmta.mapuse.UseMapper;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.tree.XMTO;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.TimingElement;
import xmtutil.DOMUtil;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * This is the Java source code for default mapping of XMT-omega to XMT-A 
 * authoring constructs.
 *
 * <p>
 * NOTE: this java program is based on ISO/IEC 14496-1 (N2501 + DCOR1) and 
 * XMT WD 3.0
 *
 * <p><b>History:</b><ul>
 * <li>18 Aug 2000 - Initial creation.
 * <li>31 Aug 2000 - Incorporated code into product.
 *                 - Separate file processing to make this process converting from
 *                   XMT-O dom tree to XMT-A dom tree.
 *                 - Fix reading timing info and creating timing output.
 * <li>08 Sep 2000 - Separate traversing for OD and Scene objects.
 *                 - Change ids from int to String.
 *                 - Add body to doc.
 *                 - Change OD, add various classes, "auto" option to some attributes, 
 *                   StreamSource, separate individual OD updates, add remove OD
 * <li>11 Sep 2000 - Eliminate random number generation for ids.  Guarantee their uniqueness by
 *                   incrementing a number.
 *                 - Change IDs to be strings not integers.  Use prefix to distinguish types of IDs
 *                 - Explicitly convert attributes for Transform2D, not just copy values.
 *                 - Convert color to backColor attribute for Background2D.
 *                 - Convert Material2D attributes, color to emissiveColor and value for filled goes from true/false to TRUE/FALSE
 * <li>12 Sep 2000 - Change from using nodes to elements, since it is easier to query element attributes.
 *                 - Use prefixs for other DEF attributes (ids).  Don't use random number generator.
 *                 - Fix transform2D to specify the appropriate attributes.
 *                 - Start setting attributes only if they aren't the default attributes.
 *                   NOTE: This means that the XMT-A dom tree that is passed in must 
 *                   be created with a DOM parser.  If you don't use a dom parser then 
 *                   the dtd is not parsed to determine the default attributes.	 
 *                 - Separate the DOMParameter conversions into XMT-O and XMT-A conversions.
 *                   The conversions can be different depending on type of tree, e.g. color
 *                   the XMT-O sets it as "x000000" value where as the XMT-A sets it using
 *                   3 floats.
 * <li>14 Sep 2000 - Convert color for background and remove generating its id
 *                 - Fix bug where we weren't generating elements for background.
 *                 - Change ID to DEF attribute for circle and rectangle.
 *                 - Add support for multiple animationColors, was only doing first animation.
 * <li>15 Sep 2000 - Do some fix up on the color animator.
 * <li>16 Sep 2000 - Move DOM & XMT utilities to xmtutil.* package to share files.
 * <li>18 Sep 2000 - Finished creating valid xmt-a for simple case of color animation.
 *                 - Fixed checking XMT tag for sizes during the OD traversal not Scene traversal.
 *                 - Add preliminary support for animating motion.
 * <li>19 Sep 2000 - Change int parameter from string "0xFF" to "255" this is for the ODs.
 * <li>20 Sep 2000 - Add support for JPEG and Video Decoder Specific Info.
 *                 - Correct OD types for various media objects.
 * <li>25 Sep 2000 - Add font support for text.
 * <li>27 Sep 2000 - Fix Switch element. Need choice element under it.
 * <li>02 Oct 2000 - Change Top document of XMT to XMT-O.
 * <li>02 Oct 2000 - Fix losing second parameter for Transform2D translation.
 *                 - Change location of Sensors and Interpolators to be under the children node
 *                   of the Transform2D node.
 * <li>03 Oct 2000 - Only write out Initial OD Scene if there are ODs.
 * <li>03 Oct 2000 - Use SFTime (double in seconds) in XMT-A instead of SMIL timing (13ms)
 * <li>03 Oct 2000 - Fix URL fields to be correct syntax.  For files they are now prefixed by
 *                   "file:///", for ODID references url fields for Background2D, ImageTexture 
 *                   and MovieTexture the prefix by convention is "odid://".
 * <li>03 Oct 2000 - Change all references to uppercase true or false to lowercase.
 * <li>05 Oct 2000 - Add error handling and getting messages from resource files.
 *                 - Make all naming conventions exactly as in MPEG-4 Sys spec. Including 
 *                   all of the anomalies.                 
 *                 - Get rid of some flags in ODs. Flag can be determined whether the 
 *                   field is set or not.                      
 *                 - Create Header and move InitialObjectDescriptor to it.
 *                 - Nest removal of OD under par which creates them.
 * <li>05 Oct 2000 - Change naming of ids in IOD to guarantee uniqueness.
 *                 - Change creating AudioClip to be AudioSource.
 * <li>07 Oct 2000 - Change name of xmt-a to xmt-o converter to make it
 *                   more consistent.
 *                 - Add user interaction support of setting visibility on and off
 *                   for an object.
 *                 - Move location of ROUTE to be under a scene, insert or replace to be
 *                   consistent with X3D.
 * <li>09 Oct 2000 - OD nesting element changes.
 *                 - Convert filenames from \ to /
 * <li>12 Oct 2000 - Change video OD.
 * <li>16 Oct 2000 - Fix bug in end time.
 * <li>18 Oct 2000 - Add z-order support.
 * <li>18 Oct 2000 - Change font name to check for MFString with Quotes, if it doesn't have 
 *                   quotes then add it.
 * <li>27 Oct 2000 - Add validation of XMT-O and XMT-A input documents.
 *                 - Change tracing to use error handler to allow calling method to log traces
 *                   to a file.
 * <li>30 Oct 2000 - Change XMT-O to have text placed in the attribute textLines instead of
 *                   text nodes.
 * <li>16 Nov 2000 - Add support for deleting the position field for the z-order support.
 * <li>13 Dec 2000 - Add support for mp3 files.
 * <li>13 Dec 2000 - Started split up of Converter files. Keeping the old history even though
 *                   it probably doesn't apply.
 * <li>20 Apr 2001 - Remove resolve model, replaced by TimeScheduler.
 * <li>06 Jun 2001 - Finished conversion to new XMT-A spec.
 * <li>20 Jun 2001 - Remove old methodology of performing mapping passes.
 *                 - Remove support for old XMT-A format.
 * </ul><p>
 * NOTE: TBD color animator still needs some work.
 * When color animates when another obj is clicked we need to set the touch sensor up correctly.
 * We need toget the id from the begin value. search for node with this id and insert a TouchSensor 
 * to it. Thus we shouldn't be attaching it to the targetNode but thenode indicated by the beginValue.
 *********************************************************************/
public class Xmto2Xmta {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;
	static final private boolean debugPrintTree_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private Document xmtADoc_;

	/*********************************************************************** 
	 **********************************************************************/
	private Document xmtODoc_;

	/*********************************************************************** 
	 **********************************************************************/
	private XMTO convertTreeRoot_;

	/***********************************************************************
	 * Hash table to map XMT-O element names to classes.  Some XMT-O element
	 * names are invalid names for java classes, thus they need to be mapped
	 * to class names which can be created.  For example switch and XMT-O
	 * can't be class names and thus must be mapped to valid names.
	 * The prime number 89 was chosen as the hash table initialize size because it was
	 * mentioned as a better starting hash size for optimization than the default 101.
	 * The element2ClassMapTable_ table is composed of (key, data)
	 *   The key is the XMT-O element name.
	 *   The data is the java class name which maps to the element.
	 **********************************************************************/
	private Hashtable element2ClassMapTable_ = new Hashtable(89);
	private String packagePrefix_;
	private static final String[][] element2ClassMappings_ = {
	/* */ {DOMXMTO.XMT_dash_O, "XMTO"},
	/* */ {"switch", "Switch"},
	/* */};

	/*********************************************************************** 
	 * The prime number 89 was chosen as the hash table initialize size because it was
	 * mentioned as a better starting hash size for optimization than the default 101.
	 **********************************************************************/
	private Hashtable xmtoIds_ = new Hashtable(89);

	/*********************************************************************** 
	 * The prime number 89 was chosen as the hash table initialize size because it was
	 * mentioned as a better starting hash size for optimization than the default 101.
	 **********************************************************************/
	private Hashtable xmtaIds_ = new Hashtable(89);

	/*********************************************************************** 
	 * The prime number 89 was chosen as the hash table initialize size because it was
	 * mentioned as a better starting hash size for optimization than the default 101.
	 **********************************************************************/
	private Hashtable convertIndexTable_ = new Hashtable(89);
//	private ResolveModel resolveModel_ = null;
	private TimeScheduler timeScheduler_ = null;
	private EventModel eventModel_ = null;

	/*********************************************************************** 
	 * The prime number 89 was chosen as the hash table initialize size because it was
	 * mentioned as a better starting hash size for optimization than the default 101.
	 **********************************************************************/
	//	private Hashtable treeIds_;

	/*********************************************************************** 
	 * List of listeners. to listen to error messages.
	 **********************************************************************/
	private EventListenerList listenerList_ = new EventListenerList();

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());

	/* XMTO document's path name.  All src attribute references in the doc are relative to this */
	private String xmtODocPath_ = null;

/**********************************************************************
 *********************************************************************/
public Xmto2Xmta() {
	for (int i = 0, count = element2ClassMappings_.length; i < count; i++) {
		element2ClassMapTable_.put(element2ClassMappings_[i][0], element2ClassMappings_[i][1]);
	}
	String className = this.getClass().getName();
	int index = className.lastIndexOf('.');
	packagePrefix_ = className.substring(0, index + 1) + "tree.";
}

/**********************************************************************
 * The path name of the XMTO document file.
 *********************************************************************/
public String getXmtODocPath() {
	return xmtODocPath_;
}
public void setXmtODocPath(String xmtODocPath) {
	xmtODocPath_ = xmtODocPath;
}

/*********************************************************************** 
 * addErrrorListener method comment.
 **********************************************************************/
public void addErrorListener(ErrorListener errorListener) {
	listenerList_.add(ErrorListener.class, errorListener);
}
/***********************************************************************
 * Convert the passed XMT-O document into and XMT-A document.  The XMT-O 
 * document should be validated before calling this routine.   The XMT-A 
 * document should only contain the root element.
 * XMT-A document is as follows:
 * <pre>
 * &lt;XMT-A&gt;
 * &lt;/XMT-A&gt;
 * </pre>
 **********************************************************************/
public void convert(Document xmto, Document xmta) throws Exception {
	if (debug_)
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.convert() - entered"));

	// Reset the ids
	IdManager.resetIds();

	// Validate the input
	if (null == xmto || null == xmta || null == xmto.getDocumentElement() || null == xmta.getDocumentElement()) {
		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_NO_DOCUMENT));
		String errorDocument;
		if (null == xmto || null == xmto.getDocumentElement())
			errorDocument = DOMXMTO.XMT_dash_O;
		else
			errorDocument = DOMXMTA.XMT_dash_A;
		Object[] msgArgs = {errorDocument};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.FATAL_ERROR, 0, message));
		if (debug_)
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.convert() - exiting failure=" + message));
		throw new Exception(message);
	}

	// Keep around the documents
	xmtADoc_ = xmta;
	xmtODoc_ = xmto;

	// Validate that this is an XMT-O document.  Signal an error if it isn't
	String xmtoRootName = xmtODoc_.getDocumentElement().getNodeName();
	if (null == xmtoRootName || !xmtoRootName.equals(DOMXMTO.XMT_dash_O)) {
		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT));
		if (null == xmtoRootName)
			xmtoRootName = "";
		Object[] msgArgs = {DOMXMTO.XMT_dash_O, xmtoRootName};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.FATAL_ERROR, 0, message));
		if (debug_)
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.convert() - exiting failure=" + message));
		throw new Exception(message);
	}

	// Validate that this is an XMT-A document.  Signal an error
	Element root = xmtADoc_.getDocumentElement();
	String xmtaRootName = root.getNodeName();
	if (null == xmtaRootName || !xmtaRootName.equals(DOMXMTA.XMT_dash_A)) {
		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT));
		if (null == xmtaRootName)
			xmtaRootName = "";
		Object[] msgArgs = {DOMXMTA.XMT_dash_A, xmtaRootName};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.FATAL_ERROR, 0, message));
		if (debug_)
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.convert() - exiting failure=" + message));
		throw new Exception(message);
	}

	// Create the tree for processing
	createTree();

	//
//	if (debug_)
//		printInfo();

	// Do the processing of the various passes.
	performPasses();

	//
//	if (debug_)
//		printInfo();

	//
	if (debug_)
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.convert() - exiting"));
}
/***********************************************************************
 * Create an Convert element for the specified XMT-O element.
 * 
 * @param element XMT-O element which needs to have a convert element created.
 * @return the created Convert element.
 **********************************************************************/
private ConvertElement createElement(Element element) {
	// Get the name of the element
	String nodeName = element.getNodeName();

	// Create the name of the class for the element.
	// First check if it is in the mapping hash table
	String className = (String) element2ClassMapTable_.get(nodeName);
	if (null == className) {
		// class name isn't in the mapping table so create it.
		className = nodeName;
	}
	className = packagePrefix_ + className;

	// Now try to create an convert object for the element.
	ConvertElement convertElement;
	try {
		// First create an array of the constructor parameters
		Object[] parameters = {element, xmtoIds_, xmtaIds_, convertIndexTable_, xmtADoc_, this};

		// Find the constructor for the parameters we want to use.
		// There is a method on class getConstructor(), but when passing the
		// classes of the objects it could not find the constructor.
		// It may be that it is looking for an exact match not just an
		// instance of, thus loop through the constructors looking for
		// the constructor we want.
		Constructor[] constructors = Class.forName(className).getConstructors();
		Constructor constructor = null;
		for (int i = 0;(i < constructors.length && null == constructor); i++) {
			Class[] parameterTypes = constructors[i].getParameterTypes();
			if (parameterTypes.length == parameters.length) {
				boolean parametersSame = true;
				for (int j = 0;(j < parameterTypes.length && parametersSame); j++) {
					if (!(parameterTypes[j].isInstance(parameters[j])))
						parametersSame = false;
				}
				if (parametersSame)
					constructor = constructors[i];
			}
		}

		//
		if (null == constructor) {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_WARNING_ELEMENT_NOT_SUPPORTED));
			Object[] msgArgs = {nodeName};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.WARNING, 0, message));
			return null;
		}

		// create the object with the parameters
		convertElement = (ConvertElement) constructor.newInstance(parameters);
	} catch (Exception e) {
		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_WARNING_ELEMENT_NOT_SUPPORTED));
		Object[] msgArgs = {nodeName};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.WARNING, 0, message));
		if (debug_) {
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.createElement() - exception=" + e));
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.createElement() - exiting failure=" + message));
			e.printStackTrace();
		}
		return null;
	}

	// Now create the children's convert element if necessary
	if (convertElement.createChildren()) {
		NodeList nl = element.getChildNodes();
		if (null != nl) {
			for (int i = 0, count = nl.getLength(); i < count; i++) {
				Node node = nl.item(i);
				if (node instanceof Element) {
					element = (Element) node;
					ConvertElement child = createElement(element);
					if (null != child)
						convertElement.addChild(child);
				}
			}
		}
	}

	//
	return convertElement;
}
/***********************************************************************
 * Create the convert tree with the Convert element for the specified 
 * XMT-O elements.
 **********************************************************************/
private void createTree() throws Exception {
	ConvertElement root = createElement(xmtODoc_.getDocumentElement());

	//
	if (null == root || !(root instanceof XMTO)) {
		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT));
		String xmtoRootName = xmtODoc_.getDocumentElement().getNodeName();
		if (null == xmtoRootName)
			xmtoRootName = "";
		Object[] msgArgs = {DOMXMTO.XMT_dash_O, xmtoRootName};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.FATAL_ERROR, 0, message));
		if (debug_)
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.convert() - exiting failure=" + message));
		throw new Exception(message);
	}

	//
	convertTreeRoot_ = (XMTO) root;
}
/***********************************************************************
 * Call Validate on the convert element and traverse through the children.
 **********************************************************************/
private void doInsertIntoXmtaTree(ConvertElement element) {
	boolean success = element.insertIntoXmtaTree();
	if (!success)
		return;

	// Now go through the validation on the children
	Vector clone = element.getChildren();
	for (Enumeration children = clone.elements(); children.hasMoreElements();) {
		ConvertElement child = (ConvertElement)children.nextElement();
		doInsertIntoXmtaTree(child);
	}
}
/***********************************************************************
 * Call Validate on the convert element and traverse through the children.
 **********************************************************************/
private void doValidateSyntax(ConvertElement element) {
	boolean success = element.validateSyntax();
	if (!success)
		return;


	// Now go through the validation on the children
	Vector clone = element.getChildren();
	for (Enumeration children = clone.elements(); children.hasMoreElements();) {
		ConvertElement child = (ConvertElement) children.nextElement();
		doValidateSyntax(child);
	}
}
/*********************************************************************** 
 * Forward the given notification event to all ErrorListeners that registered 
 * themselves as listeners for this converter. 
 * @see #addErrorListener
 * @see ErrorEvent
 * @see EventListenerList
 **********************************************************************/
public void fireErrorEvent(ErrorEvent e) {
	// Guaranteed to return a non-null array
	Object[] listeners = listenerList_.getListenerList();

	// Process the listeners last to first, notifying
	// those that are interested in this event
	for (int i = listeners.length - 2; i >= 0; i -= 2) {
		if (ErrorListener.class == listeners[i]) {
			switch (e.getErrorLevel()) {
				case ErrorEvent.OK :
				 	((ErrorListener) listeners[i + 1]).oK(e);
					break;
				case ErrorEvent.TRACE :
				 	((ErrorListener) listeners[i + 1]).trace(e);
					break;
				case ErrorEvent.ERROR :
				 	((ErrorListener) listeners[i + 1]).error(e);
					break;
				case ErrorEvent.FATAL_ERROR :
				 	((ErrorListener) listeners[i + 1]).fatalError(e);
					break;
				case ErrorEvent.WARNING :
				default :
				 	((ErrorListener) listeners[i + 1]).warning(e);
					break;
			}
		}
	}
}
/***********************************************************************
 * Create the convert tree with the Convert element for the specified 
 * XMT-O elements.
 **********************************************************************/
private XMTO getConvertTreeRoot() {
	return convertTreeRoot_;
}
/***********************************************************************
 **********************************************************************/
public EventModel getEventModel() {
	return eventModel_;
}
/***********************************************************************
 **********************************************************************/
public TimeScheduler getTimeScheduler() {
	return timeScheduler_;
}
/***********************************************************************
 * Go through convert tree performing the various passes on the 
 * different elements.
 **********************************************************************/
private void performPasses() {
	if (!(convertTreeRoot_ instanceof XMTO))
		return;

	//		
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - entered"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin doValidateSyntax"));
	}
	doValidateSyntax(convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended doValidateSyntax"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin timeScheduler"));
	}

	//
	timeScheduler_ = new TimeScheduler(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended timeScheduler"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin eventModel"));
	}

	//
	eventModel_ = new EventModel(this, xmtoIds_, convertIndexTable_, convertTreeRoot_, timeScheduler_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended eventModel"));
		if (debugPrintTree_) {
			printInfo();
			printTreeWithTimings(convertTreeRoot_, 0);
		}
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin StructureMapper"));
	}

	// Set up the infrastructure for building the XMT-A document
	//	((XMTO) convertTreeRoot_).createXmtaInfrastructure();
	StructureMapper sm = new StructureMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended StructureMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin LayoutMapper"));
	}

	//
	//	LayoutMapper lm = new LayoutMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended LayoutMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin GroupMapper"));
	}

	//
	GroupMapper gm = new GroupMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended GroupMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin MediaMapper"));
	}

	//
	MediaMapper mm = new MediaMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended MediaMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin XmediaChildrenMapper"));
	}

	//
	XmediaChildrenMapper xm = new XmediaChildrenMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended XmediaChildrenMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin XmtaMediaMapper"));
	}

	//
	XmtaMediaMapper xa = new XmtaMediaMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended XmtaMediaMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin AugmentationMapper"));
	}

	//
	AugmentationMapper au = new AugmentationMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended AugmentationMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin UseMapper"));
	}

	//
	UseMapper uu = new UseMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended UseMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin AnimationMapper"));
	}


	//
	AnimationMapper am = new AnimationMapper(this, xmtoIds_, convertIndexTable_, convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended AnimationMapper"));
		if (debugPrintTree_)
			printInfo();
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - begin doInsertIntoXmtaTree"));
	}

	//
	doInsertIntoXmtaTree(convertTreeRoot_);
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - ended doInsertIntoXmtaTree"));
		if (debugPrintTree_) {
			printInfo();
			printTreeWithTimings(convertTreeRoot_, 0);
		}
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.performPasses() - exiting"));
	}
}
/***********************************************************************
 * Create an Convert element for the specified XMT-O element.
 * 
 * @param element XMT-O element which needs to have a convert element created.
 * @return the created Convert element.
 **********************************************************************/
private void printInfo() {
	if (debug_) {
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.printInfo() - entered", null, null));
		String blankLine = "                           ";

		// Now go through the XMTO ids
		for (Enumeration keys = xmtoIds_.keys(); keys.hasMoreElements();) {
			Object key = keys.nextElement();
			ConvertElement element = (ConvertElement) xmtoIds_.get(key);
			StringBuffer sb = new StringBuffer();
			sb.append("Xmto2Xmta.printInfo() - xmto <");
			sb.append(element.getXmtoElement().getNodeName());
			sb.append(blankLine.substring(0, blankLine.length() - element.getXmtoElement().getNodeName().length()));
			sb.append(" id=").append(key).append(">");
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, sb.toString(), null, null));
		}

		// Now go through the XMTA ids
		for (Enumeration keys = xmtaIds_.keys(); keys.hasMoreElements();) {
			Object key = keys.nextElement();
			Element element = (Element) xmtaIds_.get(key);
			StringBuffer sb = new StringBuffer();
			sb.append("Xmto2Xmta.printInfo() - xmta <");
			sb.append(element.getNodeName());
			sb.append(blankLine.substring(0, blankLine.length() - element.getNodeName().length()));
			sb.append(" id=").append(key).append(">");
			fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, sb.toString(), null, null));
		}
		printTree(convertTreeRoot_, 0);

		//
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, "Xmto2Xmta.printInfo() - exiting", null, null));
	}
}
/***********************************************************************
 * Call Validate on the convert element and traverse through the children.
 **********************************************************************/
private void printTree(ConvertElement element, int nestingLevel) {
	if (debug_) {
		String name = element.getXmtoElement().getNodeName();
		//		TimingAttribute begin = element.getBeginTime();
		//		TimingAttribute end = element.getEndTime();
		StringBuffer buffer = new StringBuffer();
		buffer.append("Xmto2Xmta.printTree() - ");
		for (int i = 0; i < nestingLevel * 2; i++)
			buffer.append(" ");
		buffer.append("<").append(name);
		//		if (null != begin)
		//			buffer.append(" absoluteBegin=").append(begin);
		//		if (null != end)
		//			buffer.append(" absoluteEnd=").append(end);
		buffer.append(">");
		fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, buffer.toString(), null, null));

		// Now go through the validation on the children
		Vector clone = element.getChildren();
		for (Enumeration children = clone.elements(); children.hasMoreElements();) {
			ConvertElement child = (ConvertElement) children.nextElement();
			printTree(child, nestingLevel + 1);
		}
	}
}
/***********************************************************************
 * Call Validate on the convert element and traverse through the children.
 **********************************************************************/
private void printTreeWithTimings(ConvertElement element, int nestingLevel) {
	if (debug_) {
		if (element instanceof TimingElement) {
			//
			TimingElement timingElement = (TimingElement) element;
			Integer teIndex = timingElement.getIndex();
			String name = element.getXmtoElement().getNodeName();
			String id = element.getXmtoAttrId();
			StringBuffer elementString = new StringBuffer();
			elementString.append(" <").append(name);
			if (null != id && !id.equals(""))
				elementString.append(" id=").append(id);
			elementString.append(">");

			//			
			Vector lifeCycles = timeScheduler_.getLifeCycles(teIndex);
			int lifeCycleCount = 0;
			for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
				LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
				int activeIntervalCount = 0;
				for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
					ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();
					StringBuffer sb = new StringBuffer();
					sb.append("Xmto2Xmta.printTreeWithTimings() - ");
					for (int i = 0; i < nestingLevel * 2; i++)
						sb.append(" ");
					sb.append(elementString);
					sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
					sb.append(" activeInterval[").append(activeIntervalCount).append("]");
					activeIntervalCount++;
					sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
					sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
					sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
					sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
					sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
					for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
						TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
						sb.append(" repeatTime=").append(repeatTime);
					}
					fireErrorEvent(new ErrorEvent(this, ErrorEvent.TRACE, 0, sb.toString(), null, null));
				}
				lifeCycleCount++;
			}
		}

		// Now go through the validation on the children
		Vector clone = element.getChildren();
		for (Enumeration children = clone.elements(); children.hasMoreElements();) {
			ConvertElement child = (ConvertElement) children.nextElement();
			printTreeWithTimings(child, nestingLevel + 1);
		}
	}
}
/*********************************************************************** 
 * removeErrorListener method comment.
 **********************************************************************/
public void removeErrorListener(ErrorListener errorListener) {
	listenerList_.remove(ErrorListener.class, errorListener);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
