/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  AniAnimate.java
//
//=============================================================================
package xmto2xmta.mapanimation;

import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.tree.animate;
import xmto2xmta.tree.ConvertElement;
/*********************************************************************** 
 * Class to group together the various type of media elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>19 Apr 2001 - Add simple lifecycle support.  
 * <li>22 Jun 2001 - Convert to look for object that is ColorAnimatable
 *                   instead of looking only for material nodes.
 * <li>30 Jun 2001 - Add interpolator support
 * </ul><p>
 *
 *********************************************************************/
public class AniAnimate extends AniAnimationElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private animate treeAnimate_ = null;
/*********************************************************************** 
 * Return the name of the attribute to be animated.
 **********************************************************************/
protected String getXmtoAttrAttributeName() {
	return treeAnimate_.getXmtoAttrAttributeName();
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof animate)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimate.init() - element=" + element + " is not an instance of animate");
		throw new Exception("");
	}

	//
	treeAnimate_ = (animate) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimate.init() - treeAnimate_=" + treeAnimate_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
