/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  AniAnimationBaseElement.java
//
//=============================================================================
package xmto2xmta.mapanimation;

import xmto2xmta.MapRoot;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.XMTO;
/*********************************************************************** 
 * Class to group together the various type of media elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>19 Apr 2001 - Add simple lifecycle support.  
 * <li>22 Jun 2001 - Convert to look for object that is ColorAnimatable
 *                   instead of looking only for material nodes.
 * </ul><p>
 *
 *********************************************************************/
public abstract class AniAnimationBaseElement extends MapRoot {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtaNodeContainerId_ = null;
/*********************************************************************** 
 **********************************************************************/
protected String getXmtaNodeContainerId() {
	if (null != xmtaNodeContainerId_)
		return xmtaNodeContainerId_;
	// We can put all the new nodes in the xmta group which holds the defs elements.
	ConvertElement parent = (ConvertElement) getConvertElement().getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			xmtaNodeContainerId_ = ((XMTO) parent).getAnimationGroupId();
			break;
		}
		parent = (ConvertElement) parent.getParent();
	}

	// We should never not beable to find the defs group but just in case check anyway and set to an empty string.
	if (null == xmtaNodeContainerId_)
		xmtaNodeContainerId_ = "";

	//
	return xmtaNodeContainerId_;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
