/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  AniAnimationElement.java
//
//=============================================================================
package xmto2xmta.mapanimation;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.LifeCycle;
import xmto2xmta.MapRoot;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.AnimateColorRes;
import xmto2xmta.resources.AnimationElementRes;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.Animatable;
import xmto2xmta.tree.AnimationElement;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.TimingElement;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.XMTO;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to group together the various type of media elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>19 Apr 2001 - Add simple lifecycle support.  
 * <li>22 Jun 2001 - Convert to look for object that is ColorAnimatable
 *                   instead of looking only for material nodes.
 * <li>30 Jun 2001 - Add interpolator support
 * </ul><p>
 *********************************************************************/
public abstract class AniAnimationElement extends AniAnimationBaseElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private AnimationElement treeAnimationElement_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
	private ResourceBundle animationElementRes_ = ResourceBundle.getBundle("xmto2xmta.resources.AnimationElementRes", Locale.getDefault());
	private ResourceBundle animateColorRes_ = ResourceBundle.getBundle("xmto2xmta.resources.AnimateColorRes", Locale.getDefault());
/***********************************************************************
 * Delete the nodes created for animation.  Delete the TimeSensor and ColorIntropolator nodes.
 * Create the animation infrastructure.
 * What has to be deleted are:
 * <ol>
 * <li>Disable the TimeSensor (during the fill time).
 * <li>Reset the initial value for the object.
 * <li>Delete the TimeSensor and Interpolator.
 * </ol>
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Replace atNode="???" atField="enabled" value="false"/&gt;
 *   &lt;/par&gt;
 *   &lt;par begin="???"&gt;
 *     &lt;Delete atNode="???"/&gt;
 *     &lt;Delete atNode="???"/&gt;
 *   &lt;/par&gt;
 * <pre>
 **********************************************************************/
private boolean createDeleteMapping(ELASid elasId) {
	//
	Document xmtaDoc = treeAnimationElement_.getXmtaDoc();

	// get active interval information
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());

	// Get the insert time 
	//	int sdIndex = elasId.getSimpleDurIndex().intValue();
	//	TimingAttribute insertTime = null;
	//	if (0 == sdIndex)
	//		insertTime = activeInterval.getObservedBeginTime();
	//	else
	//		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

	// get the delete time
	TimingAttribute fillEndTime = activeInterval.getFillEndTime();
	TimingAttribute activeEndTime = activeInterval.getActiveEnd();

	// If this active End time doesn't end then we don't need to worry about disabling or deleting it.
	if (activeEndTime.getTimeValueType() == TimingAttribute.INDEFINITE_VALUE)
		return true;

	//
	String timeSensorId = treeAnimationElement_.getXmtaTimeSensorId(elasId);

	// If the fill end isn't at the end of the duration then we need to disable the timesensor.
	if (!activeEndTime.equals(fillEndTime)) {
		// <Replace atNode="???" atField="enabled" value="true"/>
		Element parNodes = xmtaDoc.createElement(DOMXMTA.par);
		parNodes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(activeEndTime.getClockValue()));
		treeAnimationElement_.addToParNode(parNodes);
		// replace the field on the TimeSensor to disable the timesensor from running
		Element replace = xmtaDoc.createElement(DOMXMTA.Replace);
		replace.setAttribute(DOMXMTA.atNode, timeSensorId);
		replace.setAttribute(DOMXMTA.atField, DOMXMTA.enabled);
		replace.setAttribute(DOMXMTA.value, DOMXMTA.string_false);
		parNodes.appendChild(replace);
	}

	// If this fill time doesn't end then we don't need to worry about deleting it.
	if (fillEndTime.getTimeValueType() == TimingAttribute.INDEFINITE_VALUE)
		return true;

	// Create the par for the delete command.  Don't place it in the XMT-A tree yet.
	// <par begin="???>
	//   <Delete atNode="???">
	//   <Delete atNode="???">
	// </par>
	Element parDelete = xmtaDoc.createElement(DOMXMTA.par);
	parDelete.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(fillEndTime.getClockValue()));
	treeAnimationElement_.addToParNode(parDelete);

	// Create delete Node for time sensor
	Element deleteNode = xmtaDoc.createElement(DOMXMTA.Delete);
	deleteNode.setAttribute(DOMXMTA.atNode, timeSensorId);
	parDelete.appendChild(deleteNode);

	// Create delete Node for Interpolator.
	deleteNode = xmtaDoc.createElement(DOMXMTA.Delete);
	String interpolatorId = treeAnimationElement_.getXmtaInterpolatorId(elasId);
	deleteNode.setAttribute(DOMXMTA.atNode, interpolatorId);
	parDelete.appendChild(deleteNode);

	// Reset the value to what it was at the begining
	boolean success = createResetTargetValues(fillEndTime);

	//
	return success;
}
/***********************************************************************
 * Create the animation infrastructure.
 * NOTE: the timing for animations done differently than the other dealings with
 * intervals.  Here we have one mapping for an active interval, other places we have
 * a mapping for each simple interval
 * Map the timings using the entire activeInterval.  Thus only pass in the first
 * elasid to the routines
 * <p>
 * What has to be created:
 * <ol>
 * <li>Create the TimeSensor and set up the times to be when it starts/ends and loop times if any.
 * <li>Create the Interpolator and set up the values for it.
 * <li>Create the routes between the interpolator and the target object to be animated.  It is 
 *     possible that there are more than one target object simple duration, active interval or 
 *     lifecycle  for this animation.  This is because the timing of the animation (creation and
 *     deletion) can be in a different tree from the animation.
 * </ol>
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       &lt;TimeSensor DEF="???" cycleInterval="???"/&gt;
 *       &lt;???Interpolator??? DEF="???" key="???" keyValue="???"/&gt;
 *       &lt;ROUTE fromField="fraction_changed" fromNode="???" toField="set_fraction"  toNode="???"/&gt;
 *     &lt;/Insert&gt;
 *   &lt;/par&gt;
 * <pre>
 **********************************************************************/
private boolean createInsertMapping(ELASid elasId) {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimationElement.createInsertMapping() - entered");

	//
	Document xmtaDoc = treeAnimationElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeAnimationElement_.getXmtaIds();

	// get active interval information
	TimeScheduler timeScheduler = treeAnimationElement_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());

	// Get the insert time
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	TimingAttribute insertTime = null;
	TimingAttribute computedInsertTime = null;
	if (0 == sdIndex) {
		insertTime = activeInterval.getObservedBeginTime();
		computedInsertTime = activeInterval.getComputedBeginTime();
	} else {
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);
		computedInsertTime = insertTime;
	}

	// get the delete time
	TimingAttribute computedDeleteTime = null;
	if (activeInterval.getELASids().size() - 1 == sdIndex)
		computedDeleteTime = activeInterval.getComputedEndTime();
	else
		computedDeleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);

	// Get the end of the fill time to know when is the last time routes can be inserted.
	TimingAttribute endFillTime = activeInterval.getFillEndTime();

	// Create the par for the insert Nodes command.
	// Insert TimeSensor and add Interpolator 
	// <par begin="???">
	//   <Insert atNode="???">
	//     <TimeSensor DEF="???" cycleInterval="???"/>
	//     <???Interpolator??? DEF="???" key="???" keyValue="???"/>
	//     <ROUTE fromField="fraction_changed" fromNode="???" toField="set_fraction"  toNode="???"/>
	//   </Insert>
	// </par>
	Element parInsertNodes = xmtaDoc.createElement(DOMXMTA.par);
	parInsertNodes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));
	treeAnimationElement_.addToParNode(parInsertNodes);

	// Create insert Node for the Time Sensor
	// Insert group isn't known yet so don't fill it in.
	Element insertNodes = xmtaDoc.createElement(DOMXMTA.Insert);
	insertNodes.setAttribute(DOMXMTA.atNode, getXmtaNodeContainerId());
	parInsertNodes.appendChild(insertNodes);

	// Create the time sensor.
	Element timeSensor = xmtaDoc.createElement(DOMXMTA.TimeSensor);
	String timeSensorId = treeAnimationElement_.getXmtaTimeSensorId(elasId);
	timeSensor.setAttribute(DOMXMTA.DEF, timeSensorId);
	// If the insert time and the computed begin time aren't the same then we want to start
	// the time sensor x number of seconds into the cycle.  Since the startTime is no longer
	// going to be 0 then the stopTime also has to be set to the end time instead of 0, otherwise
	// the animation will stop at 0.
	if (!computedInsertTime.equals(insertTime)) {
		TimingAttribute startTime = computedInsertTime.subtract(insertTime);
		TimingAttribute stopTime = startTime.add(treeAnimationElement_.getXmtaAttrCycleInterval());
		timeSensor.setAttribute(DOMXMTA.startTime, DOMXMTAParameter.time2Param(startTime.getClockValue()));
		timeSensor.setAttribute(DOMXMTA.stopTime, DOMXMTAParameter.time2Param(stopTime.getClockValue()));
	}
	// set the cycle time to be the duration of the animation motion, so that it cycles through once.
	timeSensor.setAttribute(DOMXMTA.cycleInterval, DOMXMTAParameter.time2Param(treeAnimationElement_.getXmtaAttrCycleInterval().getClockValue()));
	timeSensor.setAttribute(DOMXMTA.loop, treeAnimationElement_.getXmtaAttrLoop());
	xmtaIdTable.put(timeSensorId, timeSensor);
	insertNodes.appendChild(timeSensor);

	// Create the Interpolator.
	//     <???Interpolator??? DEF="???" key="???" keyValue="???"/>
	Animatable animatableObject = treeAnimationElement_.getXmtoAttrTargetObject();
	String xmtoAttrAttributeName = getXmtoAttrAttributeName();
	int xmtaType = animatableObject.getAnimXmtoAttrFieldType(xmtoAttrAttributeName);
	Element interpolator = null;
	switch (xmtaType) {
		case Animatable.TYPE_SFColor :
			interpolator = xmtaDoc.createElement(DOMXMTA.ColorInterpolator);
			break;
		case Animatable.TYPE_SFVec2f :
			interpolator = xmtaDoc.createElement(DOMXMTA.PositionInterpolator2D);
			break;
		default :
			interpolator = xmtaDoc.createElement(DOMXMTA.ScalarInterpolator);
			break;
	}
	// Set the interpolator's id
	String interpolatorId = treeAnimationElement_.getXmtaInterpolatorId(elasId);
	interpolator.setAttribute(DOMXMTA.DEF, interpolatorId);
	// Set the interpolator's interpolation
	interpolator.setAttribute(DOMXMTA.key, treeAnimationElement_.getXmtaAttrKey());
	interpolator.setAttribute(DOMXMTA.keyValue, treeAnimationElement_.getXmtaAttrKeyValue());
	xmtaIdTable.put(interpolatorId, interpolator);
	insertNodes.appendChild(interpolator);

	// create the routes from and to the interpolator
	// Create route from TimeSensor to ColorInterpolator
	// <ROUTE fromField="fraction_changed" fromNode="???" toField="set_fraction"  toNode="???"/>
	// Since this is a route which we know the begin and end node creation is the two above nodes 
	// we can add the the route to the insert into par for nodes instead of in the par for routes
	Element timeColorRoute = xmtaDoc.createElement(DOMXMTA.ROUTE);
	timeColorRoute.setAttribute(DOMXMTA.fromNode, timeSensorId);
	timeColorRoute.setAttribute(DOMXMTA.fromField, DOMXMTA.fraction_changed);
	timeColorRoute.setAttribute(DOMXMTA.toNode, interpolatorId);
	timeColorRoute.setAttribute(DOMXMTA.toField, DOMXMTA.set_fraction);
	insertNodes.appendChild(timeColorRoute);

	// Now we need to create the routes from the interpolator to the target object.
	boolean success = createRoutesFromInterpoator(interpolatorId, insertTime, endFillTime);
	if (!success) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimationElement.createInsertMapping() - exiting success=" + success);
		return success;
	}

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimationElement.createInsertMapping() - exiting success=true");
	return true;
}
/*********************************************************************** 
 * Reset the target element's value.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Replace atNode="???" atField="??" value="???"/&gt;
 *   &lt;/par&gt;
 * <pre>
 **********************************************************************/
private boolean createResetTargetValues(TimingAttribute animationFillEndTime) {
	//
	Document xmtaDoc = treeAnimationElement_.getXmtaDoc();

	// First get the target object.
	Animatable animatableObject = treeAnimationElement_.getXmtoAttrTargetObject();

	// Attempt to get the elasId for the object.
	ConvertElement convertElement = (ConvertElement) animatableObject;
	// if the target object isn't a timing element then search up its parent tree to find the
	// parent who has the timing elasIds.
	while (null != convertElement && !(convertElement instanceof TimingElement))
		convertElement = (ConvertElement) convertElement.getParent();

	// The target doesn't have an elasId. This is possible if it is 
	// in the defs tree and is an xMediaChildren element.  
	// In that case create the route from the begining.
	if (null == convertElement) {
		// Create the par for the replace command to replace the initial value.
		// <par begin="???>
		//   <Replace atNode="???" atField="???" value="???"/>
		// </par>
		Element parNodes = xmtaDoc.createElement(DOMXMTA.par);
		parNodes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(animationFillEndTime.getClockValue()));
		treeAnimationElement_.addToParRoutes(parNodes);

		// Create replace of field with default values
		boolean success = fillResetDefsTargetValue(parNodes, animatableObject);
		return success;
	}

	// Get the intervals of the target object
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(convertElement.getIndex());

	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// First check to see if the target element interval comes after the animationFillEndTime
			// If so then we don't need to continue looping through the target intervals.
			// Thus return.
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			if (observedBeginTime.greaterThan(animationFillEndTime)) {
				//     animFillEndTime  Target
				//     *                ****
				return true;
			}

			// If the target element interval occurs before the animationFillEndTime we don't need to check this active interval.
			// Thus if we only want to check this interval if the end of the target element comes
			// after the begining of the interpolator.  Otherwise we will check the next active interval
			// If they occur at the same time then there is no reason to set the value if it is just going to be
			// deleted anyway.
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (fillEndTime.greaterThan(animationFillEndTime)) {
				//           animFillEndTime
				//         target
				//         **** 
				//           *   
				// Now we need to loop through the simple durations (ELASids) for this active interval
				Vector elasIds = activeInterval.getELASids();
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					int sdIndex = elasId.getSimpleDurIndex().intValue();

					// get the insert time
					TimingAttribute targetInsertTime = null;
					if (0 == sdIndex) {
						targetInsertTime = activeInterval.getObservedBeginTime();
					} else
						targetInsertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

					// get the delete time
					TimingAttribute targetDeleteTime = null;
					if (elasIds.size() - 1 == sdIndex)
						targetDeleteTime = activeInterval.getFillEndTime();
					else
						targetDeleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);

					// Like above check that the target element simple interval contain with the time of the animationFillEndTime
					// First check to see if the target element interval comes after the animationFillEndTime
					// If so then we don't need to continue looping through the target intervals.
					// Thus return.
					if (targetInsertTime.greaterThan(animationFillEndTime)) {
						//     animFillEndTime  Target
						//     *                ****
						return true;
					}

					// If the target element interval occurs before the animationFillEndTime we don't need to check this simple interval.
					// Thus if we only want to check this interval if the end of the target element comes
					// after the animationFillEndTime.  Otherwise we will check the next simple interval
					// If they occur at the same time then there is no reason to set the value if it is just going to be
					// deleted anyway.
					if (targetDeleteTime.greaterThan(animationFillEndTime)) {
						//           animFillEndTime
						//         target
						//         **** 
						//           *   


						// Create the par for the replace command to replace the initial value.
						// <par begin="???>
						//   <Replace atNode="???" atField="???" value="???"/>
						// </par>
						Element parNodes = xmtaDoc.createElement(DOMXMTA.par);
						parNodes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(animationFillEndTime.getClockValue()));
						treeAnimationElement_.addToParRoutes(parNodes);

						// Create replace of field with default values
						boolean success = fillResetTimingTargetValue(parNodes, animatableObject, elasId);
						// since there can only be one simple interval during at animationFillEndTime we are finished.
						return success;
					}
				}
			}
		}
	}

	// 
	return true;
}
/*********************************************************************** 
 * Create the routes from interpolator to the target object.
 * What this requires is finding all the interval times of the target
 * object which overlap with the interpolator.  Then at the begining
 * of the overlaps create a route from the interpolator to the target.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert&gt;
 *       &lt;ROUTE fromField="value_changed"    fromNode="???" toField="???"           toNode="???"/&gt;
 *     &lt;/Insert&gt;
 *   &lt;/par&gt;
 * <pre>
 **********************************************************************/
private boolean createRoutesFromInterpoator(String interpolatorId, TimingAttribute interpolatorInsertTime, TimingAttribute interpolatorEndFillTime) {
	//
	Document xmtaDoc = treeAnimationElement_.getXmtaDoc();

	// First get the target object.
	Animatable animatableObject = treeAnimationElement_.getXmtoAttrTargetObject();

	// Attempt to get the elasId for the object.
	ConvertElement convertElement = (ConvertElement) animatableObject;
	// if the target object isn't a timing element then search up its parent tree to find the
	// parent who has the timing elasIds.
	while (null != convertElement && !(convertElement instanceof TimingElement))
		convertElement = (ConvertElement) convertElement.getParent();

	// The target doesn't have an elasId. This is possible if it is 
	// in the defs tree and is an xMediaChildren element.  
	// In that case create the route from the begining.
	if (null == convertElement) {
		// Create the par for the route insert command.
		// <par begin="???>
		//   <Insert>
		//     <ROUTE fromField="value_changed"    fromNode="???" toField="???"           toNode="???"/>
		//   </Insert>
		// </par>
		Element parInsertRoutes = xmtaDoc.createElement(DOMXMTA.par);
		parInsertRoutes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));
		treeAnimationElement_.addToParRoutes(parInsertRoutes);

		// Create insert Node 
		Element insertRoutes = xmtaDoc.createElement(DOMXMTA.Insert);
		parInsertRoutes.appendChild(insertRoutes);

		// Create route from intepolator to animation target nodeId/field
		boolean success = fillInRoutesFromInterpoatorToDefsTarget(insertRoutes, interpolatorId, animatableObject);
		return success;
	}

	// Get the intervals of the target object
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(convertElement.getIndex());

	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// First check to see if the target element interval comes after the interpolator
			// If so then we don't need to continue looping through the target intervals.
			// Thus return.
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			if (observedBeginTime.greaterThan(interpolatorEndFillTime)) {
				//     interpolator target
				//     ****         ****
				return true;
			}

			// If the target element interval occurs before the interpolator we don't need to check this active interval.
			// Thus if we only want to check this interval if the end of the target element comes
			// after the begining of the interpolator.  Otherwise we will check the next active interval
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (fillEndTime.greaterThan(interpolatorInsertTime)) {
				//           interpolator 
				//         target
				//         **** 
				//           ****
				// Now we need to loop through the simple durations (ELASids) for this active interval
				Vector elasIds = activeInterval.getELASids();
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					int sdIndex = elasId.getSimpleDurIndex().intValue();

					// get the insert time
					TimingAttribute targetInsertTime = null;
					if (0 == sdIndex) {
						targetInsertTime = activeInterval.getObservedBeginTime();
					} else
						targetInsertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

					// get the delete time
					TimingAttribute targetDeleteTime = null;
					if (elasIds.size() - 1 == sdIndex)
						targetDeleteTime = activeInterval.getFillEndTime();
					else
						targetDeleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);

					// Like above check that the target element simple interval overlaps with the time of the interval
					// First check to see if the target element interval comes after the interpolator
					// If so then we don't need to continue looping through the target intervals.
					// Thus return.
					if (targetInsertTime.greaterThan(interpolatorEndFillTime)) {
						//     interpolator target
						//     ****         ****
						return true;
					}

					// If the target element interval occurs before the interpolator we don't need to check this simple interval.
					// Thus if we only want to check this interval if the end of the target element comes
					// after the begining of the interpolator.  Otherwise we will check the next simple interval
					if (targetDeleteTime.greaterThan(interpolatorInsertTime)) {
						//           interpolator 
						//         target
						//         **** 
						//           ****
						TimingAttribute insertTime = null;
						if (targetInsertTime.greaterThan(interpolatorInsertTime))
							insertTime = targetInsertTime;
						else
							insertTime = interpolatorInsertTime;

						// Create the par for the route insert command.
						// <par begin="???>
						//   <Insert>
						//     <ROUTE fromField="value_changed"    fromNode="???" toField="???"           toNode="???"/>
						//   </Insert>
						// </par>
						Element parInsertRoutes = xmtaDoc.createElement(DOMXMTA.par);
						parInsertRoutes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));
						treeAnimationElement_.addToParRoutes(parInsertRoutes);

						// Create insert Node 
						Element insertRoutes = xmtaDoc.createElement(DOMXMTA.Insert);
						parInsertRoutes.appendChild(insertRoutes);

						// Create route from PositionIntepolator2D to Transform2D
						boolean success = fillInRoutesFromInterpoatorToTimingTarget(insertRoutes, interpolatorId, animatableObject, elasId);
					}
				}
			}
		}
	}

	// 
	return true;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimationElement.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doTimingMapping();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimationElement.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
private boolean doTimingMapping() {
	boolean success = true;

	//
	//	EventModel eventModel = getConvertElement().getXmto2Xmta().getEventModel();
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(getConvertElement().getIndex());

	// debug purposes
	// Go through it once for printing out debug info irregardless of whether it is correct or not.
	if (debug_) {
		int lifeCycleCount = 0;
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
			int activeIntervalCount = 0;
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				//
				StringBuffer sb = new StringBuffer();
				sb.append("AniAnimationElement.doTimingMapping() -");
				sb.append(" <").append(getConvertElement().getXmtoElement().getNodeName());
				String id = getConvertElement().getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
				sb.append(" activeInterval[").append(activeIntervalCount).append("]");
				sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
				sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
				sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
				sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
				sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
				ELASid parentElasId = lifeCycle.getParentELASid();
				sb.append(" parentElasId=(");
				if (null == parentElasId) {
					sb.append("null");
				} else {
					sb.append(parentElasId.getElementIndex());
					sb.append(",").append(parentElasId.getLifeCycleIndex());
					sb.append(",").append(parentElasId.getIntervalIndex());
					sb.append(",").append(parentElasId.getSimpleDurIndex());
				}
				sb.append(")");
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					sb.append(" repeatTime=").append(repeatTime);
				}
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					sb.append(" elasId=(").append(elasId.getElementIndex());
					sb.append(",").append(elasId.getLifeCycleIndex());
					sb.append(",").append(elasId.getIntervalIndex());
					sb.append(",").append(elasId.getSimpleDurIndex());
					sb.append(")");
				}
				fireErrorEvent(ErrorEvent.TRACE, 0, sb.toString(), getConvertElement().getXmtoElement());

				//
				activeIntervalCount++;
			}
			lifeCycleCount++;
		}
	}


	// Now go through for real.
	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// Check to make sure the timing values are valid
			boolean error = false;
			TimingAttribute computedBeginTime = activeInterval.getComputedBeginTime();
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			TimingAttribute activeDuration = activeInterval.getActiveDuration();
			//			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			//				|| null == fillEndTime)
			if (null == computedBeginTime || null == observedBeginTime || null == activeDuration)
				error = true;
			if (!error) {
				switch (computedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (observedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (activeDuration.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			//			if (!error) {
			//				switch (fillEndTime.getTimeValueType()) {
			//					case TimingAttribute.CLOCK_VALUE :
			//					case TimingAttribute.INDEFINITE_VALUE :
			//						break;
			//					default :
			//						error = true;
			//				}
			//			}
			if (!error) {
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					switch (repeatTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
			}

			// If there is a problem then report back the problem and remove the object.
			if (error) {
				MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
				StringBuffer buffer = new StringBuffer();
				buffer.append(getConvertElement().getXmtoElement().getNodeName());
				if (null != getConvertElement().getXmtoAttrId() && !getConvertElement().getXmtoAttrId().equals(""))
					buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getConvertElement().getXmtoAttrId()).append("\"");
				Object[] msgArgs = {buffer};
				String message = msgFormat.format(msgArgs);
				fireErrorEvent(ErrorEvent.ERROR, 0, message, getConvertElement().getXmtoElement());

				// Since we don't support the timings for this element remove it from the tree
				getConvertElement().remove();
				return false;
			}

			// All the timings check out OK so now loop through the ELASids and create the nodes.
			// NOTE: the timing for animations done differently than the other dealings with
			// intervals.  Here we have one mapping for an active interval, other places we have
			// a mapping for each simple interval
			// Map the timings using the entire activeInterval.  Thus only pass in the first
			// elasid to the routines
			Vector elasIds = activeInterval.getELASids();
			ELASid elasId = (ELASid) elasIds.firstElement();

			// Check if the simple duration is indefinite.  If it is then the interval for this
			// animation can't be determined and is dropped.  All we need to do is
			// get the first simple duration to check the duration of it.
			TimingAttribute computedInsertTime = activeInterval.getComputedBeginTime();
			// get the delete time
			TimingAttribute computedDeleteTime = null;
			if (1 == elasIds.size())
				computedDeleteTime = activeInterval.getComputedEndTime();
			else
				computedDeleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(0);
			// get the simple Duration (delete minus insert times)
			TimingAttribute simpleDuration = computedDeleteTime.subtract(computedInsertTime);
			switch (simpleDuration.getTimeValueType()) {
				case TimingAttribute.CLOCK_VALUE :
					// Create the insert Mapping.
					success = createInsertMapping(elasId);
					if (!success)
						return success;

					// Create the delete Mapping.
					success = createDeleteMapping(elasId);
					if (!success)
						return success;
					break;
				case TimingAttribute.INDEFINITE_VALUE :
					// Duration is indefinite and needs to be dropped.  This can happen on the last
					// simple duration in the interval.
					MessageFormat msgFormat = new MessageFormat(animationElementRes_.getString(AnimationElementRes.MSG_WARNING_ANIMATION_DURATION_INDEFINTE));
					StringBuffer sbElement = new StringBuffer();
					sbElement.append(getConvertElement().getXmtoElement().getNodeName());
					if (null != getConvertElement().getXmtoAttrId() && !getConvertElement().getXmtoAttrId().equals(""))
						sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getConvertElement().getXmtoAttrId()).append("\"");
					String startTime = activeInterval.getComputedBeginTime().toString();
					Object[] msgArgs = {sbElement, startTime};
					String message = msgFormat.format(msgArgs);
					fireErrorEvent(ErrorEvent.WARNING, 0, message, getConvertElement().getXmtoElement());
					break;
			}
		}
	}

	// 
	return true;
}
/***********************************************************************
 * Create the route from the interpolator to the correct node and field.
 * <pre>
 *   &lt;ROUTE fromField="value_changed"    fromNode="???" toField="???"           toNode="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillInRoutesFromInterpoator(Element insertRoutes, String interpolatorId, String targetNodeId, String targetField) {
	//
	Document xmtaDoc = getConvertElement().getXmtaDoc();

	// Create route from intepolator to target node/field
	Element positionTransformRoute = xmtaDoc.createElement(DOMXMTA.ROUTE);
	positionTransformRoute.setAttribute(DOMXMTA.fromNode, interpolatorId);
	positionTransformRoute.setAttribute(DOMXMTA.fromField, DOMXMTA.value_changed);
	positionTransformRoute.setAttribute(DOMXMTA.toNode, targetNodeId);
	positionTransformRoute.setAttribute(DOMXMTA.toField, targetField);
	insertRoutes.appendChild(positionTransformRoute);

	//
	return true;
}
/*********************************************************************** 
 * Create the route from the interpolator to the correct node and field.
 * <pre>
 *   &lt;ROUTE fromField="value_changed"    fromNode="???" toField="???"           toNode="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillInRoutesFromInterpoatorToDefsTarget(Element insertRoutes, String interpolatorId, Animatable target) {
	//
	String xmtoAttrAttributeName = getXmtoAttrAttributeName();
	String animXmtaNodeId = target.getAnimXmtaDefsNodeId(xmtoAttrAttributeName);
	String animXmtaAttrName = target.getAnimXmtaAttrName(xmtoAttrAttributeName);

	// Create route from intepolator to target node/field
	boolean success = fillInRoutesFromInterpoator(insertRoutes, interpolatorId, animXmtaNodeId, animXmtaAttrName);

	//
	return success;
}
/***********************************************************************
 * Create the route from the interpolator to the correct node and field.
 * <pre>
 *   &lt;ROUTE fromField="value_changed"    fromNode="???" toField="???"           toNode="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillInRoutesFromInterpoatorToTimingTarget(Element insertRoutes, String interpolatorId, Animatable target, ELASid targetElasId) {
	//
	String xmtoAttrAttributeName = getXmtoAttrAttributeName();
	String animXmtaNodeId = target.getAnimXmtaNodeId(xmtoAttrAttributeName, targetElasId);
	String animXmtaAttrName = target.getAnimXmtaAttrName(xmtoAttrAttributeName);

	// Create route from intepolator to target node/field
	boolean success = fillInRoutesFromInterpoator(insertRoutes, interpolatorId, animXmtaNodeId, animXmtaAttrName);

	//
	return success;
}
/*********************************************************************** 
 * Reset the target element's value.
 * <pre>
 *   &lt;Replace atNode="???" atField="??" value="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillResetDefsTargetValue(Element parNodes, Animatable target) {
	//
	String xmtoAttrAttributeName = getXmtoAttrAttributeName();
	String animXmtaNodeId = target.getAnimXmtaDefsNodeId(xmtoAttrAttributeName);
	String animXmtaAttrName = target.getAnimXmtaAttrName(xmtoAttrAttributeName);
	String animXmtaAttrInitialValue = target.getAnimXmtaAttrInitialValue(xmtoAttrAttributeName);

	// Create route from intepolator to target node/field
	boolean success = fillResetTargetValue(parNodes, animXmtaNodeId, animXmtaAttrName, animXmtaAttrInitialValue);

	//
	return success;
}
/*********************************************************************** 
 * Reset the target element's value.
 * <pre>
 *   &lt;Replace atNode="???" atField="??" value="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillResetTargetValue(Element parNodes, String animXmtaNodeId, String animXmtaAttrName, String animXmtaAttrInitialValue) {
	//
	Document xmtaDoc = getConvertElement().getXmtaDoc();

	// Create route from intepolator to target node/field
	Element replace = xmtaDoc.createElement(DOMXMTA.Replace);
	replace.setAttribute(DOMXMTA.atNode, animXmtaNodeId);
	replace.setAttribute(DOMXMTA.atField, animXmtaAttrName);
	replace.setAttribute(DOMXMTA.value, animXmtaAttrInitialValue);
	parNodes.appendChild(replace);

	//
	return true;
}
/*********************************************************************** 
 * Reset the target element's value.
 * <pre>
 *   &lt;Replace atNode="???" atField="??" value="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillResetTimingTargetValue(Element parNodes, Animatable target, ELASid targetElasId) {
	//
	String xmtoAttrAttributeName = getXmtoAttrAttributeName();
	String animXmtaNodeId = target.getAnimXmtaNodeId(xmtoAttrAttributeName, targetElasId);
	String animXmtaAttrName = target.getAnimXmtaAttrName(xmtoAttrAttributeName);
	String animXmtaAttrInitialValue = target.getAnimXmtaAttrInitialValue(xmtoAttrAttributeName);

	// Create route from intepolator to target node/field
	boolean success = fillResetTargetValue(parNodes, animXmtaNodeId, animXmtaAttrName, animXmtaAttrInitialValue);

	//
	return success;
}
/*********************************************************************** 
 * Return the name of the attribute to be animated.
 **********************************************************************/
abstract protected String getXmtoAttrAttributeName();
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof AnimationElement)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimationElement.init() - element=" + element + " is not an instance of AnimationElement");
		throw new Exception("");
	}

	//
	treeAnimationElement_ = (AnimationElement) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniAnimationElement.init() - treeAnimationElement_=" + treeAnimationElement_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
