/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  AniSet.java
//                                                                             
//=============================================================================
package xmto2xmta.mapanimation;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.MapRoot;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.resources.SetRes;
import xmto2xmta.tree.Animatable;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.group;
import xmto2xmta.tree.MediaElement;
import xmto2xmta.tree.Sensorable;
import xmto2xmta.tree.TimingElement;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.set;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to handle the conversion of the set element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>19 Apr 2001 - Add simple lifecycle support.  
 * <li>20 Apr 2000 - Remove resolve model, replaced by TimeScheduler.
 *                 - add simple support for begin event (for &lt;set&gt;)
 * <li>06 Jun 2001 - Fix bug, delete time is start+activeduration on last simple interval.
 * <li>21 Jun 2001 - Enable the touch sensor only when it is used.
 * <li>30 Jun 2001 - Fix bug, animatable is now the parent not the media object.
 * <li>02 Jul 2001 - Add support for setting all animatable attributes.
 * </ul><p>
 *********************************************************************/
public class AniSet extends AniAnimationBaseElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private set treeSet_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
	private ResourceBundle setRes_ = ResourceBundle.getBundle("xmto2xmta.resources.SetRes", Locale.getDefault());
/***********************************************************************
 * Delete the group containing the conditionals and valuator.
 * This method also should reset the value but we are not determining
 * the interval time of the set.
 * For now the timing of the set can be figured out by the time of its
 * parent timing element.  
 * <p>
 * All the conditionals and the valuator will be created under a group to make it easier
 * to remove them all at the end of the set interval.
 * <pre>
 * &lt;par begin="???"&gt;
 *   &lt;Delete atNode"???"/&gt;
 * &lt;/par&gt;
 * </pre>  
 **********************************************************************/
private boolean createDeleteEventMapping(String groupId) {

	//
	TimeScheduler timeScheduler = treeSet_.getXmto2Xmta().getTimeScheduler();

	//
	Document xmtaDoc = treeSet_.getXmtaDoc();

	// Get the parent timing element for the set
	TimingAttribute setDeleteTime = null;
	TreeElement parent = treeSet_.getParent();
	while (null != parent && !(parent instanceof TimingElement))
		parent = parent.getParent();
	if (null != parent) {
		TimingElement timingElement = null;
		timingElement = (TimingElement) parent;
		// Now determine the lastest time the set can exist.  We don't
		// need to worry about different set (or parent) intervals.  But we need to know
		// the latest time it can exist.

		// First loop through the life cycles
		Vector lifeCycles = timeScheduler.getLifeCycles(timingElement.getIndex());
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

			// Loop through the active intervals
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				// setDeleteTime is the end of interval (endFillTime)
				setDeleteTime = activeInterval.getFillEndTime();
			}
		}
	}

	//		
	if (null == setDeleteTime) {
		// report back the problem
		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
		StringBuffer buffer = new StringBuffer();
		buffer.append(treeSet_.getXmtoElement().getNodeName());
		String id = treeSet_.getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		Object[] msgArgs = {buffer};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.ERROR, 0, message, treeSet_.getXmtoElement());

		// Since we don't support the timings for this element remove it from the tree
		treeSet_.remove();
		return false;
	}

	// if the delete time is indefinite then we don't need to delete the object.
	if (TimingAttribute.INDEFINITE_VALUE == setDeleteTime.getTimeValueType())
		return true;

	//	Create the par delte Node
	// <par begin="???">
	//   <Delete atNode"???"/>
	// </par>
	Element parDeleteNode = xmtaDoc.createElement(DOMXMTA.par);
	parDeleteNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(setDeleteTime.getClockValue()));
	treeSet_.addToParNode(parDeleteNode);
	// create the delete
	Element deleteNodes = xmtaDoc.createElement(DOMXMTA.Delete);
	deleteNodes.setAttribute(DOMXMTA.atNode, groupId);
	parDeleteNode.appendChild(deleteNodes);

	//
	return true;
}
/***********************************************************************
 * Don't do anything for the deletion.
 **********************************************************************/
private boolean createDeleteMapping(ELASid elasId) {
	return true;
}
/***********************************************************************
 * Create the event mapping for the set element.  For the event we have three timings to 
 * consider, the timing of the set, the target (receiver) and the sensor
 * object.  
 * For now the timing of the set can be figured out by the time of its
 * parent timing element.  The timing of the target and sensor can be
 * obtained by those objects.
 * <p>
 * In essence the structure of the set is to have a sensor object (from
 * touchSensor) route an event to a valuator.  The valuator converts the
 * time event into a activate event.  The activate event is routed to the
 * conditional which contains the command to replace the value.  The replace
 * is the actual setting of the value.
 * <p>
 * The problem is that there can be multiple timing intervals for both the
 * target and the sensor objects.  There can also be multiple timing intervals
 * for the set, but for now the event timing is only determined by the parent
 * maximum begin and end.  Thus it can look like the following:
 * <pre>
 * timeline           0    5   10   15   20   25
 * sensor intervals   ****  ****  **** **** ****
 * set interval              *************
 * target intervals     ** ** ** ** ** ** ** **
 * </pre>
 * <p>
 * In order to take care the multiple interval problems:
 * <ul>
 * <li>The valuator will last for the entire interval of the set.  
 * <li>Create a route from each time sensor to the valuator.  
 *     The route will disappear when the time sensor is deleted.
 * <li>Create a conditional for each target interval.  Each conditional has to have
 *     a replace specific to each target interval.  A route will be set up between
 *     the valuator and each conditional.  In the case of a def object there is only
 *     one target interval.
 * </ul>
 * <p>
 * All the conditionals and the valuator will be created under a group to make it easier
 * to remove them all at the end of the set interval.
 * <p>
 * Thus the normal event set infrastructure is:
 * <ol>
 * <li>Group to hold the valuator and conditionals,
 * <li>Valuator.
 * <li>List of conditionals each with a replace.  There is a conditional/replace
 *     for each target interval. with a route from the valuator.
 * <li>List of sensor routes.  Each sensor route goes from the valuator to each
 *     sensor interval.
 * </ol>
 * 
 * <pre>
 * &lt;par begin="???"&gt;
 *   &lt;Insert atNode"???"&gt;
 *     &lt;group DEF="???"&gt;
 *       &lt;children&gt;
 *         &lt;Valuator DEF="???" Factor1="0" Offset1="1"/&gt;
 *       &lt;/children&gt;
 *     &lt;/group&gt;
 *   &lt;/Insert&gt;
 *
 *   &lt;Insert atNode"???"&gt;
 *       &lt;Conditional DEF="???"&gt;
 *         &lt;buffer&gt;
 *           &lt;Replace atNode="???" atField="??" value="???"/&gt;
 *         &lt;/buffer&gt;
 *       &lt;/Conditional&gt;
 *       &lt;ROUTE fromField="outSFBool" fromNode="???" toField="activate" toNode="???"/&gt;
 *   &lt;/Insert&gt;
 * &lt;/par&gt;
 *
 * &lt;par begin="???"&gt;
 *   &lt;Insert&gt;
 *     &lt;ROUTE fromField="touchTime" fromNode="???" toField="inSFTime" toNode="???"/&gt;
 *   &lt;/Insert&gt;
 * &lt;/par&gt;
 * </pre>  
 **********************************************************************/
private boolean createInsertEventMapping(String groupId) {

	//
	EventModel eventModel = treeSet_.getXmto2Xmta().getEventModel();
	TimeScheduler timeScheduler = treeSet_.getXmto2Xmta().getTimeScheduler();

	//
	Document xmtaDoc = treeSet_.getXmtaDoc();
	Hashtable xmtaIdTable = treeSet_.getXmtaIds();

	// Get the targe object that receives the set animation
	Animatable targetElement = treeSet_.getXmtoAttrTargetObject();

	// Get the parent timing element for the set
	TimingAttribute setInsertTime = null;
	TimingAttribute setDeleteTime = null;
	TreeElement parent = treeSet_.getParent();
	while (null != parent && !(parent instanceof TimingElement))
		parent = parent.getParent();
	if (null != parent) {
		TimingElement timingElement = null;
		timingElement = (TimingElement) parent;
		// Now determine the earliest and lastest time the set can exist.  We don't
		// need to worry about different set (or parent) intervals.  We will create
		// one valuator and connect all target and sensors up to it.  But we need to know
		// the earliest and latest time it can exist.

		// First loop through the life cycles
		Vector lifeCycles = timeScheduler.getLifeCycles(timingElement.getIndex());
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

			// Loop through the active intervals
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				// first observed begin time is the setInsertTime.  This is the
				// first observed begin time if we haven't yet initialized
				// setInsertTime.
				if (null == setInsertTime)
					setInsertTime = activeInterval.getObservedBeginTime();

				// setDeleteTime is the end of interval (endFillTime)
				setDeleteTime = activeInterval.getFillEndTime();
			}
		}
	}

	// Get the sensor object that sends the signal.
	TimingAttribute eventBeginTime = eventModel.getSceneBeginTime(treeSet_.getIndex());
	Sensorable sensorElement = null;
	if (null != eventBeginTime && TimingAttribute.EVENT_VALUE == eventBeginTime.getTimeValueType()) {
		switch (eventBeginTime.getEventSymbol()) {
			case TimingAttribute.SYMBOL_EVENT_CLICK :
			case TimingAttribute.SYMBOL_EVENT_ACTIVATE_EVENT :
				String sensorId = eventBeginTime.getIdValue();
				if (null == sensorId || sensorId.equals("")) {
					if (treeSet_ instanceof Sensorable)
						sensorElement = (Sensorable) treeSet_;
				} else {
					Object obj = treeSet_.getXmtoIds().get(sensorId);
					if (obj instanceof Sensorable && obj instanceof TimingElement)
						sensorElement = (Sensorable) obj;
				}
				break;
		}
	}

	//		
	if (null == targetElement || null == sensorElement || null == setInsertTime || null == setDeleteTime) {
		// report back the problem
		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
		StringBuffer buffer = new StringBuffer();
		buffer.append(treeSet_.getXmtoElement().getNodeName());
		String id = treeSet_.getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		Object[] msgArgs = {buffer};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.ERROR, 0, message, treeSet_.getXmtoElement());

		// Since we don't support the timings for this element remove it from the tree
		treeSet_.remove();
		return false;
	}

	//	// Create the par insert Node
	// <par begin="???">
	//   <Insert atNode"???">
	//     <group DEF="???">
	//       <Valuator DEF="???" Factor1="0" Offset1="1"/>
	//     </group>
	//   </Insert>
	// </par>
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(setInsertTime.getClockValue()));
	treeSet_.addToParNode(parInsertNode);
	// create the insert
	Element insertNodes = xmtaDoc.createElement(DOMXMTA.Insert);
	String containerId = getXmtaNodeContainerId();
	insertNodes.setAttribute(DOMXMTA.atNode, containerId);
	parInsertNode.appendChild(insertNodes);
	// create the group
	Element group = xmtaDoc.createElement(DOMXMTA.Group);
	group.setAttribute(DOMXMTA.DEF, groupId);
	xmtaIdTable.put(groupId, group);
	insertNodes.appendChild(group);
	// 
	Node children = group.appendChild(xmtaDoc.createElement(DOMXMTA.children));
	//
	String valuatorId = IdManager.getNextId(IdManager.ValuatorID);
	String id = treeSet_.getXmtoAttrId();
	if (null != id && !id.equals(""))
		valuatorId = valuatorId + ":" + id;
	// create the valuator
	Element valuator = xmtaDoc.createElement(DOMXMTA.Valuator);
	valuator.setAttribute(DOMXMTA.DEF, valuatorId);
	valuator.setAttribute(DOMXMTA.Factor1, "0");
	valuator.setAttribute(DOMXMTA.Offset1, "1");
	xmtaIdTable.put(valuatorId, valuator);
	children.appendChild(valuator);

	// Now create the insert for the conditionals
	// Create a separate insert node instead of placing them directly
	// in the group node.  This is done so that all the Replaces can
	// be added to the insert by passing in the Insert node.  If the
	// group was passed in the routes would have to be handled specially.
	//   <Insert atNode"???">
	//       <Conditional DEF="???">
	//         <buffer>
	//           <Replace atNode="???" atField="??" value="???"/>
	//         </buffer>
	//       </Conditional>
	//       <ROUTE fromField="outSFBool" fromNode="???" toField="activate" toNode="???"/>
	//   </Insert>
	// create the insert
	insertNodes = xmtaDoc.createElement(DOMXMTA.Insert);
	insertNodes.setAttribute(DOMXMTA.atNode, groupId);
	parInsertNode.appendChild(insertNodes);
	// Now add the conditionals to the insert node.
	boolean success = createInsertEventMappingTarget(insertNodes, valuatorId, setInsertTime, setDeleteTime, targetElement);
	if (!success)
		return success;

	// Now create all the routes between the sensor element elasIds and the valuator
	// <par begin="???">
	//   <Insert>
	//     <ROUTE fromField="touchTime" fromNode="???" toField="inSFTime" toNode="???">
	//   </Insert>
	// </par>
	success = createInsertEventMappingSensor(valuatorId, setInsertTime, setDeleteTime, sensorElement);
	if (!success)
		return success;

	//
	return true;
}
/***********************************************************************
 * Create the sensor mapping for the set infrastructure for event timing.
 * This is to create a route from the sensor object to the valuator.
 * This method goes through all the intervals of the sensor element
 * adding a route from the sensor's touchsensor to the valuator.
 * It is also necessary to enable the touchsensor.
 * <p>
 * The route has to be inserted on the later of the insert time of the set interval
 * and the insert time of the sensor interval.
 * <pre>
 *   &lt;par begin="???"&gt;
 *	   &lt;Insert&gt;
 *       &lt;ROUTE fromField="touchTime" fromNode="???" toField="inSFTime" toNode="???"/&gt;
 *	   &lt;/Insert&gt;
 *   &lt;/par&gt;
 *   &lt;par begin="???"&gt;
 *	   &lt;Replace atNode="???" atField="enabled" value="true"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createInsertEventMappingSensor(String valuatorId, TimingAttribute setInsertTime, TimingAttribute setDeleteTime, Sensorable sensorObject) {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingSensor() - entered");

	//
	Document xmtaDoc = treeSet_.getXmtaDoc();
	String setId = treeSet_.getXmtoAttrId();


	// Attempt to get the elasId for the object.
	if (!(sensorObject instanceof TimingElement)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingSensor() - exiting sensor not a TimingElement success=false");
		return false;
	}
	TimingElement sensorTimingElement = (TimingElement) sensorObject;

	// Get the intervals of the sensor object
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(sensorTimingElement.getIndex());

	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// First check to see if the Sensor element interval comes after the setDeleteTime
			// If so then we don't need to continue looping through the Sensor intervals.
			// Thus return.
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			if (observedBeginTime.greaterThan(setDeleteTime)) {
				//     setDeleteTime  Sensor
				//     *              ****
				if (debug_)
					fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingSensor() - exiting success=true");
				return true;
			}

			// If the Sensor element interval occurs before the setInsertTime we don't need to check this active interval.
			// Thus if we only want to check this interval if the end of the Sensor element comes
			// after the set.  Otherwise we will check the next active interval
			// If they occur at the same time then there is no reason to set the value if it is just going to be
			// deleted anyway.
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (fillEndTime.greaterThan(setInsertTime)) {
				//           setInsertTime
				//         Sensor
				//         **** 
				//           *   
				// Now we need to loop through the simple durations (ELASids) for this active interval
				Vector elasIds = activeInterval.getELASids();
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					int sdIndex = elasId.getSimpleDurIndex().intValue();

					// get the insert time
					TimingAttribute sensorInsertTime = null;
					if (0 == sdIndex) {
						sensorInsertTime = activeInterval.getObservedBeginTime();
					} else
						sensorInsertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

					// get the delete time
					TimingAttribute sensorDeleteTime = null;
					if (elasIds.size() - 1 == sdIndex)
						sensorDeleteTime = activeInterval.getFillEndTime();
					else
						sensorDeleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);

					// Like above check that the Sensor element simple interval contain with the time of the setInsertTime
					// First check to see if the Sensor element interval comes after the setDeleteTime
					// If so then we don't need to continue looping through the Sensor intervals.
					// Thus return.
					if (sensorInsertTime.greaterThan(setDeleteTime)) {
						//     setDeleteTime  Sensor
						//     *              ****
						if (debug_)
							fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingSensor() - exiting success=true");
						return true;
					}

					// If the Sensor element interval occurs before the setInsertTime we don't need to check this simple interval.
					// Thus if we only want to check this interval if the end of the Sensor element comes
					// after the setInsertTime.  Otherwise we will check the next simple interval
					// If they occur at the same time then there is no reason to set the value if it is just going to be
					// deleted anyway.
					if (sensorDeleteTime.greaterThan(setInsertTime)) {
						//           setInsertTime
						//         Sensor
						//         **** 
						//           *   

						// 
						// The route has to be inserted on the later of the insert time of the set interval
						// and the insert time of the sensor interval.
						TimingAttribute insertTime = sensorInsertTime.greaterThan(setInsertTime) ? sensorInsertTime : setInsertTime;

						//	// Create the par insert Route
						// <par begin="???">
						//   <Insert>
						//     <ROUTE fromField="touchTime" fromNode="???" toField="inSFTime" toNode="???"/>
						//   </Insert>
						// </par>
						Element parInsertRoute = xmtaDoc.createElement(DOMXMTA.par);
						parInsertRoute.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));
						treeSet_.addToParRoutes(parInsertRoute);
						// create the insert
						Node insertRoute = parInsertRoute.appendChild(xmtaDoc.createElement(DOMXMTA.Insert));
						// create the route from the touchsensor to the valuator 
						Element route = xmtaDoc.createElement(DOMXMTA.ROUTE);
						String touchSensorId = sensorObject.getXmtaTouchSensorId(elasId);
						route.setAttribute(DOMXMTA.fromNode, touchSensorId);
						route.setAttribute(DOMXMTA.fromField, DOMXMTA.touchTime);
						route.setAttribute(DOMXMTA.toNode, valuatorId);
						route.setAttribute(DOMXMTA.toField, DOMXMTA.inSFTime);
						insertRoute.appendChild(route);

						// Enable the touch sensor
						// <par begin="???">
						//   <Replace atNode="???" atField="enabled" value="true"/>
						// </par>
						// Create the par for the replace command.
						// NOTE: add it to the par routes to guarantee that it is added after
						// the nodes are created on the tree
						Element parReplace = xmtaDoc.createElement(DOMXMTA.par);
						parReplace.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(sensorInsertTime.getClockValue()));
						treeSet_.addToParRoutes(parReplace);

						// Create replace Node, enabling the touch sensor
						Element replace = xmtaDoc.createElement(DOMXMTA.Replace);
						replace.setAttribute(DOMXMTA.atNode, touchSensorId);
						replace.setAttribute(DOMXMTA.atField, DOMXMTA.enabled);
						replace.setAttribute(DOMXMTA.value, DOMXMTA.string_true);
						parReplace.appendChild(replace);



						// don't return here becuase the set could occur
						// in more than one Sensor interval.
					}
				}
			}
		}
	}

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingSensor() - exiting success=true");
	return true;
}
/***********************************************************************
 * Create the target mapping for the set infrastructure for event timing.
 * The normal set infrastructure is a replace command.  For the
 * event timing wrap the replace in a conditional which can be
 * executed when ever the touch sensor is activated.
 * <p>
 * This method goes through all the intervals of the target element
 * adding a conditional to replace the value, then  connecting the
 * conditional to the valuator with a route.
 * In the case of a def object there is only one target interval.
 * <p>
 * Thus the steps for each target interval are:
 * <ol>
 * <li>Add the conditional with the replace command into the scene
 * <li>Create a route from the valuator to the conditional
 * </ol>
 * <pre>
 *     &lt;Conditional DEF="???"&gt;
 *       &lt;buffer&gt;
 *         &lt;Replace atNode="???" atField="??" value="???"/&gt;
 *       &lt;/buffer&gt;
 *     &lt;/Conditional&gt;
 *     &lt;ROUTE fromField="outSFBool" fromNode="???" toField="activate" toNode="???"/&gt;
 * </pre>
 **********************************************************************/
private boolean createInsertEventMappingTarget(Element insertNodes, String valuatorId, TimingAttribute setInsertTime, TimingAttribute setDeleteTime, Animatable animatableObject) {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingTarget() - entered");

	//
	Document xmtaDoc = treeSet_.getXmtaDoc();
	String setId = treeSet_.getXmtoAttrId();


	// Attempt to get the elasId for the object.
	ConvertElement convertElement = (ConvertElement) animatableObject;
	// if the target object isn't a timing element then search up its parent tree to find the
	// parent who has the timing elasIds.
	while (null != convertElement && !(convertElement instanceof TimingElement))
		convertElement = (ConvertElement) convertElement.getParent();

	// The target doesn't have an elasId. This is possible if it is 
	// in the defs tree and is an xMediaChildren element.  
	// In that case create the route from the begining.
	if (null == convertElement) {
		// Create the conditional for the replace command
		//    <Conditional DEF="???">
		//      <buffer>
		//        <Replace atNode="???" atField="??" value="???"/>
		//      </buffer>
		//    </Conditional>
		//    <ROUTE fromField="outSFBool" fromNode="???" toField="activate" toNode="???"/>
		Element conditional = xmtaDoc.createElement(DOMXMTA.Conditional);
		String conditionalId = IdManager.getNextId(IdManager.ConditionalID);
		if (null != setId && !setId.equals(""))
			conditionalId = conditionalId + ":" + setId;
		conditional.setAttribute(DOMXMTA.DEF, conditionalId);
		insertNodes.appendChild(conditional);
		Element buffer = xmtaDoc.createElement(DOMXMTA.buffer);
		conditional.appendChild(buffer);
		// Create replace of field
		boolean success = fillInReplaceDefsTarget(buffer, animatableObject);
		if (!success)
			return success;

		// create the route from the valuator to the conditional
		Element route = xmtaDoc.createElement(DOMXMTA.ROUTE);
		route.setAttribute(DOMXMTA.fromNode, valuatorId);
		route.setAttribute(DOMXMTA.fromField, DOMXMTA.outSFBool);
		route.setAttribute(DOMXMTA.toNode, conditionalId);
		route.setAttribute(DOMXMTA.toField, DOMXMTA.activate);
		insertNodes.appendChild(route);

		//
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingTarget() - exiting success=true");
		return true;
	}



	// Get the intervals of the target object
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(((ConvertElement) convertElement).getIndex());

	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// First check to see if the target element interval comes after the setDeleteTime
			// If so then we don't need to continue looping through the target intervals.
			// Thus return.
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			if (observedBeginTime.greaterThan(setDeleteTime)) {
				//     setDeleteTime  Target
				//     *              ****
				if (debug_)
					fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingTarget() - exiting success=true");
				return true;
			}

			// If the target element interval occurs before the setInsertTime we don't need to check this active interval.
			// Thus if we only want to check this interval if the end of the target element comes
			// after the set.  Otherwise we will check the next active interval
			// If they occur at the same time then there is no reason to set the value if it is just going to be
			// deleted anyway.
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (fillEndTime.greaterThan(setInsertTime)) {
				//           setInsertTime
				//         target
				//         **** 
				//           *   
				// Now we need to loop through the simple durations (ELASids) for this active interval
				Vector elasIds = activeInterval.getELASids();
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					int sdIndex = elasId.getSimpleDurIndex().intValue();

					// get the insert time
					TimingAttribute targetInsertTime = null;
					if (0 == sdIndex) {
						targetInsertTime = activeInterval.getObservedBeginTime();
					} else
						targetInsertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

					// get the delete time
					TimingAttribute targetDeleteTime = null;
					if (elasIds.size() - 1 == sdIndex)
						targetDeleteTime = activeInterval.getFillEndTime();
					else
						targetDeleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);

					// Like above check that the target element simple interval contain with the time of the setInsertTime
					// First check to see if the target element interval comes after the setDeleteTime
					// If so then we don't need to continue looping through the target intervals.
					// Thus return.
					if (targetInsertTime.greaterThan(setDeleteTime)) {
						//     setDeleteTime  Target
						//     *              ****
						if (debug_)
							fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingTarget() - exiting success=true");
						return true;
					}

					// If the target element interval occurs before the setInsertTime we don't need to check this simple interval.
					// Thus if we only want to check this interval if the end of the target element comes
					// after the setInsertTime.  Otherwise we will check the next simple interval
					// If they occur at the same time then there is no reason to set the value if it is just going to be
					// deleted anyway.
					if (targetDeleteTime.greaterThan(setInsertTime)) {
						//           setInsertTime
						//         target
						//         **** 
						//           *   

						// Create the conditional for the replace command
						//    <Conditional DEF="???">
						//      <buffer>
						//        <Replace atNode="???" atField="??" value="???"/>
						//      </buffer>
						//    </Conditional>
						//    <ROUTE fromField="outSFBool" fromNode="???" toField="activate" toNode="???"/>
						Element conditional = xmtaDoc.createElement(DOMXMTA.Conditional);
						String conditionalId = IdManager.getNextId(IdManager.ConditionalID);
						if (null != setId && !setId.equals(""))
							conditionalId = conditionalId + ":" + setId;
						conditional.setAttribute(DOMXMTA.DEF, conditionalId);
						insertNodes.appendChild(conditional);
						Element buffer = xmtaDoc.createElement(DOMXMTA.buffer);
						conditional.appendChild(buffer);
						// Create replace of field
						boolean success = fillInReplaceTimingTarget(buffer, animatableObject, elasId);
						if (!success) {
							if (debug_)
								fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingTarget() - exiting success=" + success);
							return success;
						}

						// create the route from the valuator to the conditional
						Element route = xmtaDoc.createElement(DOMXMTA.ROUTE);
						route.setAttribute(DOMXMTA.fromNode, valuatorId);
						route.setAttribute(DOMXMTA.fromField, DOMXMTA.outSFBool);
						route.setAttribute(DOMXMTA.toNode, conditionalId);
						route.setAttribute(DOMXMTA.toField, DOMXMTA.activate);
						insertNodes.appendChild(route);

						// don't return here becuase the set could occur
						// in more than one target interval.
					}
				}
			}
		}
	}

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertEventMappingTarget() - exiting success=true");
	return true;
}
/***********************************************************************
 * Create the set infrastructure.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Replace atNode="???" atField="inSFInt32" value="1"/&gt;
 *	   &lt;Delete atNode="???"/&gt;
 *   &lt;/par&gt;
 * <pre>
 **********************************************************************/
private boolean createInsertMapping(ELASid elasId) {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertMapping() - entered");

	//
	Document xmtaDoc = treeSet_.getXmtaDoc();
	Hashtable xmtaIdTable = treeSet_.getXmtaIds();

	// get active interval information
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());

	//
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getObservedBeginTime();
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

	// create the replace
	boolean success = createReplaceTarget(insertTime);

	//	
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.createInsertMapping() - exiting success=" + success);
	return success;
}
/*********************************************************************** 
 * Reset the target element's value.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Replace atNode="???" atField="??" value="???"/&gt;
 *   &lt;/par&gt;
 * <pre>
 **********************************************************************/
private boolean createReplaceTarget(TimingAttribute setInsertTime) {
	//
	Document xmtaDoc = treeSet_.getXmtaDoc();

	// First get the target object.
	Animatable animatableObject = treeSet_.getXmtoAttrTargetObject();

	// Attempt to get the elasId for the object.
	ConvertElement convertElement = (ConvertElement) animatableObject;
	// if the target object isn't a timing element then search up its parent tree to find the
	// parent who has the timing elasIds.
	while (null != convertElement && !(convertElement instanceof TimingElement))
		convertElement = (ConvertElement) convertElement.getParent();

	// The target doesn't have an elasId. This is possible if it is 
	// in the defs tree and is an xMediaChildren element.  
	// In that case create the route from the begining.
	if (null == convertElement) {
		// Create the par for the replace command to replace the initial value.
		// <par begin="???>
		//   <Replace atNode="???" atField="???" value="???"/>
		// </par>
		Element parNodes = xmtaDoc.createElement(DOMXMTA.par);
		parNodes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(setInsertTime.getClockValue()));
		treeSet_.addToParRoutes(parNodes);

		// Create replace of field with default values
		boolean success = fillInReplaceDefsTarget(parNodes, animatableObject);
		return success;
	}

	// Get the intervals of the target object
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(convertElement.getIndex());

	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// First check to see if the target element interval comes after the setInsertTime
			// If so then we don't need to continue looping through the target intervals.
			// Thus return.
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			if (observedBeginTime.greaterThan(setInsertTime)) {
				//     setInsertTime  Target
				//     *              ****
				return true;
			}

			// If the target element interval occurs before the setInsertTime we don't need to check this active interval.
			// Thus if we only want to check this interval if the end of the target element comes
			// after the set.  Otherwise we will check the next active interval
			// If they occur at the same time then there is no reason to set the value if it is just going to be
			// deleted anyway.
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (fillEndTime.greaterThan(setInsertTime)) {
				//           setInsertTime
				//         target
				//         **** 
				//           *   
				// Now we need to loop through the simple durations (ELASids) for this active interval
				Vector elasIds = activeInterval.getELASids();
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					int sdIndex = elasId.getSimpleDurIndex().intValue();

					// get the insert time
					TimingAttribute targetInsertTime = null;
					if (0 == sdIndex) {
						targetInsertTime = activeInterval.getObservedBeginTime();
					} else
						targetInsertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

					// get the delete time
					TimingAttribute targetDeleteTime = null;
					if (elasIds.size() - 1 == sdIndex)
						targetDeleteTime = activeInterval.getFillEndTime();
					else
						targetDeleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);

					// Like above check that the target element simple interval contain with the time of the setInsertTime
					// First check to see if the target element interval comes after the setInsertTime
					// If so then we don't need to continue looping through the target intervals.
					// Thus return.
					if (targetInsertTime.greaterThan(setInsertTime)) {
						//     setInsertTime  Target
						//     *              ****
						return true;
					}

					// If the target element interval occurs before the setInsertTime we don't need to check this simple interval.
					// Thus if we only want to check this interval if the end of the target element comes
					// after the setInsertTime.  Otherwise we will check the next simple interval
					// If they occur at the same time then there is no reason to set the value if it is just going to be
					// deleted anyway.
					if (targetDeleteTime.greaterThan(setInsertTime)) {
						//           setInsertTime
						//         target
						//         **** 
						//           *   


						// Create the par for the replace command to replace the initial value.
						// <par begin="???>
						//   <Replace atNode="???" atField="???" value="???"/>
						// </par>
						Element parNodes = xmtaDoc.createElement(DOMXMTA.par);
						parNodes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(setInsertTime.getClockValue()));
						treeSet_.addToParRoutes(parNodes);

						// Create replace of field with default values
						boolean success = fillInReplaceTimingTarget(parNodes, animatableObject, elasId);
						// since there can only be one simple interval during at animationFillEndTime we are finished.
						return success;
					}
				}
			}
		}
	}

	// 
	return true;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doTimingMapping();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
private boolean doTimingMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doTimingMapping() - entered");

	//
	boolean success = true;

	//
	//	EventModel eventModel = getConvertElement().getXmto2Xmta().getEventModel();
	TimeScheduler timeScheduler = treeSet_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(getConvertElement().getIndex());

	// debug purposes
	// Go through it once for printing out debug info irregardless of whether it is correct or not.
	if (debug_) {
		int lifeCycleCount = 0;
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
			int activeIntervalCount = 0;
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				//
				StringBuffer sb = new StringBuffer();
				sb.append("AniSet.doTimingMapping() -");
				sb.append(" <").append(getConvertElement().getXmtoElement().getNodeName());
				String id = getConvertElement().getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
				sb.append(" activeInterval[").append(activeIntervalCount).append("]");
				sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
				sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
				sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					sb.append(" repeatTime=").append(repeatTime);
				}
				sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
				sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					sb.append(" elasId=(").append(elasId.getElementIndex());
					sb.append(",").append(elasId.getLifeCycleIndex());
					sb.append(",").append(elasId.getIntervalIndex());
					sb.append(",").append(elasId.getSimpleDurIndex());
					sb.append(")");
				}
				fireErrorEvent(ErrorEvent.TRACE, 0, sb.toString(), getConvertElement().getXmtoElement());

				//
				activeIntervalCount++;
			}
			lifeCycleCount++;
		}
	}


	//	// Only support visibility attribute
	//	if (!treeSet_.getXmtoAttrAttributeName().equals(DOMXMTO.visibility)) {
	//		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
	//		Object[] msgArgs = {treeSet_.getXmtoElement().getNodeName(), DOMXMTO.attributeName, treeSet_.getXmtoAttrAttributeName()};
	//		String message = msgFormat.format(msgArgs);
	//		fireErrorEvent(ErrorEvent.ERROR, 0, message, treeSet_.getXmtoElement(), DOMXMTO.attributeName);
	//		treeSet_.remove();
	//		if (debug_)
	//			fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doTimingMapping() - exiting success=false");
	//		return false;
	//	}

	//	// Only support visibility attribute to true/false
	//	if (!treeSet_.getXmtoAttrTo().equals(DOMXMTO.string_true) && !treeSet_.getXmtoAttrTo().equals(DOMXMTO.string_false)) {
	//		MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
	//		Object[] msgArgs = {treeSet_.getXmtoElement().getNodeName(), DOMXMTO.to, treeSet_.getXmtoAttrTo()};
	//		String message = msgFormat.format(msgArgs);
	//		fireErrorEvent(ErrorEvent.ERROR, 0, message, treeSet_.getXmtoElement(), DOMXMTO.to);
	//		treeSet_.remove();
	//		if (debug_)
	//			fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doTimingMapping() - exiting success=false");
	//		return false;
	//	}


	// Now go through for real.
	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// Check to make sure the timing values are valid
			boolean error = false;
			TimingAttribute computedBeginTime = activeInterval.getComputedBeginTime();
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			TimingAttribute activeDuration = activeInterval.getActiveDuration();
			//			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			//				|| null == fillEndTime)
			if (null == computedBeginTime || null == observedBeginTime || null == activeDuration)
				error = true;
			//			if (!error) {
			//				switch (computedBeginTime.getTimeValueType()) {
			//					case TimingAttribute.CLOCK_VALUE :
			//						break;
			//					default :
			//						error = true;
			//				}
			//			}
			if (!error) {
				switch (observedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			//			if (!error) {
			//				switch (activeDuration.getTimeValueType()) {
			//					case TimingAttribute.CLOCK_VALUE :
			//						break;
			//					default :
			//						error = true;
			//				}
			//			}
			//			if (!error) {
			//				switch (fillEndTime.getTimeValueType()) {
			//					case TimingAttribute.CLOCK_VALUE :
			//					case TimingAttribute.INDEFINITE_VALUE :
			//						break;
			//					default :
			//						error = true;
			//				}
			//			}
			if (!error) {
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					switch (repeatTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
			}

			// If there is a problem then report back the problem and remove the object.
			if (error) {
				MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
				StringBuffer buffer = new StringBuffer();
				buffer.append(getConvertElement().getXmtoElement().getNodeName());
				String id = getConvertElement().getXmtoAttrId();
				if (null != id && !id.equals(""))
					buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
				Object[] msgArgs = {buffer};
				String message = msgFormat.format(msgArgs);
				fireErrorEvent(ErrorEvent.ERROR, 0, message, getConvertElement().getXmtoElement());

				// Since we don't support the timings for this element remove it from the tree
				getConvertElement().remove();
				if (debug_)
					fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doTimingMapping() - exiting success=false");
				return false;
			}

			// All the timings check out OK so now loop through the ELASids and create the nodes.
			for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
				ELASid elasId = (ELASid) elasIdEnum.nextElement();

				// Create the insert Mapping.
				success = createInsertMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doTimingMapping() - exiting success=" + success);
					return success;
				}

				// Create the delete Mapping.
				success = createDeleteMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doTimingMapping() - exiting success=" + success);
					return success;
				}
			}
		}
	}

	// Check to see if it is a click
	EventModel eventModel = getConvertElement().getXmto2Xmta().getEventModel();
	if (eventModel.isBeginEvent(treeSet_.getIndex())) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doTimingMapping() - timing is eventModel.", getConvertElement().getXmtoElement());
		// make the group ids based on object id or make them unique.
		// We don't have an elasId thus we create the id here to make
		// sure it is unique for insert and delete.  We don't need it
		// any other time.
		String groupId = IdManager.getNextId(IdManager.GroupID);
		String id = treeSet_.getXmtoAttrId();
		if (null != id && !id.equals(""))
			groupId = groupId + ":" + id;
		createInsertEventMapping(groupId);
		createDeleteEventMapping(groupId);
	}


	// 
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.doTimingMapping() - exiting success=true");
	return true;
}
/*********************************************************************** 
 * Reset the target element's value.
 * <pre>
 *   &lt;Replace atNode="???" atField="??" value="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillInReplace(Element parNode, String animXmtaNodeId, String animXmtaAttrName, String animXmtaAttrValue) {
	//
	Document xmtaDoc = getConvertElement().getXmtaDoc();

	// Create route from intepolator to target node/field
	Element replace = xmtaDoc.createElement(DOMXMTA.Replace);
	replace.setAttribute(DOMXMTA.atNode, animXmtaNodeId);
	replace.setAttribute(DOMXMTA.atField, animXmtaAttrName);
	replace.setAttribute(DOMXMTA.value, animXmtaAttrValue);
	parNode.appendChild(replace);

	//
	return true;
}
/*********************************************************************** 
 * Reset the target element's value.
 * <pre>
 *   &lt;Replace atNode="???" atField="??" value="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillInReplaceDefsTarget(Element parNodes, Animatable target) {
	//
	String xmtoAttrAttributeName = getXmtoAttrAttributeName();
	String animXmtaNodeId = target.getAnimXmtaDefsNodeId(xmtoAttrAttributeName);
	String animXmtaAttrName = target.getAnimXmtaAttrName(xmtoAttrAttributeName);
	int animXmtoType = target.getAnimXmtoAttrFieldType(xmtoAttrAttributeName);
	int animXmtaType = target.getAnimXmtaAttrFieldType(xmtoAttrAttributeName);

	//
	String xmtoAttrTo = treeSet_.getXmtoAttrTo();
	String attribute = target.getAnimConvertValueXmto2Xmta(xmtoAttrAttributeName, xmtoAttrTo, animXmtoType, animXmtaType);
	String xmtaAttrTo = null;
	switch (animXmtaType) {
		case Animatable.TYPE_SFColor :
			{
				xmtaAttrTo = DOMXMTAParameter.color2Param(DOMXMTOParameter.param2Color(attribute));
				break;
			}
		case Animatable.TYPE_SFFloat :
			{
				Float sfFloat = DOMXMTOParameter.param2Float(attribute);
				if (null != sfFloat)
					xmtaAttrTo = DOMXMTAParameter.float2Param(sfFloat.floatValue());
				else
					xmtaAttrTo = "";
				break;
			}
		case Animatable.TYPE_SFVec2f :
			{
				Vector sfVec2f = DOMXMTOParameter.param2TwoFloats(attribute);
				if (null != sfVec2f) {
					Float sfFloat1 = (Float) sfVec2f.elementAt(0);
					Float sfFloat2 = (Float) sfVec2f.elementAt(1);
					xmtaAttrTo = DOMXMTAParameter.twoFloats2Param(sfFloat1.floatValue(), sfFloat2.floatValue());
				} else
					xmtaAttrTo = "";
				break;
			}
		case Animatable.TYPE_SFInt32 :
			{
				Integer sfInt32 = DOMXMTOParameter.param2Integer(attribute);
				if (null != sfInt32) {
					xmtaAttrTo = "" + sfInt32;
				} else
					xmtaAttrTo = "";
				break;
			}
		case Animatable.TYPE_MFBool :
			{
				xmtaAttrTo = DOMXMTAParameter.mfBool2Param(DOMXMTOParameter.param2MFBool(attribute));
				break;
			}
		case Animatable.TYPE_MFColor :
			{
				xmtaAttrTo = DOMXMTAParameter.colors2Param(DOMXMTOParameter.param2Colors(attribute));
				break;
			}
		case Animatable.TYPE_MFFloat :
			{
				xmtaAttrTo = DOMXMTAParameter.mfFloat2Param(DOMXMTOParameter.param2MFFloat(attribute));
				break;
			}
		case Animatable.TYPE_MFVec2f :
			{
				xmtaAttrTo = DOMXMTAParameter.arrayOfTwoFloats2Param(DOMXMTOParameter.param2ArrayOfTwoFloats(attribute));
				break;
			}
		case Animatable.TYPE_MFInt32 :
			{
				xmtaAttrTo = DOMXMTAParameter.integers2Param(DOMXMTOParameter.param2Integers(attribute));
				break;
			}
			//
		case Animatable.TYPE_SFBool :
		case Animatable.TYPE_SFString :
		case Animatable.TYPE_MFString :
		default :
			xmtaAttrTo = attribute;
			break;
	}

	// Create route from intepolator to target node/field
	boolean success = fillInReplace(parNodes, animXmtaNodeId, animXmtaAttrName, xmtaAttrTo);

	//
	return success;
}
/*********************************************************************** 
 * Reset the target element's value.
 * <pre>
 *   &lt;Replace atNode="???" atField="??" value="???"/&gt;
 * <pre>
 **********************************************************************/
private boolean fillInReplaceTimingTarget(Element parNodes, Animatable target, ELASid targetElasId) {
	//
	String xmtoAttrAttributeName = getXmtoAttrAttributeName();
	String animXmtaNodeId = target.getAnimXmtaNodeId(xmtoAttrAttributeName, targetElasId);
	String animXmtaAttrName = target.getAnimXmtaAttrName(xmtoAttrAttributeName);
	int animXmtoType = target.getAnimXmtoAttrFieldType(xmtoAttrAttributeName);
	int animXmtaType = target.getAnimXmtaAttrFieldType(xmtoAttrAttributeName);

	//
	String xmtoAttrTo = treeSet_.getXmtoAttrTo();
	String attribute = target.getAnimConvertValueXmto2Xmta(xmtoAttrAttributeName, xmtoAttrTo, animXmtoType, animXmtaType);
	String xmtaAttrTo = null;
	switch (animXmtaType) {
		case Animatable.TYPE_SFColor :
			{
				xmtaAttrTo = DOMXMTAParameter.color2Param(DOMXMTOParameter.param2Color(attribute));
				break;
			}
		case Animatable.TYPE_SFFloat :
			{
				Float sfFloat = DOMXMTOParameter.param2Float(attribute);
				if (null != sfFloat)
					xmtaAttrTo = DOMXMTAParameter.float2Param(sfFloat.floatValue());
				else
					xmtaAttrTo = "";
				break;
			}
		case Animatable.TYPE_SFVec2f :
			{
				Vector sfVec2f = DOMXMTOParameter.param2TwoFloats(attribute);
				if (null != sfVec2f) {
					Float sfFloat1 = (Float) sfVec2f.elementAt(0);
					Float sfFloat2 = (Float) sfVec2f.elementAt(1);
					xmtaAttrTo = DOMXMTAParameter.twoFloats2Param(sfFloat1.floatValue(), sfFloat2.floatValue());
				} else
					xmtaAttrTo = "";
				break;
			}
		case Animatable.TYPE_SFInt32 :
			{
				Integer sfInt32 = DOMXMTOParameter.param2Integer(attribute);
				if (null != sfInt32) {
					xmtaAttrTo = "" + sfInt32;
				} else
					xmtaAttrTo = "";
				break;
			}
		case Animatable.TYPE_MFBool :
			{
				xmtaAttrTo = DOMXMTAParameter.mfBool2Param(DOMXMTOParameter.param2MFBool(attribute));
				break;
			}
		case Animatable.TYPE_MFColor :
			{
				xmtaAttrTo = DOMXMTAParameter.colors2Param(DOMXMTOParameter.param2Colors(attribute));
				break;
			}
		case Animatable.TYPE_MFFloat :
			{
				xmtaAttrTo = DOMXMTAParameter.mfFloat2Param(DOMXMTOParameter.param2MFFloat(attribute));
				break;
			}
		case Animatable.TYPE_MFVec2f :
			{
				xmtaAttrTo = DOMXMTAParameter.arrayOfTwoFloats2Param(DOMXMTOParameter.param2ArrayOfTwoFloats(attribute));
				break;
			}
		case Animatable.TYPE_MFInt32 :
			{
				xmtaAttrTo = DOMXMTAParameter.integers2Param(DOMXMTOParameter.param2Integers(attribute));
				break;
			}
			//
		case Animatable.TYPE_SFBool :
		case Animatable.TYPE_SFString :
		case Animatable.TYPE_MFString :
		default :
			xmtaAttrTo = attribute;
			break;
	}

	// Create route from intepolator to target node/field
	boolean success = fillInReplace(parNodes, animXmtaNodeId, animXmtaAttrName, xmtaAttrTo);

	//
	return success;
}
/*********************************************************************** 
 * Return the name of the attribute to be animated.
 **********************************************************************/
private String getXmtoAttrAttributeName() {
	return treeSet_.getXmtoAttrAttributeName();
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof set)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.init() - element=" + element + " is not an instance of set");
		throw new Exception("");
	}

	//
	treeSet_ = (set) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AniSet.init() - treeSet_=" + treeSet_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<set>" + super.toString() + "</set>";
}
}
