/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  AugBackdrop.java
//
//=============================================================================
package xmto2xmta.mapaugmentation;

import java.awt.Color;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmtutil.MediaObjectInformation;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.backdrop;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.XMTO;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to handle the conversion of the body element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>16 Apr 2000 - Change resolve timings to use TimeScheduler.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>01 May 2001 - Changed too much of the old stuff, video descriptor.
 * <li>20 Jun 2001 - Remove support for old XMT-A format.
 * <li>28 May 2002 - odid url prefix is od:// in new spec.
 * </ul><p>
 *********************************************************************/
public class AugBackdrop extends AugAugmentationElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	static private final String ODID_URL_PREFIX = "od://";
	//static private final String FILE_URL_PREFIX = "file:///";
	static private final String FILE_URL_PREFIX = "";

	/*********************************************************************** 
	 **********************************************************************/
	private backdrop treeBackdrop_= null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the deleting of the background node.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Replace atField="set_bind" atNode="???" value="false"/&gt;
 *     &lt;Delete atNode="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createDeleteMapping(ELASid elasId) {
	TimeScheduler timeScheduler = treeBackdrop_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute deleteTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (activeInterval.getELASids().size() - 1 == sdIndex)
		deleteTime = activeInterval.getFillEndTime();
	else
		deleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);
	// If this simple duration doesn't end then we don't need to worry about deleting it.
	if (deleteTime.getTimeValueType() == TimingAttribute.INDEFINITE_VALUE)
		return true;

	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();

	// Create the par for the delete command.  Don't place it in the XMT-A tree yet.
	Element parDelete = xmtaDoc.createElement(DOMXMTA.par);
	parDelete.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(deleteTime.getClockValue()));
	treeBackdrop_.addToParNode(parDelete);

	
	// UnBind the background to the stack.
	// <Replace atNode="???" atField="set_bind" value="false"/>
	Element replace = xmtaDoc.createElement(DOMXMTA.Replace);
	String background2dId = treeBackdrop_.getXmtaBackground2dId(elasId);
	replace.setAttribute(DOMXMTA.atNode, background2dId);
	replace.setAttribute(DOMXMTA.atField, DOMXMTA.set_bind);
	replace.setAttribute(DOMXMTA.value, "false");
	parDelete.appendChild(replace);

	// Create delete Node 
	Element deleteNode = xmtaDoc.createElement(DOMXMTA.Delete);
	deleteNode.setAttribute(DOMXMTA.atNode, background2dId);
	parDelete.appendChild(deleteNode);

	//
	return true;
}
/***********************************************************************
 * Create the deleting of the background OD.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;ObjectDescriptorRemove objectDescriptorId="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createDeleteOdMapping(ELASid elasId) {
	TimeScheduler timeScheduler = treeBackdrop_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute deleteTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (activeInterval.getELASids().size() - 1 == sdIndex) {
		TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
		TimingAttribute activeDuration = activeInterval.getActiveDuration();
		deleteTime = observedBeginTime.add(activeDuration);
	} else
		deleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);
	// If this simple duration doesn't end then we don't need to worry about deleting it.
	if (deleteTime.getTimeValueType() == TimingAttribute.INDEFINITE_VALUE)
		return true;

	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();

	//
	Element parRemove = xmtaDoc.createElement(DOMXMTA.par);
	parRemove.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(deleteTime.getClockValue()));
	treeBackdrop_.addToParOd(parRemove);

	//
	Element objectDescriptorRemove = xmtaDoc.createElement(DOMXMTA.ObjectDescriptorRemove);
	String objectDescriptorId = treeBackdrop_.getXmtaObjectDescriptorId(elasId);
	objectDescriptorRemove.setAttribute(DOMXMTA.objectDescriptorId, objectDescriptorId);
	parRemove.appendChild(objectDescriptorRemove);

	//
	return true;
}
/***********************************************************************
 * Create the background infrastructure.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       &lt;Background2D DEF="???" backColor="???" src="???"/&gt;
 *     &lt;/Insert&gt;
 *    &lt;Replace atField="set_bind" atNode="???" value="true"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createInsertMapping(ELASid elasId) {
	//
	Document xmtaDoc = treeBackdrop_.getXmtaDoc();
	Hashtable xmtaIdTable = treeBackdrop_.getXmtaIds();

	// Find the Xmta parent group element which this node should be inserted into.
	XMTO xmtoElement = null;
	TreeElement parent = treeBackdrop_.getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			xmtoElement = (XMTO) parent;
			// break out of the loop
			break;
		}
		parent = parent.getParent();
	}

	// Find the background group element which this node should be inserted into.
	String backgroundGroupId = xmtoElement.getBackgroundGroupId();

	// if there are any errors	
	if (null == xmtoElement || null == backgroundGroupId) {
		// There always should be at least a xmto root so we should never get here.  
		// If we have then the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeBackdrop_.getXmtoElement());
		treeBackdrop_.remove();
		return false;
	}

	//
	String background2dId = treeBackdrop_.getXmtaBackground2dId(elasId);

	// The par node for the insertion contains the background.
	// <par begin="???">
	//   <Insert atNode="???">
	//     <Background2D DEF="???" backColor="???" src="???"/>
	//   </Insert>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeBackdrop_.addToParNode(parInsertNode);

	// Set the insert time.
	TimeScheduler timeScheduler = treeBackdrop_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getObservedBeginTime();
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element insertAtNode = xmtaDoc.createElement(DOMXMTA.Insert);
	insertAtNode.setAttribute(DOMXMTA.atNode, backgroundGroupId);
	parInsertNode.appendChild(insertAtNode);

	// Create background2d to allow the object to be moved
	Element background2D = xmtaDoc.createElement(DOMXMTA.Background2D);
	background2D.setAttribute(DOMXMTA.DEF, background2dId);
	xmtaIdTable.put(background2dId, background2D);
	insertAtNode.appendChild(background2D);

	// Set the color check default value
	String xmtaAttrColor = treeBackdrop_.getXmtaAttrColor();
	if (null != xmtaAttrColor) {
		String defaultValue = background2D.getAttribute(DOMXMTA.backColor);
		if (null == defaultValue || !defaultValue.equals(xmtaAttrColor))
			background2D.setAttribute(DOMXMTA.backColor, xmtaAttrColor);
	}

	// Check to see if we have a src attribute.  
	String xmtoAttrSrc = treeBackdrop_.getXmtoAttrSrc();
	if (null != xmtoAttrSrc && !xmtoAttrSrc.equals("")) {
		String objectDescriptorId = treeBackdrop_.getXmtaObjectDescriptorId(elasId);
		// set up the url back to the odid
		background2D.setAttribute(DOMXMTA.url, ODID_URL_PREFIX + objectDescriptorId);
	}

	// Bind the background to the stack.
	// <Replace atNode="???" atField="set_bind" value="true"/>
	Element replace = xmtaDoc.createElement(DOMXMTA.Replace);
	replace.setAttribute(DOMXMTA.atNode, background2dId);
	replace.setAttribute(DOMXMTA.atField, DOMXMTA.set_bind);
	replace.setAttribute(DOMXMTA.value, "true");
	parInsertNode.appendChild(replace);


	//
	return true;
}
/***********************************************************************
 * Create the background od infrastructure.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;ObjectDescriptorUpdate&gt;
 *       &lt;OD&gt;
 *         &lt;ObjectDescriptor ObjectDescriptorID="???"&gt;
 *           &lt;esDescr&gt;
 *             &lt;ES_Descriptor ES_ID="???"&gt;
 *               &lt;StreamSource url="???"/&gt;
 *               &lt;decConfigDescr&gt;
 *                 &lt;DecoderConfigDescriptor bufferSizeDB="auto" objectTypeIndication="???" streamType="???"&gt;
 *                   &lt;decSpecificInfo&gt;
 *                     &lt;DecoderSpecificInfo type="auto"/&gt;
 *                   &lt;/decSpecificInfo&gt;
 *                 &lt;/DecoderConfigDescriptor&gt;
 *               &lt;/decConfigDescr&gt;
 *               &lt;slConfigDescr&gt;
 *                 &lt;SLConfigDescriptor timeStampLength="auto" timeStampResolution="auto" useAccessUnitStartFlag="true"&gt;
 *               &lt;/slConfigDescr&gt;
 *             &lt;/ES_Descriptor&gt;
 *           &lt;/esDescr&gt;
 *         &lt;/ObjectDescriptor&gt;
 *       &lt;OD/&gt;
 *     &lt;/ObjectDescriptorUpdate&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createInsertOdMapping(ELASid elasId) {
	//
	Document xmtaDoc = treeBackdrop_.getXmtaDoc();
	Hashtable xmtaIdTable = treeBackdrop_.getXmtaIds();

	// Find the Xmta parent group element which this node should be inserted into.
	XMTO xmtoElement = null;
	TreeElement parent = treeBackdrop_.getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			xmtoElement = (XMTO) parent;
			// break out of the loop
			break;
		}
		parent = parent.getParent();
	}

	// Find the background group element which this node should be inserted into.
	String backgroundGroupId = xmtoElement.getBackgroundGroupId();

	// if there are any errors	
	if (null == xmtoElement || null == backgroundGroupId) {
		// There always should be at least a xmto root so we should never get here.  
		// If we have then the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeBackdrop_.getXmtoElement());
		treeBackdrop_.remove();
		return false;
	}

	// The par node for the insertion contains the background.
	// <par begin="???">
	//   <Insert atNode="???">
	//     <Background2D DEF="???" backColor="???" src="???"/>
	//   </Insert>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parUpdate = xmtaDoc.createElement(DOMXMTA.par);
	treeBackdrop_.addToParOd(parUpdate);

	// Set the insert time.
	TimeScheduler timeScheduler = treeBackdrop_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getComputedBeginTime();
		if (insertTime.lessThan(0))
			insertTime = new TimingAttribute(0);
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);
	parUpdate.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	//
	Node objectDescriptorUpdate = parUpdate.appendChild(xmtaDoc.createElement(DOMXMTA.ObjectDescriptorUpdate));

	//
	Node oD = objectDescriptorUpdate.appendChild(xmtaDoc.createElement(DOMXMTA.OD));

	// write ObjectDescriptor
	Element objectDescriptor = xmtaDoc.createElement(DOMXMTA.ObjectDescriptor);
	String objectDescriptorId = treeBackdrop_.getXmtaObjectDescriptorId(elasId);
	objectDescriptor.setAttribute(DOMXMTA.objectDescriptorID, objectDescriptorId);
	xmtaIdTable.put(objectDescriptorId, objectDescriptor);
	oD.appendChild(objectDescriptor);

	//
	Node esDescr = null;
	Node descr = objectDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.Descr));
	esDescr = descr.appendChild(xmtaDoc.createElement(DOMXMTA.esDescr));

	// write ESDescriptor
	Element es_Descriptor = xmtaDoc.createElement(DOMXMTA.ES_Descriptor);
	String esDescriptorId = treeBackdrop_.getXmtaEsDescriptorId(elasId);
	es_Descriptor.setAttribute(DOMXMTA.ES_ID, esDescriptorId);
	xmtaIdTable.put(esDescriptorId, es_Descriptor);
	esDescr.appendChild(es_Descriptor);

	//
	Node decConfigDescr = es_Descriptor.appendChild(xmtaDoc.createElement(DOMXMTA.decConfigDescr));

	// Determine it this is a video or image
	String xmtoAttrSrc = treeBackdrop_.getXmtoAttrSrc();
	String xmtoAttrSrcCanonical = treeBackdrop_.getXmtoAttrSrcCanonical();
	MediaObjectInformation moi = new MediaObjectInformation(xmtoAttrSrcCanonical);

	// write DecoderConfigDescriptor
	Element decoderConfigDescriptor = xmtaDoc.createElement(DOMXMTA.DecoderConfigDescriptor);
	decoderConfigDescriptor.setAttribute(DOMXMTA.bufferSizeDB, DOMXMTA.auto);
	decoderConfigDescriptor.setAttribute(DOMXMTA.objectTypeIndication, Integer.toString(moi.getMediaObjectType()));
	decoderConfigDescriptor.setAttribute(DOMXMTA.streamType, Integer.toString(moi.getMediaStreamType()));
	decConfigDescr.appendChild(decoderConfigDescriptor);

	//
	Node decSpecificInfo = decoderConfigDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.decSpecificInfo));
	// write DecoderSpecificInfo
	Element decoderSpecificInfo = xmtaDoc.createElement(DOMXMTA.DecoderSpecificInfo);
	decoderSpecificInfo.setAttribute(DOMXMTA.type, DOMXMTA.auto);
	decSpecificInfo.appendChild(decoderSpecificInfo);

	// Create the Sl Config Descriptor
	Node slConfigDescr = es_Descriptor.appendChild(xmtaDoc.createElement(DOMXMTA.slConfigDescr));

	//
	Element slConfigDescriptor = xmtaDoc.createElement(DOMXMTA.SLConfigDescriptor);
	slConfigDescr.appendChild(slConfigDescriptor);
	Element custom = xmtaDoc.createElement(DOMXMTA.custom);
	custom.setAttribute(DOMXMTA.useAccessUnitStartFlag, DOMXMTA.string_true);
	custom.setAttribute(DOMXMTA.timeStampResolution, DOMXMTA.auto);
	custom.setAttribute(DOMXMTA.timeStampLength, DOMXMTA.auto);
	slConfigDescriptor.appendChild(custom);

	//  write the StreamSource element
	Element streamSource = xmtaDoc.createElement(DOMXMTA.StreamSource);
	String uri = xmtoAttrSrc.replace('\\', '/');
	streamSource.setAttribute(DOMXMTA.url, FILE_URL_PREFIX + uri);
	es_Descriptor.appendChild(streamSource);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doTimingMapping();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
private boolean doTimingMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doTimingMapping() - entered", getConvertElement().getXmtoElement());

	//	
	boolean success = true;

	//
	//	EventModel eventModel = getConvertElement().getXmto2Xmta().getEventModel();
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(getConvertElement().getIndex());

	// debug purposes
	// Go through it once for printing out debug info irregardless of whether it is correct or not.
	if (debug_) {
		int lifeCycleCount = 0;
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
			int activeIntervalCount = 0;
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				//
				StringBuffer sb = new StringBuffer();
				sb.append("AugBackdrop.doTimingMapping() -");
				sb.append(" <").append(getConvertElement().getXmtoElement().getNodeName());
				String id = getConvertElement().getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
				sb.append(" activeInterval[").append(activeIntervalCount).append("]");
				sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
				sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
				sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
				sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
				sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
				ELASid parentElasId = lifeCycle.getParentELASid();
				sb.append(" parentElasId=(");
				if (null == parentElasId) {
					sb.append("null");
				} else {
					sb.append(parentElasId.getElementIndex());
					sb.append(",").append(parentElasId.getLifeCycleIndex());
					sb.append(",").append(parentElasId.getIntervalIndex());
					sb.append(",").append(parentElasId.getSimpleDurIndex());
				}
				sb.append(")");
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					sb.append(" repeatTime=").append(repeatTime);
				}
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					sb.append(" elasId=(").append(elasId.getElementIndex());
					sb.append(",").append(elasId.getLifeCycleIndex());
					sb.append(",").append(elasId.getIntervalIndex());
					sb.append(",").append(elasId.getSimpleDurIndex());
					sb.append(")");
				}
				fireErrorEvent(ErrorEvent.TRACE, 0, sb.toString(), getConvertElement().getXmtoElement());

				//
				activeIntervalCount++;
			}
			lifeCycleCount++;
		}
	}


	// Now go through for real.
	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// Check to make sure the timing values are valid
			boolean error = false;
			TimingAttribute computedBeginTime = activeInterval.getComputedBeginTime();
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			TimingAttribute activeDuration = activeInterval.getActiveDuration();
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (null == computedBeginTime || null == observedBeginTime || null == fillEndTime || null == activeDuration)
				error = true;
			if (!error) {
				switch (computedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (observedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (activeDuration.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (fillEndTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					switch (repeatTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
			}

			// If there is a problem then report back the problem and remove the object.
			if (error) {
				MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
				StringBuffer buffer = new StringBuffer();
				buffer.append(getConvertElement().getXmtoElement().getNodeName());
				if (null != getConvertElement().getXmtoAttrId() && !getConvertElement().getXmtoAttrId().equals(""))
					buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getConvertElement().getXmtoAttrId()).append("\"");
				Object[] msgArgs = {buffer};
				String message = msgFormat.format(msgArgs);
				fireErrorEvent(ErrorEvent.ERROR, 0, message, getConvertElement().getXmtoElement());

				// Since we don't support the timings for this element remove it from the tree
				getConvertElement().remove();
				if (debug_)
					fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doTimingMapping() - exiting success=false");
				return false;
			}

			// All the timings check out OK so now loop through the ELASids and create the nodes.
			for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
				ELASid elasId = (ELASid) elasIdEnum.nextElement();

				// Create the insert Mapping.
				success = createInsertMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doTimingMapping() - exiting success=false");
					return success;
				}

				// Create the delete Mapping.
				success = createDeleteMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doTimingMapping() - exiting success=false");
					return success;
				}

				// Create the OD
				String xmtoAttrSrc = treeBackdrop_.getXmtoAttrSrc();
				if (null != xmtoAttrSrc && !xmtoAttrSrc.equals("")) {
					success = createInsertOdMapping(elasId);
					if (!success) {
						if (debug_)
							fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doTimingMapping() - exiting success=false");
						return success;
					}
					success = createDeleteOdMapping(elasId);
					if (!success) {
						if (debug_)
							fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doTimingMapping() - exiting success=false");
						return success;
					}
				}
			}
		}
	}

	// 
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.doTimingMapping() - exiting success=true");
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof backdrop)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.init() - element=" + element + " is not an instance of backdrop");
		throw new Exception("");
	}

	//
	treeBackdrop_ = (backdrop) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "AugBackdrop.init() - treeBackdrop_ =" + treeBackdrop_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<backdrop>" + super.toString() + "</backdrop>";
}
}
