/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MedAudio.java
//
//=============================================================================
package xmto2xmta.mapmedia;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.AudioRes;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to handle the conversion of the rectangle element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>23 Apr 2001 - Remove check of audio type, just use what parser passes back.
 * <li>26 Apr 2001 - Update object descriptors to Jan XMT-A spec.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>03 May 2001 - Fix when repeat start time is before begining of presentation
 *                   ie. t<0.  Issue a warning and set start to t=0.
 *                 - Fix when video doesn't end (indefinite) don't remove
 *                   od.
 * </ul><p>
 *********************************************************************/
public class MedAudio extends MedExternalElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	//	private Vector parInserts_ = new Vector();
	//	private Vector parDeletes_ = new Vector();
	//	private Vector parUpdates_ = new Vector();
	//	private Vector parRemoves_ = new Vector();

	/*********************************************************************** 
	 **********************************************************************/
	//	private Element xmtaParNodes_ = null;
	//	private Element xmtaParOds_ = null;
	//	private XMTO xmtoXmtoElement_ = null;
	private xmto2xmta.tree.audio treeAudio_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle audioRes_ = ResourceBundle.getBundle("xmto2xmta.resources.AudioRes", Locale.getDefault());
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the rectangle infrastructure.
 * <pre>
 * &lt;Sound2D DEF="???"&gt;
 *   &lt;source&gt;
 *     &lt;AudioSource DEF="???" url="???"/&gt;
 *   &lt;/source&gt;
 * &lt;/Sound2D&gt;
 * </pre>
 **********************************************************************/
protected boolean fillInTransform2dChildren(Element transform2dChildren, ELASid elasId) {
	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();
	//	Element root = xmtaDoc.getDocumentElement();
	Hashtable xmtaIdTable = getConvertElement().getXmtaIds();

	// Create the shape for the video
	// <par begin="???">
	//   <Insert atNode="???">
	//     <Sound2D DEF="???">
	//       <source>
	//         <AudioSource DEF="???" url="???"/>
	//       </source>
	//     </Sound2D>
	//   </Insert>
	// </par>
	//
	//
	Element sound2D = xmtaDoc.createElement(DOMXMTA.Sound2D);
	String sound2dId = treeAudio_.getXmtaSound2dId(elasId);
	sound2D.setAttribute(DOMXMTA.DEF, sound2dId);
	xmtaIdTable.put(sound2dId, sound2D);
	transform2dChildren.appendChild(sound2D);
	//
	Node source = sound2D.appendChild(xmtaDoc.createElement(DOMXMTA.source));
	//
	Element audioSource = xmtaDoc.createElement(DOMXMTA.AudioSource);
	String objectDescriptorId = treeAudio_.getXmtaObjectDescriptorId(elasId);
	audioSource.setAttribute(DOMXMTA.url, ODID_URL_PREFIX + objectDescriptorId);
	String audioSourceId = treeAudio_.getXmtaAudioSourceId(elasId);
	audioSource.setAttribute(DOMXMTA.DEF, audioSourceId);
	xmtaIdTable.put(audioSourceId, audioSource);
	source.appendChild(audioSource);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, xmto2xmta.tree.ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof xmto2xmta.tree.audio)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "MedAudio.init() - element=" + element + " is not an instance of xmto2xmta.tree.audio");
		throw new Exception("");
	}

	//
	treeAudio_ = (xmto2xmta.tree.audio) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "MedAudio.init() - treeAudio_=" + treeAudio_);
}
 /*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
 static public void setDebug(boolean debug) {
		debug_ = debug;
 } 
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<MedAudio>" + super.toString() + "</MedAudio>";
}
}
