/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MedCurve.java
//
//=============================================================================
package xmto2xmta.mapmedia;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class to handle the conversion of the curve element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public class MedCurve extends MedInternalElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private xmto2xmta.tree.curve treeCurve_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the curve infrastructure.
 * <pre>
 * &lt;Curve2D DEF="???" points="???" fineness="???" type="???"/&gt;
 * </pre>
 **********************************************************************/
protected boolean fillInGeometry(Element geometry, ELASid elasId) {
	//	
	Document xmtaDoc = treeCurve_.getXmtaDoc();
	Hashtable xmtaIdTable = treeCurve_.getXmtaIds();

	// make the switch and transform ids based on object id or make them unique.
	String curveId = treeCurve_.getXmtaCurveId(elasId);

	//
	Element aCurve2D = xmtaDoc.createElement(DOMXMTA.Curve2D);
	aCurve2D.setAttribute(DOMXMTA.DEF, curveId);
	xmtaIdTable.put(curveId, aCurve2D);
	String point = treeCurve_.getXmtaAttrPoint();
	if (null != point) {
		Element aPoint = xmtaDoc.createElement(DOMXMTA.point);
		aCurve2D.appendChild(aPoint);
		Element aCoordinate2D = xmtaDoc.createElement(DOMXMTA.Coordinate2D);
		String coordinate2DId = treeCurve_.getXmtaCoordinate2DId(elasId);
		aCoordinate2D.setAttribute(DOMXMTA.DEF, coordinate2DId);
		xmtaIdTable.put(coordinate2DId, aCoordinate2D);
		aPoint.appendChild(aCoordinate2D);
		// Check if the default value is what we are already setting it to.
		String defaultValue = aCoordinate2D.getAttribute(DOMXMTA.point);
		if (null == defaultValue || !defaultValue.equals(point))
			aCoordinate2D.setAttribute(DOMXMTA.point, point);
	}
	String fineness = treeCurve_.getXmtaAttrFineness();
	if (null != fineness) {
		// Check if the default value is what we are already setting it to.
		String defaultValue = aCurve2D.getAttribute(DOMXMTA.fineness);
		if (null == defaultValue || !defaultValue.equals(fineness))
			aCurve2D.setAttribute(DOMXMTA.fineness, fineness);
	}
	String type = treeCurve_.getXmtaAttrType();
	if (null != type) {
		// Check if the default value is what we are already setting it to.
		String defaultValue = aCurve2D.getAttribute(DOMXMTA.type);
		if (null == defaultValue || !defaultValue.equals(type))
			aCurve2D.setAttribute(DOMXMTA.type, type);
	}
	geometry.appendChild(aCurve2D);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, xmto2xmta.tree.ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof xmto2xmta.tree.curve)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "MedCurve.init() - element=" + element + " is not an instance of xmto2xmta.tree.curve");
		throw new Exception("");
	}

	//
	treeCurve_ = (xmto2xmta.tree.curve) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "MedCurve.init() - treeCurve_=" + treeCurve_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<MedCurve>" + super.toString() + "</MedCurve>";
}
}
