/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MedInternalElement.java
//
//=============================================================================
package xmto2xmta.mapmedia;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.InternalElement;
import xmtutil.DOMXMTA;
/*********************************************************************** 
 * Class to group together the various type media elements whose data is internal to the XMT file.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>06 Jun 2001 - Add id on Appearance element
 * </ul><p>
 *********************************************************************/
public abstract class MedInternalElement extends MedMediaElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

/***********************************************************************
 **********************************************************************/
abstract protected boolean fillInGeometry(Element geometry, ELASid elasId);
/***********************************************************************
 * Create the rectangle infrastructure.
 * <pre>
 * &lt;Shape&gt;
 *   &lt;geometry/&gt;
 *   &lt;appearance&gt;
 *     &lt;Appearance/&gt;
 *   &lt;/appearance&gt;
 * &lt;/Shape&gt;
 * </pre>
 **********************************************************************/
protected boolean fillInTransform2dChildren(Element transform2dChildren, ELASid elasId) {
	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();
	//	Element root = xmtaDoc.getDocumentElement();
	Hashtable xmtaIdTable = getConvertElement().getXmtaIds();

	//
	Node shape = transform2dChildren.appendChild(xmtaDoc.createElement(DOMXMTA.Shape));

	//
	Element geometry = xmtaDoc.createElement(DOMXMTA.geometry);
	shape.appendChild(geometry);

	//
	boolean success = fillInGeometry(geometry, elasId);
	if (!success)
		return success;

	//
	Node appearance = shape.appendChild(xmtaDoc.createElement(DOMXMTA.appearance));

	//
	Element appearanceChild = xmtaDoc.createElement(DOMXMTA.Appearance);
	String appearanceId = ((InternalElement) getConvertElement()).getXmtaAppearanceId(elasId);
	appearanceChild.setAttribute(DOMXMTA.DEF, appearanceId);
	xmtaIdTable.put(appearanceId, appearanceChild);
	appearance.appendChild(appearanceChild);

	//
	return true;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
