/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MedMediaElement.java
//
//=============================================================================
package xmto2xmta.mapmedia;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.LifeCycle;
import xmto2xmta.MapRoot;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.tree.MediaElement;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.ZorderSupport;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.EventActiveInterval;
import xmto2xmta.IdManager;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to group together the various type of media elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>21 Jun 2001 - Disable the touchsensor by default.  Enable it when it is used.
 * </ul><p>
 *********************************************************************/
public abstract class MedMediaElement extends MapRoot{
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private MediaElement treeMediaElement_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());

/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the par node for the deletion.
 * The par for the deletion, deletes the top level group for the element.  It also
 * takes care of the position in the ordered group field (for z-order)
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Delete atNode="???"/&gt;
 *     &lt;Delete atField="order" atNode="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createDeleteMapping(ELASid elasId) {

	// JSB: TBD
	// for now ignore events
	//
	//	if (endResolved) {
	//	} else {
	//		// Not so easy way.  In this case we have an event so end with a conditional and
	//		// set route from touch sensor.
	//		success = timeMapEndEvent();
	//		if (!success)
	//			return success;
	//	}

	TimeScheduler timeScheduler = treeMediaElement_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute deleteTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (activeInterval.getELASids().size() - 1 == sdIndex)
		deleteTime = activeInterval.getFillEndTime();
	else
		deleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);
	return generateDeleteMapping(elasId, lifeCycle.getParentELASid(), deleteTime);
}

/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the par node for the deletion.
 * The par for the deletion, deletes the top level group for the element.  It also
 * takes care of the position in the ordered group field (for z-order)
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Delete atNode="???"/&gt;
 *     &lt;Delete atField="order" atNode="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createDeleteMapping(ELASid elasId, ELASid parentELASid, TimingAttribute deleteTime) {
	return generateDeleteMapping(elasId, parentELASid, deleteTime);
}

private boolean generateDeleteMapping(ELASid elasId, ELASid parentELASid, TimingAttribute deleteTime) {
	//	
	// If this simple duration doesn't end then we don't need to worry about deleting it.
	if (deleteTime.getTimeValueType() == TimingAttribute.INDEFINITE_VALUE)
		return true;
	Document xmtaDoc = getConvertElement().getXmtaDoc();
	Hashtable xmtaIdTable = getConvertElement().getXmtaIds();

	// Find the Xmta region or parent group element which this node should be inserted into.
	// Determine whether we should be under a region or under the parent group
	ZorderSupport xmtaParentGroupElement = null;
	String xmtoAttrRegion = treeMediaElement_.getXmtoAttrRegion();
	if (null == xmtoAttrRegion || xmtoAttrRegion.equals("")) {
		// No region so must be under the parent group
		// Find the Xmta parent group element which this node should be inserted into.
		TreeElement parent = treeMediaElement_.getParent();
		while (null != parent) {
			if (parent instanceof ZorderSupport) {
				xmtaParentGroupElement = (ZorderSupport) parent;
				// break out of the loop
				break;
			}
			parent = parent.getParent();
		}
	} else {
		// Get the region
		Hashtable xmtoIdTable = treeMediaElement_.getXmtoIds();
		Object obj = xmtoIdTable.get(xmtoAttrRegion);
		if (obj instanceof ZorderSupport)
			xmtaParentGroupElement = (ZorderSupport) obj;
	}

	//	
	if (null == xmtaParentGroupElement) {
		// We should always beable to find the parent ordered group.  
		// Should be able to find the body or the region
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), getConvertElement().getXmtoElement());
		getConvertElement().remove();
		return false;
	}

	// Get the ordered group of the parent we were inserted into.
	String xmtaParentOrderedGroupId = xmtaParentGroupElement.getXmtaOrderedGroupId(parentELASid);
	String groupId = treeMediaElement_.getXmtaGroupId(elasId);

	// The par for the deletion, deletes the top level group for the element.  It also
	// takes care of the position in the ordered group field (for z-order)
	// <par begin="???">
	//   <Delete atNode="???"/>
	//   <Delete atField="order" atNode="???"/>
	// </par>

	// Create the par for the delete command.  Don't place it in the XMT-A tree yet.
	Element parDeleteNode = xmtaDoc.createElement(DOMXMTA.par);
	parDeleteNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(deleteTime.getClockValue()));
	treeMediaElement_.addToParNode(parDeleteNode);

	// Create delete Node
	Element deleteNode = xmtaDoc.createElement(DOMXMTA.Delete);
	deleteNode.setAttribute(DOMXMTA.atNode, groupId);
	parDeleteNode.appendChild(deleteNode);

	// Do we use z-order positioning? If so then we need to delete the OrderedGroup's atField.
	if (xmtaParentGroupElement.usePositions()) {
		// Create delete Field.
		// Insert group isn't known yet so don't fill it in.
		// Also don't know the position it will be in at deletion time so don't fill it in.
		Element deleteField = xmtaDoc.createElement(DOMXMTA.Delete);
		deleteField.setAttribute(DOMXMTA.atNode, xmtaParentOrderedGroupId);
		deleteField.setAttribute(DOMXMTA.atField, DOMXMTA.order);
		parDeleteNode.appendChild(deleteField);

		// get the position the deleted element is in so that we delete the correct filed position.
		int deleteFieldPosition = xmtaParentGroupElement.getDeleteFieldPosition(elasId);
		deleteField.setAttribute(DOMXMTA.position, "" + deleteFieldPosition);
	}

	//
	return true;
}
/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the par node for the insertion.
 * The par node for the insertion contains the top level group which will contain the
 * information for the element.  This also takes care of the position in the ordered group
 * field (for z-order).
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       &lt;Group DEF="???"&gt;
 *         &lt;children&gt;
 *           &lt;TouchSensor DEF="???" enabled="false"&gt;
 *         &lt;/children&gt;
 *       &lt;/Group&gt;
 *     &lt;/Insert&gt;
 *     &lt;Insert atField="order" atNode="???" position="END" value="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createInsertMapping(ELASid elasId) {
	// JSB: TBD
	// for now ignore events
	//	// Attach the switch into the group
	//	if (beginResolved) {
	//		// Easy way timing is resolved so attach switch directly
	//		groupChildren_.appendChild(switch_);
	//	} else {
	//		// Not so easy way.  In this case we have an event so attach switch inside conditional and
	//		// set route from touch sensor.
	//		success = timeMapBeginEvent();
	//		if (!success)
	//			return success;
	//	}

	// Get the insert time.
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getObservedBeginTime();
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);
	return generateInsertMapping(elasId, lifeCycle.getParentELASid(), insertTime, null);
}

/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the par node for the insertion.
 * The par node for the insertion contains the top level group which will contain the
 * information for the element.  This also takes care of the position in the ordered group
 * field (for z-order).
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       &lt;Group DEF="???"&gt;
 *         &lt;children&gt;
 *           &lt;TouchSensor DEF="???" enabled="false"&gt;
 *         &lt;/children&gt;
 *       &lt;/Group&gt;
 *     &lt;/Insert&gt;
 *     &lt;Insert atField="order" atNode="???" position="END" value="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createInsertMapping(ELASid elasId, ELASid parentELASid, TimingAttribute insertTime, TimingAttribute simpleDuration) {
	return generateInsertMapping(elasId, parentELASid, insertTime, simpleDuration);
}

private boolean generateInsertMapping(ELASid elasId, ELASid parentELASid, TimingAttribute insertTime, TimingAttribute simpleDuration) {
	//
	Document xmtaDoc = treeMediaElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeMediaElement_.getXmtaIds();

	// Find the Xmta region or parent group element which this node should be inserted into.
	// Determine whether we should be under a region or under the parent group
	ZorderSupport xmtaParentGroupElement = null;
	String xmtoAttrRegion = treeMediaElement_.getXmtoAttrRegion();
	if (null == xmtoAttrRegion || xmtoAttrRegion.equals("")) {
		// No region so must be under the parent group
		// Find the Xmta parent group element which this node should be inserted into.
		TreeElement parent = treeMediaElement_.getParent();
		while (null != parent) {
			if (parent instanceof ZorderSupport) {
				xmtaParentGroupElement = (ZorderSupport) parent;
				// break out of the loop
				break;
			}
			parent = parent.getParent();
		}
	} else {
		// Get the region
		Hashtable xmtoIdTable = treeMediaElement_.getXmtoIds();
		Object obj = xmtoIdTable.get(xmtoAttrRegion);
		if (obj instanceof ZorderSupport)
			xmtaParentGroupElement = (ZorderSupport) obj;
	}
	if (null == xmtaParentGroupElement) {
		// We should always beable to find the parent ordered group.  
		// Should be able to find the body or the region
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeMediaElement_.getXmtoElement());
		getConvertElement().remove();
		return false;
	}


	// Get the ordered group of the parent we are going to be inserted into.
	String parentOrderedGroupId = xmtaParentGroupElement.getXmtaOrderedGroupId(parentELASid);

	// The par node for the insertion contains the top level group which will contain the
	// information for the element.  This also takes care of the position in the ordered group
	// field (for z-order).
	// <par begin="???">
	//   <Insert atNode="???">
	//     <Group DEF="???">
	//       <children>
	//         <TouchSensor DEF="???" enabled="false">
	//       </children>
	//     </Group>
	//   </Insert>
	//   <Insert atField="order" atNode="???" position="END" value="???"/>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeMediaElement_.addToParNode(parInsertNode);
	// Set the insert time.
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element insertAtNode = xmtaDoc.createElement(DOMXMTA.Insert);
	insertAtNode.setAttribute(DOMXMTA.atNode, parentOrderedGroupId);
	parInsertNode.appendChild(insertAtNode);

	// Create the top level group node
	Element group = xmtaDoc.createElement(DOMXMTA.Group);
	String groupId = treeMediaElement_.getXmtaGroupId(elasId);
	group.setAttribute(DOMXMTA.DEF, groupId);
	xmtaIdTable.put(groupId, group);
	insertAtNode.appendChild(group);

	//
	Element groupChildren = xmtaDoc.createElement(DOMXMTA.children);
	group.appendChild(groupChildren);

	//
	Element switchElement = createXmtaMapping(elasId);
	groupChildren.appendChild(switchElement);

	if (simpleDuration != null && simpleDuration.getTimeValueType() == TimingAttribute.CLOCK_VALUE) {
		// delete the group node after the simple duration
		Element aParNode = xmtaDoc.createElement(DOMXMTA.par);
		parInsertNode.appendChild(aParNode);
		// Set the insert time.
		aParNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(simpleDuration.getClockValue()));
		deleteNodeAtEndOfDuration(aParNode, groupId, 0);
	}
	
	// Create a touch sensor.
	Element touchSensor = xmtaDoc.createElement(DOMXMTA.TouchSensor);
	// make the switch and transform ids based on object id or make them unique.
	String touchSensorId = treeMediaElement_.getXmtaTouchSensorId(elasId);
	// Set up the id.  
	touchSensor.setAttribute(DOMXMTA.DEF, touchSensorId);
	if (treeMediaElement_.touchSensorEnabled()) {
		touchSensor.setAttribute(DOMXMTA.enabled, DOMXMTA.string_true);
	} else {
		touchSensor.setAttribute(DOMXMTA.enabled, DOMXMTA.string_false);
	}
	xmtaIdTable.put(touchSensorId, touchSensor);
	groupChildren.appendChild(touchSensor);

	// Do we use z-order positioning? If so then we need to create an insert to modify the OrderedGroup's atField.
	if (xmtaParentGroupElement.usePositions()) {
		// Create insert Field.  
		// Insert group isn't known yet so don't fill it in.
		// Also don't know the zOrder so don't fill it in.
		Element insertAtField = xmtaDoc.createElement(DOMXMTA.Insert);
		insertAtField.setAttribute(DOMXMTA.atNode, parentOrderedGroupId);
		insertAtField.setAttribute(DOMXMTA.atField, DOMXMTA.order);
		insertAtField.setAttribute(DOMXMTA.position, DOMXMTA.END);
		insertAtField.setAttribute(DOMXMTA.value, "-1"); // give it a default position value
		parInsertNode.appendChild(insertAtField);


		// Set the transformation information (need transform2d, switch and insertfield).
		// Do this dynamically since we don't want to require a material class in the build.
		try {
			Class treeTransformationClass = Class.forName("xmto2xmta.tree.transformation");
			Class transformationClass = Class.forName("xmto2xmta.mapmedia.MedTransformation");
			MedXmediaChildrenElement xmediaChildrenElement_ = (MedXmediaChildrenElement) transformationClass.newInstance();
			Vector children = getConvertElement().getChildren();
			for (Enumeration enum = children.elements(); enum.hasMoreElements();) {
				xmto2xmta.tree.ConvertElement child = (xmto2xmta.tree.ConvertElement) enum.nextElement();
				if (treeTransformationClass.isInstance(child))
					xmediaChildrenElement_.setTransformation(child, null, null, insertAtField);
			}
		} catch (Exception e) {
			if (debug_)
				fireErrorEvent(ErrorEvent.TRACE, 0, "MedMediaElement.createXmtaMapping() - Unable to create class=transformation exception=" + e);
		}
	}

	//
	return true;
}
/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the switch and the transform are the standard base nodes for the element.  
 * Where it is placed in the tree is dependent on the begin and end timing for the element.
 * </ul>
 *   &lt;Switch DEF="???"&gt;
 *     &lt;choice&gt;
 *       &lt;Transform2D DEF="???"&gt;
 *         &lt;children/&gt;
 *       &lt;/Transform2D&gt;
 *     &lt;/choice&gt;
 *   &lt;/Switch&gt;
 * </pre>
 **********************************************************************/
private Element createXmtaMapping(ELASid elasId) {
	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();
	Hashtable xmtaIdTable = getConvertElement().getXmtaIds();

	// make the switch and transform ids based on object id or make them unique.
	String switchId = treeMediaElement_.getXmtaSwitchId(elasId);
	String transform2dId = treeMediaElement_.getXmtaTransform2dId(elasId);

	// The switch and the transform are the standard base nodes for the element.  
	// Where it is placed in the tree is dependent on the begin and end timing for the element.
	// <Switch DEF="???">
	//   <choice>
	//     <Transform2D DEF="???">
	//       <children/>
	//     </Transform2D>
	//   </choice>
	// </Switch>

	// Create switch to allow the object to be turned on or off
	Element switchElement = xmtaDoc.createElement(DOMXMTA.Switch);
	switchElement.setAttribute(DOMXMTA.DEF, switchId);
	switchElement.setAttribute(DOMXMTA.whichChoice, "0"); // default is on
	xmtaIdTable.put(switchId, switchElement);

	// Create the choice node under the switch
	Node choice = switchElement.appendChild(xmtaDoc.createElement(DOMXMTA.choice));

	// Create transform2d to allow the object to be moved
	Element transform2D = xmtaDoc.createElement(DOMXMTA.Transform2D);
	transform2D.setAttribute(DOMXMTA.DEF, transform2dId);
	xmtaIdTable.put(transform2dId, transform2D);
	choice.appendChild(transform2D);

	//
	Element transform2dChildren = xmtaDoc.createElement(DOMXMTA.children);
	transform2D.appendChild(transform2dChildren);

	//
	fillInTransform2dChildren(transform2dChildren, elasId);

	// Set the transformation information (need transform2d, switch and insertfield).
	// Do this dynamically since we don't want to require a material class in the build.
	try {
		Class treeTransformationClass = Class.forName("xmto2xmta.tree.transformation");
		Class transformationClass = Class.forName("xmto2xmta.mapmedia.MedTransformation");
		MedXmediaChildrenElement xmediaChildrenElement_ = (MedXmediaChildrenElement) transformationClass.newInstance();
		Vector children = getConvertElement().getChildren();
		for (Enumeration enum = children.elements(); enum.hasMoreElements();) {
			xmto2xmta.tree.ConvertElement child = (xmto2xmta.tree.ConvertElement) enum.nextElement();
			if (treeTransformationClass.isInstance(child)) {
				xmediaChildrenElement_.setTransformation(child, switchElement, transform2D, null);
				break;
			}
		}
	} catch (Exception e) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "MedMediaElement.createXmtaMapping() - Unable to create class=transformation exception=" + e);
	}

	//
	return switchElement;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "MedMediaElement.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doTimingMapping();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "MedMediaElement.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
private boolean doTimingMapping() {
	boolean success = true;

	//
	//	EventModel eventModel = getConvertElement().getXmto2Xmta().getEventModel();
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(getConvertElement().getIndex());

	// debug purposes
	// Go through it once for printing out debug info irregardless of whether it is correct or not.
	if (debug_) {
		int lifeCycleCount = 0;
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
			int activeIntervalCount = 0;
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				//
				StringBuffer sb = new StringBuffer();
				sb.append("MedMediaElement.determineTimingMapping() -");
				sb.append(" <").append(getConvertElement().getXmtoElement().getNodeName());
				String id = getConvertElement().getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
				sb.append(" activeInterval[").append(activeIntervalCount).append("]");
				sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
				sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
				sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
				sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
				sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
				ELASid parentElasId = lifeCycle.getParentELASid();
				sb.append(" parentElasId=(");
				if (null == parentElasId) {
					sb.append("null");
				} else {
					sb.append(parentElasId.getElementIndex());
					sb.append(",").append(parentElasId.getLifeCycleIndex());
					sb.append(",").append(parentElasId.getIntervalIndex());
					sb.append(",").append(parentElasId.getSimpleDurIndex());
				}
				sb.append(")");
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					sb.append(" repeatTime=").append(repeatTime);
				}
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					sb.append(" elasId=(").append(elasId.getElementIndex());
					sb.append(",").append(elasId.getLifeCycleIndex());
					sb.append(",").append(elasId.getIntervalIndex());
					sb.append(",").append(elasId.getSimpleDurIndex());
					sb.append(")");
				}
				fireErrorEvent(ErrorEvent.TRACE, 0, sb.toString(), getConvertElement().getXmtoElement());

				//
				activeIntervalCount++;
			}
			lifeCycleCount++;
		}
	}


	// Now go through for real.
	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		if (lifeCycle.getActiveIntervals().size() > 0) {
			// Loop through the active intervals
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				// Check to make sure the timing values are valid
				boolean error = false;
				TimingAttribute computedBeginTime = activeInterval.getComputedBeginTime();
				TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
				TimingAttribute fillEndTime = activeInterval.getFillEndTime();
				if (null == computedBeginTime || null == observedBeginTime || null == fillEndTime)
					error = true;
				if (!error) {
					switch (computedBeginTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
				if (!error) {
					switch (observedBeginTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
				if (!error) {
					switch (fillEndTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
						case TimingAttribute.INDEFINITE_VALUE :
							break;
						default :
							error = true;
					}
				}
				if (!error) {
					for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
						TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
						switch (repeatTime.getTimeValueType()) {
							case TimingAttribute.CLOCK_VALUE :
								break;
							default :
								error = true;
						}
					}
				}

				// If there is a problem then report back the problem and remove the object.
				if (error) {
					MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
					StringBuffer buffer = new StringBuffer();
					buffer.append(getConvertElement().getXmtoElement().getNodeName());
					if (null != getConvertElement().getXmtoAttrId() && !getConvertElement().getXmtoAttrId().equals(""))
						buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getConvertElement().getXmtoAttrId()).append("\"");
					Object[] msgArgs = {buffer};
					String message = msgFormat.format(msgArgs);
					fireErrorEvent(ErrorEvent.ERROR, 0, message, getConvertElement().getXmtoElement());

					// Since we don't support the timings for this element remove it from the tree
					getConvertElement().remove();
					return false;
				}

				// All the timings check out OK so now loop through the ELASids and create the nodes.
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();	

					// Create the insert Mapping.
					success = createInsertMapping(elasId);
					if (!success)
						return success;

					// Create the delete Mapping.
					success = createDeleteMapping(elasId);
					if (!success)
						return success;
				}
			}
		} else { // look for events
			doEventTimingMapping();
		}
	}

	// 
	return true;
}
/*********************************************************************** 
 **********************************************************************/
abstract protected boolean fillInTransform2dChildren(Element transform2dChildren, ELASid elasId);
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, xmto2xmta.tree.ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof xmto2xmta.tree.MediaElement)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "MedMediaElement.init() - element=" + element + " is not an instance of xmto2xmta.tree.MediaElement");
		throw new Exception("");
	}

	//
	treeMediaElement_ = (xmto2xmta.tree.MediaElement) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "MedMediaElement.init() - treeMediaElement_=" + treeMediaElement_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}

private boolean doEventTimingMapping() {
	boolean error = false;
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	EventActiveInterval eventInterval = timeScheduler.getEventActiveInterval(getConvertElement().getIndex());
	if (eventInterval == null) return false;
	TimingAttribute sceneInsertTime = eventInterval.getObservedBeginTime();
	TimingAttribute eventBeginTime = eventInterval.getBeginEvent();
	TimingAttribute sceneDeleteTime = eventInterval.getFillEndTime();
	TimingAttribute eventEndTime = eventInterval.getEndEvent();
	ELASid elasId = eventInterval.getELASid();	
	ELASid parentELASid = eventInterval.getParentELASid();
	TimingAttribute simpleDuration = eventInterval.getSimpleDuration();
	if (eventBeginTime == null) {	// begin time is resolved
		if (null == sceneInsertTime)
			error = true;
		if (!error) {
			switch (sceneInsertTime.getTimeValueType()) {
				case TimingAttribute.CLOCK_VALUE :
					break;
				default :
					error = true;
			}
		}
		if (!error) {
			// Create the insert Mapping.
			if (!createInsertMapping(elasId, parentELASid, sceneInsertTime, simpleDuration)) {
				error = true;
			}
		}
	} else {
		if (!createEventInsertMapping(elasId, parentELASid)) {
			error = true;
		}
	}

	if (!error) {
		if (eventEndTime == null) {	// end time is resolved
			if (null == sceneDeleteTime)
				error = true;
			if (!error) {
				switch (sceneDeleteTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error && (sceneDeleteTime.getTimeValueType() != TimingAttribute.INDEFINITE_VALUE)) {
				// Create the delete Mapping.
				if (!createDeleteMapping(elasId, parentELASid, sceneDeleteTime)) {
					error = true;
				}
			}
		} else {
			if (sceneDeleteTime != null) {
				switch (sceneDeleteTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				if (!createEventDeleteMapping(elasId, parentELASid)) {
					error = true;
				}
			}
		}
	}
	return !error;
}

/*********************************************************************** 
 * Attach the switch for the element based on the begin event.  The begining 
 * of the object is based on an event thus attach switch 
 * The begining of the object is based on an event thus attach switch 
 * inside conditional and set the route from a touch sensor.  If there
 * is an offset time then also need a timer conditional.
 **********************************************************************/
protected boolean createEventInsertMapping(ELASid elasId, ELASid parentELASid) {
	Document xmtaDoc = treeMediaElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeMediaElement_.getXmtaIds();

	// Find the Xmta region or parent group element which this node should be inserted into.
	// Determine whether we should be under a region or under the parent group
	ZorderSupport xmtaParentGroupElement = null;
	String xmtoAttrRegion = treeMediaElement_.getXmtoAttrRegion();
	if (null == xmtoAttrRegion || xmtoAttrRegion.equals("")) {
		// No region so must be under the parent group
		// Find the Xmta parent group element which this node should be inserted into.
		TreeElement parent = treeMediaElement_.getParent();
		while (null != parent) {
			if (parent instanceof ZorderSupport) {
				xmtaParentGroupElement = (ZorderSupport) parent;
				// break out of the loop
				break;
			}
			parent = parent.getParent();
		}
	} else {
		// Get the region
		Hashtable xmtoIdTable = treeMediaElement_.getXmtoIds();
		Object obj = xmtoIdTable.get(xmtoAttrRegion);
		if (obj instanceof ZorderSupport)
			xmtaParentGroupElement = (ZorderSupport) obj;
	}
	if (null == xmtaParentGroupElement) {
		// We should always beable to find the parent ordered group.  
		// Should be able to find the body or the region
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeMediaElement_.getXmtoElement());
		getConvertElement().remove();
		return false;
	}

	// Get the insert time.
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	EventActiveInterval eventInterval = timeScheduler.getEventActiveInterval(elasId.getElementIndex());
	TimingAttribute eventAttribute = eventInterval.getBeginEvent();
	TimingAttribute observedBegin = eventInterval.getObservedBeginTime();
	TimingAttribute simpleDuration = eventInterval.getSimpleDuration();
	if (eventAttribute == null || eventAttribute.getTimeValueType() != TimingAttribute.MULTI_VALUE) {
		return false;
	}
	Vector timingAttributes = eventAttribute.getMultiTimingAttributes();
	if (timingAttributes.size() < 3) {
		return false;
	}
	TimingAttribute sceneInsertTime = (TimingAttribute) timingAttributes.elementAt(0);
	TimingAttribute eventBeginTime = (TimingAttribute) timingAttributes.elementAt(2);
	// For right now we only support time values
	boolean error = false;
	if (null == sceneInsertTime)
	  	error = true;
	else
	  	switch (sceneInsertTime.getTimeValueType()) {
	     	case TimingAttribute.CLOCK_VALUE :
	     		break;
	     	default :
	     		error = true;
	   }
	if (null == observedBegin)
	  	error = true;
	else
	  	switch (observedBegin.getTimeValueType()) {
	     	case TimingAttribute.CLOCK_VALUE :
	     	case TimingAttribute.INDEFINITE_VALUE :
	     		break;
	     	default :
	     		error = true;
	   }
	String touchSensorId = null;
	xmto2xmta.tree.MediaElement touchSensorElement = null;
	boolean sensorElementMapped = true;
	if (!error) {
		if (observedBegin.getTimeValueType() == TimingAttribute.CLOCK_VALUE && observedBegin.greaterThan(sceneInsertTime)) {
			sceneInsertTime = observedBegin;
		}
	  	if (null == eventBeginTime || TimingAttribute.EVENT_VALUE != eventBeginTime.getTimeValueType())
	  		error = true;
	  	else {
	  		switch (eventBeginTime.getEventSymbol()) {
	  			case TimingAttribute.SYMBOL_EVENT_CLICK :
	  			case TimingAttribute.SYMBOL_EVENT_ACTIVATE_EVENT :
	  				String sensorId = eventBeginTime.getIdValue();
	  				Object obj = getConvertElement().getXmtoIds().get(sensorId);
	  				if (null == obj)
	  					error = true;
	  				else {
	  					if (obj instanceof xmto2xmta.tree.Sensorable) {
							touchSensorElement = (xmto2xmta.tree.MediaElement) obj;
							EventActiveInterval touchEventInterval = timeScheduler.getEventActiveInterval(touchSensorElement.getIndex()); 
							if (touchEventInterval != null) {
								ELASid touchELASid = touchEventInterval.getELASid();
	  							touchSensorId = touchSensorElement.getXmtaTouchSensorId(touchELASid);
								touchSensorElement.enableTouchSensor(true);
								Element touchSensor = (Element) touchSensorElement.getXmtaIds().get(touchSensorId);
								if (touchSensor != null) {
									touchSensor.setAttribute(DOMXMTA.enabled, DOMXMTA.string_true);
								} else {
									sensorElementMapped = false;
								}
							} else {
								error = true;
							}
	  					} else {
	  						error = true;
	  					}
	  				}
	  				break;
	  			default :
	  				error = true;
	  		}
	  	}
	}
   if (error) {
  	// report back the problem
   	MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
     	StringBuffer buffer = new StringBuffer();
     	buffer.append(getConvertElement().getXmtoElement().getNodeName());
     	if (null != getConvertElement().getXmtoAttrId() && !getConvertElement().getXmtoAttrId().equals(""))
     		buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getConvertElement().getXmtoAttrId()).append("\"");
     	Object[] msgArgs = {buffer};
     	String message = msgFormat.format(msgArgs);
     	fireErrorEvent(ErrorEvent.ERROR, 0, message, getConvertElement().getXmtoElement());
     	// Since we don't support the timings for this element remove it from the tree
     	getConvertElement().remove();
     	return false;
    }
	

	// Get the ordered group of the parent we are going to be inserted into.
	String parentOrderedGroupId = xmtaParentGroupElement.getXmtaOrderedGroupId(parentELASid);

	// The par node for the insertion contains the top level group which will contain the
	// information for the element.  This also takes care of the position in the ordered group
	// field (for z-order).
	// <par begin="???">
	//   <Insert atNode="???">
	//     <Group DEF="???">
	//       <children>
	//         <TouchSensor DEF="???" enabled="false">
	//       </children>
	//     </Group>
	//   </Insert>
	//   <Insert atField="order" atNode="???" position="END" value="???"/>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeMediaElement_.addToParNode(parInsertNode);
	// Set the insert time.
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(sceneInsertTime.getClockValue()));

	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element insertAtNode = xmtaDoc.createElement(DOMXMTA.Insert);
	insertAtNode.setAttribute(DOMXMTA.atNode, parentOrderedGroupId);
	parInsertNode.appendChild(insertAtNode);

	// Create the top level group node
	Element group = xmtaDoc.createElement(DOMXMTA.Group);
	String groupId = treeMediaElement_.getXmtaGroupId(elasId);
	group.setAttribute(DOMXMTA.DEF, groupId);
	xmtaIdTable.put(groupId, group);
	insertAtNode.appendChild(group);

	//
	Element groupChildren = xmtaDoc.createElement(DOMXMTA.children);
	group.appendChild(groupChildren);

	//
	// make the ids based on object id or make them unique.
	String conditionalInsertId = treeMediaElement_.getXmtaConditionalInsertId(elasId);
	String conditionalInsertTimerId = treeMediaElement_.getXmtaConditionalInsertTimerId(elasId);
	String timeSensorBeginOffsetId = treeMediaElement_.getXmtaTimeSensorBeginOffsetId(elasId);
	// String valuatorId = treeMediaElement_.getXmtaValuatorId(elasId);
	String valuatorId = IdManager.getNextId(IdManager.ValuatorID);
	
	// Create the conditional for the insert command and add the switch to it.
	// Also add a delete of the conditional at the end of the conditional (delete itself).
	// <Conditional DEF="???">
	//   <buffer>
	//     <Insert atNode="???"/>
	//     <Delete atNode="???"/>
	//   </buffer>
	// </Conditional>
	// Create conditional for Insert
	Element conditional = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Conditional);
	conditional.setAttribute(DOMXMTA.DEF, conditionalInsertId);
	xmtaIdTable.put(conditionalInsertId, conditional);
	groupChildren.appendChild(conditional);
	// Create the buffer
	Node buffer = conditional.appendChild(getConvertElement().getXmtaDoc().createElement(DOMXMTA.buffer));
	// Create Insert
	Element insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	insert.setAttribute(DOMXMTA.atNode, groupId);
	buffer.appendChild(insert);
	// Insert the switch
	Element switchElement = createXmtaMapping(elasId);
	insert.appendChild(switchElement);

	if (simpleDuration != null && simpleDuration.getTimeValueType() == TimingAttribute.CLOCK_VALUE) {
		// delete the group node after the simple duration
		deleteNodeAtEndOfDuration(buffer, groupId, simpleDuration.getClockValue());
	}
	
	// Create Delete
	Element delete = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Delete);
	delete.setAttribute(DOMXMTA.atNode, conditionalInsertId);
	buffer.appendChild(delete);
	// If this is started at an offset of then create a timer to delay the start.
	long offset = eventBeginTime.getClockValue();
	if (offset > 0) {
	 	// <Conditional DEF="???">
	  	//   <buffer>
		//     <Insert atNode="???">
	  	//       <TimeSensor DEF="???" cycleInterval="???"/>
	  	//     </Insert>
	  	//     <Insert>
	  	//       <ROUTE fromField="isActive" fromNode="???" toField="reverseActivate" toNode="???"/>
	  	//     </Insert>
	  	//     <Delete atNode="???"/>
	  	//   </buffer>
	  	// </Conditional>
	 	// Create conditional for Insert
	  	conditional = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Conditional);
	  	conditional.setAttribute(DOMXMTA.DEF, conditionalInsertTimerId);
	  	xmtaIdTable.put(conditionalInsertTimerId, conditional);
	  	groupChildren.appendChild(conditional);
	  	// Create the buffer
	  	buffer = conditional.appendChild(getConvertElement().getXmtaDoc().createElement(DOMXMTA.buffer));
	  	// Create Insert
	  	insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	  	insert.setAttribute(DOMXMTA.atNode, groupId);
	  	buffer.appendChild(insert);
	 	// Create time sensor
	  	Element timeSensor = getConvertElement().getXmtaDoc().createElement(DOMXMTA.TimeSensor);
	  	timeSensor.setAttribute(DOMXMTA.DEF, timeSensorBeginOffsetId);
	  	timeSensor.setAttribute(DOMXMTA.cycleInterval, DOMXMTAParameter.time2Param(offset));
		xmtaIdTable.put(timeSensorBeginOffsetId, timeSensor);
	  	insert.appendChild(timeSensor);
	  	// Create Insert
	  	insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	  	buffer.appendChild(insert);
	  	// Create route
	  	Element route = getConvertElement().getXmtaDoc().createElement(DOMXMTA.ROUTE);
	  	route.setAttribute(DOMXMTA.fromField, DOMXMTA.isActive);
	  	route.setAttribute(DOMXMTA.fromNode, timeSensorBeginOffsetId);
	  	route.setAttribute(DOMXMTA.toField, DOMXMTA.reverseActivate);
	  	route.setAttribute(DOMXMTA.toNode, conditionalInsertId);
	  	insert.appendChild(route);
	  	// Create Delete
	  	delete = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Delete);
	  	delete.setAttribute(DOMXMTA.atNode, conditionalInsertTimerId);
	  	buffer.appendChild(delete);
	}
	// Now create the route from the touch sensor to the conditional
	// <Valuator DEF="???" Factor1="0" Offset1="1"/>
	// <par begin="???">
	//   <Insert>
	//     <ROUTE fromField="touchTime" fromNode="???" toField="inSFTime" toNode="???"/>
	//     <ROUTE fromField="outSFBool" fromNode="???" toField="activate" toNode="???"/>
	//   </Insert>
	// </par>
	TimingAttribute routeSceneInsertTime = sceneInsertTime;
	// create the valuator
	Element valuator = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Valuator);
	valuator.setAttribute(DOMXMTA.DEF, valuatorId);
	valuator.setAttribute(DOMXMTA.Factor1, "0");
	valuator.setAttribute(DOMXMTA.Offset1, "1");
	groupChildren.appendChild(valuator);
	// create the par for the insert routes
	Element parInsertBeginRoutes = getConvertElement().getXmtaDoc().createElement(DOMXMTA.par);
	parInsertBeginRoutes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(routeSceneInsertTime.getClockValue()));
	if (sensorElementMapped) {
		treeMediaElement_.addToParRoutes(parInsertBeginRoutes);
	} else {	// attach the par to the sensor element
		touchSensorElement.addToParRoutes(parInsertBeginRoutes);
	}
	// create the insert
	insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	parInsertBeginRoutes.appendChild(insert);
	// create the route from the touch sensor to the valuator
	Element route = getConvertElement().getXmtaDoc().createElement(DOMXMTA.ROUTE);
	route.setAttribute(DOMXMTA.fromField, DOMXMTA.touchTime);
	route.setAttribute(DOMXMTA.fromNode, touchSensorId);
	route.setAttribute(DOMXMTA.toField, DOMXMTA.inSFTime);
	route.setAttribute(DOMXMTA.toNode, valuatorId);
	insert.appendChild(route);
	parInsertBeginRoutes = getConvertElement().getXmtaDoc().createElement(DOMXMTA.par);
	parInsertBeginRoutes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(routeSceneInsertTime.getClockValue()));
	treeMediaElement_.addToParRoutes(parInsertBeginRoutes);
	// create the insert
	insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	parInsertBeginRoutes.appendChild(insert);

	// create the route from the valuator to the conditional (either the insert or the insert timer)
	route = getConvertElement().getXmtaDoc().createElement(DOMXMTA.ROUTE);
	route.setAttribute(DOMXMTA.fromField, DOMXMTA.outSFBool);
	route.setAttribute(DOMXMTA.fromNode, valuatorId);
	route.setAttribute(DOMXMTA.toField, DOMXMTA.activate);
	if (offset <= 0)
	 	route.setAttribute(DOMXMTA.toNode, conditionalInsertId);
	else
	 	route.setAttribute(DOMXMTA.toNode, conditionalInsertTimerId);
	insert.appendChild(route);

	// Create a touch sensor.
	Element touchSensor = xmtaDoc.createElement(DOMXMTA.TouchSensor);
	// make the switch and transform ids based on object id or make them unique.
	touchSensorId = treeMediaElement_.getXmtaTouchSensorId(elasId);
	// Set up the id.  
	touchSensor.setAttribute(DOMXMTA.DEF, touchSensorId);
	if (treeMediaElement_.touchSensorEnabled()) {
		touchSensor.setAttribute(DOMXMTA.enabled, DOMXMTA.string_true);
	} else {
		touchSensor.setAttribute(DOMXMTA.enabled, DOMXMTA.string_false);
	}
	xmtaIdTable.put(touchSensorId, touchSensor);
	groupChildren.appendChild(touchSensor);

	// Do we use z-order positioning? If so then we need to create an insert to modify the OrderedGroup's atField.
	if (xmtaParentGroupElement.usePositions()) {
		// Create insert Field.  
		// Insert group isn't known yet so don't fill it in.
		// Also don't know the zOrder so don't fill it in.
		Element insertAtField = xmtaDoc.createElement(DOMXMTA.Insert);
		insertAtField.setAttribute(DOMXMTA.atNode, parentOrderedGroupId);
		insertAtField.setAttribute(DOMXMTA.atField, DOMXMTA.order);
		insertAtField.setAttribute(DOMXMTA.position, DOMXMTA.END);
		insertAtField.setAttribute(DOMXMTA.value, "-1"); // give it a default position value
		parInsertNode.appendChild(insertAtField);


		// Set the transformation information (need transform2d, switch and insertfield).
		// Do this dynamically since we don't want to require a material class in the build.
		try {
			Class treeTransformationClass = Class.forName("xmto2xmta.tree.transformation");
			Class transformationClass = Class.forName("xmto2xmta.mapmedia.MedTransformation");
			MedXmediaChildrenElement xmediaChildrenElement_ = (MedXmediaChildrenElement) transformationClass.newInstance();
			Vector children = getConvertElement().getChildren();
			for (Enumeration enum = children.elements(); enum.hasMoreElements();) {
				xmto2xmta.tree.ConvertElement child = (xmto2xmta.tree.ConvertElement) enum.nextElement();
				if (treeTransformationClass.isInstance(child))
					xmediaChildrenElement_.setTransformation(child, null, null, insertAtField);
			}
		} catch (Exception e) {
			if (debug_)
				fireErrorEvent(ErrorEvent.TRACE, 0, "MedMediaElement.createXmtaMapping() - Unable to create class=transformation exception=" + e);
		}
	}

	//
	return true;
}

/*********************************************************************** 
 **********************************************************************/
private boolean deleteNodeAtEndOfDuration(Node parent, String groupId, long duration) {
	Element insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	insert.setAttribute(DOMXMTA.atNode, groupId);
	parent.appendChild(insert);
  	Element timeSensor = getConvertElement().getXmtaDoc().createElement(DOMXMTA.TimeSensor);
	String timeSensorId = IdManager.getNextId(IdManager.TimeSensorID);
	timeSensor.setAttribute(DOMXMTA.DEF, timeSensorId);
  	timeSensor.setAttribute(DOMXMTA.cycleInterval, DOMXMTAParameter.time2Param(duration));
  	insert.appendChild(timeSensor);
	insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	insert.setAttribute(DOMXMTA.atNode, groupId);
	parent.appendChild(insert);
	Element conditional = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Conditional);
	String conditionalId = IdManager.getNextId(IdManager.ConditionalID);
	conditional.setAttribute(DOMXMTA.DEF, conditionalId);
  	insert.appendChild(conditional);
  	Node buffer = conditional.appendChild(getConvertElement().getXmtaDoc().createElement(DOMXMTA.buffer));
	conditional.appendChild(buffer);
	Element delete = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Delete);
	delete.setAttribute(DOMXMTA.atNode, groupId);
	buffer.appendChild(delete);
	insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	parent.appendChild(insert);
  	Element route = getConvertElement().getXmtaDoc().createElement(DOMXMTA.ROUTE);
  	route.setAttribute(DOMXMTA.fromField, DOMXMTA.isActive);
  	route.setAttribute(DOMXMTA.fromNode, timeSensorId);
  	route.setAttribute(DOMXMTA.toField, DOMXMTA.reverseActivate);
  	route.setAttribute(DOMXMTA.toNode, conditionalId);
  	insert.appendChild(route);	
	return true;
}

/*********************************************************************** 
 * Map the timing of the end of the object based on the end event(s).
 * The end could be a single event or more likely a list of end events
 * or clock timing.  The first one which occurs will remove the event and
 * the rest will be ignored (or fall through without effect).
 **********************************************************************/
protected boolean createEventDeleteMapping(ELASid elasId, ELASid parentELASid) {
	Document xmtaDoc = treeMediaElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeMediaElement_.getXmtaIds();

	// Find the Xmta region or parent group element which this node should be inserted into.
	// Determine whether we should be under a region or under the parent group
	ZorderSupport xmtaParentGroupElement = null;
	String xmtoAttrRegion = treeMediaElement_.getXmtoAttrRegion();
	if (null == xmtoAttrRegion || xmtoAttrRegion.equals("")) {
		// No region so must be under the parent group
		// Find the Xmta parent group element which this node should be inserted into.
		TreeElement parent = treeMediaElement_.getParent();
		while (null != parent) {
			if (parent instanceof ZorderSupport) {
				xmtaParentGroupElement = (ZorderSupport) parent;
				// break out of the loop
				break;
			}
			parent = parent.getParent();
		}
	} else {
		// Get the region
		Hashtable xmtoIdTable = treeMediaElement_.getXmtoIds();
		Object obj = xmtoIdTable.get(xmtoAttrRegion);
		if (obj instanceof ZorderSupport)
			xmtaParentGroupElement = (ZorderSupport) obj;
	}
	if (null == xmtaParentGroupElement) {
		// We should always beable to find the parent ordered group.  
		// Should be able to find the body or the region
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeMediaElement_.getXmtoElement());
		getConvertElement().remove();
		return false;
	}

	// Get the delete time.
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	EventActiveInterval eventInterval = timeScheduler.getEventActiveInterval(elasId.getElementIndex());
	TimingAttribute eventAttribute = eventInterval.getEndEvent();
	TimingAttribute sceneDeleteTime = eventInterval.getFillEndTime();
	if (eventAttribute == null || eventAttribute.getTimeValueType() != TimingAttribute.MULTI_VALUE) {
		return false;
	}
	Vector timingAttributes = eventAttribute.getMultiTimingAttributes();
	if (timingAttributes.size() < 3) {
		return false;
	}
	TimingAttribute sceneInsertTime = (TimingAttribute) timingAttributes.elementAt(0);
	TimingAttribute eventBeginTime = (TimingAttribute) timingAttributes.elementAt(2);
	// For right now we only support time values
	boolean error = false;
	if (null == sceneInsertTime)
	  	error = true;
	else
	  	switch (sceneInsertTime.getTimeValueType()) {
	     	case TimingAttribute.CLOCK_VALUE :
	     		break;
	     	default :
	     		error = true;
	   }
	if (sceneDeleteTime != null) {
	  	switch (sceneDeleteTime.getTimeValueType()) {
	     	case TimingAttribute.CLOCK_VALUE :
	     	case TimingAttribute.INDEFINITE_VALUE :
	     		break;
	     	default :
	     		error = true;
	   }
	}

	String touchSensorId = null;
	xmto2xmta.tree.MediaElement touchSensorElement = null;
	boolean sensorElementMapped = true;
	if (!error) {
	  	if (null == eventBeginTime || TimingAttribute.EVENT_VALUE != eventBeginTime.getTimeValueType())
	  		error = true;
	  	else {
	  		switch (eventBeginTime.getEventSymbol()) {
	  			case TimingAttribute.SYMBOL_EVENT_CLICK :
	  			case TimingAttribute.SYMBOL_EVENT_ACTIVATE_EVENT :
	  				String sensorId = eventBeginTime.getIdValue();
	  				Object obj = getConvertElement().getXmtoIds().get(sensorId);
	  				if (null == obj)
	  					error = true;
	  				else {
	  					if (obj instanceof xmto2xmta.tree.Sensorable) {
							touchSensorElement = (xmto2xmta.tree.MediaElement) obj;
							EventActiveInterval touchEventInterval = timeScheduler.getEventActiveInterval(touchSensorElement.getIndex()); 
							if (touchEventInterval != null) {
								ELASid touchELASid = touchEventInterval.getELASid();
	  							touchSensorId = touchSensorElement.getXmtaTouchSensorId(touchELASid);
								touchSensorElement.enableTouchSensor(true);
								Element touchSensor = (Element) touchSensorElement.getXmtaIds().get(touchSensorId);
								if (touchSensor != null) {
									touchSensor.setAttribute(DOMXMTA.enabled, DOMXMTA.string_true);
								} else {
									sensorElementMapped = false;
								}
							} else {
								error = true;
							}
	  					} else {
	  						error = true;
	  					}
	  				}
	  				break;
	  			default :
	  				error = true;
	  		}
	  	}
	}
   if (error) {
  	// report back the problem
   	MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
     	StringBuffer buffer = new StringBuffer();
     	buffer.append(getConvertElement().getXmtoElement().getNodeName());
     	if (null != getConvertElement().getXmtoAttrId() && !getConvertElement().getXmtoAttrId().equals(""))
     		buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getConvertElement().getXmtoAttrId()).append("\"");
     	Object[] msgArgs = {buffer};
     	String message = msgFormat.format(msgArgs);
     	fireErrorEvent(ErrorEvent.ERROR, 0, message, getConvertElement().getXmtoElement());
     	// Since we don't support the timings for this element remove it from the tree
     	getConvertElement().remove();
     	return false;
    }
	
	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parDeleteNode = xmtaDoc.createElement(DOMXMTA.par);
	treeMediaElement_.addToParNode(parDeleteNode);
	// Set the delete time.
	parDeleteNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(sceneInsertTime.getClockValue()));

	String groupId = treeMediaElement_.getXmtaGroupId(elasId);
	String switchId = treeMediaElement_.getXmtaSwitchId(elasId);
	// make the ids based on object id or make them unique.
	String conditionalDeleteId = treeMediaElement_.getXmtaConditionalDeleteId(elasId);
	String conditionalDeleteTimerId = treeMediaElement_.getXmtaConditionalDeleteTimerId(elasId);
	String timeSensorEndOffsetId = treeMediaElement_.getXmtaTimeSensorEndOffsetId(elasId);
	String valuatorId = IdManager.getNextId(IdManager.ValuatorID);

	Element mainInsert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	mainInsert.setAttribute(DOMXMTA.atNode, groupId);
	parDeleteNode.appendChild(mainInsert);
	
	Element conditional = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Conditional);
	conditional.setAttribute(DOMXMTA.DEF, conditionalDeleteId);
	xmtaIdTable.put(conditionalDeleteId, conditional);
	mainInsert.appendChild(conditional);
	// Create the buffer
	Node buffer = conditional.appendChild(getConvertElement().getXmtaDoc().createElement(DOMXMTA.buffer));
	// Create delete switch node 
	Element delete = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Delete);
	//delete.setAttribute(DOMXMTA.atNode, switchId);
	delete.setAttribute(DOMXMTA.atNode, groupId);
	buffer.appendChild(delete);
	// Create Delete conditional
	// delete = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Delete);
	// delete.setAttribute(DOMXMTA.atNode, conditionalDeleteId);
	// buffer.appendChild(delete);

	// If this is started at an offset of then create a timer to delay the start.
	long offset = eventBeginTime.getClockValue();
	if (offset > 0) {
	 	// <Conditional DEF="???">
	  	//   <buffer>
		//     <Insert atNode="???">
	  	//       <TimeSensor DEF="???" cycleInterval="???"/>
	  	//     </Insert>
	  	//     <Insert>
	  	//       <ROUTE fromField="isActive" fromNode="???" toField="reverseActivate" toNode="???"/>
	  	//     </Insert>
	  	//     <Delete atNode="???"/>
	  	//   </buffer>
	  	// </Conditional>
	 	// Create conditional for Insert
	  	conditional = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Conditional);
	  	conditional.setAttribute(DOMXMTA.DEF, conditionalDeleteTimerId);
	  	xmtaIdTable.put(conditionalDeleteTimerId, conditional);
	  	mainInsert.appendChild(conditional);
	  	// Create the buffer
	  	buffer = conditional.appendChild(getConvertElement().getXmtaDoc().createElement(DOMXMTA.buffer));
	  	// Create Insert
	  	Element insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	  	insert.setAttribute(DOMXMTA.atNode, groupId);
	  	buffer.appendChild(insert);
	 	// Create time sensor
	  	Element timeSensor = getConvertElement().getXmtaDoc().createElement(DOMXMTA.TimeSensor);
	  	timeSensor.setAttribute(DOMXMTA.DEF, timeSensorEndOffsetId);
	  	timeSensor.setAttribute(DOMXMTA.cycleInterval, DOMXMTAParameter.time2Param(offset));
	  	xmtaIdTable.put(timeSensorEndOffsetId, timeSensor);
	  	insert.appendChild(timeSensor);
	  	// Create Insert
	  	insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	  	buffer.appendChild(insert);
	  	// Create route
	  	Element route = getConvertElement().getXmtaDoc().createElement(DOMXMTA.ROUTE);
	  	route.setAttribute(DOMXMTA.fromField, DOMXMTA.isActive);
	  	route.setAttribute(DOMXMTA.fromNode, timeSensorEndOffsetId);
	  	route.setAttribute(DOMXMTA.toField, DOMXMTA.reverseActivate);
	  	route.setAttribute(DOMXMTA.toNode, conditionalDeleteId);
	  	insert.appendChild(route);
	  	// Create Delete
	  	// delete = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Delete);
	  	// delete.setAttribute(DOMXMTA.atNode, conditionalDeleteTimerId);
	  	// buffer.appendChild(delete);
	}
	// Now create the route from the touch sensor to the conditional
	// <Valuator DEF="???" Factor1="0" Offset1="1"/>
	// <par begin="???">
	//   <Insert>
	//     <ROUTE fromField="touchTime" fromNode="???" toField="inSFTime" toNode="???"/>
	//     <ROUTE fromField="outSFBool" fromNode="???" toField="activate" toNode="???"/>
	//   </Insert>
	// </par>
	TimingAttribute routeSceneInsertTime = sceneInsertTime;
	// create the valuator
	Element valuator = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Valuator);
	valuator.setAttribute(DOMXMTA.DEF, valuatorId);
	valuator.setAttribute(DOMXMTA.Factor1, "0");
	valuator.setAttribute(DOMXMTA.Offset1, "1");
	mainInsert.appendChild(valuator);
	// create the par for the insert routes
	Element parInsertEndRoutes = getConvertElement().getXmtaDoc().createElement(DOMXMTA.par);
	parInsertEndRoutes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(routeSceneInsertTime.getClockValue()));
	if (sensorElementMapped) {
		treeMediaElement_.addToParRoutes(parInsertEndRoutes);
	} else {
		touchSensorElement.addToParRoutes(parInsertEndRoutes);
	}
	// create the insert
	Element insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	parInsertEndRoutes.appendChild(insert);
	// create the route from the touch sensor to the valuator
	Element route = getConvertElement().getXmtaDoc().createElement(DOMXMTA.ROUTE);
	route.setAttribute(DOMXMTA.fromField, DOMXMTA.touchTime);
	route.setAttribute(DOMXMTA.fromNode, touchSensorId);
	route.setAttribute(DOMXMTA.toField, DOMXMTA.inSFTime);
	route.setAttribute(DOMXMTA.toNode, valuatorId);
	insert.appendChild(route);
	parInsertEndRoutes = getConvertElement().getXmtaDoc().createElement(DOMXMTA.par);
	parInsertEndRoutes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(routeSceneInsertTime.getClockValue()));
	treeMediaElement_.addToParRoutes(parInsertEndRoutes);
	// create the insert
	insert = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Insert);
	parInsertEndRoutes.appendChild(insert);
	// create the route from the valuator to the conditional (either the insert or the insert timer)
	route = getConvertElement().getXmtaDoc().createElement(DOMXMTA.ROUTE);
	route.setAttribute(DOMXMTA.fromField, DOMXMTA.outSFBool);
	route.setAttribute(DOMXMTA.fromNode, valuatorId);
	route.setAttribute(DOMXMTA.toField, DOMXMTA.activate);
	if (offset <= 0)
	 	route.setAttribute(DOMXMTA.toNode, conditionalDeleteId);
	else
	 	route.setAttribute(DOMXMTA.toNode, conditionalDeleteTimerId);
	insert.appendChild(route);

	if (sceneDeleteTime != null && sceneDeleteTime.getTimeValueType() == TimingAttribute.CLOCK_VALUE) {
		parDeleteNode = xmtaDoc.createElement(DOMXMTA.par);
		treeMediaElement_.addToParNode(parDeleteNode);
		parDeleteNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(sceneDeleteTime.getClockValue()));
		// Create delete switch node 
		delete = getConvertElement().getXmtaDoc().createElement(DOMXMTA.Delete);
		//delete.setAttribute(DOMXMTA.atNode, switchId);
		delete.setAttribute(DOMXMTA.atNode, groupId);
		parDeleteNode.appendChild(delete);
	}

	//
	return true;
}

}
