/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MedPoints.java
//
//=============================================================================
package xmto2xmta.mapmedia;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class to handle the conversion of the points element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public class MedPoints extends MedInternalElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private xmto2xmta.tree.points treePoints_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the points infrastructure.
 * <pre>
 * &lt;PointSet2D DEF="???" color="???"/&gt;
 * </pre>
 **********************************************************************/
protected boolean fillInGeometry(Element geometry, ELASid elasId) {
	//	
	Document xmtaDoc = treePoints_.getXmtaDoc();
	Hashtable xmtaIdTable = treePoints_.getXmtaIds();

	// make the switch and transform ids based on object id or make them unique.
	String pointsId = treePoints_.getXmtaPointsId(elasId);

	//
	Element aPointSet2D = xmtaDoc.createElement(DOMXMTA.PointSet2D);
	aPointSet2D.setAttribute(DOMXMTA.DEF, pointsId);
	xmtaIdTable.put(pointsId, aPointSet2D);
	String point = treePoints_.getXmtaAttrPoint();
	if (null != point) {
		Element aCoord = xmtaDoc.createElement(DOMXMTA.coord);
		aPointSet2D.appendChild(aCoord);
		Element aCoordinate2D = xmtaDoc.createElement(DOMXMTA.Coordinate2D);
		String coordinate2DId = treePoints_.getXmtaCoordinate2DId(elasId);
		aCoordinate2D.setAttribute(DOMXMTA.DEF, coordinate2DId);
		xmtaIdTable.put(coordinate2DId, aCoordinate2D);
		aCoord.appendChild(aCoordinate2D);
		// Check if the default value is what we are already setting it to.
		String defaultValue = aCoordinate2D.getAttribute(DOMXMTA.point);
		if (null == defaultValue || !defaultValue.equals(point))
			aCoordinate2D.setAttribute(DOMXMTA.point, point);
	}
	String color = treePoints_.getXmtaAttrColor();
	if (null != color) {
		Element aColor = xmtaDoc.createElement(DOMXMTA.color);
		aPointSet2D.appendChild(aColor);
		Element aColorNode = xmtaDoc.createElement(DOMXMTA.Color);
		String colorId = treePoints_.getXmtaColorId(elasId);
		aColorNode.setAttribute(DOMXMTA.DEF, colorId);
		xmtaIdTable.put(colorId, aColorNode);
		aColor.appendChild(aColorNode);
		// Check if the default value is what we are already setting it to.
		String defaultValue = aColorNode.getAttribute(DOMXMTA.color);
		if (null == defaultValue || !defaultValue.equals(color))
			aColorNode.setAttribute(DOMXMTA.color, color);
	}
	geometry.appendChild(aPointSet2D);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, xmto2xmta.tree.ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof xmto2xmta.tree.points)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "MedPoints.init() - element=" + element + " is not an instance of xmto2xmta.tree.points");
		throw new Exception("");
	}

	//
	treePoints_ = (xmto2xmta.tree.points) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "MedPoints.init() - treePoints_=" + treePoints_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<MedPoints>" + super.toString() + "</MedPoints>";
}
}
