/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MedPolygons.java
//
//=============================================================================
package xmto2xmta.mapmedia;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class to handle the conversion of the polygons element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public class MedPolygons extends MedInternalElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private xmto2xmta.tree.polygons treePolygons_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the IndexedFaceSet2D infrastructure.
 * <pre>
 * &lt;IndexedFaceSet2D DEF="???" color="???"/&gt;
 * </pre>
 **********************************************************************/
protected boolean fillInGeometry(Element geometry, ELASid elasId) {
	//	
	Document xmtaDoc = treePolygons_.getXmtaDoc();
	Hashtable xmtaIdTable = treePolygons_.getXmtaIds();

	// make the switch and transform ids based on object id or make them unique.
	String polygonsId = treePolygons_.getXmtaPolygonsId(elasId);

	//
	Element aIndexedFaceSet2D = xmtaDoc.createElement(DOMXMTA.IndexedFaceSet2D);
	aIndexedFaceSet2D.setAttribute(DOMXMTA.DEF, polygonsId);
	xmtaIdTable.put(polygonsId, aIndexedFaceSet2D);
	String point = treePolygons_.getXmtaAttrPoint();
	if (null != point) {
		Element aCoord = xmtaDoc.createElement(DOMXMTA.coord);
		aIndexedFaceSet2D.appendChild(aCoord);
		Element aCoordinate2D = xmtaDoc.createElement(DOMXMTA.Coordinate2D);
		String coordinate2DId = treePolygons_.getXmtaCoordinate2DId(elasId);
		aCoordinate2D.setAttribute(DOMXMTA.DEF, coordinate2DId);
		xmtaIdTable.put(coordinate2DId, aCoordinate2D);
		aCoord.appendChild(aCoordinate2D);
		// Check if the default value is what we are already setting it to.
		String defaultValue = aCoordinate2D.getAttribute(DOMXMTA.point);
		if (null == defaultValue || !defaultValue.equals(point))
			aCoordinate2D.setAttribute(DOMXMTA.point, point);
	}
	String color = treePolygons_.getXmtaAttrColor();
	if (null != color) {
		Element aColor = xmtaDoc.createElement(DOMXMTA.color);
		aIndexedFaceSet2D.appendChild(aColor);
		Element aColorNode = xmtaDoc.createElement(DOMXMTA.Color);
		String colorId = treePolygons_.getXmtaColorId(elasId);
		aColorNode.setAttribute(DOMXMTA.DEF, colorId);
		xmtaIdTable.put(colorId, aColorNode);
		aColor.appendChild(aColorNode);
		// Check if the default value is what we are already setting it to.
		String defaultValue = aColorNode.getAttribute(DOMXMTA.color);
		if (null == defaultValue || !defaultValue.equals(color))
			aColorNode.setAttribute(DOMXMTA.color, color);
	}
	String colorPerVertex = treePolygons_.getXmtaAttrColorPerVertex();
	if (null != colorPerVertex) {
		// Check if the default value is what we are already setting it to.
		String defaultValue = aIndexedFaceSet2D.getAttribute(DOMXMTA.colorPerVertex);
		if (null == defaultValue || !defaultValue.equals(colorPerVertex))
			aIndexedFaceSet2D.setAttribute(DOMXMTA.colorPerVertex, colorPerVertex);
	}
	String colorIndex = treePolygons_.getXmtaAttrColorIndex();
	if (null != colorIndex) {
		// Check if the default value is what we are already setting it to.
		String defaultValue = aIndexedFaceSet2D.getAttribute(DOMXMTA.colorIndex);
		if (null == defaultValue || !defaultValue.equals(colorIndex))
			aIndexedFaceSet2D.setAttribute(DOMXMTA.colorIndex, colorIndex);
	}
	String coordIndex = treePolygons_.getXmtaAttrCoordIndex();
	if (null != coordIndex) {
		// Check if the default value is what we are already setting it to.
		String defaultValue = aIndexedFaceSet2D.getAttribute(DOMXMTA.coordIndex);
		if (null == defaultValue || !defaultValue.equals(coordIndex))
			aIndexedFaceSet2D.setAttribute(DOMXMTA.coordIndex, coordIndex);
	}
	String texCoord = treePolygons_.getXmtaAttrTexCoord();
	if (null != texCoord) {
		Element aTexCoord = xmtaDoc.createElement(DOMXMTA.texCoord);
		aIndexedFaceSet2D.appendChild(aTexCoord);
		Element aCoordinate2D = xmtaDoc.createElement(DOMXMTA.Coordinate2D);
		String coordinate2DId = treePolygons_.getXmtaTextureCoordinate2DId(elasId);
		aCoordinate2D.setAttribute(DOMXMTA.DEF, coordinate2DId);
		xmtaIdTable.put(coordinate2DId, aCoordinate2D);
		aTexCoord.appendChild(aCoordinate2D);
		// Check if the default value is what we are already setting it to.
		String defaultValue = aCoordinate2D.getAttribute(DOMXMTA.point);
		if (null == defaultValue || !defaultValue.equals(texCoord))
			aCoordinate2D.setAttribute(DOMXMTA.point, texCoord);
	}
	String texCoordIndex = treePolygons_.getXmtaAttrTexCoordIndex();
	if (null != texCoordIndex) {
		// Check if the default value is what we are already setting it to.
		String defaultValue = aIndexedFaceSet2D.getAttribute(DOMXMTA.texCoordIndex);
		if (null == defaultValue || !defaultValue.equals(texCoordIndex))
			aIndexedFaceSet2D.setAttribute(DOMXMTA.texCoordIndex, texCoordIndex);
	}
	geometry.appendChild(aIndexedFaceSet2D);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, xmto2xmta.tree.ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof xmto2xmta.tree.polygons)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "MedPolygons.init() - element=" + element + " is not an instance of xmto2xmta.tree.polygons");
		throw new Exception("");
	}

	//
	treePolygons_ = (xmto2xmta.tree.polygons) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "MedPolygons.init() - treePolygons_=" + treePolygons_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<MedPolygons>" + super.toString() + "</MedPolygons>";
}
}
