/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MedTransformation.java
//
//=============================================================================
package xmto2xmta.mapmedia;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.resources.TransformationRes;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
/*********************************************************************** 
 * Class to handle the conversion of the transformation element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public class MedTransformation extends MedXmediaChildrenElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
	private ResourceBundle transformationRes_ = ResourceBundle.getBundle("xmto2xmta.resources.TransformationRes", Locale.getDefault());
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
		debug_ = debug;
}
/***********************************************************************
 * Create the material infrastructure. 
 **********************************************************************/
public void setTransformation(xmto2xmta.tree.ConvertElement treeConvertElement, Element switchElement, Element transform2D, Element insertAtField) {
	if (!(treeConvertElement instanceof xmto2xmta.tree.transformation))
		return;
	xmto2xmta.tree.transformation treeTransformationElement = (xmto2xmta.tree.transformation) treeConvertElement;

	//
	if (null != insertAtField) {
		// Set the z-order (order field on insert atField tag), check default value
		String orderAttribute = treeTransformationElement.getXmtoAttrOrder();
		if (null != orderAttribute) {
			String defaultValue = insertAtField.getAttribute(DOMXMTA.value);
			if (null == defaultValue || !defaultValue.equals(orderAttribute))
				insertAtField.setAttribute(DOMXMTA.value, orderAttribute);
		}
	}
	if (null != switchElement) {
		// Set the visibility (choice on switch node), check default value
		String whichChoiceAttribute = treeTransformationElement.getXmtaAttrWhichChoice();
		if (null != whichChoiceAttribute) {
			String defaultValue = switchElement.getAttribute(DOMXMTA.whichChoice);
			if (null == defaultValue || !defaultValue.equals(whichChoiceAttribute))
				switchElement.setAttribute(DOMXMTA.whichChoice, whichChoiceAttribute);
		}
	}
	if (null != transform2D) {
		// Set the center offset, check default value
		String centerAttribute = treeTransformationElement.getXmtaAttrCenter();
		if (null != centerAttribute) {
			String defaultValue = transform2D.getAttribute(DOMXMTA.center);
			if (null == defaultValue || !defaultValue.equals(centerAttribute))
				transform2D.setAttribute(DOMXMTA.center, centerAttribute);
		}

		// Set the scale value, check default value
		String scaleAttribute = treeTransformationElement.getXmtaAttrScale();
		if (null != scaleAttribute) {
			String defaultValue = transform2D.getAttribute(DOMXMTA.scale);
			if (null == defaultValue || !defaultValue.equals(scaleAttribute))
				transform2D.setAttribute(DOMXMTA.scale, scaleAttribute);
		}

		// Set the scale orientation offset, check default value
		String scaleOrientationAttribute = treeTransformationElement.getXmtaAttrScaleOrientation();
		if (null != scaleOrientationAttribute) {
			String defaultValue = transform2D.getAttribute(DOMXMTA.scaleOrientation);
			if (null == defaultValue || !defaultValue.equals(scaleOrientationAttribute))
				transform2D.setAttribute(DOMXMTA.scaleOrientation, scaleOrientationAttribute);
		}

		// Set the translation offset, check default value
		String translationAttribute = treeTransformationElement.getXmtaAttrTranslation();
		if (null != translationAttribute) {
			String defaultValue = transform2D.getAttribute(DOMXMTA.translation);
			if (null == defaultValue || !defaultValue.equals(translationAttribute))
				transform2D.setAttribute(DOMXMTA.translation, translationAttribute);
		}

		// Set the rotation angle, check default value
		String rotationAttribute = treeTransformationElement.getXmtaAttrRotation();
		if (null != rotationAttribute) {
			String defaultValue = transform2D.getAttribute(DOMXMTA.rotationAngle);
			if (null == defaultValue || !defaultValue.equals(rotationAttribute))
				transform2D.setAttribute(DOMXMTA.rotationAngle, rotationAttribute);
		}
	}
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<MedTransformation>" + super.toString() + "</MedTransformation>";
}
}
