/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  StrLayout.java
//
//=============================================================================
package xmto2xmta.mapstructure;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmtutil.DOMXMTA;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.LayoutRes;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.layout;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.XMTO;

/*********************************************************************** 
 * Class to handle the conversion of the XMT-O element.  The class
 * name is XMTO instead of XMT-O since java doesn't allow a dash character 
 * in the class name.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>20 Jun 2001 - Remove support for old XMT-A format.
 * </ul><p>
 *********************************************************************/
public class StrLayout extends StrStructureElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private boolean xmtoUsePixelMetric_ = true;
	private String xmtoAttrType_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private layout treeLayout_= null;
	
	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle layoutRes_ = ResourceBundle.getBundle("xmto2xmta.resources.LayoutRes", Locale.getDefault());
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doMetrics();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
public boolean doMetrics() {
	// Find the XMTO root node
	XMTO treeXmto = null;
	TreeElement parent = treeLayout_.getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			treeXmto = (XMTO) parent;
		}
		parent = parent.getParent();
	}
	if (null == treeXmto) {
		// There always should be at least a xmto root so we should never get here.  
		// If we have then the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeLayout_.getXmtoElement());
		treeLayout_.remove();
		return false;
	}

	//
	boolean usePixelMetrics = treeLayout_.getXmtaAttrUsePixelMetrics();

	//
	Element commandStream = treeXmto.getXmtaCommandStream();
	commandStream.setAttribute(DOMXMTA.pixelMetric, usePixelMetrics ? DOMXMTA.string_true : DOMXMTA.string_false);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof layout)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "StrLayout.init() - element=" + element + " is not an instance of layout");
		throw new Exception("");
	}

	//
	treeLayout_ = (layout) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrLayout.init() - treeLayout_  =" + treeLayout_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<layout>" + super.toString() + "</layout>";
}
}
