/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  StrTopLayout.java
//
//=============================================================================
package xmto2xmta.mapstructure;

import java.awt.Color;
import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.topLayout;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.XMTO;

/*********************************************************************** 
 * Class to handle the conversion of the XMT-O element.  The class
 * name is XMTO instead of XMT-O since java doesn't allow a dash character 
 * in the class name.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>20 Jun 2001 - Remove support for old XMT-A format.
 * </ul><p>
 *********************************************************************/
public class StrTopLayout extends StrStructureElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private topLayout treeTopLayout_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the background infrastructure.
 * The infrastructure contains the par node for the insertion.
 * The par node for the insertion contains the background.
 * <pre>
 *   &lt;par begin="0"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       &lt;Background2D DEF="???" backColor="???"/&gt;
 *     &lt;/Insert&gt;
 *    &lt;Replace atField="set_bind" atNode="???" value="true"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
public boolean createBackground() {
	//
	String color = treeTopLayout_.getXmtaAttrBackgroundColor();
	if (null == color || color.equals(""))
		return true;

	//
	Document xmtaDoc = treeTopLayout_.getXmtaDoc();
	Hashtable xmtaIdTable = treeTopLayout_.getXmtaIds();

	// Find the XMTO root node
	XMTO treeXmto = null;
	TreeElement parent = treeTopLayout_.getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			treeXmto = (XMTO) parent;
			break;
		}
		parent = parent.getParent();
	}
	if (null == treeXmto) {
		// There always should be at least a xmto root so we should never get here.  
		// If we have then the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeTopLayout_.getXmtoElement());
		treeTopLayout_.remove();
		return false;
	}

	// The par node for the insertion contains the background.
	// <par begin="0">
	//   <Insert atNode="???">
	//     <Background2D DEF="???" backColor="???"/>
	//   </Insert>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeTopLayout_.addToParNode(parInsertNode);
	// Set the insert time.
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));

	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element insertAtNode = xmtaDoc.createElement(DOMXMTA.Insert);
	String backgroundGroupId = treeXmto.getBackgroundGroupId();
	insertAtNode.setAttribute(DOMXMTA.atNode, backgroundGroupId);
	parInsertNode.appendChild(insertAtNode);

	// Create background2d to allow the object to be moved
	Element background2D = xmtaDoc.createElement(DOMXMTA.Background2D);
	//
	String background2dId = treeTopLayout_.getXmtaBackground2dId();
	background2D.setAttribute(DOMXMTA.DEF, background2dId);
	xmtaIdTable.put(background2dId, background2D);
	insertAtNode.appendChild(background2D);
	// Set the color check default value
	background2D.setAttribute(DOMXMTA.backColor, color);

	// Bind the background to the stack.
	// <Replace atNode="???" atField="set_bind" value="true"/>
	Element replace = xmtaDoc.createElement(DOMXMTA.Replace);
	replace.setAttribute(DOMXMTA.atNode, background2dId);
	replace.setAttribute(DOMXMTA.atField, DOMXMTA.set_bind);
	replace.setAttribute(DOMXMTA.value, "true");
	parInsertNode.appendChild(replace);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doSize();
	if (!success) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.doMapping() - exiting success=" + success);
		return;
	}

	//
	success = createBackground();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
public boolean doSize() {
	// Find the XMTO root node
	XMTO treeXmto = null;
	TreeElement parent = treeTopLayout_.getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			treeXmto = (XMTO) parent;
		}
		parent = parent.getParent();
	}
	if (null == treeXmto) {
		// There always should be at least a xmto root so we should never get here.  
		// If we have then the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeTopLayout_.getXmtoElement());
		treeTopLayout_.remove();
		return false;
	}

	//
	String width = treeTopLayout_.getXmtoAttrWidth();
	String height = treeTopLayout_.getXmtoAttrHeight();

	//	
	Element commandStream = treeXmto.getXmtaCommandStream();
	//
	Element size = treeTopLayout_.getXmtaDoc().createElement(DOMXMTA.size);
	size.setAttribute(DOMXMTA.pixelWidth, width);
	size.setAttribute(DOMXMTA.pixelHeight, height);
	commandStream.appendChild(size);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof topLayout)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "StrTopLayout.init() - element=" + element + " is not an instance of topLayout");
		throw new Exception("");
	}

	//
	treeTopLayout_ = (topLayout) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrTopLayout.init() - treeTopLayout_ =" + treeTopLayout_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<topLayout>" + super.toString() + "</topLayout>";
}
}
