/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  StrXMTO.java
//
//=============================================================================
package xmto2xmta.mapstructure;

import java.lang.reflect.Method;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.TreeClassObjects;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.XMTO;

/*********************************************************************** 
 * Class to handle the conversion of the XMT-O element.  The class
 * name is XMTO instead of XMT-O since java doesn't allow a dash character 
 * in the class name.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>06 Jun 2001 - Move defs group under a switch so that it can be hidden.
 * <li>17 Jun 2001 - Add %&t;texture&gt; as one of the classes to search for
 *                   in determining whether or not we have Object Descriptors.
 * <li>20 Jun 2001 - Remove support for old XMT-A format.
 * <li>21 Jun 2001 - Add checking %&t;xmtaMedia&gt; to see if it has object descriptors.
 * <li>28 Jun 2001 - Changed SLConfigDescriptor to use predefined option.
 * </ul><p>
 *********************************************************************/
public class StrXMTO extends StrStructureElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private XMTO treeXmto_= null;	

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the body infrastructure. 
 * <pre>
 *   &lt;Body&gt;
 *     &lt;par begin="0"/&gt;
 *     &lt;par begin="0"&gt;
 *       &lt;Replace&gt;
 *         &lt;Scene&gt;
 *           &lt;OrderedGroup&gt;
 *             &lt;children&gt;
 *               &lt;Group DEF="???"&gt;
 *                 &lt;children/&gt;
 *               &lt;/Group&gt;
 *               &lt;Switch whichChoice="-1"&gt;
 *                 &lt;choice&gt;
 *                   &lt;Group DEF="???"&gt;
 *                     &lt;children/&gt;
 *                   &lt;/Group&gt;
 *                 &lt;/choice&gt;
 *               &lt;Switch&gt;
 *               &lt;Switch whichChoice="-1"&gt;
 *                 &lt;choice&gt;
 *                   &lt;Group DEF="???"&gt;
 *                     &lt;children/&gt;
 *                   &lt;/Group&gt;
 *                 &lt;/choice&gt;
 *               &lt;Switch&gt;
 *               &lt;OrderedGroup DEF="???"&gt;
 *                 &lt;children/&gt;
 *               &lt;/OrderedGroup&gt;
 *               &lt;OrderedGroup DEF="???"&gt;
 *                 &lt;children/&gt;
 *               &lt;/OrderedGroup&gt;
 *             &lt;/children&gt;
 *           &lt;/Group&gt;
 *         &lt;/Scene&gt;
 *       &lt;/Replace&gt;
 *     &lt;/par&gt;
 *     &lt;par begin="0"/&gt;
 *     &lt;par begin="0"/&gt;
 *   &lt;/Body&gt;
 * </pre>
 **********************************************************************/
private boolean createXmtaBodyInfrasture() {
	Document xmtaDoc = treeXmto_.getXmtaDoc();
	Element root = xmtaDoc.getDocumentElement();
	Hashtable xmtaIdTable = treeXmto_.getXmtaIds();

	// Create the Body
	Element body = xmtaDoc.createElement(DOMXMTA.Body);
	root.appendChild(body);

	// Create the ODs par.
	Element parOds = treeXmto_.getXmtaParOds();
	parOds.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));
	body.appendChild(parOds);

	// Create the Nodes par.
	Element parNodes = treeXmto_.getXmtaParNodes();
	parNodes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));
	body.appendChild(parNodes);

	// Create the Animation Nodes par.
	// Create the animation nodes's par separate from the Node's par so that we can guarantee that the 
	// nodes in the animation occur at a specific time are added after the nodes.  This is so that we don't
	// have forward referencing of node ids, where the node may reference a node before the
	// node is added to the tree.  The XMT-O document can have forward referenceing of the target for the
	// for the animation.
	Element parAnimationNodes = treeXmto_.getXmtaParAnimationNodes();
	parAnimationNodes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));
	body.appendChild(parAnimationNodes);

	// Create the Routes par.
	// Create the route's par separate from the Node's par so that we can guarantee that the 
	// routes at a specific time are added after the nodes.  This is so that we don't
	// have forward referencing of ids, where the route may reference a node before the
	// node is added to the tree.
	Element parRoutes = treeXmto_.getXmtaParRoutes();
	parRoutes.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));
	body.appendChild(parRoutes);

	// Create the Nodes par.
	Element par = xmtaDoc.createElement(DOMXMTA.par);
	par.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));
	parNodes.appendChild(par);

	// Create the top level replace node.  This should be under the par which is passed
	// in as the targetNode.
	Node replace = par.appendChild(xmtaDoc.createElement(DOMXMTA.Replace));
	//
	Node scene = replace.appendChild(xmtaDoc.createElement(DOMXMTA.Scene));
	// Create group to hold group for backgrounds and media objects (media objects need z-order)
	Node bottomOrderedGroup = scene.appendChild(xmtaDoc.createElement(DOMXMTA.OrderedGroup));
	//
	Node children = bottomOrderedGroup.appendChild(xmtaDoc.createElement(DOMXMTA.children));

	// Create the Group which holds the subsequent background updates.
	Element backgroundGroup = xmtaDoc.createElement(DOMXMTA.Group);
	String backgroundGroupID = treeXmto_.getBackgroundGroupId();
	backgroundGroup.setAttribute(DOMXMTA.DEF, backgroundGroupID);
	xmtaIdTable.put(backgroundGroupID, backgroundGroup);
	children.appendChild(backgroundGroup);
	//
	backgroundGroup.appendChild(xmtaDoc.createElement(DOMXMTA.children));

	// Create the switch which hides the group for the defs.
	// This allows the defs to be turn off and not seen.
	Element switchElement = xmtaDoc.createElement(DOMXMTA.Switch);
	switchElement.setAttribute(DOMXMTA.whichChoice, "-1"); // hidden
	children.appendChild(switchElement);
	// Create the choice node under the switch
	Node choice = switchElement.appendChild(xmtaDoc.createElement(DOMXMTA.choice));
	// Create the Group which holds the subsequent defs updates.
	Element defsGroup = xmtaDoc.createElement(DOMXMTA.Group);
	String defsGroupID = treeXmto_.getDefsGroupId();
	defsGroup.setAttribute(DOMXMTA.DEF, defsGroupID);
	xmtaIdTable.put(defsGroupID, defsGroup);
	choice.appendChild(defsGroup);
	//
	defsGroup.appendChild(xmtaDoc.createElement(DOMXMTA.children));

	// Create the switch which hides the group for the nodes used in animations.
	// This allows the nodes to be turn off and not seen.
	switchElement = xmtaDoc.createElement(DOMXMTA.Switch);
	switchElement.setAttribute(DOMXMTA.whichChoice, "-1"); // hidden
	children.appendChild(switchElement);
	// Create the choice node under the switch
	choice = switchElement.appendChild(xmtaDoc.createElement(DOMXMTA.choice));
	// Create the Group which holds the subsequent updates needed to for nodes used in animations.
	Element animationGroup = xmtaDoc.createElement(DOMXMTA.Group);
	String animationGroupID = treeXmto_.getAnimationGroupId();
	animationGroup.setAttribute(DOMXMTA.DEF, animationGroupID);
	xmtaIdTable.put(animationGroupID, animationGroup);
	choice.appendChild(animationGroup);
	//
	animationGroup.appendChild(xmtaDoc.createElement(DOMXMTA.children));


	// Create the OrderedGroup which holds the subsequent scene node updates.
	Element orderedGroup = xmtaDoc.createElement(DOMXMTA.OrderedGroup);
	String orderedGroupID = treeXmto_.getMediaOrderedGroupId();
	orderedGroup.setAttribute(DOMXMTA.DEF, orderedGroupID);
	xmtaIdTable.put(orderedGroupID, orderedGroup);
	children.appendChild(orderedGroup);
	//
	orderedGroup.appendChild(xmtaDoc.createElement(DOMXMTA.children));

	// Create the OrderedGroup which holds the subsequent layout updates.
	Element layoutOrderedGroup = xmtaDoc.createElement(DOMXMTA.OrderedGroup);
	String layoutOrderedGroupID = treeXmto_.getLayoutOrderedGroupId();
	layoutOrderedGroup.setAttribute(DOMXMTA.DEF, layoutOrderedGroupID);
	xmtaIdTable.put(layoutOrderedGroupID, layoutOrderedGroup);
	children.appendChild(layoutOrderedGroup);
	//
	layoutOrderedGroup.appendChild(xmtaDoc.createElement(DOMXMTA.children));

	//
	return true;
}
/***********************************************************************
 * Create the XMT-A infrasturcture.
 * <pre>
 * &lt;Header&gt;
 *   &lt;InitialObjectDescriptor ObjectDescriptorID="???"&gt;
 *     &lt;esDescr&gt;
 *       &lt;ES_Descriptor ES_ID="???"&gt;
 *         &lt;decConfigDescr&gt;
 *           &lt;DecoderConfigDescriptor bufferSizeDB="auto" objectTypeIndication="255" streamType="3"&gt;
 *             &lt;decSpecificInfo&gt;
 *               &lt;BIFSConfig nodeIDbits="auto" routeIDbits="auto"/&gt;
 *                 &lt;commandStream pixelMetric="???"&gt;
 *                   &lt;size pixelWidth="???" pixelHeight="???"/&gt;
 *                 &lt;/commandStream&gt;
 *               &lt;/BIFSConfig&gt;
 *             &lt;/decSpecificInfo&gt;
 *           &lt;/DecoderConfigDescriptor&gt;
 *         &lt;/decConfigDescr&gt;
 *         &lt;slConfigDescr&gt;
 *           &lt;SLConfigDescriptor&gt;
 *             &lt;predefined value="2"/&gt;
 *           &lt;/SLConfigDescriptor&gt;
 *         &lt;/slConfigDescr&gt;
 *       &lt;/ES_Descriptor&gt;
 *       &lt;ES_Descriptor ES_ID="???"&gt;
 *         &lt;decConfigDescr&gt;
 *           &lt;DecoderConfigDescriptor bufferSizeDB="auto" objectTypeIndication="255" streamType="1"/&gt;
 *         &lt;/decConfigDescr&gt;
 *         &lt;slConfigDescr&gt;
 *           &lt;SLConfigDescriptor&gt;
 *             &lt;predefined value="2"/&gt;
 *           &lt;/SLConfigDescriptor&gt;
 *         &lt;/slConfigDescr&gt;
 *       &lt;/ES_Descriptor&gt;
 *     &lt;/esDescr&gt;
 *   &lt;/InitialObjectDescriptor&gt;
 * &lt;/Header&gt;
 * </pre>
 **********************************************************************/
private boolean createXmtaHeaderInfrastructure() {
	Document xmtaDoc = treeXmto_.getXmtaDoc();
	Element root = xmtaDoc.getDocumentElement();
	Hashtable xmtaIdTable = treeXmto_.getXmtaIds();

	// Create the header 
	Element header = xmtaDoc.createElement(DOMXMTA.Header);
	root.appendChild(header);

	// write initialOD
	Element initialObjectDescriptor = xmtaDoc.createElement(DOMXMTA.InitialObjectDescriptor);
	String iodID = IdManager.getNextId(IdManager.IodID);
	initialObjectDescriptor.setAttribute(DOMXMTA.objectDescriptorID, iodID);
	xmtaIdTable.put(iodID, initialObjectDescriptor);
	header.appendChild(initialObjectDescriptor);

	//
	Element profiles = xmtaDoc.createElement(DOMXMTA.Profiles);
	profiles.setAttribute(DOMXMTA.ODProfileLevelIndication, DOMXMTA.Unspecified);
	profiles.setAttribute(DOMXMTA.sceneProfileLevelIndication, DOMXMTA.Unspecified);
	profiles.setAttribute(DOMXMTA.audioProfileLevelIndication, DOMXMTA.Unspecified);
	profiles.setAttribute(DOMXMTA.visualProfileLevelIndication, DOMXMTA.Unspecified);
	profiles.setAttribute(DOMXMTA.includeInlineProfileLevelFlag, DOMXMTA.string_false);
	initialObjectDescriptor.appendChild(profiles);

	//
	Element esDescr = xmtaDoc.createElement(DOMXMTA.esDescr);
	Node descr = initialObjectDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.Descr));
	descr.appendChild(esDescr);

	// write ESDescriptor for bifs stream
	Element bifsESDescriptor = xmtaDoc.createElement(DOMXMTA.ES_Descriptor);
	String iodBifsESID = IdManager.getNextId(IdManager.IodBifsESID);
	bifsESDescriptor.setAttribute(DOMXMTA.ES_ID, iodBifsESID);
	xmtaIdTable.put(iodBifsESID, bifsESDescriptor);
	esDescr.appendChild(bifsESDescriptor);

	//
	Node decConfigDescr = bifsESDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.decConfigDescr));

	//
	Element decoderConfigDescriptor = xmtaDoc.createElement(DOMXMTA.DecoderConfigDescriptor);
	decoderConfigDescriptor.setAttribute(DOMXMTA.objectTypeIndication, DOMXMTA.Unspecified);
	decoderConfigDescriptor.setAttribute(DOMXMTA.streamType, DOMXMTA.SceneDescription);
	decoderConfigDescriptor.setAttribute(DOMXMTA.bufferSizeDB, DOMXMTA.auto);
	decConfigDescr.appendChild(decoderConfigDescriptor);

	//
	Node decSpecificInfo = decoderConfigDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.decSpecificInfo));

	// Determine the information concerning metrics
	Element bifsConfig = treeXmto_.getXmtaBIFSConfig();
	bifsConfig.setAttribute(DOMXMTA.nodeIDbits, DOMXMTA.auto);
	bifsConfig.setAttribute(DOMXMTA.routeIDbits, DOMXMTA.auto);
	decSpecificInfo.appendChild(bifsConfig);
	Element commandStream = treeXmto_.getXmtaCommandStream();
	commandStream.setAttribute(DOMXMTA.pixelMetric, DOMXMTA.string_true);
	bifsConfig.appendChild(commandStream);
	// use the default.  The layout will over write it if it is specified.

	//
	Node slConfigDescr = bifsESDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.slConfigDescr));

	//
	Element slConfigDescriptor = xmtaDoc.createElement(DOMXMTA.SLConfigDescriptor);
	slConfigDescr.appendChild(slConfigDescriptor);
	Element predefined = xmtaDoc.createElement(DOMXMTA.predefined);
	predefined.setAttribute(DOMXMTA.value, "2");
	slConfigDescriptor.appendChild(predefined);

	// Only write out the Initial OD  Scene if there are any Object Descriptors.
	boolean hasObjectDescriptors = determineHasObjectDescriptors();
	if (hasObjectDescriptors) {
		// write ESDescriptor for od stream.
		Element odESDescriptor = xmtaDoc.createElement(DOMXMTA.ES_Descriptor);
		String iodOdESID = IdManager.getNextId(IdManager.IodOdESID);
		odESDescriptor.setAttribute(DOMXMTA.ES_ID, iodOdESID);
		xmtaIdTable.put(iodOdESID, odESDescriptor);
		esDescr.appendChild(odESDescriptor);

		//
		decConfigDescr = odESDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.decConfigDescr));

		//	
		decoderConfigDescriptor = xmtaDoc.createElement(DOMXMTA.DecoderConfigDescriptor);
		decoderConfigDescriptor.setAttribute(DOMXMTA.objectTypeIndication, DOMXMTA.Unspecified);
		decoderConfigDescriptor.setAttribute(DOMXMTA.streamType, DOMXMTA.ObjectDescriptor);
		decoderConfigDescriptor.setAttribute(DOMXMTA.bufferSizeDB, DOMXMTA.auto);
		decConfigDescr.appendChild(decoderConfigDescriptor);

		//
		slConfigDescr = odESDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.slConfigDescr));

		//
		slConfigDescriptor = xmtaDoc.createElement(DOMXMTA.SLConfigDescriptor);
		slConfigDescr.appendChild(slConfigDescriptor);
		predefined = xmtaDoc.createElement(DOMXMTA.predefined);
		predefined.setAttribute(DOMXMTA.value, "2");
		slConfigDescriptor.appendChild(predefined);
	}

	//
	return true;
}
/***********************************************************************
 * Create the XMT-A infrasturcture.
 **********************************************************************/
private boolean createXmtaInfrastructure() {
	boolean success = true;
	success = createXmtaHeaderInfrastructure();
	if (!success)
		return success;
	success = createXmtaBodyInfrasture();
	if (!success)
		return success;
	return true;
}
/***********************************************************************
 **********************************************************************/
private boolean determineHasBackdropSrc(TreeElement treeElement, Class backdropClass, Method getXmtoAttrSrcMethod) {
	if (backdropClass.isInstance(treeElement)) {
		try {
			String xmtoAttrSrc = (String) getXmtoAttrSrcMethod.invoke(treeElement, null);
			if (null != xmtoAttrSrc && !xmtoAttrSrc.equals(""))
				return true;
		} catch (Exception e) {
		}
	}

	// This element isn't one on the lsit so we have to recurse through the children until we find one.
	for (Enumeration enum = treeElement.getChildren().elements(); enum.hasMoreElements();) {
		boolean hasClass = determineHasBackdropSrc((TreeElement) enum.nextElement(), backdropClass, getXmtoAttrSrcMethod);
		if (hasClass)
			return true;
	}
	return false;
}
/***********************************************************************
 **********************************************************************/
private boolean determineHasClass(TreeElement treeElement, Vector classes) {
	for (Enumeration enum = classes.elements(); enum.hasMoreElements();) {
		if (((Class) enum.nextElement()).isInstance(treeElement))
			return true;
	}

	// This element isn't one on the lsit so we have to recurse through the children until we find one.
	for (Enumeration enum = treeElement.getChildren().elements(); enum.hasMoreElements();) {
		boolean hasClass = determineHasClass((TreeElement) enum.nextElement(), classes);
		if (hasClass)
			return true;
	}
	return false;
}
/***********************************************************************
 **********************************************************************/
private boolean determineHasObjectDescriptors() {
	// Create a list of the class we want to search for.
	// We're looking for img, video or audio stream.
	Vector treeClasses = new Vector();
	int[] classObjIds = {
	/* */ TreeClassObjects.XMTO2XMTA_TREE_audio,
	/* */ TreeClassObjects.XMTO2XMTA_TREE_video,
	/* */ TreeClassObjects.XMTO2XMTA_TREE_img,
	/* */ TreeClassObjects.XMTO2XMTA_TREE_texture,
	/* */};
	for (int i = 0; i < classObjIds.length; i++) {
		// Try and load the classes	
		Class treeClass = TreeClassObjects.getClassObject(classObjIds[i]);
		// If there were no problems then add the class to the list
		if (null != treeClass)
			treeClasses.add(treeClass);
	}

	//	
	boolean hasObjectDescriptors = determineHasClass(treeXmto_, treeClasses);

	// Check for src file on background tag
	if (!hasObjectDescriptors) {
		Class backdropClass = TreeClassObjects.getClassObject(TreeClassObjects.XMTO2XMTA_TREE_backdrop);
		Method getXmtoAttrSrcMethod = TreeClassObjects.getMethodObject(TreeClassObjects.XMTO2XMTA_TREE_backdrop_getXmtoAttrSrc);
		if (null != backdropClass && null != getXmtoAttrSrcMethod)
			hasObjectDescriptors = determineHasBackdropSrc(treeXmto_, backdropClass, getXmtoAttrSrcMethod);
	}
	// check for hasOd on xmtaMedia tag
	if (!hasObjectDescriptors) {
		Class xmtaMediaClass = TreeClassObjects.getClassObject(TreeClassObjects.XMTO2XMTA_TREE_xmtaMedia);
		Method getXmtaAttrHasODMethod = TreeClassObjects.getMethodObject(TreeClassObjects.XMTO2XMTA_TREE_xmtaMedia_getXmtaAttrHasOD);
		hasObjectDescriptors = determineHasXmtaMediaHasOD(treeXmto_, xmtaMediaClass, getXmtaAttrHasODMethod);
	}

	//
	return hasObjectDescriptors;
}
/***********************************************************************
 **********************************************************************/
private boolean determineHasXmtaMediaHasOD(TreeElement treeElement, Class xmtaMediaClass, Method getXmtoAttrHasODMethod) {
	if (xmtaMediaClass.isInstance(treeElement)) {
		try {
			Boolean xmtoAttrHasOD = (Boolean) getXmtoAttrHasODMethod.invoke(treeElement, null);
			if (null != xmtoAttrHasOD && xmtoAttrHasOD.booleanValue())
				return true;
		} catch (Exception e) {
		}
	}

	// This element isn't one on the lsit so we have to recurse through the children until we find one.
	for (Enumeration enum = treeElement.getChildren().elements(); enum.hasMoreElements();) {
		boolean hasClass = determineHasXmtaMediaHasOD((TreeElement) enum.nextElement(), xmtaMediaClass, getXmtoAttrHasODMethod);
		if (hasClass)
			return true;
	}
	return false;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = createXmtaInfrastructure();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.doMapping() - exiting success=" + success);
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof XMTO)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.init() - element=" + element + " is not an instance of XMTO");
		throw new Exception("");
	}

	//
	treeXmto_ = (XMTO) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "StrXMTO.init() - treeXmto_ =" + treeXmto_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<XMT-O>" + super.toString() + "</XMT-O>";
}
}
