/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  UseFontStyle.java
//
//=============================================================================
package xmto2xmta.mapuse;

import java.lang.reflect.Method;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.TreeClassObjects;
import xmtutil.DOMXMTA;
/*********************************************************************** 
 * Class to handle the conversion of the material element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public class UseFontStyle extends UseXmediaChildrenDefsElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;
/***********************************************************************
 * return the name of element which needs to be created.
 **********************************************************************/
protected String getElementName() {
	return DOMXMTA.FontStyle;
}
/***********************************************************************
 * Get the XMTA field id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtFieldAttr() {
	return DOMXMTA.fontStyle;
}
/***********************************************************************
 * Get the XMTA node id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtNodeAttr(ELASid elasId) {
	if (null == elasId)
		return "";

	// get the text element
	ConvertElement convertElement = (ConvertElement) getConvertElement().getConvertIndexTable().get(elasId.getElementIndex());

	// Check to make sure parent is a string class
	Class stringClass = TreeClassObjects.getClassObject(TreeClassObjects.XMTO2XMTA_TREE_string);
	if (null == stringClass || !stringClass.isInstance(convertElement))
		return "";

	// Now get the id
	Method stringGetXmtaTextId = TreeClassObjects.getMethodObject(TreeClassObjects.XMTO2XMTA_TREE_string_getXmtaTextId);
	Object args[] = {elasId};
	try {
		String xmtaTextId = (String) stringGetXmtaTextId.invoke(convertElement, args);
		if (null != xmtaTextId && !xmtaTextId.equals(""))
			return xmtaTextId;
	} catch (Exception e) {
		return "";
	}

	//
	return "";
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
