/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  UseMediaGroupElement.java
//
//=============================================================================
package xmto2xmta.mapuse;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.DefsObject;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.use;
import xmto2xmta.tree.ZorderSupport;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to handle the conversion of the body element.
 *
 * <p><b>History:</b><ul>
 * <li>12 Jun 2001 - Created file.
 *                 - Add use of media object and group support.
 * <li>21 Jun 2001 - Disable the touchsensor by default.  Enable it when it is used.
 * </ul><p>
 *********************************************************************/
public class UseMediaGroupElement extends UseTimingElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private use treeUse_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the par node for the deletion.
 * The par for the deletion, deletes the top level group for the element.  It also
 * takes care of the position in the ordered group field (for z-order)
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Delete atNode="???"/&gt;
 *     &lt;Delete atField="order" atNode="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createDeleteMapping(ELASid elasId) {

	// JSB: TBD
	// for now ignore events
	//
	//	if (endResolved) {
	//	} else {
	//		// Not so easy way.  In this case we have an event so end with a conditional and
	//		// set route from touch sensor.
	//		success = timeMapEndEvent();
	//		if (!success)
	//			return success;
	//	}

	TimeScheduler timeScheduler = treeUse_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute deleteTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (activeInterval.getELASids().size() - 1 == sdIndex)
		deleteTime = activeInterval.getFillEndTime();
	else
		deleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);
	// If this simple duration doesn't end then we don't need to worry about deleting it.
	if (deleteTime.getTimeValueType() == TimingAttribute.INDEFINITE_VALUE)
		return true;

	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();
	Hashtable xmtaIdTable = getConvertElement().getXmtaIds();

	// Find the Xmta region or parent group element which this node should be inserted into.
	// Determine whether we should be under a region or under the parent group
	ZorderSupport xmtaParentGroupElement = null;
	String xmtoAttrRegion = treeUse_.getXmtoAttrRegion();
	if (null == xmtoAttrRegion || xmtoAttrRegion.equals("")) {
		// No region so must be under the parent group
		// Find the Xmta parent group element which this node should be inserted into.
		TreeElement parent = treeUse_.getParent();
		while (null != parent) {
			if (parent instanceof ZorderSupport) {
				xmtaParentGroupElement = (ZorderSupport) parent;
				// break out of the loop
				break;
			}
			parent = parent.getParent();
		}
	} else {
		// Get the region
		Hashtable xmtoIdTable = treeUse_.getXmtoIds();
		Object obj = xmtoIdTable.get(xmtoAttrRegion);
		if (obj instanceof ZorderSupport)
			xmtaParentGroupElement = (ZorderSupport) obj;
	}

	//	
	if (null == xmtaParentGroupElement) {
		// We should always beable to find the parent ordered group.  
		// Should be able to find the body or the region
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), getConvertElement().getXmtoElement());
		getConvertElement().remove();
		return false;
	}

	// Get the ordered group of the parent we were inserted into.
	String xmtaParentOrderedGroupId = xmtaParentGroupElement.getXmtaOrderedGroupId(lifeCycle.getParentELASid());
	String groupId = treeUse_.getXmtaGroupId(elasId);

	// The par for the deletion, deletes the top level group for the element.  It also
	// takes care of the position in the ordered group field (for z-order)
	// <par begin="???">
	//   <Delete atNode="???"/>
	//   <Delete atField="order" atNode="???"/>
	// </par>

	// Create the par for the delete command.  Don't place it in the XMT-A tree yet.
	Element parDeleteNode = xmtaDoc.createElement(DOMXMTA.par);
	parDeleteNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(deleteTime.getClockValue()));
	treeUse_.addToParNode(parDeleteNode);

	// Create delete Node
	Element deleteNode = xmtaDoc.createElement(DOMXMTA.Delete);
	deleteNode.setAttribute(DOMXMTA.atNode, groupId);
	parDeleteNode.appendChild(deleteNode);

	// Do we use z-order positioning? If so then we need to delete the OrderedGroup's atField.
	if (xmtaParentGroupElement.usePositions()) {
		// Create delete Field.
		// Insert group isn't known yet so don't fill it in.
		// Also don't know the position it will be in at deletion time so don't fill it in.
		Element deleteField = xmtaDoc.createElement(DOMXMTA.Delete);
		deleteField.setAttribute(DOMXMTA.atNode, xmtaParentOrderedGroupId);
		deleteField.setAttribute(DOMXMTA.atField, DOMXMTA.order);
		parDeleteNode.appendChild(deleteField);

		// get the position the deleted element is in so that we delete the correct filed position.
		int deleteFieldPosition = xmtaParentGroupElement.getDeleteFieldPosition(elasId);
		deleteField.setAttribute(DOMXMTA.position, "" + deleteFieldPosition);
	}

	//
	return true;
}
/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the par node for the insertion.
 * The par node for the insertion contains the top level group which will contain the
 * information for the element.  This also takes care of the position in the ordered group
 * field (for z-order).
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       &lt;Group DEF="???"&gt;
 *         &lt;children&gt;
 *           &lt;TouchSensor DEF="???" enabled="false"&gt;
 *         &lt;/children&gt;
 *       &lt;/Group&gt;
 *     &lt;/Insert&gt;
 *     &lt;Insert atField="order" atNode="???" position="END" value="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createInsertMapping(ELASid elasId) {
	// JSB: TBD
	// for now ignore events
	//	// Attach the switch into the group
	//	if (beginResolved) {
	//		// Easy way timing is resolved so attach switch directly
	//		groupChildren_.appendChild(switch_);
	//	} else {
	//		// Not so easy way.  In this case we have an event so attach switch inside conditional and
	//		// set route from touch sensor.
	//		success = timeMapBeginEvent();
	//		if (!success)
	//			return success;
	//	}


	//
	Document xmtaDoc = treeUse_.getXmtaDoc();
	Hashtable xmtaIdTable = treeUse_.getXmtaIds();

	// Find the Xmta region or parent group element which this node should be inserted into.
	// Determine whether we should be under a region or under the parent group
	ZorderSupport xmtaParentGroupElement = null;
	String xmtoAttrRegion = treeUse_.getXmtoAttrRegion();
	if (null == xmtoAttrRegion || xmtoAttrRegion.equals("")) {
		// No region so must be under the parent group
		// Find the Xmta parent group element which this node should be inserted into.
		TreeElement parent = treeUse_.getParent();
		while (null != parent) {
			if (parent instanceof ZorderSupport) {
				xmtaParentGroupElement = (ZorderSupport) parent;
				// break out of the loop
				break;
			}
			parent = parent.getParent();
		}
	} else {
		// Get the region
		Hashtable xmtoIdTable = treeUse_.getXmtoIds();
		Object obj = xmtoIdTable.get(xmtoAttrRegion);
		if (obj instanceof ZorderSupport)
			xmtaParentGroupElement = (ZorderSupport) obj;
	}
	if (null == xmtaParentGroupElement) {
		// We should always beable to find the parent ordered group.  
		// Should be able to find the body or the region
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeUse_.getXmtoElement());
		getConvertElement().remove();
		return false;
	}

	// Get the insert time.
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getObservedBeginTime();
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

	// Get the ordered group of the parent we are going to be inserted into.
	String parentOrderedGroupId = xmtaParentGroupElement.getXmtaOrderedGroupId(lifeCycle.getParentELASid());

	// The par node for the insertion contains the top level group which will contain the
	// information for the element.  This also takes care of the position in the ordered group
	// field (for z-order).
	// <par begin="???">
	//   <Insert atNode="???">
	//     <Group DEF="???">
	//       <children>
	//         <TouchSensor DEF="???" enabled="false">
	//       </children>
	//     </Group>
	//   </Insert>
	//   <Insert atField="order" atNode="???" position="END" value="???"/>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeUse_.addToParNode(parInsertNode);
	// Set the insert time.
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element insertAtNode = xmtaDoc.createElement(DOMXMTA.Insert);
	insertAtNode.setAttribute(DOMXMTA.atNode, parentOrderedGroupId);
	parInsertNode.appendChild(insertAtNode);

	// Create the top level group node
	Element group = xmtaDoc.createElement(DOMXMTA.Group);
	String groupId = treeUse_.getXmtaGroupId(elasId);
	group.setAttribute(DOMXMTA.DEF, groupId);
	xmtaIdTable.put(groupId, group);
	insertAtNode.appendChild(group);

	//
	Element groupChildren = xmtaDoc.createElement(DOMXMTA.children);
	group.appendChild(groupChildren);

	//
	Element switchElement = createXmtaMapping(elasId);
	groupChildren.appendChild(switchElement);

	// Create a touch sensor.
	Element touchSensor = xmtaDoc.createElement(DOMXMTA.TouchSensor);
	// make the switch and transform ids based on object id or make them unique.
	String touchSensorId = treeUse_.getXmtaTouchSensorId(elasId);
	// Set up the id.  
	touchSensor.setAttribute(DOMXMTA.DEF, touchSensorId);
	// Disable the touchsensor by default.
	touchSensor.setAttribute(DOMXMTA.enabled, DOMXMTA.string_false);
	xmtaIdTable.put(touchSensorId, touchSensor);
	groupChildren.appendChild(touchSensor);

	// Do we use z-order positioning? If so then we need to create an insert to modify the OrderedGroup's atField.
	if (xmtaParentGroupElement.usePositions()) {
		// Create insert Field.  
		// Insert group isn't known yet so don't fill it in.
		// Also don't know the zOrder so don't fill it in.
		Element insertAtField = xmtaDoc.createElement(DOMXMTA.Insert);
		insertAtField.setAttribute(DOMXMTA.atNode, parentOrderedGroupId);
		insertAtField.setAttribute(DOMXMTA.atField, DOMXMTA.order);
		
		insertAtField.setAttribute(DOMXMTA.position, DOMXMTA.END);
		insertAtField.setAttribute(DOMXMTA.value, "-1"); // give it a default position value
		parInsertNode.appendChild(insertAtField);
	}

	//
	return true;
}
/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the switch and the transform are the standard base nodes for the element.  
 * Where it is placed in the tree is dependent on the begin and end timing for the element.
 * </ul>
 *   &lt;Switch DEF="???"&gt;
 *     &lt;choice&gt;
 *       &lt;Transform2D DEF="???"&gt;
 *         &lt;children/&gt;
 *       &lt;/Transform2D&gt;
 *     &lt;/choice&gt;
 *   &lt;/Switch&gt;
 * </pre>
 **********************************************************************/
private Element createXmtaMapping(ELASid elasId) {
	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();
	Hashtable xmtaIdTable = getConvertElement().getXmtaIds();

	// make the switch and transform ids based on object id or make them unique.
	String switchId = treeUse_.getXmtaSwitchId(elasId);
	String transform2dId = treeUse_.getXmtaTransform2dId(elasId);

	// The switch and the transform are the standard base nodes for the element.  
	// Where it is placed in the tree is dependent on the begin and end timing for the element.
	// <Switch DEF="???">
	//   <choice>
	//     <Transform2D DEF="???">
	//       <children/>
	//     </Transform2D>
	//   </choice>
	// </Switch>

	// Create switch to allow the object to be turned on or off
	Element switchElement = xmtaDoc.createElement(DOMXMTA.Switch);
	switchElement.setAttribute(DOMXMTA.DEF, switchId);
	switchElement.setAttribute(DOMXMTA.whichChoice, "0"); // default is on
	xmtaIdTable.put(switchId, switchElement);

	// Create the choice node under the switch
	Node choice = switchElement.appendChild(xmtaDoc.createElement(DOMXMTA.choice));

	// Create transform2d to allow the object to be moved
	Element transform2D = xmtaDoc.createElement(DOMXMTA.Transform2D);
	transform2D.setAttribute(DOMXMTA.DEF, transform2dId);
	xmtaIdTable.put(transform2dId, transform2D);
	choice.appendChild(transform2D);

	//
	Element transform2dChildren = xmtaDoc.createElement(DOMXMTA.children);
	transform2D.appendChild(transform2dChildren);

	//
	fillInTransform2dChildren(transform2dChildren, elasId);

	//
	return switchElement;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaGroupElement.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doTimingMapping();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaGroupElement.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
private boolean doTimingMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaElement.doTimingMapping() - entered", getConvertElement().getXmtoElement());

	//	
	boolean success = true;

	//
	//	EventModel eventModel = getConvertElement().getXmto2Xmta().getEventModel();
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(getConvertElement().getIndex());

	// debug purposes
	// Go through it once for printing out debug info irregardless of whether it is correct or not.
	if (debug_) {
		int lifeCycleCount = 0;
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
			int activeIntervalCount = 0;
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				//
				StringBuffer sb = new StringBuffer();
				sb.append("UseMediaGroupElement.doTimingMapping() -");
				sb.append(" <").append(getConvertElement().getXmtoElement().getNodeName());
				String id = getConvertElement().getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
				sb.append(" activeInterval[").append(activeIntervalCount).append("]");
				sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
				sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
				sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
				sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
				sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
				ELASid parentElasId = lifeCycle.getParentELASid();
				sb.append(" parentElasId=(");
				if (null == parentElasId) {
					sb.append("null");
				} else {
					sb.append(parentElasId.getElementIndex());
					sb.append(",").append(parentElasId.getLifeCycleIndex());
					sb.append(",").append(parentElasId.getIntervalIndex());
					sb.append(",").append(parentElasId.getSimpleDurIndex());
				}
				sb.append(")");
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					sb.append(" repeatTime=").append(repeatTime);
				}
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					sb.append(" elasId=(").append(elasId.getElementIndex());
					sb.append(",").append(elasId.getLifeCycleIndex());
					sb.append(",").append(elasId.getIntervalIndex());
					sb.append(",").append(elasId.getSimpleDurIndex());
					sb.append(")");
				}
				fireErrorEvent(ErrorEvent.TRACE, 0, sb.toString(), getConvertElement().getXmtoElement());

				//
				activeIntervalCount++;
			}
			lifeCycleCount++;
		}
	}


	// Now go through for real.
	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// Check to make sure the timing values are valid
			boolean error = false;
			TimingAttribute computedBeginTime = activeInterval.getComputedBeginTime();
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			TimingAttribute activeDuration = activeInterval.getActiveDuration();
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (null == computedBeginTime || null == observedBeginTime || null == fillEndTime || null == activeDuration)
				error = true;
			if (!error) {
				switch (computedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (observedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (activeDuration.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (fillEndTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					switch (repeatTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
			}

			// If there is a problem then report back the problem and remove the object.
			if (error) {
				MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
				StringBuffer buffer = new StringBuffer();
				buffer.append(getConvertElement().getXmtoElement().getNodeName());
				if (null != getConvertElement().getXmtoAttrId() && !getConvertElement().getXmtoAttrId().equals(""))
					buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getConvertElement().getXmtoAttrId()).append("\"");
				Object[] msgArgs = {buffer};
				String message = msgFormat.format(msgArgs);
				fireErrorEvent(ErrorEvent.ERROR, 0, message, getConvertElement().getXmtoElement());

				// Since we don't support the timings for this element remove it from the tree
				getConvertElement().remove();
				if (debug_)
					fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaGroupElement.doTimingMapping() - exiting success=false");
				return false;
			}

			// All the timings check out OK so now loop through the ELASids and create the nodes.
			for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
				ELASid elasId = (ELASid) elasIdEnum.nextElement();

				// Create the insert Mapping.
				success = createInsertMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaGroupElement.doTimingMapping() - exiting success=false");
					return success;
				}

				// Create the delete Mapping.
				success = createDeleteMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaGroupElement.doTimingMapping() - exiting success=false");
					return success;
				}
			}
		}
	}

	// 
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaGroupElement.doTimingMapping() - exiting success=true");
	return true;
}
/***********************************************************************
 * Create the rectangle infrastructure.
 * <pre>
 * &lt;Group USE="???"/&gt;
 * </pre>
 **********************************************************************/
protected boolean fillInTransform2dChildren(Element transform2dChildren, ELASid elasId) {
	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();

	//
	Element group = xmtaDoc.createElement(DOMXMTA.Group);
	DefsObject defsObject = treeUse_.getXmtoAttrXlinkHrefDefsObject();
	String defsId = defsObject.getXmtaDefsId();
	group.setAttribute(DOMXMTA.USE, defsId);
	transform2dChildren.appendChild(group);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof use)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaGroupElement.init() - element=" + element + " is not an instance of use");
		throw new Exception("");
	}

	//
	treeUse_ = (use) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseMediaGroupElement.init() - treeUse_ =" + treeUse_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<UseMediaGroupElement>" + super.toString() + "</UseMediaGroupElement>";
}
}
