/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  UseUse.java
//
//=============================================================================
package xmto2xmta.mapuse;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import xmto2xmta.ErrorEvent;
import xmto2xmta.MapInterface;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.UseRes;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.DefsObject;
import xmto2xmta.tree.MediaElement;
import xmto2xmta.tree.TreeClassObjects;
import xmto2xmta.tree.use;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class to handle the conversion of the body element.
 *
 * <p><b>History:</b><ul>
 * <li>12 Jun 2001 - Created file.
 *                 - Add use of media object and group support.
 * <li>21 Jun 2001 - Add xmtaMedia element.
 * </ul><p>
 *********************************************************************/
public class UseUse extends UseTimingElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private use treeUse_ = null;
	private Xmto2Xmta xmto2xmta_ = null;

	/***********************************************************************
	 * Hash table containing the layout mapping classes which correspond to 
	 * the XMT-O tree elements elements.
	 * The prime number 89 was chosen as the hash table initialize size because it was
	 * mentioned as a better starting hash size for optimization than the default 101.
	 * The layoutTable_ table is composed of (key, data)
	 *   The key is the XMT-O tree element index.
	 *   The data is the layout mapping class.
	 **********************************************************************/
	private Hashtable mpTable_ = new Hashtable(89);

	/***********************************************************************
	 * Used to create Lists for class mappings
	 **********************************************************************/
	static private boolean classListInitialized_ = false;
	static private final String treeClassesPackage_ = "xmto2xmta.tree.";
	static private final String mpClassesPackage_ = "xmto2xmta.mapuse.";
	static private Vector treeClasses_ = new Vector();
	static private Vector mpClasses_ = new Vector();
	static private final String[] treeClassNames_ = {
	/* 01 */ "MediaElement",
	/* 02 */ "group",
	/* 03 */ "xmtaMedia",
	/* 04 */ "fontStyle",
	/* 05 */ "material",
	/* 06 */ "outline",
	/* 07 */ "chromakey",
	/* 08 */ "texture",
	/*    */};
	static private final String[] mpClassNames_ = {
	/* 01 */ "UseMediaGroupElement",
	/* 02 */ "UseMediaGroupElement",
	/* 03 */ "UseMediaGroupElement",
	/* 04 */ "UseFontStyle",
	/* 05 */ "UseMaterial",
	/* 06 */ "UseOutline",
	/* 07 */ "UseChromakey",
	/* 08 */ "UseTexture",
	/*    */};

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle useRes_ = ResourceBundle.getBundle("xmto2xmta.resources.UseRes", Locale.getDefault());
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseUse.doMapping() - entered convertElement=" + getConvertElement());

	// Get the defs object which we are using.
	DefsObject defsObject = treeUse_.getXmtoAttrXlinkHrefDefsObject();
	mapClass(getConvertElement(), defsObject);

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseUse.doMapping() - exiting");
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof use)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "UseUse.init() - element=" + element + " is not an instance of use");
		throw new Exception("");
	}

	//
	treeUse_ = (use) element;
	xmto2xmta_ = xmto2xmta;

	//
	if (!classListInitialized_)
		initializeClassLists(this);

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseUse.init() - treeUse_ =" + treeUse_);
}
/*******************************************************************
 *******************************************************************/
static private void initializeClassLists(UseUse useUse) {
	if (classListInitialized_)
		return;

	//
	classListInitialized_ = true;

	//	
	if (treeClassNames_.length != mpClassNames_.length)
		useUse.fireErrorEvent(ErrorEvent.FATAL_ERROR, 0, "UseUse.initializeClassLists() - Internal error mapping lists different sizes");

	//
	for (int i = 0; i < treeClassNames_.length; i++) {
		Class treeClass = null;
		Class mpClass = null;

		// Try and load the classes	
		try {
			treeClass = Class.forName(treeClassesPackage_ + treeClassNames_[i]);
		} catch (Exception e) {
			if (debug_)
				useUse.fireErrorEvent(ErrorEvent.TRACE, 0, "UseUse.initializeClassLists() - Unable to create class=" + (treeClassesPackage_ + treeClassNames_[i]) + " exception=" + e);
		}
		try {
			mpClass = Class.forName(mpClassesPackage_ + mpClassNames_[i]);
		} catch (Exception e) {
			if (debug_)
			useUse.	fireErrorEvent(ErrorEvent.TRACE, 0, "UseUse.initializeClassLists() - Unable to create class=" + (mpClassesPackage_ + mpClassNames_[i]) + " exception=" + e);
		}

		// If there were no problems then add the files to the list
		if (null != treeClass && null != mpClass) {
			treeClasses_.add(treeClass);
			mpClasses_.add(mpClass);
		}
	}
	if (debug_) {
		for (int i = 0; i < treeClasses_.size(); i++) {
			Class treeClass = (Class) treeClasses_.get(i);
			Class mpClass = (Class) mpClasses_.get(i);
useUse.			fireErrorEvent(ErrorEvent.TRACE, 0, "UseUse.initializeClassLists() - Created treeClass[" + i + "]=" + treeClass + "" + " mpClass[" + i + "]=" + mpClass);
		}
	}
}
/*******************************************************************
 * Do a second level of mapping.  This maps the &lt;use&gt; object
 * to the type of usage it is performing.  The type of useage it is
 * performing is based on the defs object associated with the use.
 * thus a usage of a media object get used like a media object.  But
 * the usage of a fontStyle is used in a very different way, in the
 * way a fontstyle is used.
 *<p>The Media objects and group are used in the same way because
 * they all have as their root a xmt-a group element.  And the
 * usage of a standard group can be used.
 *******************************************************************/
private void mapClass(ConvertElement element, DefsObject defsObject) {
	for (int i = 0; i < treeClasses_.size(); i++) {
		Class treeClass = (Class) treeClasses_.get(i);
		if (treeClass.isInstance(defsObject)) {
			Class mpClass = (Class) mpClasses_.get(i);
			try {
				// Create the class 
				MapInterface mpObject = (MapInterface) mpClass.newInstance();
				mpObject.init(xmto2xmta_, element);

				// Add the class to the list of layout classes.
				mpTable_.put(element.getIndex(), mpObject);

				// Do the layout on the class
				mpObject.doMapping();
				return;
			} catch (Exception e) {
				if (debug_) {
					fireErrorEvent(ErrorEvent.TRACE, 0, "UseUse.mapClass() - unable to create class=" + mpClass.getName() + " exception=" + e);
					e.printStackTrace();
				}
			}
		}
	}
	if (debug_) {
		MessageFormat msgFormat = new MessageFormat(useRes_.getString(UseRes.MSG_WARMING_NO_MAPPING_FOR_DEFS_OBJECT));
		StringBuffer buffer = new StringBuffer();
		buffer.append(treeUse_.getXmtoElement().getNodeName());
		String id = treeUse_.getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		StringBuffer defsBuffer = new StringBuffer();
		defsBuffer.append(((ConvertElement) defsObject).getXmtoElement().getNodeName());
		id = ((ConvertElement) defsObject).getXmtoAttrId();
		if (null != id && !id.equals(""))
			defsBuffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		Object[] msgArgs = {buffer, defsBuffer};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, treeUse_.getXmtoElement());
		treeUse_.remove();
	}
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<UseUse>" + super.toString() + "</UseUse>";
}
}
