/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  UseXmediaChildrenDefsElement.java
//
//=============================================================================
package xmto2xmta.mapuse;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.LifeCycle;
import xmto2xmta.MapRoot;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.DefsObject;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.TimingElement;
import xmto2xmta.tree.use;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;

/*********************************************************************** 
 * Class to group together the Structure type of elements.
 *
 * <p><b>History:</b><ul>
 * <li>12 Jun 2001 - Created file.
 *                 - Add use of media object and group support.
 * </ul><p>
 *********************************************************************/
public abstract class UseXmediaChildrenDefsElement extends MapRoot {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;
	
	/*********************************************************************** 
	 **********************************************************************/
	private use treeUse_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * Create the xMedia Child Element infrastructure.
 * The infrastructure contains the par node for the insertion.
 * This insertMapping is because the XMT-O xMediaChild element is found
 * underneath an mediaObject (or some other timing object) element.  
 * Thus we need to replace the XMT-A field with the appropriate XMT-A node.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Replace atField="???" atNode="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createInsertMapping(ELASid elasId) {
	//
	Document xmtaDoc = treeUse_.getXmtaDoc();
	Hashtable xmtaIdTable = treeUse_.getXmtaIds();

	// Get the insert time.
	TimeScheduler timeScheduler = treeUse_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getObservedBeginTime();
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

	// The par node for the insertion contains the top level group which will contain the
	// information for the element.  This also takes care of the position in the ordered group
	// field (for z-order).
	// <par begin="???">
	//   <Replace atField="???" atNode="???"/>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parReplaceNode = xmtaDoc.createElement(DOMXMTA.par);
	treeUse_.addToParNode(parReplaceNode);
	// Set the insert time.
	parReplaceNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element replaceAtField = xmtaDoc.createElement(DOMXMTA.Replace);
	replaceAtField.setAttribute(DOMXMTA.atNode, getReplaceAtNodeAttr(elasId));
	replaceAtField.setAttribute(DOMXMTA.atField, getReplaceAtFieldAttr());
	parReplaceNode.appendChild(replaceAtField);

	//
	Element xMediaChildElement = xmtaDoc.createElement(getElementName());
	DefsObject defsObject = treeUse_.getXmtoAttrXlinkHrefDefsObject();
	String defsId = defsObject.getXmtaDefsId();
	xMediaChildElement.setAttribute(DOMXMTA.USE, defsId);
	replaceAtField.appendChild(xMediaChildElement);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseXmediaChildrenDefsElement.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doTimingMapping();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseXmediaChildrenDefsElement.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
private boolean doTimingMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseXmediaChildrenDefsElement.doTimingMapping() - entered", getConvertElement().getXmtoElement());

	//	
	boolean success = true;

	//  This object is under a media object and thus we have to create it using the parent's timing intervals.
	TimeScheduler timeScheduler = treeUse_.getXmto2Xmta().getTimeScheduler();
	TreeElement parent = treeUse_.getParent();
	while (null != parent && !(parent instanceof TimingElement))
		parent = parent.getParent();
	if (null == parent) {
		return false;
	}
	ConvertElement parentElement = (ConvertElement) parent;
	Integer parentIndex = parentElement.getIndex();
	Vector lifeCycles = timeScheduler.getLifeCycles(parentIndex);

	// debug purposes
	// Go through it once for printing out debug info irregardless of whether it is correct or not.
	if (debug_) {
		int lifeCycleCount = 0;
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
			int activeIntervalCount = 0;
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				//
				StringBuffer sb = new StringBuffer();
				sb.append("UseXmediaChildrenDefsElement.doTimingMapping() -");
				sb.append(" <").append(treeUse_.getXmtoElement().getNodeName());
				String id = treeUse_.getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" parent=<").append(parentElement.getXmtoElement().getNodeName());
				id = parentElement.getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
				sb.append(" activeInterval[").append(activeIntervalCount).append("]");
				sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
				sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
				sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
				sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
				sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
				ELASid parentElasId = lifeCycle.getParentELASid();
				sb.append(" parentElasId=(");
				if (null == parentElasId) {
					sb.append("null");
				} else {
					sb.append(parentElasId.getElementIndex());
					sb.append(",").append(parentElasId.getLifeCycleIndex());
					sb.append(",").append(parentElasId.getIntervalIndex());
					sb.append(",").append(parentElasId.getSimpleDurIndex());
				}
				sb.append(")");
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					sb.append(" repeatTime=").append(repeatTime);
				}
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					sb.append(" elasId=(").append(elasId.getElementIndex());
					sb.append(",").append(elasId.getLifeCycleIndex());
					sb.append(",").append(elasId.getIntervalIndex());
					sb.append(",").append(elasId.getSimpleDurIndex());
					sb.append(")");
				}
				fireErrorEvent(ErrorEvent.TRACE, 0, sb.toString(), treeUse_.getXmtoElement());

				//
				activeIntervalCount++;
			}
			lifeCycleCount++;
		}
	}


	// Now go through for real.
	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// Check to make sure the timing values are valid
			boolean error = false;
			TimingAttribute computedBeginTime = activeInterval.getComputedBeginTime();
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			TimingAttribute activeDuration = activeInterval.getActiveDuration();
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (null == computedBeginTime || null == observedBeginTime || null == fillEndTime || null == activeDuration)
				error = true;
			if (!error) {
				switch (computedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (observedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (activeDuration.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (fillEndTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					switch (repeatTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
			}

			// If there is a problem then report back the problem and remove the object.
			if (error) {
				MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
				StringBuffer buffer = new StringBuffer();
				buffer.append(getConvertElement().getXmtoElement().getNodeName());
				String id = treeUse_.getXmtoAttrId();
				if (null != id && !id.equals(""))
					buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
				Object[] msgArgs = {buffer};
				String message = msgFormat.format(msgArgs);
				fireErrorEvent(ErrorEvent.ERROR, 0, message, treeUse_.getXmtoElement());

				// Since we don't support the timings for this element remove it from the tree
				treeUse_.remove();
				if (debug_)
					fireErrorEvent(ErrorEvent.TRACE, 0, "UseXmediaChildrenDefsElement.doTimingMapping() - exiting success=false");
				return false;
			}

			// All the timings check out OK so now loop through the ELASids and create the nodes.
			for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
				ELASid elasId = (ELASid) elasIdEnum.nextElement();

				// Create the insert Mapping.
				success = createInsertMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "UseXmediaChildrenDefsElement.doTimingMapping() - exiting success=false");
					return success;
				}

			}
		}
	}

	// 
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseXmediaChildrenDefsElement.doTimingMapping() - exiting success=true");
	return true;
}
/***********************************************************************
 * return the name of element which needs to be created.
 **********************************************************************/
abstract protected String getElementName();
/***********************************************************************
 * Get the XMTA field id in which the xMediaChild should be placed into.
 **********************************************************************/
abstract protected String getReplaceAtFieldAttr();
/***********************************************************************
 * Get the XMTA node id in which the xMediaChild should be placed into.
 **********************************************************************/
abstract protected String getReplaceAtNodeAttr(ELASid elasId);
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof use)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "UseXmediaChildrenDefsElement.init() - element=" + element + " is not an instance of use");
		throw new Exception("");
	}

	//
	treeUse_ = (use) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "UseXmediaChildrenDefsElement.init() - treeUse_ =" + treeUse_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
