/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  XchChromakey.java
//
//=============================================================================
package xmto2xmta.mapxmediachildren;

import xmto2xmta.tree.ExternalVisualElement;
import xmto2xmta.tree.chromakey;
import java.util.Hashtable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.ConvertElement;
//import xmto2xmta.tree.material;
//import xmto2xmta.tree.VisualElement;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class to handle the conversion of the material element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>                                                                         
 *********************************************************************/
public class XchChromakey extends XchXmediaChildrenDefsElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private chromakey treeChromakey_ = null;
/***********************************************************************
 * Create the xMedia Children Element Defs infrastructure.
 * This creates the infrastructure necessary for the XMT-A Material2D element
 * to fit in the defs section.
 * <pre>
 * &lt;Shape&gt;
 *   &lt;appearance&gt;
 *     &lt;Appearance&gt;
 *       &lt;material/&gt;
 *     &lt;/Appearance&gt;
 *   &lt;/appearance&gt;
 * &lt;/Shape&gt;
 * </pre>
 **********************************************************************/
protected Element createXmtaDefsMapping() {
	Document xmtaDoc = treeChromakey_.getXmtaDoc();

	// Create the material infrastructure for defs section
	// <Shape>
	//   <appearance>
	//     <Appearance>
	//       <material/>
	//     </Appearance>
	//   </appearance>
	// </Shape>

	// Create the Shape element.
	Element shape = xmtaDoc.createElement(DOMXMTA.Shape);

	//
	Node appearance = shape.appendChild(xmtaDoc.createElement(DOMXMTA.appearance));
	//
	Node appearanceChild = appearance.appendChild(xmtaDoc.createElement(DOMXMTA.Appearance));
	//
	Node material = appearanceChild.appendChild(xmtaDoc.createElement(DOMXMTA.material));

	// get the materialKey
	Element materialKey = createXmtaMapping(null);
	material.appendChild(materialKey);

	// return the created shape
	return shape;
}
/***********************************************************************
 * Create the Material2D infrastructure. 
 **********************************************************************/
protected Element createXmtaMapping(ELASid elasId) {
	Document xmtaDoc = treeChromakey_.getXmtaDoc();
	Hashtable xmtaIdTable = treeChromakey_.getXmtaIds();

	// make the material id object id, parents id or make them unique.
	String materialKeyId = null;
	if (null != elasId)
		materialKeyId = treeChromakey_.getXmtaMaterialKeyId(elasId);
	else
		materialKeyId = treeChromakey_.getXmtaDefsId();

	// Create the MaterialKey element.
	Element materialKey = xmtaDoc.createElement(DOMXMTA.MaterialKey);
	materialKey.setAttribute(DOMXMTA.DEF, materialKeyId);
	xmtaIdTable.put(materialKeyId, materialKey);

	// Set the keyColor check default value
	String keyColorAttribute = treeChromakey_.getXmtaAttrKeyColor();
	if (null != keyColorAttribute) {
		String defaultValue = materialKey.getAttribute(DOMXMTA.keyColor);
		if (null == defaultValue || !defaultValue.equals(keyColorAttribute))
			materialKey.setAttribute(DOMXMTA.keyColor, keyColorAttribute);
	}

	// Set the isKeyed check default value
	String isKeyedAttribute = treeChromakey_.getXmtoAttrIsKeyed();
	if (null != isKeyedAttribute) {
		String defaultValue = materialKey.getAttribute(DOMXMTA.isKeyed);
		if (null == defaultValue || !defaultValue.equals(isKeyedAttribute))
			materialKey.setAttribute(DOMXMTA.isKeyed, isKeyedAttribute);
	}

	// Set the isRGB check default value
	String isRGBAttribute = treeChromakey_.getXmtoAttrIsRGB();
	if (null != isRGBAttribute) {
		String defaultValue = materialKey.getAttribute(DOMXMTA.isRGB);
		if (null == defaultValue || !defaultValue.equals(isRGBAttribute))
			materialKey.setAttribute(DOMXMTA.isRGB, isRGBAttribute);
	}
	// Set the lowThreshold check default value
	String lowThresholdAttribute = treeChromakey_.getXmtoAttrLowThreshold();
	if (null != lowThresholdAttribute) {
		String defaultValue = materialKey.getAttribute(DOMXMTA.lowThreshold);
		if (null == defaultValue || !defaultValue.equals(lowThresholdAttribute))
			materialKey.setAttribute(DOMXMTA.lowThreshold, lowThresholdAttribute);
	}
	// Set the highThreshold check default value
	String highThresholdAttribute = treeChromakey_.getXmtoAttrHighThreshold();
	if (null != highThresholdAttribute) {
		String defaultValue = materialKey.getAttribute(DOMXMTA.highThreshold);
		if (null == defaultValue || !defaultValue.equals(highThresholdAttribute))
			materialKey.setAttribute(DOMXMTA.highThreshold, highThresholdAttribute);
	}
	// Set the transparency check default value
	String transparencyAttribute = treeChromakey_.getXmtoAttrTransparency();
	if (null != transparencyAttribute) {
		String defaultValue = materialKey.getAttribute(DOMXMTA.transparency);
		if (null == defaultValue || !defaultValue.equals(transparencyAttribute))
			materialKey.setAttribute(DOMXMTA.transparency, transparencyAttribute);
	}

	// return the created materialKey
	return materialKey;
}
/***********************************************************************
 * Get the XMTA field id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtFieldAttr() {
	return DOMXMTA.material;
}
/***********************************************************************
 * Get the XMTA node id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtNodeAttr(ELASid elasId) {
	if (null == elasId)
		return "";

	// get the text element
	Hashtable xmtoIdTable = treeChromakey_.getXmtoIds();
	ConvertElement convertElement = (ConvertElement) treeChromakey_.getConvertIndexTable().get(elasId.getElementIndex());

	// get the id from the visual element
	if (null != convertElement && convertElement instanceof ExternalVisualElement)
		return ((ExternalVisualElement) convertElement).getXmtaAppearanceId(elasId);

	//
	return "";
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof chromakey)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "XchChromakey.init() - element=" + element + " is not an instance of chromakey");
		throw new Exception("");
	}

	//
	treeChromakey_ = (chromakey) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchChromakey.init() - treeChromakey_=" + treeChromakey_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<XchChromakey>" + super.toString() + "</XchChromakey>";
}
}
