/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  XchFontStyle.java
//
//=============================================================================
package xmto2xmta.mapxmediachildren;

import java.lang.reflect.Method;
import java.util.Hashtable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.fontStyle;
import xmto2xmta.tree.TreeClassObjects;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
/*********************************************************************** 
 * Class to handle the conversion of the material element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public class XchFontStyle extends XchXmediaChildrenDefsElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private fontStyle treeFontStyle_ = null;
/***********************************************************************
 * Create the xMedia Children Element Defs infrastructure.
 * This creates the infrastructure necessary for the XMT-A FontStyle element
 * to fit int he defs section.
 * <pre>
 * &lt;Shape&gt;
 *   &lt;geometry&gt;
 *     &lt;Text&gt;
 *       &lt;fontStyle/&gt;
 *     &lt;/Text&gt;
 *   &lt;/geometry&gt;
 * &lt;/Shape&gt;
 * </pre>
 **********************************************************************/
protected Element createXmtaDefsMapping() {
	Document xmtaDoc = treeFontStyle_.getXmtaDoc();

	// Create the FontStyle infrastructure for defs section
	// <Shape>
	//   <geometry>
	//     <Text/>
	//   </geometry>
	// </Shape>

	// Create the Shape element.
	Element shape = xmtaDoc.createElement(DOMXMTA.Shape);

	//
	Node geometry = shape.appendChild(xmtaDoc.createElement(DOMXMTA.geometry));
	//
	Node text = geometry.appendChild(xmtaDoc.createElement(DOMXMTA.Text));
	//
	Node fontStyle = text.appendChild(xmtaDoc.createElement(DOMXMTA.fontStyle));

	// get the FontStyle
	Element childFontStyle = createXmtaMapping(null);
	fontStyle.appendChild(childFontStyle );

	// return the created shape
	return shape;
}
/***********************************************************************
 * Create the FontStyle infrastructure. 
 **********************************************************************/
protected Element createXmtaMapping(ELASid elasId) {
	Document xmtaDoc = treeFontStyle_.getXmtaDoc();
	Hashtable xmtaIdTable = treeFontStyle_.getXmtaIds();

	// make the material id object id, parents id or make them unique.
	String fontStyleId = null;
	if (null != elasId)
		fontStyleId = treeFontStyle_.getXmtaFontStyleId(elasId);
	else
		fontStyleId = treeFontStyle_.getXmtaDefsId();

	// Create the FontStyle element.
	Element fontStyle = xmtaDoc.createElement(DOMXMTA.FontStyle);
	fontStyle.setAttribute(DOMXMTA.DEF, fontStyleId);
	xmtaIdTable.put(fontStyleId, fontStyle);

	// Set the font family check default value
	String familyAttribute = treeFontStyle_.getXmtoAttrFamily();
	if (null != familyAttribute) {
                familyAttribute = DOMXMTAParameter.textLines2Param(DOMXMTOParameter.param2TextLines(familyAttribute));
		String defaultValue = fontStyle.getAttribute(DOMXMTA.family);
		if (null == defaultValue || !defaultValue.equals(familyAttribute))
			fontStyle.setAttribute(DOMXMTA.family, familyAttribute);
	}
	// Set the font horizontal alinment check default value
	String horizontalAttribute = treeFontStyle_.getXmtoAttrHorizontal();
	if (null != horizontalAttribute) {
		String defaultValue = fontStyle.getAttribute(DOMXMTA.horizontal);
		if (null == defaultValue || !defaultValue.equals(horizontalAttribute))
			fontStyle.setAttribute(DOMXMTA.horizontal, horizontalAttribute);
	}
	// Set the font justify positions check default value
	String justifyAttribute = treeFontStyle_.getXmtoAttrJustify();
	if (null != justifyAttribute) {
                String horzAlign = DOMXMTOParameter.param2HorizontalAlignment(justifyAttribute);
                String vertAlign = DOMXMTOParameter.param2VerticalAlignment(justifyAttribute);
                justifyAttribute = DOMXMTAParameter.alignment2Param(horzAlign, vertAlign);
		String defaultValue = fontStyle.getAttribute(DOMXMTA.justify);
		if (null == defaultValue || !defaultValue.equals(justifyAttribute))
			fontStyle.setAttribute(DOMXMTA.justify, justifyAttribute);
	}
	// Set the font language check default value
	String languageAttribute = treeFontStyle_.getXmtoAttrLanguage();
	if (null != languageAttribute) {
		String defaultValue = fontStyle.getAttribute(DOMXMTA.language);
		if (null == defaultValue || !defaultValue.equals(languageAttribute))
			fontStyle.setAttribute(DOMXMTA.language, languageAttribute);
	}
	// Set the font leftToRight direction check default value
	String leftToRightAttribute = treeFontStyle_.getXmtoAttrLeftToRight();
	if (null != leftToRightAttribute) {
		String defaultValue = fontStyle.getAttribute(DOMXMTA.leftToRight);
		if (null == defaultValue || !defaultValue.equals(leftToRightAttribute))
			fontStyle.setAttribute(DOMXMTA.leftToRight, leftToRightAttribute);
	}
	// Set the font size check default value
	String sizeAttribute = treeFontStyle_.getXmtoAttrSize();
	if (null != sizeAttribute) {
		String defaultValue = fontStyle.getAttribute(DOMXMTA.size);
		if (null == defaultValue || !defaultValue.equals(sizeAttribute))
			fontStyle.setAttribute(DOMXMTA.size, sizeAttribute);
	}
	// Set the font spacing check default value
	String spacingAttribute = treeFontStyle_.getXmtoAttrSpacing();
	if (null != spacingAttribute) {
		String defaultValue = fontStyle.getAttribute(DOMXMTA.spacing);
		if (null == defaultValue || !defaultValue.equals(spacingAttribute))
			fontStyle.setAttribute(DOMXMTA.spacing, spacingAttribute);
	}
	// Set the font style check default value
	String styleAttribute = treeFontStyle_.getXmtoAttrStyle();
	Integer styleAttr = DOMXMTOParameter.param2FontStyle(styleAttribute);
        if (null != styleAttribute && null != styleAttr) {
                styleAttribute = DOMXMTAParameter.fontStyle2Param(styleAttr.intValue());
		String defaultValue = fontStyle.getAttribute(DOMXMTA.style);
		if (null == defaultValue || !defaultValue.equals(styleAttribute))
			fontStyle.setAttribute(DOMXMTA.style, styleAttribute);
	}
	// Set the font topToBottom check default value
	String topToBottomAttribute = treeFontStyle_.getXmtoAttrTopToBottom();
	if (null != topToBottomAttribute) {
		String defaultValue = fontStyle.getAttribute(DOMXMTA.topToBottom);
		if (null == defaultValue || !defaultValue.equals(topToBottomAttribute))
			fontStyle.setAttribute(DOMXMTA.topToBottom, topToBottomAttribute);
	}

	// return the created fontstyle
	return fontStyle;
}
/***********************************************************************
 * Get the XMTA field id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtFieldAttr() {
	return DOMXMTA.fontStyle;
}
/***********************************************************************
 * Get the XMTA node id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtNodeAttr(ELASid elasId) {
	if (null == elasId)
		return "";

	// get the text element
	Hashtable xmtoIdTable = treeFontStyle_.getXmtoIds();
	ConvertElement convertElement = (ConvertElement) treeFontStyle_.getConvertIndexTable().get(elasId.getElementIndex());

	// Check to make sure parent is a string class
	Class stringClass = TreeClassObjects.getClassObject(TreeClassObjects.XMTO2XMTA_TREE_string);
	if (null == stringClass || !stringClass.isInstance(convertElement))
		return "";

	// Now get the id
	Method stringGetXmtaTextId = TreeClassObjects.getMethodObject(TreeClassObjects.XMTO2XMTA_TREE_string_getXmtaTextId);
	Object args[] = {elasId};
	try {
		String xmtaTextId = (String) stringGetXmtaTextId.invoke(convertElement, args);
		if (null != xmtaTextId && !xmtaTextId.equals(""))
			return xmtaTextId;
	} catch (Exception e) {
		return "";
	}

	//
	return "";
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof fontStyle)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "XchFontStyle.init() - element=" + element + " is not an instance of fontStyle");
		throw new Exception("");
	}

	//
	treeFontStyle_ = (fontStyle) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchFontStyle.init() - treeFontStyle_=" + treeFontStyle_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<XchFontStyle>" + super.toString() + "</XchFontStyle>";
}
}
