/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  XchMaterial.java
//
//=============================================================================
package xmto2xmta.mapxmediachildren;

import java.util.Hashtable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.material;
import xmto2xmta.tree.VisualElement;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class to handle the conversion of the material element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public class XchMaterial extends XchXmediaChildrenDefsElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private material treeMaterial_ = null;
/***********************************************************************
 * Create the xMedia Children Element Defs infrastructure.
 * This creates the infrastructure necessary for the XMT-A Material2D element
 * to fit in the defs section.
 * <pre>
 * &lt;Shape&gt;
 *   &lt;appearance&gt;
 *     &lt;Appearance&gt;
 *       &lt;material/&gt;
 *     &lt;/Appearance&gt;
 *   &lt;/appearance&gt;
 * &lt;/Shape&gt;
 * </pre>
 **********************************************************************/
protected Element createXmtaDefsMapping() {
	Document xmtaDoc = treeMaterial_.getXmtaDoc();

	// Create the material infrastructure for defs section
	// <Shape>
	//   <appearance>
	//     <Appearance>
	//       <material/>
	//     </Appearance>
	//   </appearance>
	// </Shape>

	// Create the Shape element.
	Element shape = xmtaDoc.createElement(DOMXMTA.Shape);

	//
	Node appearance = shape.appendChild(xmtaDoc.createElement(DOMXMTA.appearance));
	//
	Node appearanceChild = appearance.appendChild(xmtaDoc.createElement(DOMXMTA.Appearance));
	//
	Node material = appearanceChild.appendChild(xmtaDoc.createElement(DOMXMTA.material));

	// get the FontStyle
	Element material2D = createXmtaMapping(null);
	material.appendChild(material2D);

	// return the created shape
	return shape;
}
/***********************************************************************
 * Create the Material2D infrastructure. 
 **********************************************************************/
protected Element createXmtaMapping(ELASid elasId) {
	Document xmtaDoc = treeMaterial_.getXmtaDoc();
	Hashtable xmtaIdTable = treeMaterial_.getXmtaIds();

	// make the material id object id, parents id or make them unique.
	String material2dId = null;
	if (null != elasId)
		material2dId = treeMaterial_.getXmtaMaterial2dId(elasId);
	else
		material2dId = treeMaterial_.getXmtaDefsId();

	// Create the FontStyle element.
	Element material2d = xmtaDoc.createElement(DOMXMTA.Material2D);
	material2d.setAttribute(DOMXMTA.DEF, material2dId);
	xmtaIdTable.put(material2dId, material2d);

	// Set the color check default value
	String emissiveColorAttribute = treeMaterial_.getXmtaAttrEmissiveColor();
	if (null != emissiveColorAttribute) {
		String defaultValue = material2d.getAttribute(DOMXMTA.emissiveColor);
		if (null == defaultValue || !defaultValue.equals(emissiveColorAttribute))
			material2d.setAttribute(DOMXMTA.emissiveColor, emissiveColorAttribute);
	}

	// Set the filled check default value
	String filledAttribute = treeMaterial_.getXmtoAttrFilled();
	if (null != filledAttribute) {
		String defaultValue = material2d.getAttribute(DOMXMTA.filled);
		if (null == defaultValue || !defaultValue.equals(filledAttribute))
			material2d.setAttribute(DOMXMTA.filled, filledAttribute);
	}

	// Set the transparency check default value
	String transparencyAttribute = treeMaterial_.getXmtoAttrTransparency();
	if (null != transparencyAttribute) {
		String defaultValue = material2d.getAttribute(DOMXMTA.transparency);
		if (null == defaultValue || !defaultValue.equals(transparencyAttribute))
			material2d.setAttribute(DOMXMTA.transparency, transparencyAttribute);
	}

	// return the created material2d
	return material2d;
}
/***********************************************************************
 * Get the XMTA field id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtFieldAttr() {
	return DOMXMTA.material;
}
/***********************************************************************
 * Get the XMTA node id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtNodeAttr(ELASid elasId) {
	if (null == elasId)
		return "";

	// get the text element
	Hashtable xmtoIdTable = treeMaterial_.getXmtoIds();
	ConvertElement convertElement = (ConvertElement) treeMaterial_.getConvertIndexTable().get(elasId.getElementIndex());

	// get the id from the visual element
	if (null != convertElement && convertElement instanceof VisualElement)
		return ((VisualElement) convertElement).getXmtaAppearanceId(elasId);

	//
	return "";
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof material)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "XchMaterial.init() - element=" + element + " is not an instance of material");
		throw new Exception("");
	}

	//
	treeMaterial_ = (material) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchMaterial.init() - treeMaterial_=" + treeMaterial_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<XchMaterial>" + super.toString() + "</XchMaterial>";
}
}
