/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  XchOutline.java
//
//=============================================================================
package xmto2xmta.mapxmediachildren;

import java.lang.reflect.Method;
import java.util.Hashtable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.outline;
import xmto2xmta.tree.TreeClassObjects;
import xmto2xmta.tree.VisualElement;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class to handle the conversion of the material element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public class XchOutline extends XchXmediaChildrenDefsElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private outline treeOutline_ = null;
/***********************************************************************
 * Create the xMedia Children Element Defs infrastructure.
 * This creates the infrastructure necessary for the XMT-A LineProperties element
 * to fit in the defs section.
 * <pre>
 * &lt;Shape&gt;
 *   &lt;appearance&gt;
 *     &lt;Appearance&gt;
 *       &lt;material&gt;
 *         &lt;Material2D&gt;
 *           &lt;lineProps&gt;
 *             u&lt;LineProperties/&gt;
 *           &lt;/lineProps&gt;
 *         &lt;/Material2D&gt;
 *       &lt;/material&gt;
 *     &lt;/Appearance&gt;
 *   &lt;/appearance&gt;
 * &lt;/Shape&gt;
 * </pre>
 **********************************************************************/
protected Element createXmtaDefsMapping() {
	Document xmtaDoc = treeOutline_.getXmtaDoc();

	// Create the LineProperties infrastructure for defs section
	// <Shape>
	//   <appearance>
	//     <Appearance>
	//       <material>
	//         <Material2D>
	//           <lineProps>
	//             <LineProperties/>
	//           </lineProps>
	//         </Material2D>
	//       </material>
	//     </Appearance>
	//   </appearance>
	// </Shape>

	// Create the Shape element.
	Element shape = xmtaDoc.createElement(DOMXMTA.Shape);

	//
	Node appearance = shape.appendChild(xmtaDoc.createElement(DOMXMTA.appearance));
	//
	Node appearanceChild = appearance.appendChild(xmtaDoc.createElement(DOMXMTA.Appearance));
	//
	Node material = appearanceChild.appendChild(xmtaDoc.createElement(DOMXMTA.material));
	Node material2D = material.appendChild(xmtaDoc.createElement(DOMXMTA.Material2D));
	Node lineProps = material2D.appendChild(xmtaDoc.createElement(DOMXMTA.lineProps));

	// get the LineProperties
	Element lineProperties = createXmtaMapping(null);
	lineProps.appendChild(lineProperties);

	// return the created shape
	return shape;
}
/***********************************************************************
 * Create the LineProperties infrastructure. 
 **********************************************************************/
protected Element createXmtaMapping(ELASid elasId) {
	Document xmtaDoc = treeOutline_.getXmtaDoc();
	Hashtable xmtaIdTable = treeOutline_.getXmtaIds();

	// make the material id object id, parents id or make them unique.
	String linePropertiesId = null;
	if (null != elasId)
		linePropertiesId = treeOutline_.getXmtaLinePropertiesId(elasId);
	else
		linePropertiesId = treeOutline_.getXmtaDefsId();

	// Create the LineProperties element.
	Element lineProperties = xmtaDoc.createElement(DOMXMTA.LineProperties);
	lineProperties.setAttribute(DOMXMTA.DEF, linePropertiesId);
	xmtaIdTable.put(linePropertiesId, lineProperties);

	// Set the line color check default value
	String lineColor = treeOutline_.getXmtaAttrLineColor();
	if (null != lineColor) {
		String defaultValue = lineProperties.getAttribute(DOMXMTA.lineColor);
		if (null == defaultValue || !defaultValue.equals(lineColor))
			lineProperties.setAttribute(DOMXMTA.lineColor, lineColor);
	}

	// Set the line style check default value
	String lineStyle = treeOutline_.getXmtaAttrLineStyle();
	if (null != lineStyle) {
		String defaultValue = lineProperties.getAttribute(DOMXMTA.lineStyle);
		if (null == defaultValue || !defaultValue.equals(lineStyle))
			lineProperties.setAttribute(DOMXMTA.lineStyle, lineStyle);
	}

	// Set the filled check default value
	String width = treeOutline_.getXmtoAttrWidth();
	if (null != width) {
		String defaultValue = lineProperties.getAttribute(DOMXMTA.width);
		if (null == defaultValue || !defaultValue.equals(width))
			lineProperties.setAttribute(DOMXMTA.width, width);
	}

	// return the created LineProperties
	return lineProperties;
}
/***********************************************************************
 * Get the XMTA field id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtFieldAttr() {
	return DOMXMTA.lineProps;
}
/***********************************************************************
 * Get the XMTA node id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtNodeAttr(ELASid elasId) {
	if (null == elasId)
		return "";

	// get the material element
	Hashtable xmtoIdTable = treeOutline_.getXmtoIds();
	ConvertElement convertElement = (ConvertElement) treeOutline_.getParent();

	// Check to make sure parent is a material class
	Class materialClass = TreeClassObjects.getClassObject(TreeClassObjects.XMTO2XMTA_TREE_material);
	if (null == materialClass || !materialClass.isInstance(convertElement))
		return "";

	// Now get the id
	Method materialGetXmtaMaterial2dId = TreeClassObjects.getMethodObject(TreeClassObjects.XMTO2XMTA_TREE_material_getXmtaMaterial2dId);
	Object args[] = {elasId};
	try {
		String xmtaMaterial2dId = (String) materialGetXmtaMaterial2dId.invoke(convertElement, args);
		if (null != xmtaMaterial2dId && !xmtaMaterial2dId.equals(""))
			return xmtaMaterial2dId;
	} catch (Exception e) {
		return "";
	}

	//
	return "";
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof outline)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "XchOutline.init() - element=" + element + " is not an instance of outline");
		throw new Exception("");
	}

	//
	treeOutline_ = (outline) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchOutline.init() - treeOutline_=" + treeOutline_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<XchOutline>" + super.toString() + "</XchOutline>";
}
}
