/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  XchTexture.java
//
//=============================================================================
package xmto2xmta.mapxmediachildren;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmtutil.MediaObjectInformation;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.LifeCycle;
import xmto2xmta.MapRoot;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.XchTextureRes;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.InternalElement;
import xmto2xmta.tree.texture;
import xmto2xmta.tree.TimingElement;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.XmediaChildrenDefsElement;
import xmto2xmta.tree.ZorderSupport;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;

/*********************************************************************** 
 * Class to group together the Structure type of elements.
 *
 * <p><b>History:</b><ul>
 * <li>12 Jun 2001 - Created file.
 *                 - Add use of media object and group support.
 * </ul><p>
 *********************************************************************/
public class XchTexture extends XchXmediaChildrenExternalElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;
	
	/*********************************************************************** 
	 **********************************************************************/
	private texture treeTexture_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
	private ResourceBundle xchTextureRes_ = ResourceBundle.getBundle("xmto2xmta.resources.XchTextureRes", Locale.getDefault());
/***********************************************************************
 * Create the xMedia Child Element infrastructure.
 * The infrastructure contains the par node for the insertion.
 * This insertMapping is because the XMT-O xMediaChild element is found
 * underneath a &lt;defs&gt; element instead of a mediaObject 
 * (or some other timing object) element.  
 * Thus we need to insert into the defs infrastructure. 
 * The place in the defs infrastructure is the same for each xMediaChild element.
 * Each xMediaChildStructure is going to have to have some xmta structure in order
 * to be placed int the defs structure.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       &lt;Shape&gt;
 *         &lt;appearance&gt;
 *           &lt;Appearance&gt;
 *             &lt;texture&gt;
 *             &lt;/texture&gt;
 *             &lt;textureTransform&gt;
 *             &lt;/textureTransform&gt;
 *           &lt;/Appearance&gt;
 *         &lt;/appearance&gt;
 *       &lt;/Shape&gt;
 *     &lt;/Insert&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createInsertMapping() {
	boolean success = super.createInsertMapping();
	if (!success)
		return success;
	//
	Document xmtaDoc = treeTexture_.getXmtaDoc();
	Hashtable xmtaIdTable = treeTexture_.getXmtaIds();

	// Parent is <defs>
	TreeElement parent = treeTexture_.getParent();
	if (!treeTexture_.isDefsParent())
		return false;
	if (!(parent instanceof ZorderSupport))
		return false;
	String xmtaDefsId = ((ZorderSupport) parent).getXmtaOrderedGroupId(null);

	// The par node for the insertion contains the top level group which will contain the
	// information for the element.  
	// <par begin="???">
	//   <Insert atNode="???"/>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeTexture_.addToParNode(parInsertNode);
	// Set the insert time.
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));


	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element insertAtNode = xmtaDoc.createElement(DOMXMTA.Insert);
	insertAtNode.setAttribute(DOMXMTA.atNode, xmtaDefsId);
	parInsertNode.appendChild(insertAtNode);

	//
	Element xMediaChildDefsInfrastructure = createXmtaDefsMapping();
	insertAtNode.appendChild(xMediaChildDefsInfrastructure);

	//
	return true;
}
/***********************************************************************
 * Create the xMedia Child Element infrastructure.
 * The infrastructure contains the par node for the insertion.
 * This insertMapping is because the XMT-O xMediaChild element is found
 * underneath an mediaObject (or some other timing object) element.  
 * Thus we need to replace the XMT-A field with the appropriate XMT-A node.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Replace atField="???" atNode="???"/&gt;
 *     &lt;Replace atField="???" atNode="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createInsertMapping(ELASid elasId) {
	boolean success = super.createInsertMapping(elasId);
	if (!success)
		return success;
	//
	Document xmtaDoc = treeTexture_.getXmtaDoc();
	Hashtable xmtaIdTable = treeTexture_.getXmtaIds();

	// Get the insert time.
	TimeScheduler timeScheduler = treeTexture_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getObservedBeginTime();
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

	// The par node for the insertion contains the top level group which will contain the
	// information for the element.  This also takes care of the position in the ordered group
	// field (for z-order).
	// <par begin="???">
	//   <Replace atField="???" atNode="???"/>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parReplaceNode = xmtaDoc.createElement(DOMXMTA.par);
	treeTexture_.addToParNode(parReplaceNode);
	// Set the insert time.
	parReplaceNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	// Create insert Node for Texture node
	Element replaceAtField = xmtaDoc.createElement(DOMXMTA.Replace);
	replaceAtField.setAttribute(DOMXMTA.atNode, getReplaceAtNodeAttr(elasId));
	replaceAtField.setAttribute(DOMXMTA.atField, DOMXMTA.texture);
	parReplaceNode.appendChild(replaceAtField);

	//
	String textureId = treeTexture_.getXmtaTextureId(elasId);
	String objectDescriptorId = treeTexture_.getXmtaObjectDescriptorId(elasId);
	replaceAtField.appendChild(createXmtaMappingTexture(textureId, objectDescriptorId));

	// Create insert Node for Texture Transform node
	replaceAtField = xmtaDoc.createElement(DOMXMTA.Replace);
	replaceAtField.setAttribute(DOMXMTA.atNode, getReplaceAtNodeAttr(elasId));
	replaceAtField.setAttribute(DOMXMTA.atField, DOMXMTA.textureTransform);
	parReplaceNode.appendChild(replaceAtField);

	//
	String textureTransformId = treeTexture_.getXmtaTextureTransformId(elasId);
	replaceAtField.appendChild(createXmtaMappingTextureTransform(textureTransformId));

	//
	return true;
}
/***********************************************************************
 * Create the xMedia Children Element Defs infrastructure.
 * This creates the infrastructure necessary for the XMT-A Material2D element
 * to fit in the defs section.
 * <pre>
 * &lt;Shape&gt;
 *   &lt;appearance&gt;
 *     &lt;Appearance&gt;
 *       &lt;texture&gt;
 *       &lt;/texture&gt;
 *       &lt;textureTransform&gt;
 *       &lt;/textureTransform&gt;
 *     &lt;/Appearance&gt;
 *   &lt;/appearance&gt;
 * &lt;/Shape&gt;
 * </pre>
 **********************************************************************/
protected Element createXmtaDefsMapping() {
	Document xmtaDoc = treeTexture_.getXmtaDoc();

	// Create the material infrastructure for defs section
	// <Shape>
	//   <appearance>
	//     <Appearance>
	//       <texture/>
	//       <textureTransform/>
	//     </Appearance>
	//   </appearance>
	// </Shape>

	// Create the Shape element.
	Element shape = xmtaDoc.createElement(DOMXMTA.Shape);

	//
	Node appearance = shape.appendChild(xmtaDoc.createElement(DOMXMTA.appearance));
	//
	Node appearanceChild = appearance.appendChild(xmtaDoc.createElement(DOMXMTA.Appearance));
	//
	Node texture = appearanceChild.appendChild(xmtaDoc.createElement(DOMXMTA.texture));

	// get the child texture element (MovieTexture or ImageTexture)
	String textureId = treeTexture_.getXmtaTextureId();
	String objectDescriptorId = treeTexture_.getXmtaObjectDescriptorId();
	texture.appendChild(createXmtaMappingTexture(textureId, objectDescriptorId));

	//
	Node textureTransform = appearanceChild.appendChild(xmtaDoc.createElement(DOMXMTA.textureTransform));

	// get the child textureTransform element
	String textureTransformId = treeTexture_.getXmtaTextureTransformId();
	textureTransform.appendChild(createXmtaMappingTextureTransform(textureTransformId));

	// return the created shape
	return shape;
}
/***********************************************************************
 * Create the texture infrastructure. 
 **********************************************************************/
protected Element createXmtaMappingTexture(String textureId, String objectDescriptorId) {
	Document xmtaDoc = treeTexture_.getXmtaDoc();
	Hashtable xmtaIdTable = treeTexture_.getXmtaIds();

	// Create the texture element.
	Element texture = null;
	MediaObjectInformation moi = getMediaObjectInformation();
	switch (moi.getMediaType()) {
		case MediaObjectInformation.VIDEO :
			texture = xmtaDoc.createElement(DOMXMTA.MovieTexture);
			break;
		case MediaObjectInformation.IMAGE :
			texture = xmtaDoc.createElement(DOMXMTA.ImageTexture);
			break;
		default :
			MessageFormat msgFormat = new MessageFormat(xchTextureRes_.getString(XchTextureRes.MSG_ERROR_INVALID_SRC_MEDIA_TYPE));
			StringBuffer buffer = new StringBuffer();
			buffer.append(treeTexture_.getXmtoElement().getNodeName());
			String id = treeTexture_.getXmtoAttrId();
			if (null != id && !id.equals(""))
				buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
			StringBuffer attrBuffer = new StringBuffer();
			attrBuffer.append(DOMXMTO.src).append("=\"").append(treeTexture_.getXmtoAttrSrc()).append("=\"");
			Object[] msgArgs = {buffer, attrBuffer};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, treeTexture_.getXmtoElement());
			treeTexture_.remove();
			if (debug_)
				fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.createXmtaMappingTexture() - exiting");
			return null;
	}
	texture.setAttribute(DOMXMTA.url, ODID_URL_PREFIX + objectDescriptorId);
	texture.setAttribute(DOMXMTA.DEF, textureId);
	xmtaIdTable.put(textureId, texture);

	// Set the repeatS check default value
	String repeatSAttribute = treeTexture_.getXmtoAttrRepeatS();
	if (null != repeatSAttribute) {
		String defaultValue = texture.getAttribute(DOMXMTA.repeatS);
		if (null == defaultValue || !defaultValue.equals(repeatSAttribute))
			texture.setAttribute(DOMXMTA.repeatS, repeatSAttribute);
	}

	// Set the repeatT check default value
	String repeatTAttribute = treeTexture_.getXmtoAttrRepeatT();
	if (null != repeatTAttribute) {
		String defaultValue = texture.getAttribute(DOMXMTA.repeatT);
		if (null == defaultValue || !defaultValue.equals(repeatTAttribute))
			texture.setAttribute(DOMXMTA.repeatT, repeatTAttribute);
	}

	// return the created textureTransform
	return texture;
}
/***********************************************************************
 * Create the texture infrastructure. 
 **********************************************************************/
protected Element createXmtaMappingTextureTransform(String textureTransformId ) {
	Document xmtaDoc = treeTexture_.getXmtaDoc();
	Hashtable xmtaIdTable = treeTexture_.getXmtaIds();

	// Create the textureTransform element.
	Element textureTransform = xmtaDoc.createElement(DOMXMTA.TextureTransform);
	textureTransform.setAttribute(DOMXMTA.DEF, textureTransformId);
	xmtaIdTable.put(textureTransformId, textureTransform);

	// Set the center check default value
	String centerAttribute = treeTexture_.getXmtoAttrCenter();
	if (null != centerAttribute) {
		String defaultValue = textureTransform.getAttribute(DOMXMTA.center);
		if (null == defaultValue || !defaultValue.equals(centerAttribute))
			textureTransform.setAttribute(DOMXMTA.center, centerAttribute);
	}

	// Set the rotation check default value
	String rotationAttribute = treeTexture_.getXmtoAttrRotation();
	if (null != rotationAttribute) {
		String defaultValue = textureTransform.getAttribute(DOMXMTA.rotation);
		if (null == defaultValue || !defaultValue.equals(rotationAttribute))
			textureTransform.setAttribute(DOMXMTA.rotation, rotationAttribute);
	}

	// Set the scale check default value
	String scaleAttribute = treeTexture_.getXmtoAttrScale();
	if (null != scaleAttribute) {
		String defaultValue = textureTransform.getAttribute(DOMXMTA.scale);
		if (null == defaultValue || !defaultValue.equals(scaleAttribute))
			textureTransform.setAttribute(DOMXMTA.scale, scaleAttribute);
	}

	// Set the translation check default value
	String translationAttribute = treeTexture_.getXmtoAttrTranslation();
	if (null != translationAttribute) {
		String defaultValue = textureTransform.getAttribute(DOMXMTA.translation);
		if (null == defaultValue || !defaultValue.equals(translationAttribute))
			textureTransform.setAttribute(DOMXMTA.translation, translationAttribute);
	}

	// return the created textureTransform
	return textureTransform;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doTimingMapping();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
private boolean doTimingMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.doTimingMapping() - entered", getConvertElement().getXmtoElement());

	//	
	boolean success = true;

	// If this is a defs object whose parent is the <defs> tag then this object does not
	// have any timing associated with it.  Thus the structure has to be created to add 
	// itself directly to the defs section in the xmt-a
	if (treeTexture_.isDefsParent()) {
		createInsertMapping();
		// 
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.doTimingMapping() - exiting success=true");
		return true;
	}

	//  This object is under a media object and thus we have to create it using the parent's timing intervals.
	TimeScheduler timeScheduler = treeTexture_.getXmto2Xmta().getTimeScheduler();
	TreeElement parent = treeTexture_.getParent();
	while (null != parent && !(parent instanceof TimingElement))
		parent = parent.getParent();
	if (null == parent) {
		return false;
	}
	ConvertElement parentElement = (ConvertElement) parent;
	Integer parentIndex = parentElement.getIndex();
	Vector lifeCycles = timeScheduler.getLifeCycles(parentIndex);

	// debug purposes
	// Go through it once for printing out debug info irregardless of whether it is correct or not.
	if (debug_) {
		int lifeCycleCount = 0;
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
			int activeIntervalCount = 0;
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				//
				StringBuffer sb = new StringBuffer();
				sb.append("XchTexture.doTimingMapping() -");
				sb.append(" <").append(treeTexture_.getXmtoElement().getNodeName());
				String id = treeTexture_.getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" parent=<").append(parentElement.getXmtoElement().getNodeName());
				id = parentElement.getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
				sb.append(" activeInterval[").append(activeIntervalCount).append("]");
				sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
				sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
				sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
				sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
				sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
				ELASid parentElasId = lifeCycle.getParentELASid();
				sb.append(" parentElasId=(");
				if (null == parentElasId) {
					sb.append("null");
				} else {
					sb.append(parentElasId.getElementIndex());
					sb.append(",").append(parentElasId.getLifeCycleIndex());
					sb.append(",").append(parentElasId.getIntervalIndex());
					sb.append(",").append(parentElasId.getSimpleDurIndex());
				}
				sb.append(")");
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					sb.append(" repeatTime=").append(repeatTime);
				}
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					sb.append(" elasId=(").append(elasId.getElementIndex());
					sb.append(",").append(elasId.getLifeCycleIndex());
					sb.append(",").append(elasId.getIntervalIndex());
					sb.append(",").append(elasId.getSimpleDurIndex());
					sb.append(")");
				}
				fireErrorEvent(ErrorEvent.TRACE, 0, sb.toString(), treeTexture_.getXmtoElement());

				//
				activeIntervalCount++;
			}
			lifeCycleCount++;
		}
	}


	// Now go through for real.
	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// Check to make sure the timing values are valid
			boolean error = false;
			TimingAttribute computedBeginTime = activeInterval.getComputedBeginTime();
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			TimingAttribute activeDuration = activeInterval.getActiveDuration();
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (null == computedBeginTime || null == observedBeginTime || null == fillEndTime || null == activeDuration)
				error = true;
			if (!error) {
				switch (computedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (observedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (activeDuration.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (fillEndTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					switch (repeatTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
			}

			// If there is a problem then report back the problem and remove the object.
			if (error) {
				MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
				StringBuffer buffer = new StringBuffer();
				buffer.append(getConvertElement().getXmtoElement().getNodeName());
				String id = treeTexture_.getXmtoAttrId();
				if (null != id && !id.equals(""))
					buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
				Object[] msgArgs = {buffer};
				String message = msgFormat.format(msgArgs);
				fireErrorEvent(ErrorEvent.ERROR, 0, message, treeTexture_.getXmtoElement());

				// Since we don't support the timings for this element remove it from the tree
				treeTexture_.remove();
				if (debug_)
					fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.doTimingMapping() - exiting success=false");
				return false;
			}

			// All the timings check out OK so now loop through the ELASids and create the nodes.
			for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
				ELASid elasId = (ELASid) elasIdEnum.nextElement();

				// Create the insert Mapping.
				success = createInsertMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.doTimingMapping() - exiting success=false");
					return success;
				}
				// Create the delete Mapping.
				success = createDeleteMapping(elasId);
				if (!success) {
					if (debug_)
						fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.doTimingMapping() - exiting success=false");
					return success;
				}
			}
		}
	}

	// 
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.doTimingMapping() - exiting success=true");
	return true;
}
/***********************************************************************
 * Get the XMTA node id in which the xMediaChild should be placed into.
 **********************************************************************/
protected String getReplaceAtNodeAttr(ELASid elasId) {
	if (null == elasId)
		return "";

	// get the media element
	Hashtable xmtoIdTable = treeTexture_.getXmtoIds();
	ConvertElement convertElement = (ConvertElement) treeTexture_.getConvertIndexTable().get(elasId.getElementIndex());

	// get the id from the internal element
	if (null != convertElement && convertElement instanceof InternalElement)
		return ((InternalElement) convertElement).getXmtaAppearanceId(elasId);

	//
	return "";
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof texture)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.init() - element=" + element + " is not an instance of texture");
		throw new Exception("");
	}

	//
	treeTexture_ = (texture) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchTexture.init() - treeTexture_=" + treeTexture_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
