/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  XchXmediaChildrenExternalElement.java
//
//=============================================================================
package xmto2xmta.mapxmediachildren;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmtutil.MediaObjectInformation;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.MapRoot;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.ExternalElementRes;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.tree.ConvertElement;
import xmto2xmta.tree.XmediaChildrenExternalElement;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to group together the various type media elements whose data is external to the XMT file.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>09 Feb 2001 - Fix bug should support indefininte duration (end time)
 * <li>28 May 2002 - odid url prefix is od:// in new spec.
 * </ul><p>
 *********************************************************************/
public abstract class XchXmediaChildrenExternalElement extends MapRoot {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	static protected final String ODID_URL_PREFIX = "od://";
	//static protected final String FILE_URL_PREFIX = "file:///";
	static protected final String FILE_URL_PREFIX = "";

	/*********************************************************************** 
	 **********************************************************************/
	private XmediaChildrenExternalElement treeXmediaChildExtElement_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private MediaObjectInformation mediaObjectInformation_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle externalElementRes_ = ResourceBundle.getBundle("xmto2xmta.resources.ExternalElementRes", Locale.getDefault());
/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the par node for the deletion.
 * The par for the deletion, deletes the top level group for the element.  It also
 * takes care of the position in the ordered group field (for z-order)
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;ObjectDescriptorRemove objectDescriptorId="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createDeleteMapping(ELASid elasId) {
	//
	TimeScheduler timeScheduler = treeXmediaChildExtElement_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute deleteTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (activeInterval.getELASids().size() - 1 == sdIndex)
		deleteTime = activeInterval.getFillEndTime();
	else
		deleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);
	// If this simple duration doesn't end then we don't need to worry about deleting it.
	if (deleteTime.getTimeValueType() == TimingAttribute.INDEFINITE_VALUE)
		return true;

	//	
	Document xmtaDoc = getConvertElement().getXmtaDoc();

	// Nest the remove par under the par that creates the object. 
	// <par begin="???">
	//   <ObjectDescriptorRemove objectDescriptorId="???"/>
	// </par>
	Element parRemove = xmtaDoc.createElement(DOMXMTA.par);
	parRemove.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(deleteTime.getClockValue()));
	treeXmediaChildExtElement_.addToParOd(parRemove);

	//
	Element objectDescriptorRemove = xmtaDoc.createElement(DOMXMTA.ObjectDescriptorRemove);
	String objectDescriptorId = treeXmediaChildExtElement_.getXmtaObjectDescriptorId(elasId);
	objectDescriptorRemove.setAttribute(DOMXMTA.objectDescriptorId, objectDescriptorId);
	parRemove.appendChild(objectDescriptorRemove);

	//
	return true;
}
/***********************************************************************
 * Create the External Element infrastructure.
 **********************************************************************/
protected boolean createInsertMapping() {

	// Do the actual creation of the OD insert
	String objectDescriptorId = treeXmediaChildExtElement_.getXmtaObjectDescriptorId();
	String esDescriptorId = treeXmediaChildExtElement_.getXmtaEsDescriptorId();
	boolean success = createInsertMappingOD(new TimingAttribute(0), objectDescriptorId, esDescriptorId);

	//
	return success;
}
/***********************************************************************
 * Create the External Element infrastructure.
 **********************************************************************/
protected boolean createInsertMapping(ELASid elasId) {

	// Get the insert time.
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getComputedBeginTime();
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

	// Do the actual creation of the OD insert
	String objectDescriptorId = treeXmediaChildExtElement_.getXmtaObjectDescriptorId(elasId);
	String esDescriptorId = treeXmediaChildExtElement_.getXmtaEsDescriptorId(elasId);
	boolean success = createInsertMappingOD(insertTime, objectDescriptorId, esDescriptorId);

	//
	return success;
}
/***********************************************************************
 * Create the External Element infrastructure.
 * The infrastructure contains the par node for the insertion of the OD.
 * NOTE: The OD is the same with all the external streams (audio, video & image)
 * in the new XMT-A specification. 
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;ObjectDescriptorUpdate&gt;
 *       &lt;OD&gt;
 *         &lt;ObjectDescriptor ObjectDescriptorID="???"&gt;
 *           &lt;esDescr&gt;
 *             &lt;ES_Descriptor ES_ID="???"&gt;
 *               &lt;StreamSource url="???"/&gt;
 *               &lt;decConfigDescr&gt;
 *                 &lt;DecoderConfigDescriptor bufferSizeDB="auto" objectTypeIndication="???" streamType="???"&gt;
 *                   &lt;decSpecificInfo&gt;
 *                     &lt;DecoderSpecificInfo type="auto"/&gt;
 *                   &lt;/decSpecificInfo&gt;
 *                 &lt;/DecoderConfigDescriptor&gt;
 *               &lt;/decConfigDescr&gt;
 *               &lt;slConfigDescr&gt;
 *                 &lt;SLConfigDescriptor timeStampLength="auto" timeStampResolution="auto" useAccessUnitStartFlag="true"&gt;
 *               &lt;/slConfigDescr&gt;
 *             &lt;/ES_Descriptor&gt;
 *           &lt;/esDescr&gt;
 *         &lt;/ObjectDescriptor&gt;
 *       &lt;OD/&gt;
 *     &lt;/ObjectDescriptorUpdate&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createInsertMappingOD(TimingAttribute insertTime, String objectDescriptorId ,String esDescriptorId ) {
	//
	Document xmtaDoc = treeXmediaChildExtElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeXmediaChildExtElement_.getXmtaIds();

	// Create the OD Update for the new object 
	// <par begin="???">
	//   <ObjectDescriptorUpdate>
	//     <OD>
	//       <ObjectDescriptor ObjectDescriptorID="???">
	//         <esDescr>
	//           <ES_Descriptor ES_ID="???">
	//             <StreamSource url="???"/>
	//             <decConfigDescr>
	//               <DecoderConfigDescriptor bufferSizeDB="auto" objectTypeIndication="???" streamType="???">
	//                 <decSpecificInfo>
	//                   <DecoderSpecificInfo type="auto"/>
	//                 </decSpecificInfo>
	//               </DecoderConfigDescriptor> 
	//             </decConfigDescr>
	//             <slConfigDescr>
	//               <SLConfigDescriptor timeStampLength="auto" timeStampResolution="auto" useAccessUnitStartFlag="true">
	//             </slConfigDescr>
	//           </ES_Descriptor>
	//         </esDescr>
	//       </ObjectDescriptor>
	//     <OD/>
	//   </ObjectDescriptorUpdate>
	// </par>
	Element parUpdate = xmtaDoc.createElement(DOMXMTA.par);

	// It is still possible that the start is before the begining of the presentation, ie. t<0s
	// If this is the case then set the start time to t=0 and issue a warning to the user.
	if (insertTime.lessThan(0)) {
		parUpdate.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(0));
		// report back the warning
		MessageFormat msgFormat = new MessageFormat(externalElementRes_.getString(ExternalElementRes.MSG_WARNING_NEGATIVE_SCENE_START));
		StringBuffer buffer = new StringBuffer();
		buffer.append("<").append(treeXmediaChildExtElement_.getXmtoElement().getNodeName());
		String id = treeXmediaChildExtElement_.getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		buffer.append(">");
		Object[] msgArgs = {buffer};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, treeXmediaChildExtElement_.getXmtoElement());
	} else {
		parUpdate.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));
	}
	treeXmediaChildExtElement_.addToParOd(parUpdate);

	//
	Node objectDescriptorUpdate = parUpdate.appendChild(xmtaDoc.createElement(DOMXMTA.ObjectDescriptorUpdate));

	//
	Node oD = objectDescriptorUpdate.appendChild(xmtaDoc.createElement(DOMXMTA.OD));

	// write ObjectDescriptor
	Element objectDescriptor = xmtaDoc.createElement(DOMXMTA.ObjectDescriptor);
	// make the switch and transform ids based on object id or make them unique.
//	String objectDescriptorId = treeXmediaChildExtElement_.getXmtaObjectDescriptorId(elasId);
	objectDescriptor.setAttribute(DOMXMTA.objectDescriptorID, objectDescriptorId);
	xmtaIdTable.put(objectDescriptorId, objectDescriptor);
	oD.appendChild(objectDescriptor);

	//
	Node esDescr = null;
	Node descr = objectDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.Descr));
	esDescr = descr.appendChild(xmtaDoc.createElement(DOMXMTA.esDescr));

	// write ESDescriptor
	Element es_Descriptor = xmtaDoc.createElement(DOMXMTA.ES_Descriptor);
//	String esDescriptorId = treeXmediaChildExtElement_.getXmtaEsDescriptorId(elasId);
	es_Descriptor.setAttribute(DOMXMTA.ES_ID, esDescriptorId);
	xmtaIdTable.put(esDescriptorId, es_Descriptor);
	esDescr.appendChild(es_Descriptor);

	//
	Node decConfigDescr = es_Descriptor.appendChild(xmtaDoc.createElement(DOMXMTA.decConfigDescr));

	// write DecoderConfigDescriptor
	Element decoderConfigDescriptor = xmtaDoc.createElement(DOMXMTA.DecoderConfigDescriptor);
	decoderConfigDescriptor.setAttribute(DOMXMTA.bufferSizeDB, DOMXMTA.auto);
	// Determine the type of the file.
	MediaObjectInformation moi = getMediaObjectInformation();
	decoderConfigDescriptor.setAttribute(DOMXMTA.objectTypeIndication, Integer.toString(moi.getMediaObjectType()));
	decoderConfigDescriptor.setAttribute(DOMXMTA.streamType, Integer.toString(moi.getMediaStreamType()));
	decConfigDescr.appendChild(decoderConfigDescriptor);

	//
	Node decSpecificInfo = decoderConfigDescriptor.appendChild(xmtaDoc.createElement(DOMXMTA.decSpecificInfo));
	// write DecoderSpecificInfo
	Element decoderSpecificInfo = xmtaDoc.createElement(DOMXMTA.DecoderSpecificInfo);
	decoderSpecificInfo.setAttribute(DOMXMTA.type, DOMXMTA.auto);
	decSpecificInfo.appendChild(decoderSpecificInfo);

	// Create the Sl Config Descriptor
	Node slConfigDescr = es_Descriptor.appendChild(xmtaDoc.createElement(DOMXMTA.slConfigDescr));
	//
	Element slConfigDescriptor = xmtaDoc.createElement(DOMXMTA.SLConfigDescriptor);
	slConfigDescr.appendChild(slConfigDescriptor);
	Element custom = xmtaDoc.createElement(DOMXMTA.custom);
	custom.setAttribute(DOMXMTA.useAccessUnitStartFlag, DOMXMTA.string_true);
	custom.setAttribute(DOMXMTA.timeStampResolution, DOMXMTA.auto);
	custom.setAttribute(DOMXMTA.timeStampLength, DOMXMTA.auto);
	slConfigDescriptor.appendChild(custom);
	//  write the StreamSource element
	Element streamSource = xmtaDoc.createElement(DOMXMTA.StreamSource);
	String uri = treeXmediaChildExtElement_.getXmtoAttrSrc().replace('\\', '/');
	streamSource.setAttribute(DOMXMTA.url, FILE_URL_PREFIX + uri);
	es_Descriptor.appendChild(streamSource);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
protected MediaObjectInformation getMediaObjectInformation() {
	if (null == mediaObjectInformation_) {
		// Determine the type of the file.
		mediaObjectInformation_ = new MediaObjectInformation(treeXmediaChildExtElement_.getXmtoAttrSrcCanonical());
	}

	//
	return mediaObjectInformation_;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof XmediaChildrenExternalElement)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "XchXmediaChildrenExternalElement.init() - element=" + element + " is not an instance of XmediaChildrenExternalElement");
		throw new Exception("");
	}

	//
	treeXmediaChildExtElement_ = (XmediaChildrenExternalElement) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XchXmediaChildrenExternalElement.init() - treeXmediaChildExtElement_=" + treeXmediaChildExtElement_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
