/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  XmaXmtaMedia.java
//
//=============================================================================
package xmto2xmta.mapxmtamedia;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.MapRoot;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.tree.xmtaMedia;
import xmto2xmta.tree.TreeElement;
import xmto2xmta.tree.ZorderSupport;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMUtil;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to group together the various type of media elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>21 Jun 2001 - Disable the touchsensor by default.  Enable it when it is used.
 * </ul><p>
 *********************************************************************/
public class XmaXmtaMedia extends MapRoot {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private xmtaMedia treeXmtaMediaElement_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/***********************************************************************
 * This will go through the cmds element in the xmtaMedia adding the
 * elements to the xmta tree element.
 **********************************************************************/
private void addCmds(Element xmtaElement) {

	// First locate the <nodes> element
	Element xmtoXmtaMedia = treeXmtaMediaElement_.getXmtoElement();
	Element xmtoCmds = DOMUtil.getChildElementByTagName(xmtoXmtaMedia, DOMXMTO.xmto_colon_cmds);


	// if there isn't a xmtoNodes element then return
	if (null == xmtoCmds)
		return;

	// Copy all the nodes from the xmto document to the new xmta document
	copy(xmtoCmds, xmtaElement);
}
/***********************************************************************
 * This will go through the nodes element in the xmtaMedia adding the
 * elements to the xmta tree element.
 **********************************************************************/
private void addNodes(Element xmtaElement) {

	// First locate the <nodes> element
	Element xmtoXmtaMedia = treeXmtaMediaElement_.getXmtoElement();
	Element xmtoNodes = DOMUtil.getChildElementByTagName(xmtoXmtaMedia, DOMXMTO.xmto_colon_nodes);


	// if there isn't a xmtoNodes element then return
	if (null == xmtoNodes)
		return;

	// Copy all the nodes from the xmto document to the new xmta document
	copy(xmtoNodes, xmtaElement);
}
private void copy(Node input, Node output) {
	if (!input.hasChildNodes())
		return;
	NodeList nodeList = input.getChildNodes();
	if (nodeList == null)
		return;

	//
	Document outputDoc = output.getOwnerDocument();
	//
	int len = nodeList.getLength();
	for (int i = 0; i < len; i++) {
		Node inputChild = nodeList.item(i);
		Node outputChild = null;
		int type = inputChild.getNodeType();
		switch (type) {
			case Node.ELEMENT_NODE :
				{
					//
					String elementNodeName = ((Element) inputChild).getNodeName();
					//
					Element element = outputDoc.createElement(elementNodeName);
					outputChild = element;
					output.appendChild(outputChild);

					//
					NamedNodeMap inputAttributes = inputChild.getAttributes();
					for (int j = 0; j < inputAttributes.getLength(); j++) {
						Node inputAttribute = inputAttributes.item(j);
						if (Node.ATTRIBUTE_NODE == inputAttribute.getNodeType()) {
							element.setAttribute(inputAttribute.getNodeName(), inputAttribute.getNodeValue());
						} else {
							fireErrorEvent(ErrorEvent.WARNING, 0, "WARNING - unsupported attribute node. Node=" + inputAttribute);
							if (debug_)
								fireErrorEvent(ErrorEvent.TRACE, 0, "XmaXmtamedia.copy() - unsupported attribute node. Node=" + inputAttribute);
						}
					}
					break;
				}
			case Node.TEXT_NODE :
				{
					String text = inputChild.getNodeValue().trim();
					if (null != text && !text.equals("")) {
						outputChild = outputDoc.createTextNode(inputChild.getNodeValue());
						output.appendChild(outputChild);
					}
					break;
				}
			case Node.ATTRIBUTE_NODE :
			case Node.CDATA_SECTION_NODE :
			case Node.COMMENT_NODE :
			case Node.DOCUMENT_NODE :
			case Node.DOCUMENT_FRAGMENT_NODE :
			case Node.DOCUMENT_TYPE_NODE :
			case Node.ENTITY_NODE :
			case Node.ENTITY_REFERENCE_NODE :
			case Node.NOTATION_NODE :
			case Node.PROCESSING_INSTRUCTION_NODE :
			default :
				fireErrorEvent(ErrorEvent.WARNING, 0, "WARNING - unsupported node. Node=" + inputChild);
				if (debug_)
					fireErrorEvent(ErrorEvent.TRACE, 0, "XmaXmtamedia.copy() - unsupported node. Node=" + inputChild);
				break;
		}
		copy(inputChild, outputChild);
	}
}
/***********************************************************************
 * Create the infrastructure to add the nodes' elements in an insert into the scene structure.
 * The infrastructure contains the par node for the insertion.
 * The insert places the nodes' elements in the container for the xmtaMedia element.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     ***** xmta elmements under the &lt;cmds&gt; element. *****
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createCmdsMapping(TimingAttribute insertTime) {
	//	
	Document xmtaDoc = treeXmtaMediaElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeXmtaMediaElement_.getXmtaIds();

	// Create the par for the insert command. 
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeXmtaMediaElement_.addToParNode(parInsertNode);
	// Set the insert time.
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	// Now add all the nodes found in the xmtaMedia element to the children element.
	addCmds(parInsertNode);

	//
	return true;
}
/***********************************************************************
 * Create the xmtaMedia Element infrastructure.
 * The infrastructure contains the par node for the insertion.
 * The par node for the insertion contains the top level group which will contain the
 * information for the element.  This also takes care of the position in the ordered group
 * field (for z-order).
 * <p>
 * The infrasturcture requires adding the elements to the BIFS scene requires 
 * creating a container to add all the elements in the xmt-o nodes element.  
 * Since we are in a 2D context if the xmtaMedia element is 3D then make the container
 * element a Layer3D so that we can switch to a 3D context.
 * If the xmtaMedia element isn't 3D just create a Group as the container.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       &lt;Group DEF="???"&gt;
 *         &lt;children&gt;
 *           &lt;TouchSensor DEF="???" enabled="false"&gt;
 *           &lt;Group DEF="???"/&gt; OR &lt;Layer3D DEF="???"/&gt; 
 *         &lt;/children&gt;
 *       &lt;/Group&gt;
 *     &lt;/Insert&gt;
 *     &lt;Insert atField="order" atNode="???" position="END" value="-1"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createContainerMapping(String parentOrderedGroupId, boolean parentUsePositions, String groupId, String touchSensorId, String containerId, TimingAttribute insertTime) {

	//
	Document xmtaDoc = treeXmtaMediaElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeXmtaMediaElement_.getXmtaIds();

	// The par node for the insertion contains the top level group which will contain the
	// information for the element.  This also takes care of the position in the ordered group
	// field (for z-order).
	// <par begin="???">
	//   <Insert atNode="???">
	//     <Group DEF="???">
	//       <children>
	//         <TouchSensor DEF="???" enabled="false">
	//         <Group DEF="???"/> OR <Layer3D DEF="???"/>
	//       </children>
	//     </Group>
	//   </Insert>
	//   <Insert atField="order" atNode="???" position="END" value="???"/>
	// </par>

	// Create the par for the insert command. Don't place it in the XMT-A tree yet.
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeXmtaMediaElement_.addToParNode(parInsertNode);
	// Set the insert time.
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element insertAtNode = xmtaDoc.createElement(DOMXMTA.Insert);
	insertAtNode.setAttribute(DOMXMTA.atNode, parentOrderedGroupId);
	parInsertNode.appendChild(insertAtNode);

	// Create the top level group node
	Element group = xmtaDoc.createElement(DOMXMTA.Group);
	group.setAttribute(DOMXMTA.DEF, groupId);
	xmtaIdTable.put(groupId, group);
	insertAtNode.appendChild(group);

	//
	Element groupChildren = xmtaDoc.createElement(DOMXMTA.children);
	group.appendChild(groupChildren);

	// If this is a switch to 3D then creat a Layer3d otherwise create a Group.
	Element container = null;
	if (treeXmtaMediaElement_.getXmtoAttrType().equals(DOMXMTO.string_3D))
		container = xmtaDoc.createElement(DOMXMTA.Layer3D);
	else
		container = xmtaDoc.createElement(DOMXMTA.Group);
	container.setAttribute(DOMXMTA.DEF, containerId);
	xmtaIdTable.put(containerId, container);
	groupChildren.appendChild(container);

	// Create a touch sensor.
	Element touchSensor = xmtaDoc.createElement(DOMXMTA.TouchSensor);
	// Set up the id.  
	touchSensor.setAttribute(DOMXMTA.DEF, touchSensorId);
	// Disable the touchsensor by default.
	touchSensor.setAttribute(DOMXMTA.enabled, DOMXMTA.string_false);
	xmtaIdTable.put(touchSensorId, touchSensor);
	groupChildren.appendChild(touchSensor);

	// Do we use z-order positioning? If so then we need to create an insert to modify the OrderedGroup's atField.
	// The xmtaMedia object doesn't support positions but if the parent requests it then we need to add a default value
	if (parentUsePositions) {
		// Create insert Field.  
		// Insert group isn't known yet so don't fill it in.
		// Also don't know the zOrder so don't fill it in.
		Element insertAtField = xmtaDoc.createElement(DOMXMTA.Insert);
		insertAtField.setAttribute(DOMXMTA.atNode, parentOrderedGroupId);
		insertAtField.setAttribute(DOMXMTA.atField, DOMXMTA.order);
		insertAtField.setAttribute(DOMXMTA.position, DOMXMTA.END);
		insertAtField.setAttribute(DOMXMTA.value, "-1"); // give it a default position value
		parInsertNode.appendChild(insertAtField);
	}

	//
	return true;
}
/***********************************************************************
 * Create the media Element infrastructure.
 * The infrastructure contains the par node for the deletion.
 * The par for the deletion, deletes the top level group for the element.  It also
 * takes care of the position in the ordered group field (for z-order)
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Delete atNode="???"/&gt;
 *     &lt;Delete atField="order" atNode="???"/&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
protected boolean createDeleteMapping(ELASid elasId) {
	//
	TimeScheduler timeScheduler = treeXmtaMediaElement_.getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute deleteTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (activeInterval.getELASids().size() - 1 == sdIndex)
		deleteTime = activeInterval.getFillEndTime();
	else
		deleteTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex);
	// If this simple duration doesn't end then we don't need to worry about deleting it.
	if (deleteTime.getTimeValueType() == TimingAttribute.INDEFINITE_VALUE)
		return true;

	//	
	Document xmtaDoc = treeXmtaMediaElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeXmtaMediaElement_.getXmtaIds();

	// Find the Xmta parent group element which this node should be inserted into.
	ZorderSupport xmtaParentGroupElement = null;
	TreeElement parent = treeXmtaMediaElement_.getParent();
	while (null != parent) {
		if (parent instanceof ZorderSupport) {
			xmtaParentGroupElement = (ZorderSupport) parent;
			// break out of the loop
			break;
		}
		parent = parent.getParent();
	}

	//	
	if (null == xmtaParentGroupElement) {
		// We should always beable to find the parent ordered group.  
		// Should be able to find the body or the region
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), getConvertElement().getXmtoElement());
		getConvertElement().remove();
		return false;
	}

	// Get the ordered group of the parent we were inserted into.
	String xmtaParentOrderedGroupId = xmtaParentGroupElement.getXmtaOrderedGroupId(lifeCycle.getParentELASid());
	String groupId = treeXmtaMediaElement_.getXmtaGroupId(elasId);

	// The par for the deletion, deletes the top level group for the element.  It also
	// takes care of the position in the ordered group field (for z-order)
	// <par begin="???">
	//   <Delete atNode="???"/>
	//   <Delete atField="order" atNode="???"/>
	// </par>

	// Create the par for the delete command.  Don't place it in the XMT-A tree yet.
	Element parDeleteNode = xmtaDoc.createElement(DOMXMTA.par);
	parDeleteNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(deleteTime.getClockValue()));
	treeXmtaMediaElement_.addToParNode(parDeleteNode);

	// Create delete Node
	Element deleteNode = xmtaDoc.createElement(DOMXMTA.Delete);
	deleteNode.setAttribute(DOMXMTA.atNode, groupId);
	parDeleteNode.appendChild(deleteNode);

	// Do we use z-order positioning? If so then we need to delete the OrderedGroup's atField.
	if (xmtaParentGroupElement.usePositions()) {
		// Create delete Field.
		// Insert group isn't known yet so don't fill it in.
		// Also don't know the position it will be in at deletion time so don't fill it in.
		Element deleteField = xmtaDoc.createElement(DOMXMTA.Delete);
		deleteField.setAttribute(DOMXMTA.atNode, xmtaParentOrderedGroupId);
		deleteField.setAttribute(DOMXMTA.atField, DOMXMTA.order);
		parDeleteNode.appendChild(deleteField);

		// get the position the deleted element is in so that we delete the correct filed position.
		int deleteFieldPosition = xmtaParentGroupElement.getDeleteFieldPosition(elasId);
		deleteField.setAttribute(DOMXMTA.position, "" + deleteFieldPosition);
	}

	//
	return true;
}
/***********************************************************************
 * Create the xmtaMedia Element infrastructure.
 * The xmtaMedia Element infrastructure consists of 3 parts, the nodes for the 
 * container, the nodes and cmds.
 * <ol>
 * <li>The container infrastructure is the wrapper for placing the xmtaMedia nodes
 *     in the bifs tree.
 * <li>The nodes infrastructure inserts all the xmta elements found in the nodes
 *     section of the xmtaMedia element.  The xmta elements are inserted into the
 *     a container created in the container infrastructure creation.
 * <li>The cmds infrastructure inserts all the xmta cmd elements.  These are wrapped
 *     by a par which contains the timing of the xmtaMedia element.
 * </ol>
 **********************************************************************/
protected boolean createInsertMapping(ELASid elasId) {

	//
	Document xmtaDoc = treeXmtaMediaElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeXmtaMediaElement_.getXmtaIds();

	// Find the parent group element which this node should be inserted into.
	ZorderSupport xmtaParentGroupElement = null;
	// Find the Xmta parent group element which this node should be inserted into.
	TreeElement parent = treeXmtaMediaElement_.getParent();
	while (null != parent) {
		if (parent instanceof ZorderSupport) {
			xmtaParentGroupElement = (ZorderSupport) parent;
			// break out of the loop
			break;
		}
		parent = parent.getParent();
	}
	if (null == xmtaParentGroupElement) {
		// We should always beable to find the parent ordered group.  
		// Should be able to find the body or the region
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), treeXmtaMediaElement_.getXmtoElement());
		getConvertElement().remove();
		return false;
	}

	// Get the insert time.
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(elasId.getElementIndex());
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.elementAt(elasId.getLifeCycleIndex().intValue());
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.elementAt(elasId.getIntervalIndex().intValue());
	TimingAttribute insertTime = null;
	int sdIndex = elasId.getSimpleDurIndex().intValue();
	if (0 == sdIndex) {
		// Insert time is the observed time not the computed time.  This is the time
		// the node is placed in the Bifs tree for the z-ordering.
		insertTime = activeInterval.getObservedBeginTime();
	} else
		insertTime = (TimingAttribute) activeInterval.getRepeatTimes().elementAt(sdIndex - 1);

	// Get the ordered group of the parent we are going to be inserted into.
	String parentOrderedGroupId = xmtaParentGroupElement.getXmtaOrderedGroupId(lifeCycle.getParentELASid());
	boolean parentUsePositions = xmtaParentGroupElement.usePositions();
	String groupId = treeXmtaMediaElement_.getXmtaGroupId(elasId);
	String touchSensorId = treeXmtaMediaElement_.getXmtaTouchSensorId(elasId);
	String containerId = treeXmtaMediaElement_.getXmtaNodeContainerId(elasId);

	// Create the infrastructure for the xmtaMedia element to place all the nodes in the scene
	boolean success = createContainerMapping(parentOrderedGroupId, parentUsePositions, groupId, touchSensorId, containerId, insertTime);
	if (!success)
		return false;

	// Create the insert for adding the nodes' elements to the scene
	success = createNodesMapping(containerId, insertTime);
	if (!success)
		return false;

	// Create the insert for adding the cmds' elements to the scene
	success = createCmdsMapping(insertTime);
	if (!success)
		return false;

	//
	return true;
}
/***********************************************************************
 * Create the infrastructure to add the nodes' elements in an insert into the scene structure.
 * The infrastructure contains the par node for the insertion.
 * The insert places the nodes' elements in the container for the xmtaMedia element.
 * <pre>
 *   &lt;par begin="???"&gt;
 *     &lt;Insert atNode="???"&gt;
 *       ***** xmta elmements under the &lt;nodes&gt; element. *****
 *     &lt;/Insert&gt;
 *   &lt;/par&gt;
 * </pre>
 **********************************************************************/
private boolean createNodesMapping(String containerId, TimingAttribute insertTime) {
	//	
	Document xmtaDoc = treeXmtaMediaElement_.getXmtaDoc();
	Hashtable xmtaIdTable = treeXmtaMediaElement_.getXmtaIds();

	// Create the par for the insert command. 
	Element parInsertNode = xmtaDoc.createElement(DOMXMTA.par);
	treeXmtaMediaElement_.addToParNode(parInsertNode);
	// Set the insert time.
	parInsertNode.setAttribute(DOMXMTA.begin, DOMXMTAParameter.time2Param(insertTime.getClockValue()));

	// Create insert Node 
	// Insert group isn't known yet so don't fill it in.
	Element insertAtNode = xmtaDoc.createElement(DOMXMTA.Insert);
	insertAtNode.setAttribute(DOMXMTA.atNode, containerId);
	parInsertNode.appendChild(insertAtNode);

	// Now add all the nodes found in the xmtaMedia element to the children element.
	addNodes(insertAtNode);

	//
	return true;
}
/*******************************************************************
 *******************************************************************/
public void doMapping() {
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XmaXmtaMedia.doMapping() - entered convertElement=" + getConvertElement());
	boolean success = true;

	//
	success = doTimingMapping();

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XmaXmtaMedia.doMapping() - exiting success=" + success);
}
/***********************************************************************
 **********************************************************************/
private boolean doTimingMapping() {
	boolean success = true;

	//
	//	EventModel eventModel = getConvertElement().getXmto2Xmta().getEventModel();
	TimeScheduler timeScheduler = getConvertElement().getXmto2Xmta().getTimeScheduler();
	Vector lifeCycles = timeScheduler.getLifeCycles(getConvertElement().getIndex());

	// debug purposes
	// Go through it once for printing out debug info irregardless of whether it is correct or not.
	if (debug_) {
		int lifeCycleCount = 0;
		for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
			LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();
			int activeIntervalCount = 0;
			for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
				ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

				//
				StringBuffer sb = new StringBuffer();
				sb.append("XmaXmtaMedia.determineTimingMapping() -");
				sb.append(" <").append(getConvertElement().getXmtoElement().getNodeName());
				String id = getConvertElement().getXmtoAttrId();
				if (null != id && !id.equals(""))
					sb.append(" id=").append(id);
				sb.append(">");
				sb.append(" lifeCycle[").append(lifeCycleCount).append("]");
				sb.append(" activeInterval[").append(activeIntervalCount).append("]");
				sb.append(" computedBeginTime=").append(activeInterval.getComputedBeginTime());
				sb.append(" observedBeginTime=").append(activeInterval.getObservedBeginTime());
				sb.append(" intrinsicDuration=").append(activeInterval.getIntrinsicDuration());
				sb.append(" activeDuration=").append(activeInterval.getActiveDuration());
				sb.append(" fillEndTime=").append(activeInterval.getFillEndTime());
				ELASid parentElasId = lifeCycle.getParentELASid();
				sb.append(" parentElasId=(");
				if (null == parentElasId) {
					sb.append("null");
				} else {
					sb.append(parentElasId.getElementIndex());
					sb.append(",").append(parentElasId.getLifeCycleIndex());
					sb.append(",").append(parentElasId.getIntervalIndex());
					sb.append(",").append(parentElasId.getSimpleDurIndex());
				}
				sb.append(")");
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					sb.append(" repeatTime=").append(repeatTime);
				}
				for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
					ELASid elasId = (ELASid) elasIdEnum.nextElement();
					sb.append(" elasId=(").append(elasId.getElementIndex());
					sb.append(",").append(elasId.getLifeCycleIndex());
					sb.append(",").append(elasId.getIntervalIndex());
					sb.append(",").append(elasId.getSimpleDurIndex());
					sb.append(")");
				}
				fireErrorEvent(ErrorEvent.TRACE, 0, sb.toString(), getConvertElement().getXmtoElement());

				//
				activeIntervalCount++;
			}
			lifeCycleCount++;
		}
	}


	// Now go through for real.
	// First loop through the life cycles
	for (Enumeration lifeCycleEnum = lifeCycles.elements(); lifeCycleEnum.hasMoreElements();) {
		LifeCycle lifeCycle = (LifeCycle) lifeCycleEnum.nextElement();

		// Loop through the active intervals
		for (Enumeration activeIntervalEnum = lifeCycle.getActiveIntervals().elements(); activeIntervalEnum.hasMoreElements();) {
			ActiveInterval activeInterval = (ActiveInterval) activeIntervalEnum.nextElement();

			// Check to make sure the timing values are valid
			boolean error = false;
			TimingAttribute computedBeginTime = activeInterval.getComputedBeginTime();
			TimingAttribute observedBeginTime = activeInterval.getObservedBeginTime();
			TimingAttribute fillEndTime = activeInterval.getFillEndTime();
			if (null == computedBeginTime || null == observedBeginTime || null == fillEndTime)
				error = true;
			if (!error) {
				switch (computedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (observedBeginTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				switch (fillEndTime.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
						break;
					default :
						error = true;
				}
			}
			if (!error) {
				for (Enumeration repeatEnum = activeInterval.getRepeatTimes().elements(); repeatEnum.hasMoreElements();) {
					TimingAttribute repeatTime = (TimingAttribute) repeatEnum.nextElement();
					switch (repeatTime.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
							break;
						default :
							error = true;
					}
				}
			}

			// If there is a problem then report back the problem and remove the object.
			if (error) {
				MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM));
				StringBuffer buffer = new StringBuffer();
				buffer.append(getConvertElement().getXmtoElement().getNodeName());
				if (null != getConvertElement().getXmtoAttrId() && !getConvertElement().getXmtoAttrId().equals(""))
					buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getConvertElement().getXmtoAttrId()).append("\"");
				Object[] msgArgs = {buffer};
				String message = msgFormat.format(msgArgs);
				fireErrorEvent(ErrorEvent.ERROR, 0, message, getConvertElement().getXmtoElement());

				// Since we don't support the timings for this element remove it from the tree
				getConvertElement().remove();
				return false;
			}

			// All the timings check out OK so now loop through the ELASids and create the nodes.
			for (Enumeration elasIdEnum = activeInterval.getELASids().elements(); elasIdEnum.hasMoreElements();) {
				ELASid elasId = (ELASid) elasIdEnum.nextElement();

				// Create the insert Mapping.
				success = createInsertMapping(elasId);
				if (!success)
					return success;

				// Create the delete Mapping.
				success = createDeleteMapping(elasId);
				if (!success)
					return success;
			}
		}
	}

	// 
	return true;
}
/*******************************************************************
 *******************************************************************/
public void init(Xmto2Xmta xmto2xmta, xmto2xmta.tree.ConvertElement element) throws Exception {
	super.init(xmto2xmta, element);

	//
	if (!(element instanceof xmtaMedia)) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "XmaXmtaMedia.init() - element=" + element + " is not an instance of xmtaMedia");
		throw new Exception("");
	}

	//
	treeXmtaMediaElement_ = (xmtaMedia) element;

	//
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, "XmaXmtaMedia.init() - treeXmtaMediaElement_=" + treeXmtaMediaElement_);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
