/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  AnimationElementRes.java
//
//=============================================================================
package xmto2xmta.resources;

import java.util.ListResourceBundle;
import java.lang.String;

/***********************************************************************
 * This is the default language resources.
 * The language is English (United States).
 * <p>
 * There is a good explanation on the National Language Support (NLS)
 * in the book "Core Java Volume II - Advanced Features".  The chapter
 * is Internationalization.
 * <p>
 * The resources are listed on two lines so that it is easier to compare
 * the various language resource files to see what strings are missing.
 * <p>
 * The resources that use parameters (like filenames or numbers) are formated
 * into MessageFormat (java.text.MessageFormat). To put a single quote (')
 * in such a resources, you have to put it twice (''). For the others
 * resources, you just have to put it once.
 *
 *
 * <p><b>History:</b><ul>
 * <li>18 Feb 2000 - Created file to handle resources for NLS.
 * </ul>
 *********************************************************************/
public class AnimationElementRes extends ListResourceBundle {

  /***********************************************************************
   * All these keys are public since this will keep accessing the string
   * standardized and lead to less problems.
   **********************************************************************/
  public static final String MSG_ERROR_ANIMATION_ELEMENT_NOT_SUPPORTED = "MsgErrorAnimationElementNotSupported";
  public static final String MSG_ERROR_ANIMATION_INVALID_INTERPOLATOR_ATTRIBUTES = "MsgErrorAnimationInvalidInterpolatorAttributes";
  public static final String MSG_WARNING_ANIMATION_DURATION_INDEFINTE = "MsgWarningAnimationDurationIndefinite";

  /***********************************************************************
   * These are the strings which should get translated.
   * The first line is the KEY and the second line is the string
   * to be translated.  These could be on the same line but having them on
   * separate lines will make it easier to compare the various language files
   * to see which items have not been translated and which lines are missing
   * from the default language file English.
   **********************************************************************/
  static final Object[][] contents = {
  /***********************************************************************
   * The text is the tag is not supported in the current structure.  Generally
   * the tag is not underneath a tag which can be animated or changed.
   * There is 1 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * </ol>
   **********************************************************************/
  /* */ {AnimationElementRes.MSG_ERROR_ANIMATION_ELEMENT_NOT_SUPPORTED,
  /* */ "<{0}> not supported in currrent structure."},
  /***********************************************************************
   * The element has an iteration that goes on indefinitely and therefore
   * the duration can't be determined.
   * There is 1 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * <li>The second parameter is a String starting time for the animation iteration.
   * </ol>
   **********************************************************************/
  /* */ {AnimationElementRes.MSG_WARNING_ANIMATION_DURATION_INDEFINTE,
  /* */ "The <{0}> starting at {1} goes on indefinitely and therefore is being dropped."},
  /***********************************************************************
   * The element has an invalid animation attributes. Thus we can't figure
   * out the interpolator.
   * There is 2 parameters.
   * <ol>
   * <li>Parameter 1 is a String for the element name.
   * <li>Parameter 2 is a String for the attribute error. NOTE: this string
   *     is a complete sentence.
   * </ol>
   **********************************************************************/
  /* */ {AnimationElementRes.MSG_ERROR_ANIMATION_INVALID_INTERPOLATOR_ATTRIBUTES,
  /* */ "The <{0}> is not able to preform the animation. {1}"},
  /* */};
/***********************************************************************
 **********************************************************************/
public Object[][] getContents() {
  return contents;
}
}
