/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  UseRes.java
//
//=============================================================================
package xmto2xmta.resources;

import java.util.ListResourceBundle;
import java.lang.String;

/***********************************************************************
 * This is the default language resources.
 * The language is English (United States).
 * <p>
 * There is a good explanation on the National Language Support (NLS)
 * in the book "Core Java Volume II - Advanced Features".  The chapter
 * is Internationalization.
 * <p>
 * The resources are listed on two lines so that it is easier to compare
 * the various language resource files to see what strings are missing.
 * <p>
 * The resources that use parameters (like filenames or numbers) are formated
 * into MessageFormat (java.text.MessageFormat). To put a single quote (')
 * in such a resources, you have to put it twice (''). For the others
 * resources, you just have to put it once.
 *
 *
 * <p><b>History:</b><ul>
 * <li>18 Feb 2000 - Created file to handle resources for NLS.
 * </ul>
 *********************************************************************/
public class UseRes extends ListResourceBundle {

  /***********************************************************************
   * All these keys are public since this will keep accessing the string
   * standardized and lead to less problems.
   **********************************************************************/
  public static final String MSG_ERROR_HREF_FILE_NOT_SUPPORTED = "MsgErrorHrefFileNotSupported";
  public static final String MSG_ERROR_HREF_ID_NOT_FOUND = "MsgErrorHrefIdNotFound";
  public static final String MSG_ERROR_HREF_ATTRIBUTE_MISSING = "MsgErrorHrefAttributeMissing";
  public static final String MSG_WARNING_TIMING_ATTRIBUTE_NOT_SUPPORTED = "MsgWarningTimingAttributeNotSupported";
  public static final String MSG_WARMING_NO_MAPPING_FOR_DEFS_OBJECT = "MsgWarningNoMappingForDefsObject";

  /***********************************************************************
   * These are the strings which should get translated.
   * The first line is the KEY and the second line is the string
   * to be translated.  These could be on the same line but having them on
   * separate lines will make it easier to compare the various language files
   * to see which items have not been translated and which lines are missing
   * from the default language file English.
   **********************************************************************/
  static final Object[][] contents = {
  /***********************************************************************
   * The text is the error message when use tag has a xlink:href with an
   * external file specification.
   * There is 2 parameters.
   * <ol>
   * <li>Parameter 1 is a String for the name/id of the tag.
   * <li>Parameter 2 is a String for the attribute name/value of the tag.
   * </ol>
   **********************************************************************/
  /* */ {UseRes.MSG_ERROR_HREF_FILE_NOT_SUPPORTED,
  /* */ "<{0}> attribute {1} does not support the external file specification."},
  /***********************************************************************
   * The text is the error message when use tag as a xlink:href with an
   * id that doesn't exist in the file.
   * There is 3 parameters.
   * <ol>
   * <li>Parameter 1 is a String for the name/id of the tag.
   * <li>Parameter 2 is a String for the attribute name/value of the tag.
   * <li>Parameter 3 is a String for invalid id.
   * </ol>
   **********************************************************************/
  /* */ {UseRes.MSG_ERROR_HREF_ID_NOT_FOUND,
  /* */ "<{0}> attribute {1} has an id \"{2}\" which can not be found <defs> section in the document."},
  /***********************************************************************
   * The text is the error message when use tag does not have a xlink:href
   * atribute.
   * There is 2 parameters.
   * <ol>
   * <li>Parameter 1 is a String for the name/id of the tag.
   * <li>Parameter 2 is a String for the xlink:href attribute name.
   * </ol>
   **********************************************************************/
  /* */ {UseRes.MSG_ERROR_HREF_ATTRIBUTE_MISSING,
  /* */ "<{0}> must have a \"{1}\" attribute."},
  /***********************************************************************
   * The text is the warning message when use tag is for an element in which
   * the useage of it does not support timing attributes.
   * There is 3 parameters.
   * <ol>
   * <li>Parameter 1 is a String for the name/id of the tag.
   * <li>Parameter 2 is a String for the name/id for the element tag which we are using.
   * </ol>
   **********************************************************************/
  /* */ {UseRes.MSG_WARNING_TIMING_ATTRIBUTE_NOT_SUPPORTED,
  /* */ "<{0}> for a {1} does not support timing attributes and are being ignored."},
  /***********************************************************************
   * The text is the warning message when there is not mapping for the defs object
   * of the use tag.
   * There is 2 parameters.
   * <ol>
   * <li>Parameter 1 is a String for the name/id of the tag.
   * <li>Parameter 2 is a String for invalid id for the defined object which there isn't a mapping.
   * </ol>
   **********************************************************************/
  /* */ {UseRes.MSG_WARMING_NO_MAPPING_FOR_DEFS_OBJECT,
  /* */ "<{0}> does not support a mapping to use <{1}>."},
  /* */};
/***********************************************************************
 **********************************************************************/
public Object[][] getContents() {
  return contents;
}
}
