/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  Xmto2XmtaRes.java
//
//=============================================================================
package xmto2xmta.resources;

import java.util.ListResourceBundle;
import java.lang.String;

/***********************************************************************
 * This is the default language resources.
 * The language is English (United States).
 * <p>
 * There is a good explanation on the National Language Support (NLS)
 * in the book "Core Java Volume II - Advanced Features".  The chapter
 * is Internationalization.
 * <p>
 * The resources are listed on two lines so that it is easier to compare
 * the various language resource files to see what strings are missing.
 * <p>
 * The resources that use parameters (like filenames or numbers) are formated
 * into MessageFormat (java.text.MessageFormat). To put a single quote (')
 * in such a resources, you have to put it twice (''). For the others
 * resources, you just have to put it once.
 *
 *
 * <p><b>History:</b><ul>
 * <li>18 Feb 2000 - Created file to handle resources for NLS.
 * </ul>
 *********************************************************************/
public class Xmto2XmtaRes extends ListResourceBundle {

  /***********************************************************************
   * All these keys are public since this will keep accessing the string
   * standardized and lead to less problems.
   **********************************************************************/
  public static final String MSG_ERROR_NO_DOCUMENT = "MsgErrorNoDocument";
  public static final String MSG_ERROR_INVALID_DOCUMENT = "MsgErrorInvalidDocument";
  public static final String MSG_ERROR_INVALID_DOCUMENT_STRUCTURE = "MsgErrorInvalidDocumentStructure";
  public static final String MSG_WARNING_ELEMENT_NOT_SUPPORTED = "MsgErrorElementNotSupported";
  public static final String MSG_WARNING_ATTRIBUTE_NOT_SUPPORTED = "MsgErrorAttributeNotSupported";
  public static final String MSG_WARNING_ATTRIBUTE_NOT_SUPPORTED_IN_LOCATION= "MsgErrorAttributeNotSupportedInLocation";
  public static final String MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED = "MsgErrorAttributeValueNotSupported";
  public static final String MSG_ERROR_TIMING_PROBLEM = "MsgErrorTimingProblem";
  public static final String MSG_WARNING_TIMING_PROBLEM_NO_BEGIN = "MsgWarningTimingProblemNoBegin";
  public static final String MSG_WARNING_TIMING_PROBLEM_NO_END = "MsgWarningTimingProblemNoEnd";
  public static final String MSG_WARNING_TIMING_PROBLEM_PARENT_FINISHES_FIRST = "MsgWarningTimingProblemParentFinishesFirst";

  /***********************************************************************
   * These are the strings which should get translated.
   * The first line is the KEY and the second line is the string
   * to be translated.  These could be on the same line but having them on
   * separate lines will make it easier to compare the various language files
   * to see which items have not been translated and which lines are missing
   * from the default language file English.
   **********************************************************************/
  static final Object[][] contents = {
  /***********************************************************************
   * The text is the error message when an XMT document is not passed
   * to the converter.
   * There is 1 parameters.
   * <ol>
   * <li>The parameter is a String for the type of document (XMT-O or XMT-A).
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_ERROR_NO_DOCUMENT,
  /* */ "No input {0} document for conversion."},
  /***********************************************************************
   * The text is the error message when an XMT document which is passed is
   * not a valid XMT document.  Not valid in that it isn't a XMT-A or XMT-O
   * document
   * There is 2 parameters.
   * <ol>
   * <li>The first parameter is a String for the type of document which was passed.
   * <li>The second parameter is a String for the type of document it should be
   *         (XMT-O or XMT-A).
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT,
  /* */ "Invalid document for conversion. A {0} was passed instead of a {1} document."},
  /* */ {Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE,
  /* */ "Invalid document structure."},
  /***********************************************************************
   * The text is the warning message when an unsupported element is found.
   * There are 1 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_WARNING_ELEMENT_NOT_SUPPORTED,
  /* */ "The <{0}> tag is not supported."},
  /***********************************************************************
   * The text is the warning message when the a unsupported attribute.
   * There is 2 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * <li>The second parameter is a String for the attribute name.
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_WARNING_ATTRIBUTE_NOT_SUPPORTED,
  /* */ "The <{0}> tag has an unsupported attribute {1}."},
  /***********************************************************************
   * The text is the error message when the an invalid attribute value.
   * There is 3 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * <li>The second parameter is a String for the attribute name.
   * <li>The third parameter is a String for the attribute value.
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED,
  /* */ "The <{0}> tag has an unsupported value of {2} for the attribute {1}."},
  /***********************************************************************
   * The text is the warning message when the a unsupported attribute in the
   * current document location.
   * There is 2 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * <li>The second parameter is a String for the attribute name.
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_WARNING_ATTRIBUTE_NOT_SUPPORTED_IN_LOCATION,
  /* */ "The <{0}> tag does not support the attribute {1} in its current location."},
  /***********************************************************************
   * The text is the error message when the a unsupported attribute.
   * There is 1 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_ERROR_TIMING_PROBLEM,
  /* */ "Unable to determine timing for <{0}> tag, and it is being ignored."},
  /***********************************************************************
   * The text is the error message when the a unsupported attribute.
   * There is 1 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_WARNING_TIMING_PROBLEM_NO_BEGIN,
  /* */ "Unable to determine begining time for <{0}> tag, using 0."},
  /***********************************************************************
   * The text is the error message when the a unsupported attribute.
   * There is 1 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_WARNING_TIMING_PROBLEM_NO_END,
  /* */ "Unable to determine ending time for <{0}> tag, setting end to be equal to begining (0 duration)."},
  /***********************************************************************
   * The text is the error message when the a unsupported attribute.
   * There are 2 parameters.
   * <ol>
   * <li>The first parameter is a String for the element name.
   * <li>The Second parameter is a String for the parent's element name.
   * </ol>
   **********************************************************************/
  /* */ {Xmto2XmtaRes.MSG_WARNING_TIMING_PROBLEM_PARENT_FINISHES_FIRST,
  /* */ "Truncating ending time for <{0}> tag to the ending of its parent <{1}>."},
  /* */};
/***********************************************************************
 **********************************************************************/
public Object[][] getContents() {
  return contents;
}
}
