/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  ContainerLife.java
//
//=============================================================================
package xmto2xmta.timegraph;

import java.util.Vector;
import java.util.Enumeration;

/*********************************************************************** 
 * Life cycle of a timing element.
 *
 * <p><b>History:</b><ul>
 * <li>15 Mar 2001 - Created file.
 * </ul><p>                                                                   
 *********************************************************************/

public class ContainerLife extends Interval {

	private TimeContainer container = null;
	private Vector lifeIntervals = new Vector(); 	// list of life intervals
	private ELASid parentELASid = null;

	public ContainerLife() {
		super();
	}

	public ContainerLife(Time begin, Time end) {
		super(begin, end);
	}

	public void setContainer(TimeContainer container) {
		this.container = container;
	}

	public TimeContainer getContainer() {
		return container;
	}

	public void setLifeIntervals(Vector lifeIntervals) {
		if (lifeIntervals != null) {
			this.lifeIntervals = lifeIntervals;
			setContainerLifeInLifeIntervals();
		}
	}

	public Vector getLifeIntervals() {
		return lifeIntervals;
	}

	public void addInterval(LifeInterval lifeInterval) {
		if (lifeInterval == null) return;
		lifeInterval.setContainerLife(this);
		lifeIntervals.add(lifeInterval);
	}

	public void removeIntervalAt(int index) {
		lifeIntervals.removeElementAt(index);
	}

	public int size() {
		return lifeIntervals.size();
	}

	public Enumeration enumerateIntervals() {
		return lifeIntervals.elements();
	}

	public void setParentELASid(ELASid elasid) {
		parentELASid = elasid;
	}

	public ELASid getParentELASid() {
		return parentELASid;
	}

	private void setContainerLifeInLifeIntervals() {
		for (Enumeration e = lifeIntervals.elements(); e.hasMoreElements();) {
			LifeInterval lifeInterval = (LifeInterval) e.nextElement();
			lifeInterval.setContainerLife(this);
		}
	}

}
