/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  ELASid.java
//
//=============================================================================
package xmto2xmta.timegraph;

import java.util.Vector;

/*********************************************************************** 
 * Element simple interval id object.  Consists of the element index,
 * lifecycle index, active interval index, and the simple duration index.
 *
 * <p><b>History:</b><ul>
 * <li>22 May 2001 - Created file.
 * </ul><p>
 *********************************************************************/
public class ELASid {

	private Integer elementIndex;
	private Integer lifeCycleIndex;
	private Integer intervalIndex;
	private Integer simpleDurIndex;

	public ELASid (
						Integer elementIndex, Integer lifeCycleIndex,
						Integer intervalIndex, Integer simpleDurIndex) {
		set(elementIndex, lifeCycleIndex, intervalIndex, simpleDurIndex);
	}

	public Integer getElementIndex() {
		return elementIndex;
	}

	public Integer getLifeCycleIndex() {
		return lifeCycleIndex;
	}

	public Integer getIntervalIndex() {
		return intervalIndex;
	}

	public Integer getSimpleDurIndex() {
		return simpleDurIndex;
	}

	public String toString() {
		return new String("("+elementIndex+","+lifeCycleIndex+","+
									intervalIndex+","+simpleDurIndex+")");
	}

	public boolean equals(Object object) {
		if ((object != null) && (object instanceof ELASid)) {
			ELASid elisid = (ELASid) object;
			return elementIndex.equals(elisid.getElementIndex()) &&
					lifeCycleIndex.equals(elisid.getLifeCycleIndex()) &&
					intervalIndex.equals(elisid.getIntervalIndex()) &&
					simpleDurIndex.equals(elisid.getSimpleDurIndex());
		} else return false;
	}

	public int hashCode() {
		return elementIndex.hashCode() + lifeCycleIndex.hashCode() +
					intervalIndex.hashCode() + simpleDurIndex.hashCode();
	}

	private void set (
						Integer elementIndex, Integer lifeCycleIndex,
						Integer intervalIndex, Integer simpleDurIndex) {
		this.elementIndex = elementIndex;
		this.lifeCycleIndex = lifeCycleIndex;
		this.intervalIndex = intervalIndex;
		this.simpleDurIndex = simpleDurIndex;
	}

}
