/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  InstanceTimeList.java
//
//=============================================================================
package xmto2xmta.timegraph;

import java.util.TreeMap;
import java.util.HashSet;

/*********************************************************************** 
 * An ordered list of instance times.
 *
 * <p><b>History:</b><ul>
 * <li>15 Mar 2001 - Created file.
 * </ul><p>
 *********************************************************************/

public class InstanceTimeList extends TreeMap {

	public void putTime(Time time, Object condition) {
		try {
			if (containsKey(time)) {
				HashSet conditions = (HashSet) get(time);
				conditions.add(condition);
			} else {
				HashSet conditions = new HashSet();
				conditions.add(condition);
				put(time, conditions);
			}
		} catch (Exception e) {}
	}

	public void removeTime(Time time, Object condition) {
		if (containsKey(time)) {
			HashSet conditions = (HashSet) get(time);
			if (conditions.contains(condition)) {
				conditions.remove(condition);
				if (conditions.isEmpty()) {
					remove(time);
				}
			}
		}
	}

	public boolean hasTime(Time from) {	// inclusive
		return (getNextTime(from) != null);
	}

	public boolean hasResolvedTime(Time from) {	// inclusive
		return (getNextResolvedTime(from) != null);
	}

	public boolean hasTimeAfter(Time from) {	// exclusive
		return (getNextTimeAfter(from) != null);
	}

	public boolean hasResolvedTimeAfter(Time from) {	// exclusive
		return (getNextResolvedTimeAfter(from) != null);
	}

	public Time getNextTime(Time from) {  // inclusive
		try {
			return (Time) tailMap(from).firstKey();
		} catch (Exception e) {
			return null;
		}
	}


	public Time getNextTimeAfter(Time from) {  // exclusive
		try {
			return (Time) tailMap(Time.add(from, new Time(1))).firstKey();
		} catch (Exception e) {
			return null;
		}
	}
	public Time getNextResolvedTime(Time from) {  // inclusive
		try {
			return (Time) subMap(from, new Time()).firstKey();
		} catch (Exception e) {
			return null;
		}
	}
		
	public Time getNextResolvedTimeAfter(Time from) {  // exclusive
		try {
			return (Time) subMap(Time.add(from, new Time(1)), new Time()).firstKey();
		} catch (Exception e) {
			return null;
		}
	}
		
}
