/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  Interval.java
//
//=============================================================================
package xmto2xmta.timegraph;

/*********************************************************************** 
 * A time interval.  Has a begin, end, and duration.
 *
 * <p><b>History:</b><ul>
 * <li>15 Mar 2001 - Created file.
 * </ul><p>                                                                   
 *********************************************************************/

public class Interval {
	
	private Time begin = null;
	private Time end = null;

	// ctors

	public Interval() {
		begin = new Time(Time.UNRESOLVED);
		end = new Time(Time.UNRESOLVED);
	}

	public Interval(Time begin, Time end) {
		setInterval(begin, end);
	}

	public void setInterval(Time begin, Time end) {
		if (begin == null) {
			begin = new Time(Time.UNRESOLVED);
		} else {
			this.begin = begin;
		}
		if (end == null) {
			end = new Time(Time.UNRESOLVED);
		} else {
			this.end = end;
		}
	}

	public void setBeginTime(Time time) {
		if (time == null) {
			begin = new Time(Time.UNRESOLVED);
		} else {
			begin = time;
		}
	}

	public Time getBeginTime() {
		return begin;
	}

	public void setEndTime(Time time) {
		if (time == null) {
			end = new Time(Time.UNRESOLVED);
		} else {
			end = time;
		}
	}

	public Time getEndTime() {
		return end;
	}

	public Time getDuration() {
		return Time.subtract(end,begin);
	}

	public boolean equals(Object interval) {
		if (interval == null || !(interval instanceof Interval)) {
			return false;
		} else {
			return begin.equals(((Interval)interval).getBeginTime()) & end.equals(((Interval)interval).getEndTime());
		}
	}

	public int hashCode() {
		return begin.hashCode() + end.hashCode();
	}

	public static Interval add(Interval interval1, Interval interval2) {
		return new Interval(Time.add(interval1.getBeginTime(), interval2.getBeginTime()),
									Time.add(interval1.getEndTime(), interval2.getEndTime()));
	}

	public static Interval add(Interval interval, Time time) {
		return Interval.add(interval, new Interval(time, time));
	}

	public static Interval subtract(Interval interval1, Interval interval2) {
		return new Interval(Time.subtract(interval1.getBeginTime(), interval2.getBeginTime()),
									Time.subtract(interval1.getEndTime(), interval2.getEndTime()));
	}

	public static Interval subtract(Interval interval, Time time) {
		return Interval.subtract(interval, new Interval(time, time));
	}

	public static Interval union(Interval interval1, Interval interval2) {
		Time newBeginTime = null;
		if (interval1.getBeginTime().isLess(interval2.getBeginTime())) {
			newBeginTime = interval1.getBeginTime();
		} else {
			newBeginTime = interval2.getBeginTime();
		}
		Time newEndTime = null;
		if (interval1.getEndTime().isGreater(interval2.getEndTime())) {
			newEndTime = interval1.getEndTime();
		} else {
			newEndTime = interval2.getEndTime();
		}
		return new Interval(newBeginTime, newEndTime);
	}

	public boolean isResolved() {
		return begin.isResolved() & end.isResolved();
	}

	public boolean point() {
		return (isResolved() && begin.isEqual(end));
	}

	public boolean negative() {
		return begin.isGreater(end);
	}

	public String toString() {
		return "("+begin.toString()+","+end.toString()+")";
	}

}	// class Interval
