/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  LifeInterval.java
//
//=============================================================================
package xmto2xmta.timegraph;

import java.util.Vector;

/***********************************************************************
 * Active interval of a timing element.
 *
 * <p><b>History:</b><ul>
 * <li>15 Mar 2001 - Created file.
 * </ul><p>
 *********************************************************************/

public class LifeInterval extends Interval {
  public static final int ACTIVE = 0;
  public static final int FILL_FREEZE = 2;
  public static final int FILL_TRANSITION = 3;

  private int type = ACTIVE;
  private Time observedBegin = new Time();
  private Vector simpleDurs = new Vector(); // of simple dur (Time)
  private Time intrinsicDur = new Time();
  private ContainerLife containerLife = null;
  private Vector elasids = new Vector();  // of ELASid
  private Time lastComputedSimpleDur = new Time();

  public LifeInterval() {
    super();
  }

  public LifeInterval(Time begin, Time end) {
    super(begin, end);
  }

  public void setType(int type) {
    this.type = type;
  }

  public int getType() {
    return type;
  }

  public void setObservedBegin(Time time) {
    observedBegin = time;
  }

  public Time getObservedBegin() {
    return observedBegin;
  }

  public void setSimpleDurs(Vector simpleDurs) {
    this.simpleDurs = simpleDurs;
  }

  public Vector getSimpleDurs() {
    return simpleDurs;
  }

  public int size() {
    return simpleDurs.size();
  }

  public void setIntrinsicDur(Time intrinsicDur) {
    this.intrinsicDur = intrinsicDur;
  }

  public Time getIntrinsicDur() {
    return intrinsicDur;
  }

  public void setContainerLife(ContainerLife containerLife) {
    this.containerLife = containerLife;
  }

  public ContainerLife getContainerLife() {
    return containerLife;
  }

  public void setELASids(Vector elasids) {
    this.elasids = elasids;
  }

  public Vector getELASids() {
    return elasids;
  }

  public void setLastComputedSimpleDur(Time simpleDur) {
    lastComputedSimpleDur = simpleDur;
  }

  public Time getLastComputedSimpleDur() {
    return lastComputedSimpleDur;
  }

}
