/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MediaTimeContainer.java
//
//=============================================================================
package xmto2xmta.timegraph;                                            

import xmtutil.MediaObjectInformation;

/*********************************************************************** 
 * Time container that has an associated media object.
 *
 * <p><b>History:</b><ul>
 * <li>15 Mar 2001 - Created file.
 * </ul><p>
 *********************************************************************/

public class MediaTimeContainer extends TimeContainer {

	private String mediaSrc = null;

	public MediaTimeContainer() {
		this(null, new Integer(-1), null);
	}

	public MediaTimeContainer(Integer id, Container parent) {
		this(null, id, parent);
	}

	public MediaTimeContainer(TimeGraph timeGraph, Integer id, Container parent) {
		super(timeGraph, id, parent);
		// Endsync default is "media".
		setEndsyncRule(TimeContainer.ENDSYNC_MEDIA);
		// setIntrinsicDur(new Time(0));
	}

	public void setMediaSrc(String mediaSrc) {
		this.mediaSrc = mediaSrc;
		// computeIntrinsicDur();
	}

	public String getMediaSrc() {
		return mediaSrc;
	}

	public Time computeIntrinsicDur() {
		if (getIntrinsicDur().isResolved()) return getIntrinsicDur();

		writer.println(getID()+" media src=" + mediaSrc);
		MediaObjectInformation info = new MediaObjectInformation(mediaSrc);
		if (info.getMediaType() != MediaObjectInformation.UNKNOWN_TYPE) {
			setIntrinsicDur(new Time(info.getDuration()));
			writer.println(getID()+" media intrinsic dur="+getIntrinsicDur());
		} else {
			setIntrinsicDur(new Time(0));
			writer.println(getID()+" unknown media type. intrinsic dur="+getIntrinsicDur());
		}
		return getIntrinsicDur();
	}

	protected Time computeImplicitDur() {
		if (getImplicitDur().isResolved()) return getImplicitDur();
		Time now = convertFromGlobalToLocalTime(getCurrentTime());
		switch (getEndsyncRule()) {
			case TimeContainer.ENDSYNC_MEDIA:
				setImplicitDur(computeIntrinsicDur());
				break;
			case TimeContainer.ENDSYNC_FIRST:
				if (computeIntrinsicDur().isLessEqual(now)) {
					setImplicitDur(computeIntrinsicDur());
				} else {
					if (timeContainerHasEnded()) {
						setImplicitDur(now);
					}
				}
				break;
			case TimeContainer.ENDSYNC_ID:
				if (timeContainerHasEnded()) {
					setImplicitDur(now);
				}
				break;
			case TimeContainer.ENDSYNC_LAST:
			case TimeContainer.ENDSYNC_ALL:
				if (timeContainerHasEnded()) {
					if (computeIntrinsicDur().isLessEqual(now)) {
						setImplicitDur(now);
					}
				}
				break;
		}
		return getImplicitDur();
	}


}   // class MediaTimeContainer

