/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  SeqTimeContainer.java
//
//=============================================================================
package xmto2xmta.timegraph;

/*********************************************************************** 
 * The seq time container.
 *
 * <p><b>History:</b><ul>
 * <li>15 Mar 2001 - Created file.
 * </ul><p>                                                                   
 *********************************************************************/

public class SeqTimeContainer extends TimeContainer {

	public SeqTimeContainer() {
		this(null, new Integer(-1), null);
	}

	public SeqTimeContainer(Integer id, Container parent) {
		this(null, id, parent);
	}

	public SeqTimeContainer(TimeGraph timeGraph, Integer id, Container parent) {
		super(timeGraph, id, parent);
		// Seq behaves as if endsync = "all".
		// This value is fixed, and cannot be changed.
		super.setEndsyncRule(TimeContainer.ENDSYNC_ALL, new Integer(-1));
	}

	public void setEndsyncRule(int endsyncRule, Integer endsyncID) {
		// ignore it.
		writer.println("ignored seq setendsyncrule");
	}

	public void setEndsyncRule(int endsyncRule) {
		// ignore it.
		writer.println("ignored seq setendsyncrule");
	}

	public void setFill(int fill) {
		if (fill == TimeContainer.FILL_TRANSITION) {
			writer.println(getID()+" fill transition ignored for par");
		} else {
			super.setFill(fill);
		}
	}

	public void setFillDefault(int fillDefault) {
		if (fillDefault == TimeContainer.FILL_TRANSITION) {
			writer.println(getID()+" fill default transition ignored for par");
		} else {
			super.setFillDefault(fillDefault);
		}
	}

}	// class SeqTimeContainer

