/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  Tracer.java
//
//=============================================================================
package xmto2xmta.timegraph;

import java.io.Writer;
import java.io.IOException;
import xmto2xmta.TimeScheduler;
import xmto2xmta.ErrorEvent;

/*********************************************************************** 
 * A java.io.Writer class that wraps the fireErrorEvent method of 
 * TimeScheduler.  
 *
 * <p><b>History:</b><ul>
 * <li>27 Mar 2001 - Created file.
 * </ul><p>
 *********************************************************************/

public class Tracer extends Writer {

	private TimeScheduler timeScheduler = null;
	private int msgLevel = ErrorEvent.TRACE;
	private StringBuffer buffer = new StringBuffer();

	public Tracer(TimeScheduler timeScheduler, int msgLevel) {
		this.timeScheduler = timeScheduler;
		this.msgLevel = msgLevel;
	}

	public void flush() throws IOException {
		if (buffer.length() > 0) {
			if (buffer.charAt(buffer.length() - 1) == '\n') {
				buffer.setLength(buffer.length() - 1);
			}
			if ((buffer.length() > 0) && timeScheduler.getDebug()) {
				timeScheduler.fireErrorEvent(msgLevel, 0, buffer.toString());
			}
			buffer.setLength(0);
		}
	}

	public void close() throws IOException {
		flush();
	}

	public void write(char[] cbuf, int off, int len) throws IOException {
		buffer.append(cbuf, off, len);
	}

	public void write(char[] cbuf) throws IOException {
		buffer.append(cbuf);
	}

	public void write(int c) throws IOException {
		buffer.append(c);
	}

	public void write(String str) throws IOException {
		buffer.append(str);
	}

	public void write(String str, int off, int len) throws IOException {
		try {
			buffer.append(str.substring(off, off+len));
		} catch (Exception e) {
			throw new IOException(e.getMessage());
		}
	}
}
