/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  Animatable.java
//
//=============================================================================
package xmto2xmta.tree;

import org.w3c.dom.Element;
import xmto2xmta.timegraph.ELASid;

/*********************************************************************** 
 * listener interface for the editor options model.
 *
 * <p><b>History:</b><ul>
 * <li>27 Oct 2000 - Add tracing support.  For messages, in case we want tracing to
 *                   to go to a trace log file instead of system console.
 * </ul>                                                                           
 *********************************************************************/
public interface Animatable {
	/*********************************************************************** 
	 **********************************************************************/
	static final public int TYPE_UNSUPPORTED = 0;
	static final public int TYPE_SFBool = 1;
	static final public int TYPE_SFColor = 2;
	static final public int TYPE_SFFloat = 3;
	static final public int TYPE_SFVec2f = 4;
	static final public int TYPE_SFInt32 = 5;
	static final public int TYPE_SFString = 6;
	static final public int TYPE_MFBool = 7;
	static final public int TYPE_MFColor = 8;
	static final public int TYPE_MFFloat = 9;
	static final public int TYPE_MFVec2f = 10;
	static final public int TYPE_MFInt32 = 11;
	static final public int TYPE_MFString = 12;
/*********************************************************************** 
 * This function converts a xmto attribute value to its xmta equivalent.
 * It then returns the converted value.  
 **********************************************************************/
public abstract String getAnimConvertValueXmto2Xmta(String xmtoAttrName, String xmtoValue, int xmtoFieldType, int xmtaFieldType);
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public abstract int getAnimXmtaAttrFieldType(String xmtoAttrName);
/*********************************************************************** 
 * This function returns the initial value of the field begin animated.  This
 * is used in case the attribute is not specified in the animaiton.  Generally
 * this is the original value so that after the animation is finished
 * the node field can be returned to its original state.  If this value
 * isn't specified then the default value will be used.
 **********************************************************************/
public abstract String getAnimXmtaAttrInitialValue(String xmtoAttrName);
/***********************************************************************
 * This function returns the MPEG-4 field being animated. 
 **********************************************************************/
public abstract String getAnimXmtaAttrName(String xmtoAttrName);
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated.
 * This method is for when there is no elasId for this instantiation of the
 * object.  This can occur for xMediaChildrenObject that are defs objects,
 * but the parent isn't a TimingElement.
 **********************************************************************/
public abstract String getAnimXmtaDefsNodeId(String xmtoAttrName);
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated. 
 **********************************************************************/
public abstract String getAnimXmtaNodeId(String xmtoAttrName, ELASid elasId);
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public abstract int getAnimXmtoAttrFieldType(String xmtoAttrName);
}
