/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  ConvertElement.java
//
//=============================================================================
package xmto2xmta.tree;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class which describes the various operations for converting from 
 * elements from XMT-O to XMT-A 
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>20 Jun 2001 - Remove old methodology of performing mapping passes.
 * </ul>                                                                       
 *********************************************************************/
public abstract class ConvertElement extends TreeElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrId_ = null;

	/**********************************************************************
	 * the unique index identifier for this element.
	 *********************************************************************/
	private Integer index_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private Element xmtoElement_;
	private Hashtable xmtoIds_;
	private Hashtable xmtaIds_;
	private Hashtable convertIndexTable_;
	private Document xmtaDoc_;
	private Xmto2Xmta xmto2Xmta_;

	/*********************************************************************** 
	 **********************************************************************/
	private Vector xmtaParNodes_ = new Vector();
	private Vector xmtaParOds_ = new Vector();
	private Vector xmtaParRoutes_ = new Vector();

	/*********************************************************************** 
	 **********************************************************************/
//	private Element xmtaParNodes_ = null;
//	private Element xmtaParOds_ = null;
//	private Element xmtaParRoutes_ = null;
	
	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public ConvertElement(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	xmtoElement_ = xmtoElement;
	xmtoIds_ = xmtoIds;
	xmtaIds_ = xmtaIds;
	xmtaDoc_ = xmtaDoc;
	xmto2Xmta_ = xmto2Xmta;
	convertIndexTable_ = convertIndexTable;

	// Add to the complete table.
	index_ = IdManager.createNewElementIndex();
	convertIndexTable_.put(index_, this);

	// Check out if there is an id field.
	xmtoAttrId_ = xmtoElement.getAttribute(DOMXMTO.id);
	if (null != xmtoAttrId_)
		if (!xmtoAttrId_.equals(""))
			xmtoIds.put(xmtoAttrId_, this);
		else
			xmtoAttrId_ = null;
}
/**********************************************************************
 * Add the node to the Par that contains the BIFS tree node changes.
 *********************************************************************/
public void addToParNode(Node node) {
	xmtaParNodes_.add(node);
}
/**********************************************************************
 * Add the node to the Par that contains the OD changes.
 *********************************************************************/
public void addToParOd(Node node) {
	xmtaParOds_.add(node);
}
/**********************************************************************
 * Add the node to the Par that contains the Routes changes.
 *********************************************************************/
public void addToParRoutes(Node node) {
	xmtaParRoutes_.add(node);
}
/***********************************************************************
 * Determine whether or not children of this DOMParser element should be parsed
 * and create children ConvertElments or not.  Probably the only place
 * this method is used to return false is in the xmtaMedia element
 * where children of the object are specific to XMT-A and not XMT-O.
 **********************************************************************/
public boolean createChildren() {
	return true;
}
/**********************************************************************
 *********************************************************************/
public void fireErrorEvent(int errorLevel, int errorId, String message) {
	fireErrorEvent(errorLevel, errorId, message, null, null);
}
/**********************************************************************
 *********************************************************************/
public void fireErrorEvent(int errorLevel, int errorId, String message, Element xmtoElement) {
	fireErrorEvent(errorLevel, errorId, message, xmtoElement, null);
}
/**********************************************************************
 *********************************************************************/
public void fireErrorEvent(int errorLevel, int errorId, String message, Element xmtoElement, String xmtoAttribute) {
	xmto2Xmta_.fireErrorEvent(new ErrorEvent(xmto2Xmta_, errorLevel, errorId, message, xmtoElement, xmtoAttribute));
}
/**********************************************************************
 * @return xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 *********************************************************************/
public Hashtable getConvertIndexTable() {
	return convertIndexTable_;
}
/**********************************************************************
 * @return the unique index identifier for this element.
 *********************************************************************/
public Integer getIndex() {
	return index_;
}
/**********************************************************************
 * @return xmtaDoc     DOM Document for the new XMT-A elements.
 *********************************************************************/
public Document getXmtaDoc() {
	return xmtaDoc_;
}
/**********************************************************************
 * @return xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 *********************************************************************/
public Hashtable getXmtaIds() {
	return xmtaIds_;
}
/**********************************************************************
 * @return the converter, used for firing error events.
 *********************************************************************/
public Xmto2Xmta getXmto2Xmta() {
	return xmto2Xmta_;
}
/**********************************************************************
 * @return XMT-O id attribute.
 *********************************************************************/
public String getXmtoAttrId() {
	return xmtoAttrId_;
}
/**********************************************************************
 * @return xmtoElement XMT-O element for conversion.
 *********************************************************************/
public Element getXmtoElement() {
	return xmtoElement_;
}
/**********************************************************************
 * @return xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 *********************************************************************/
public Hashtable getXmtoIds() {
	return xmtoIds_;
}
/*********************************************************************** 
 * Validate the syntax of the element.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	// id
	if (DOMXMTO.id.equals(attributeName)) {
		// Determine the id
		if (null != attributeValue && (!(attributeValue.equals(""))))
			xmtoAttrId_ = attributeValue;
		else {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, ""};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}
	return false;
}
/*********************************************************************** 
 **********************************************************************/
public boolean insertIntoXmtaTree() {
	Element xmtaParNode = null;
	Element xmtaParRoute = null;
	Element xmtaParOd = null;

	// The Xmta insert and delete pars's will need to be added to the root xmta body.
	TreeElement parent = this;
	while (null != parent) {
		if (parent instanceof XMTO) {
			xmtaParNode = ((XMTO) parent).getXmtaParNodes();
			xmtaParRoute = ((XMTO) parent).getXmtaParRoutes();
			xmtaParOd = ((XMTO) parent).getXmtaParOds();
			break;
		}
		parent = parent.getParent();
	}
	if (null == xmtaParNode || null == xmtaParRoute || null == xmtaParOd) {
		// There always should be a XMTO so we should never get here.  
		// If we have then the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), getXmtoElement());
		remove();
		return false;
	}

	//
	for (Enumeration enum = xmtaParNodes_.elements(); enum.hasMoreElements();) {
		Node node = (Node) enum.nextElement();
		xmtaParNode.appendChild(node);
	}
	for (Enumeration enum = xmtaParRoutes_.elements(); enum.hasMoreElements();) {
		Node node = (Node) enum.nextElement();
		xmtaParRoute.appendChild(node);
	}
	for (Enumeration enum = xmtaParOds_.elements(); enum.hasMoreElements();) {
		Node node = (Node) enum.nextElement();
		xmtaParOd.appendChild(node);
	}

	//
	return true;
}
/**********************************************************************
 * This method removes the element from the tree.  In addition to removing the element 
 * from the tree it also removes the children and removes the element from the id and index tables.
 *********************************************************************/
public void remove() {
	// First go through the children removing them.
	Vector clone = getChildren();
	for (Enumeration children = clone.elements(); children.hasMoreElements();) {
		ConvertElement child = (ConvertElement) children.nextElement();
		child.remove();
	}
	// Now remove ourselves from the tables
	convertIndexTable_.remove(index_);

	// Check out if there is an id field.
	if (null != xmtoAttrId_)
		xmtoIds_.remove(xmtoAttrId_);

	// Since we don't support the timings for this element remove it from the tree
	getParent().removeChild(this);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/*********************************************************************** 
 * Validate the syntax of the element.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.
 **********************************************************************/
public boolean validateSyntax() {
	// Loop through all the attributes checking if they are valid or not.
	NamedNodeMap children = getXmtoElement().getAttributes();
	for (int i = 0; i < children.getLength(); i++) {
		Node child = children.item(i);
		if (Node.ATTRIBUTE_NODE == child.getNodeType()) {
			String nodeName = child.getNodeName();
			String nodeValue = child.getNodeValue();
			boolean handled = false;

			//
			if (null != nodeName && null != nodeValue && !nodeName.equals("") && !nodeValue.equals(""))
				handled = handleValidateSyntax(nodeName, nodeValue);

			//
			if (!handled) {
				MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_WARNING_ATTRIBUTE_NOT_SUPPORTED));

				//
				StringBuffer elementBuffer = new StringBuffer();
				elementBuffer.append(getXmtoElement().getNodeName());
				String id = getXmtoElement().getAttribute(DOMXMTO.id);
				if (null != id && !id.equals(""))
					elementBuffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");

				//	
				StringBuffer attribBuffer = new StringBuffer();
				attribBuffer.append(nodeName);
				if (null != nodeValue && !nodeValue.equals(""))
					attribBuffer.append("=\"").append(nodeValue).append("\"");


				//	
				Object[] msgArgs = {elementBuffer.toString(), attribBuffer.toString()};
				String message = msgFormat.format(msgArgs);
				fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), nodeName);
			}
		}
	}

	// 
	return true;
}
}
