/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  DefsObject.java
//
//=============================================================================
package xmto2xmta.tree;

/*********************************************************************** 
 * listener interface for the editor options model.
 *
 * <p><b>History:</b><ul>
 * <li>27 Oct 2000 - Add tracing support.  For messages, in case we want tracing to
 *                   to go to a trace log file instead of system console.
 * <li>12 Jun 2001 - Add support for defs id.
 * </ul>
 *********************************************************************/
public interface DefsObject {
/***********************************************************************
 * Get the defs id of the object.
 * @return the unique id of this object.
 **********************************************************************/
public abstract String getXmtaDefsId();
/***********************************************************************
 * Routine to initialize the defs query flags.  This is added to provide
 * consistency for all the classes that implement the DefsObject interface.
 **********************************************************************/
public abstract void initDefsFlags();
/***********************************************************************
 * Determine whether the object is a child of the defs element or not.  
 * This means that the object is a direct child of the defs element.
 * It isn't a child if it isn't in the defs tree or is a descendent 
 * (starting at grandchild) of the defs element.
 * @return whether the object is a child of the defs element.  True if 
 * it is a child of the defs element, false otherwise.
 **********************************************************************/
public abstract boolean isDefsParent();
/***********************************************************************
 * Determine whether the object is in the defs tree or not.  This means
 * that the object is defined somewhere in the Defs tree.  It may be a
 * child or descendent of the defs element.
 * @return whether the object is in the defs tree.  True if a descendent
 * of the defs element, false otherwise.
 **********************************************************************/
public abstract boolean isInDefsTree();
}
