/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  MediaElement.java
//
//=============================================================================
package xmto2xmta.tree;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to group together the various type of media elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>08 Jun 2001 - Add support (ParContainable) support to distinguish
 *                   that this classes contains timing elements in a 
 *                   par manner.
 * </ul>
 *********************************************************************/
public abstract class MediaElement extends TimingElement implements Sensorable, ParContainable {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrRegion_ = null;
	private boolean enableTouchSensor_ = false;

	/***********************************************************************
	 **********************************************************************/
	private Hashtable xmtaGroupIds_ = new Hashtable(89);
	private Hashtable xmtaSwitchIds_ = new Hashtable(89);
	private Hashtable xmtaTransform2dIds_ = new Hashtable(89);
	private Hashtable xmtaTouchSensorIds_ = new Hashtable(89);
	private Hashtable xmtaConditionalInsertIds_ = new Hashtable(89);
	private Hashtable xmtaConditionalInsertTimerIds_ = new Hashtable(89);
	private Hashtable xmtaTimeSensorBeginOffsetIds_ = new Hashtable(89);
	private Hashtable xmtaConditionalDeleteIds_ = new Hashtable(89);
	private Hashtable xmtaConditionalDeleteTimerIds_ = new Hashtable(89);
	private Hashtable xmtaTimeSensorEndOffsetIds_ = new Hashtable(89);
	private Hashtable xmtaValuatorIds_ = new Hashtable(89);

	/*********************************************************************** 
	 **********************************************************************/
	static private Class headClass_ = null;
	static private Class groupClass_ = null;
	static private boolean classesLoaded_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public MediaElement(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * Get the id for the group element.
 **********************************************************************/
public String getXmtaGroupId(ELASid elasId) {
	String xmtaGroupId = (String) xmtaGroupIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaGroupId) {
		xmtaGroupId = IdManager.getNextId(IdManager.GroupID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaGroupId = xmtaGroupId + ":" + getXmtoAttrId();
		xmtaGroupIds_.put(elasId, xmtaGroupId);
	}

	//
	return xmtaGroupId;
}
/***********************************************************************
 * Get the id for the switch element.
 **********************************************************************/
public String getXmtaSwitchId(ELASid elasId) {
	String xmtaSwitchId = (String) xmtaSwitchIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaSwitchId) {
		xmtaSwitchId = IdManager.getNextId(IdManager.SwitchID);
		// make the switch and transform ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaSwitchId = xmtaSwitchId + ":" + getXmtoAttrId();
		xmtaSwitchIds_.put(elasId, xmtaSwitchId);
	}

	//
	return xmtaSwitchId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaTouchSensorId(ELASid elasId) {
	String xmtaTouchSensorId = (String) xmtaTouchSensorIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTouchSensorId) {
		xmtaTouchSensorId = IdManager.getNextId(IdManager.TouchSensorID);
		// make the switch and transform ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaTouchSensorId = xmtaTouchSensorId + ":" + getXmtoAttrId();
		xmtaTouchSensorIds_.put(elasId, xmtaTouchSensorId);
	}
	return xmtaTouchSensorId;
}
/***********************************************************************
 * Get the id for the transform2d element.
 **********************************************************************/
public String getXmtaTransform2dId(ELASid elasId) {
	String xmtaTransform2dId = (String) xmtaTransform2dIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTransform2dId) {
		xmtaTransform2dId = IdManager.getNextId(IdManager.Transform2dID);
		// make the transform ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaTransform2dId = xmtaTransform2dId + ":" + getXmtoAttrId();
		xmtaTransform2dIds_.put(elasId, xmtaTransform2dId);
	}
	return xmtaTransform2dId;
}
/***********************************************************************
 * Get the id for the conditional insert element.
 **********************************************************************/
public String getXmtaConditionalInsertId(ELASid elasId) {
	String xmtaId = (String) xmtaConditionalInsertIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaId) {
		xmtaId = IdManager.getNextId(IdManager.ConditionalInsertID);
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaId = xmtaId + ":" + getXmtoAttrId();
		xmtaConditionalInsertIds_.put(elasId, xmtaId);
	}
	return xmtaId;
}
/***********************************************************************
 * Get the id for the conditional insert timer.
 **********************************************************************/
public String getXmtaConditionalInsertTimerId(ELASid elasId) {
	String xmtaId = (String) xmtaConditionalInsertTimerIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaId) {
		xmtaId = IdManager.getNextId(IdManager.ConditionalInsertTimerID);
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaId = xmtaId + ":" + getXmtoAttrId();
		xmtaConditionalInsertTimerIds_.put(elasId, xmtaId);
	}
	return xmtaId;
}
/***********************************************************************
 * Get the id for the time sensor begin offset.
 **********************************************************************/
public String getXmtaTimeSensorBeginOffsetId(ELASid elasId) {
	String xmtaId = (String) xmtaTimeSensorBeginOffsetIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaId) {
		xmtaId = IdManager.getNextId(IdManager.TimeSensorBeginOffsetID);
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaId = xmtaId + ":" + getXmtoAttrId();
		xmtaTimeSensorBeginOffsetIds_.put(elasId, xmtaId);
	}
	return xmtaId;
}
/***********************************************************************
 * Get the id for the conditional delete element.
 **********************************************************************/
public String getXmtaConditionalDeleteId(ELASid elasId) {
	String xmtaId = (String) xmtaConditionalDeleteIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaId) {
		xmtaId = IdManager.getNextId(IdManager.ConditionalDeleteID);
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaId = xmtaId + ":" + getXmtoAttrId();
		xmtaConditionalDeleteIds_.put(elasId, xmtaId);
	}
	return xmtaId;
}
/***********************************************************************
 * Get the id for the conditional delete timer.
 **********************************************************************/
public String getXmtaConditionalDeleteTimerId(ELASid elasId) {
	String xmtaId = (String) xmtaConditionalDeleteTimerIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaId) {
		xmtaId = IdManager.getNextId(IdManager.ConditionalDeleteTimerID);
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaId = xmtaId + ":" + getXmtoAttrId();
		xmtaConditionalDeleteTimerIds_.put(elasId, xmtaId);
	}
	return xmtaId;
}
/***********************************************************************
 * Get the id for the time sensor end offset.
 **********************************************************************/
public String getXmtaTimeSensorEndOffsetId(ELASid elasId) {
	String xmtaId = (String) xmtaTimeSensorEndOffsetIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaId) {
		xmtaId = IdManager.getNextId(IdManager.TimeSensorEndOffsetID);
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaId = xmtaId + ":" + getXmtoAttrId();
		xmtaTimeSensorEndOffsetIds_.put(elasId, xmtaId);
	}
	return xmtaId;
}
/***********************************************************************
 * Get the id for the Valuator element.
 **********************************************************************/
public String getXmtaValuatorId(ELASid elasId) {
	String xmtaId = (String) xmtaValuatorIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaId) {
		xmtaId = IdManager.getNextId(IdManager.ValuatorID);
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaId = xmtaId + ":" + getXmtoAttrId();
		xmtaValuatorIds_.put(elasId, xmtaId);
	}
	return xmtaId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRegion() {
	return xmtoAttrRegion_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// region
	if (DOMXMTO.region.equals(attributeName)) {
		if (!classesLoaded_)
			loadClasses();
		boolean regionValid = true;
		// In order to use the region tag the object 
		// - has to be in the body section (!head section)
		// - not be under a group element
		TreeElement parent = getParent();
		while (null != parent) {
			if (null != groupClass_ && groupClass_.isInstance(parent)) {
				regionValid = false;
				break;
			} else {
				if (null != headClass_ && headClass_.isInstance(parent)) {
					regionValid = false;
					break;
				}
			}
			parent = parent.getParent();
		}
		if (regionValid)
			xmtoAttrRegion_ = attributeValue;
		else {
			//
			StringBuffer regionString = new StringBuffer();
			regionString.append(attributeName).append("=\"").append(attributeValue).append("\"");
			//
			StringBuffer nameString = new StringBuffer();
			nameString.append(getXmtoElement().getNodeName());
			String id = getXmtoAttrId();
			if (null != id && !id.equals(""))
				nameString.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");

			//
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_WARNING_ATTRIBUTE_NOT_SUPPORTED_IN_LOCATION));
			Object[] msgArgs = {nameString, regionString};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	//
	return success;
}

public boolean touchSensorEnabled() {
	return enableTouchSensor_;
}

public void enableTouchSensor(boolean enable) {
	enableTouchSensor_ = enable;
}

/*********************************************************************** 
 **********************************************************************/
static private void loadClasses() {
	//
	if (classesLoaded_)
		return;

	//
	classesLoaded_ = true;

	//
	try {
		headClass_ = Class.forName("xmto2xmta.tree.head");
	} catch (Exception e) {
	}
	//
	try {
		groupClass_ = Class.forName("xmto2xmta.tree.group");
	} catch (Exception e) {
	}
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
