/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  TimingElement.java
//
//=============================================================================
package xmto2xmta.tree;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.TimingElementRes;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to group together the various type of timing elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>23 Mar 2001 - Add support for the remaining attributes for timing .
 * <li>09 Apr 2001 - Fix handling of problem with timing.  If we don't
 *                   support the timing attribute then remove the object.
 * <li>06 Jun 2001 - Add defs support, for validating timing attributes.
 * <li>30 Jun 2001 - Fix bug, on validating syntax if problem with parent then
 *                   return directly the problem.
 * </ul><p>
 *********************************************************************/
public abstract class TimingElement extends ConvertElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private TimingAttribute xmtoAttrBegin_ = null;
	private TimingAttribute xmtoAttrDur_ = null;
	private TimingAttribute xmtoAttrEnd_ = null;
	private TimingAttribute xmtoAttrMin_ = null;
	private TimingAttribute xmtoAttrMax_ = null;
	private TimingAttribute xmtoAttrRepeatDur_ = null;
	private String xmtoAttrFill_ = null;
	private String xmtoAttrFillDefault_ = null;
	private String xmtoAttrRestart_ = null;
	private String xmtoAttrRestartDefault_ = null;
	private String xmtoAttrEndsync_ = null;
	private String xmtoAttrRepeatCount_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle timingElementRes_ = ResourceBundle.getBundle("xmto2xmta.resources.TimingElementRes", Locale.getDefault());
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/*********************************************************************** 
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public TimingElement(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 **********************************************************************/
public TimingAttribute getXmtoAttrBegin() {
	//
	return xmtoAttrBegin_;
}
/*********************************************************************** 
 **********************************************************************/
public TimingAttribute getXmtoAttrDur() {
	//
	return xmtoAttrDur_;
}
/*********************************************************************** 
 **********************************************************************/
public TimingAttribute getXmtoAttrEnd() {
	//
	return xmtoAttrEnd_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrEndsync() {
	//
	return xmtoAttrEndsync_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrFill() {
	//
	return xmtoAttrFill_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrFillDefault() {
	//
	return xmtoAttrFillDefault_;
}
/*********************************************************************** 
 **********************************************************************/
public TimingAttribute getXmtoAttrMax() {
	//
	return xmtoAttrMax_;
}
/*********************************************************************** 
 **********************************************************************/
public TimingAttribute getXmtoAttrMin() {
	//
	return xmtoAttrMin_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRepeatCount() {
	//
	return xmtoAttrRepeatCount_;
}
/*********************************************************************** 
 **********************************************************************/
public TimingAttribute getXmtoAttrRepeatDur() {
	//
	return xmtoAttrRepeatDur_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRepeatRestart() {
	//
	return xmtoAttrRestart_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRepeatRestartDefault() {
	//
	return xmtoAttrRestartDefault_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// First resolve the attribute value. This is the same whether it is a begin, dur or end
	if (DOMXMTO.begin.equals(attributeName) || DOMXMTO.dur.equals(attributeName) || DOMXMTO.end.equals(attributeName) || DOMXMTO.min.equals(attributeName) || DOMXMTO.max.equals(attributeName) || DOMXMTO.repeatDur.equals(attributeName)) {
		TimingAttribute timingAttribute = new TimingAttribute(getXmtoIds(), attributeValue);

		// Determine if the attribute type is supported by the attribute.
		boolean errorFound = false;
		if (DOMXMTO.begin.equals(attributeName)) {
			switch (timingAttribute.getTimeValueType()) {
				case TimingAttribute.CLOCK_VALUE :
				case TimingAttribute.SYNCBASE_VALUE :
				case TimingAttribute.SYNC_TO_PREV_VALUE :
				case TimingAttribute.EVENT_VALUE :
				case TimingAttribute.MULTI_VALUE :
				case TimingAttribute.REPEAT_VALUE :
					break;
				case TimingAttribute.INDEFINITE_VALUE :
				case TimingAttribute.ACCESS_KEY_VALUE : //is valid but we don't support yet 
				case TimingAttribute.MEDIA_MARKER_VALUE : //is valid but we don't support yet 
				case TimingAttribute.WALLCLOCK_VALUE : //is valid but we don't support yet 
				case TimingAttribute.MEDIA_VALUE :
				case TimingAttribute.INVALID :
				default :
					errorFound = true;
					break;
			}
		} else
			if (DOMXMTO.end.equals(attributeName)) {
				switch (timingAttribute.getTimeValueType()) {
					case TimingAttribute.CLOCK_VALUE :
					case TimingAttribute.SYNCBASE_VALUE :
					case TimingAttribute.SYNC_TO_PREV_VALUE :
					case TimingAttribute.EVENT_VALUE :
					case TimingAttribute.INDEFINITE_VALUE :
					case TimingAttribute.MULTI_VALUE :
					case TimingAttribute.REPEAT_VALUE :
						break;
					case TimingAttribute.ACCESS_KEY_VALUE : //is valid but we don't support yet 
					case TimingAttribute.MEDIA_MARKER_VALUE : //is valid but we don't support yet 
					case TimingAttribute.WALLCLOCK_VALUE : //is valid but we don't support yet 
					case TimingAttribute.MEDIA_VALUE :
					case TimingAttribute.INVALID :
					default :
						errorFound = true;
						break;
				}
			} else
				if (DOMXMTO.dur.equals(attributeName) || DOMXMTO.max.equals(attributeName)) {
					switch (timingAttribute.getTimeValueType()) {
						case TimingAttribute.CLOCK_VALUE :
						case TimingAttribute.MEDIA_VALUE :
						case TimingAttribute.INDEFINITE_VALUE :
							break;
						default :
							errorFound = true;
							break;
					}
				} else
					if (DOMXMTO.min.equals(attributeName)) {
						switch (timingAttribute.getTimeValueType()) {
							case TimingAttribute.CLOCK_VALUE :
							case TimingAttribute.MEDIA_VALUE :
								break;
							default :
								errorFound = true;
								break;
						}
					} else
						if (DOMXMTO.repeatDur.equals(attributeName)) {
							switch (timingAttribute.getTimeValueType()) {
								case TimingAttribute.CLOCK_VALUE :
								case TimingAttribute.INDEFINITE_VALUE :
									break;
								default :
									errorFound = true;
									break;
							}
						}

						// Now set the real attribute
		if (!errorFound)
			if (DOMXMTO.begin.equals(attributeName))
				xmtoAttrBegin_ = timingAttribute;
			else
				if (DOMXMTO.dur.equals(attributeName))
					xmtoAttrDur_ = timingAttribute;
				else
					if (DOMXMTO.end.equals(attributeName))
						xmtoAttrEnd_ = timingAttribute;
					else
						if (DOMXMTO.min.equals(attributeName))
							xmtoAttrMin_ = timingAttribute;
						else
							if (DOMXMTO.max.equals(attributeName))
								xmtoAttrMax_ = timingAttribute;
							else
								if (DOMXMTO.repeatDur.equals(attributeName))
									xmtoAttrRepeatDur_ = timingAttribute;
								else
									errorFound = true;
		if (errorFound) {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			StringBuffer buffer = new StringBuffer();
			buffer.append(getXmtoElement().getNodeName());
			if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
				buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
			Object[] msgArgs = {buffer.toString(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), attributeName);

			// Since we don't support the timings for this element remove it from the tree
			remove();
			return false;
		}
		// Irregardless if this is a valid timing attribute, print out any messages from the attribute.
		Vector errorMessages = timingAttribute.getErrorMessages();
		if (null != errorMessages)
			for (java.util.Enumeration enum = errorMessages.elements(); enum.hasMoreElements();) {
				fireErrorEvent(ErrorEvent.ERROR, 0, (String) enum.nextElement(), getXmtoElement(), attributeName);
			}
		Vector warningMessages = timingAttribute.getWarningMessages();
		if (null != warningMessages)
			for (java.util.Enumeration enum = warningMessages.elements(); enum.hasMoreElements();) {
				fireErrorEvent(ErrorEvent.WARNING, 0, (String) enum.nextElement(), getXmtoElement(), attributeName);
			}
			//
		return true;
	} // end timing (begin, dur or end)

	// fill
	if (DOMXMTO.fill.equals(attributeName) || DOMXMTO.fillDefault.equals(attributeName) || DOMXMTO.restart.equals(attributeName) || DOMXMTO.restartDefault.equals(attributeName) || DOMXMTO.endsync.equals(attributeName) || DOMXMTO.repeatCount.equals(attributeName)) {
		boolean errorFound = false;
		// Check it isn't null
		if (null == attributeValue || (attributeValue.equals("")))
			errorFound = true;
		else
			if (DOMXMTO.fill.equals(attributeName))
				xmtoAttrFill_ = attributeValue;
			else
				if (DOMXMTO.fillDefault.equals(attributeName))
					xmtoAttrFillDefault_ = attributeValue;
				else
					if (DOMXMTO.restart.equals(attributeName))
						xmtoAttrRestart_ = attributeValue;
					else
						if (DOMXMTO.restartDefault.equals(attributeName))
							xmtoAttrRestartDefault_ = attributeValue;
						else
							if (DOMXMTO.endsync.equals(attributeName))
								xmtoAttrEndsync_ = attributeValue;
							else
								if (DOMXMTO.repeatCount.equals(attributeName))
									xmtoAttrRepeatCount_ = attributeValue;
								else
									errorFound = true;
		if (errorFound) {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, ""};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), attributeName);
			return false;
		}
		return true;
	}

	//
	return success;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/*********************************************************************** 
 * Validate the syntack of the defs object found in the desf tree.	
 * Defs object can't have timing attributes. If so then this is an error,
 * remove them and give a message to the user.
 **********************************************************************/
private boolean validateDefsObjectTimingSyntax() {
	// Nothing to check if it isn't a DefsObject in the Defs tree
	if (!(this instanceof DefsObject) || !((DefsObject) this).isInDefsTree())
		return true;

	// Check the xmtoAttrBegin_ attribute
	if (null != xmtoAttrBegin_) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.begin).append("=\"").append(xmtoAttrBegin_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrBegin_ = null;
	}
	// Check the xmtoAttrDur_ attribute
	if (null != xmtoAttrDur_) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.dur).append("=\"").append(xmtoAttrDur_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrDur_ = null;
	}
	// Check the xmtoAttrEnd_ attribute
	if (null != xmtoAttrEnd_) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.end).append("=\"").append(xmtoAttrEnd_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrEnd_ = null;
	}
	// Check the xmtoAttrMin_ attribute
	if (null != xmtoAttrMin_) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.min).append("=\"").append(xmtoAttrMin_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrMin_ = null;
	}
	// Check the xmtoAttrMax_ attribute
	if (null != xmtoAttrMax_) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.max).append("=\"").append(xmtoAttrMax_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrMax_ = null;
	}
	// Check the xmtoAttrRepeatDur_ attribute
	if (null != xmtoAttrRepeatDur_) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.repeatDur).append("=\"").append(xmtoAttrRepeatDur_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrRepeatDur_ = null;
	}



	// Check the xmtoAttrFill_ attribute
	if (null != xmtoAttrFill_ && !xmtoAttrFill_.equals("")) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.fill).append("=\"").append(xmtoAttrFill_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrFill_ = null;
	}
	// Check the xmtoAttrFillDefault_ attribute
	if (null != xmtoAttrFillDefault_ && !xmtoAttrFillDefault_.equals("")) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.fillDefault).append("=\"").append(xmtoAttrFillDefault_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrFillDefault_ = null;
	}
	// Check the xmtoAttrRestart_ attribute
	if (null != xmtoAttrRestart_ && !xmtoAttrRestart_.equals("")) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.restart).append("=\"").append(xmtoAttrRestart_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrRestart_ = null;
	}
	// Check the xmtoAttrRestartDefault_ attribute
	if (null != xmtoAttrRestartDefault_ && !xmtoAttrRestartDefault_.equals("")) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.restartDefault).append("=\"").append(xmtoAttrRestartDefault_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrRestartDefault_ = null;
	}
	// Check the xmtoAttrEndsync_ attribute
	if (null != xmtoAttrEndsync_ && !xmtoAttrEndsync_.equals("")) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.endsync).append("=\"").append(xmtoAttrEndsync_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrEndsync_ = null;
	}
	// Check the xmtoAttrRepeatCount_ attribute
	if (null != xmtoAttrRepeatCount_ && !xmtoAttrRepeatCount_.equals("")) {
		MessageFormat msgFormat = new MessageFormat(timingElementRes_.getString(TimingElementRes.MSG_WARNING_DEFS_OBJECT_WITH_TIMING_ATTRIBUTE));
		StringBuffer sbElement = new StringBuffer();
		sbElement.append(getXmtoElement().getNodeName());
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			sbElement.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
		StringBuffer sbAttribute = new StringBuffer();
		sbAttribute.append(DOMXMTO.repeatCount).append("=\"").append(xmtoAttrRepeatCount_).append("\"");
		Object[] msgArgs = {sbElement, sbAttribute, DOMXMTO.defs};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		xmtoAttrRepeatCount_ = null;
	}



	// One last item is that we have to reset the duration to be indefinite.  This way the object sticks around forever.
	xmtoAttrDur_ = new TimingAttribute(TimingAttribute.INDEFINITE_VALUE);

	// 
	return true;
}
/*********************************************************************** 
 * Validate the syntax of the element.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.
 * Mainly check the begin, dur and end values to see they are valid timing
 * attributes.  If this is a problem then remove this node from the tree.
 **********************************************************************/
public boolean validateSyntax() {
	boolean success = super.validateSyntax();
	if (!success)
		return success;

	// Check to see if it is in the defs tree
	if (this instanceof DefsObject && ((DefsObject) this).isInDefsTree()) {
		// If we are in the <defs> tree then the none of the timing attributes can be set.  If so then
		// this is an error, remove them and give a message to the user.
		boolean rcSuccess = validateDefsObjectTimingSyntax();
		// Only overwrite success if we found a problem
		if (!rcSuccess)
			success = rcSuccess;
	}

	// 
	return success;
}
}
