/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  TreeClassObjects.java
//
//=============================================================================
package xmto2xmta.tree;

import java.lang.reflect.Method;
/***********************************************************************
 * Class to handle the class objects in order to make the tree class nodes dynamic.
 *
 * <p><b>History:</b><ul>
 * <li>06 Jun 2001 - Created file.
 * </ul>
 *********************************************************************/
public class TreeClassObjects {
  /***********************************************************************
   * Whether debug information is to be printed to the console.
   * default is not have debug information printed (false).
   **********************************************************************/
  static private boolean debug_ = false;

  /***********************************************************************
   **********************************************************************/
  static final public int XMTO2XMTA_TREE_XMTO = 0;
  static final public int XMTO2XMTA_TREE_animateColor = 1;
  static final public int XMTO2XMTA_TREE_animateMotion = 2;
  static final public int XMTO2XMTA_TREE_audio = 3;
  static final public int XMTO2XMTA_TREE_backdrop = 4;
  static final public int XMTO2XMTA_TREE_body = 5;
  static final public int XMTO2XMTA_TREE_circle = 6;
  static final public int XMTO2XMTA_TREE_defs = 7;
  static final public int XMTO2XMTA_TREE_fontStyle = 8;
  static final public int XMTO2XMTA_TREE_group = 9;
  static final public int XMTO2XMTA_TREE_head = 10;
  static final public int XMTO2XMTA_TREE_img = 11;
  static final public int XMTO2XMTA_TREE_layout = 12;
  static final public int XMTO2XMTA_TREE_material = 13;
  static final public int XMTO2XMTA_TREE_par = 14;
  static final public int XMTO2XMTA_TREE_rectangle = 15;
  static final public int XMTO2XMTA_TREE_region = 16;
  static final public int XMTO2XMTA_TREE_seq = 17;
  static final public int XMTO2XMTA_TREE_set = 18;
  static final public int XMTO2XMTA_TREE_string = 19;
  static final public int XMTO2XMTA_TREE_topLayout = 20;
  static final public int XMTO2XMTA_TREE_transformation = 21;
  static final public int XMTO2XMTA_TREE_use = 22;
  static final public int XMTO2XMTA_TREE_video = 23;
  static final public int XMTO2XMTA_TREE_xmtaMedia = 24;
  static final public int XMTO2XMTA_TREE_texture = 25;
  static private int CLASS_COUNT = 26;

  /***********************************************************************
   **********************************************************************/
  static final private String[] classNames_ = {
  /* 00 */ "xmto2xmta.tree.XMTO",
  /* 01 */ "xmto2xmta.tree.animateColor",
  /* 02 */ "xmto2xmta.tree.animateMotion",
  /* 03 */ "xmto2xmta.tree.audio",
  /* 04 */ "xmto2xmta.tree.backdrop",
  /* 05 */ "xmto2xmta.tree.body",
  /* 06 */ "xmto2xmta.tree.circle",
  /* 07 */ "xmto2xmta.tree.defs",
  /* 08 */ "xmto2xmta.tree.fontStyle",
  /* 09 */ "xmto2xmta.tree.group",
  /* 10 */ "xmto2xmta.tree.head",
  /* 11 */ "xmto2xmta.tree.img",
  /* 12 */ "xmto2xmta.tree.layout",
  /* 13 */ "xmto2xmta.tree.material",
  /* 14 */ "xmto2xmta.tree.par",
  /* 15 */ "xmto2xmta.tree.rectangle",
  /* 16 */ "xmto2xmta.tree.region",
  /* 17 */ "xmto2xmta.tree.seq",
  /* 18 */ "xmto2xmta.tree.set",
  /* 19 */ "xmto2xmta.tree.string",
  /* 20 */ "xmto2xmta.tree.topLayout",
  /* 21 */ "xmto2xmta.tree.transformation",
  /* 22 */ "xmto2xmta.tree.use",
  /* 23 */ "xmto2xmta.tree.video",
  /* 24 */ "xmto2xmta.tree.xmtaMedia",
  /* 25 */ "xmto2xmta.tree.texture",
  /*   */};

  /***********************************************************************
   **********************************************************************/
  static private boolean[] classLoaded_ = null;

  /***********************************************************************
   **********************************************************************/
  static private Class[] classObjects_ = null;

  /***********************************************************************
   **********************************************************************/
  static final public int XMTO2XMTA_TREE_string_getXmtaTextId = 0;
  static final public int XMTO2XMTA_TREE_material_getXmtaMaterial2dId = 1;
  static final public int XMTO2XMTA_TREE_backdrop_getXmtoAttrSrc = 2;
  static final public int XMTO2XMTA_TREE_xmtaMedia_getXmtaAttrHasOD = 3;
  static private int METHOD_COUNT = 4;


  /***********************************************************************
   **********************************************************************/
  static private String[] methodNames_ = {
  /* 00 */ "getXmtaTextId",
  /* 01 */ "getXmtaMaterial2dId",
  /* 02 */ "getXmtaAttrSrc",
  /* 03 */ "getXmtaAttrHasOD",
  /*    */};

  /***********************************************************************
   **********************************************************************/
  static private String[][] methodArgs_ = {
  /* 00 */ {"xmto2xmta.timegraph.ELASid",},
  /* 01 */ {"xmto2xmta.timegraph.ELASid",},
  /* 02 */ {},
  /* 03 */ {},
  /*    */};

  /***********************************************************************
   **********************************************************************/
  static private int[] methodClasses_ = {
  /* 00 */ XMTO2XMTA_TREE_string,
  /* 01 */ XMTO2XMTA_TREE_material,
  /* 02 */ XMTO2XMTA_TREE_backdrop,
  /* 03 */ XMTO2XMTA_TREE_xmtaMedia,
  /*    */};

  /***********************************************************************
   **********************************************************************/
  static private boolean[] methodLoaded_ = null;

  /***********************************************************************
   **********************************************************************/
  static private Method[] methodObjects_ = null;

  /***********************************************************************
   **********************************************************************/
  static private boolean initClasses_ = false;
/***********************************************************************
 **********************************************************************/
static public Class getClassObject(int classIndex) {
  //
  if (!initClasses_)
    initClasses();

  //
  if (classIndex < 0 || classIndex >= CLASS_COUNT)
    return null;

  //
  if (!classLoaded_[classIndex]) {
    classLoaded_[classIndex] = true;

    //
    try {
      classObjects_[classIndex] = Class.forName(classNames_[classIndex]);
    } catch (Exception e) {
    }
  }

  // return the class object
  return classObjects_[classIndex];
}
/***********************************************************************
 **********************************************************************/
static public Method getMethodObject(int methodIndex) {
  //
  if (!initClasses_)
    initClasses();

  //
  if (methodIndex < 0 || methodIndex >= METHOD_COUNT)
    return null;

  //
  if (!methodLoaded_[methodIndex]) {
    methodLoaded_[methodIndex] = true;

    //
    try {
      Class classObject = getClassObject(methodClasses_[methodIndex]);
      Class args[] = null;
      if (0 != methodArgs_[methodIndex].length) {
        args = new Class[methodArgs_[methodIndex].length];
        for (int i = 0; i < args.length; i++) {
          args[i] = Class.forName(methodArgs_[methodIndex][i]);
        }
      }
      methodObjects_[methodIndex] = classObject.getDeclaredMethod(methodNames_[methodIndex], args);
    } catch (Exception e) {
    }
  }

  // return the class object
  return methodObjects_[methodIndex];
}
/***********************************************************************
 **********************************************************************/
static private void initClasses() {
  //
  if (initClasses_)
    return;

  //  Initialize Classes
  initClasses_ = true;
  if (classNames_.length != CLASS_COUNT) {
    System.out.println("ClassObjects.initClasses() - Internal ERROR class count incorrect");
    CLASS_COUNT = classNames_.length;
  }
  //
  classLoaded_ = new boolean[CLASS_COUNT];
  classObjects_ = new Class[CLASS_COUNT];
  // initialize
  for (int i = 0; i < CLASS_COUNT; i++) {
    classLoaded_[i] = false;
    classObjects_[i] = null;
  }

  // Initialize Methods
  METHOD_COUNT = methodNames_.length;
  if (methodArgs_.length != METHOD_COUNT || methodClasses_.length != METHOD_COUNT) {
    if (methodArgs_.length < METHOD_COUNT)
      METHOD_COUNT = methodArgs_.length;
    if (methodClasses_.length < METHOD_COUNT)
      METHOD_COUNT = methodClasses_.length;
    System.out.println("ClassObjects.initClasses() - Internal ERROR method count incorrect");
  }
  methodLoaded_ = new boolean[METHOD_COUNT];
  methodObjects_ = new Method[METHOD_COUNT];
  // initialize
  for (int i = 0; i < METHOD_COUNT; i++) {
    methodLoaded_[i] = false;
    methodObjects_[i] = null;
  }
}
/***********************************************************************
 * Enable debugging information to be printed to the console.
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
  debug_ = debug;
}
}
