/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  TreeElement.java
//
//=============================================================================
package xmto2xmta.tree;

import java.util.Enumeration;
import java.util.Vector;
/*********************************************************************** 
 * Class which describes the basic tree attributes for convertion elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 JSB - Created file.
 * </ul>
 *********************************************************************/
public abstract class TreeElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private Vector children = new Vector();

	/*********************************************************************** 
	 **********************************************************************/
	private TreeElement parent_ = null;
/*********************************************************************** 
 **********************************************************************/
public void addChild(TreeElement child) {
	children.add(child);
	child.setParent(this);
}
/***********************************************************************
 * Get a clone of the vector of children.  Return a clone because the
 * children may be removed at anytime and an Enumeration of the children
 * is not kept up to date.
 **********************************************************************/
public Vector getChildren() {
	return (Vector) children.clone();
}
/*********************************************************************** 
 **********************************************************************/
public TreeElement getParent() {
	return parent_;
}
/*********************************************************************** 
 **********************************************************************/
public void removeChild(TreeElement child) {
	children.remove(child);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/*********************************************************************** 
 **********************************************************************/
public void setParent(TreeElement parent) {
	parent_ = parent;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	StringBuffer sb = new StringBuffer();
	for (Enumeration enumeration = children.elements(); enumeration.hasMoreElements();) {
		sb.append(enumeration.nextElement().toString());
	}
	return sb.toString();
}
}
