/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  ZorderSupport.java
//
//=============================================================================
package xmto2xmta.tree;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.resources.GroupElementRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to group together the various type elements who group together other elements.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * </ul><p>
 *********************************************************************/
public interface ZorderSupport {
/*********************************************************************** 
 * Determine the field position at time of deletion for the media/group element
 * in the group.
 *
 * @param  elasId id to specify which simple duration of a child element.
 * @return the field position at time of deletion for the media/group element.
 **********************************************************************/
public abstract int getDeleteFieldPosition(ELASid elasId) ;
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 *
 * @param  elasId id to specify which simple duration of a child element.
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public abstract String getXmtaOrderedGroupId(ELASid elasId);
/*********************************************************************** 
 * Return whether the z-ordering positioning is used for the children 
 * or not.
 * 
 * @return whether z-ordering is used for this GroupElement or not.
 **********************************************************************/
public abstract boolean usePositions();
}
