/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  animateColor.java
//
//=============================================================================
package xmto2xmta.tree;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.ErrorEvent;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.AnimateColorRes;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to handle the conversion of the animationMotion element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>19 Apr 2001 - Add simple lifecycle support.  
 * <li>30 Jun 2001 - Add interpolator support
 * <li>30 Jun 2001 - Fix bug, on validating syntax if problem with parent then
 *                   return directly the problem.
 * </ul>                                                                         
 *********************************************************************/
public class animateColor extends animate {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle animateColorRes_ = ResourceBundle.getBundle("xmto2xmta.resources.AnimateColorRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public animateColor(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<animateColor>" + super.toString() + "</animateColor>";
}
/*********************************************************************** 
 * Validate the syntax of the element.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.
 * Mainly check the begin, dur and end values to see they are valid timing
 * attributes.  If this is a problem then remove this node from the tree.
 **********************************************************************/
public boolean validateSyntax() {
	boolean success = super.validateSyntax();
	if (!success)
		return success;

	// The only distinction between animateColor and animate is that animateColor only works on
	// animating the color of an object.  Thus we need to make sure we are changing the color.
	// Then let the animate take care of the rest.
	// The attribute we want to change must be of a type SFColor
	String xmtoAttrAttributeName = getXmtoAttrAttributeName();
	Animatable targetObject = getXmtoAttrTargetObject();
	// Don't need to worry about error message if it was in error.  It would have been caught higher up in the heirarchy.
	if (null != xmtoAttrAttributeName && null != targetObject) {
		int fieldType = targetObject.getAnimXmtaAttrFieldType(xmtoAttrAttributeName);
		if (fieldType != Animatable.TYPE_SFColor) {
			MessageFormat msgFormat = new MessageFormat(animateColorRes_.getString(AnimateColorRes.MSG_ERROR_ANIMATE_ATTRIBUTE_NOT_A_COLOR));
			StringBuffer buffer = new StringBuffer();
			buffer.append(getXmtoElement().getNodeName());
			if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
				buffer.append(" ").append(DOMXMTO.id).append("=\"").append(getXmtoAttrId()).append("\"");
			StringBuffer attrBuffer = new StringBuffer();
			attrBuffer.append(DOMXMTO.attributeName).append("=\"").append(xmtoAttrAttributeName).append("\"");
			Object[] msgArgs = {buffer, attrBuffer};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), DOMXMTO.attributeName);
			remove();
			return false;
		}
	}

	// 
	return success;
}
}
