/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  backdrop.java
//
//=============================================================================
package xmto2xmta.tree;

import java.awt.Color;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to handle the conversion of the body element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>16 Apr 2000 - Change resolve timings to use TimeScheduler.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>01 May 2001 - Changed too much of the old stuff, video descriptor.
 * <li>12 Jun 2001 - Add support for defs id.
 * </ul>
 *********************************************************************/
public class backdrop extends AugmentationElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	static protected final String ODID_URL_PREFIX = "od://";
	//static private final String FILE_URL_PREFIX = "file:///";
	static private final String FILE_URL_PREFIX = "";

	/*********************************************************************** 
	 **********************************************************************/
	private Element parInsert_ = null;
	private Element parUpdate_ = null;
	private Element parRemove_ = null;
	private Element parDelete_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private Element xmtaParNodes_ = null;
	private Element xmtaParOds_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtaAttrColor_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrSrc_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private Hashtable xmtaBackground2dIds_ = new Hashtable(89);
	private Hashtable xmtaEsDescriptorIds_ = new Hashtable(89);
	private Hashtable xmtaObjectDescriptorIds_ = new Hashtable(89);
	
	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public backdrop(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds,Hashtable convertIndexTable,  Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrColor() {
	return xmtaAttrColor_;
}
/*********************************************************************** 
 * Get the id for the MovieTexture element.
 **********************************************************************/
public String getXmtaBackground2dId(ELASid elasId) {
	String xmtaBackground2dId = (String) xmtaBackground2dIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaBackground2dId) {
		xmtaBackground2dId = IdManager.getNextId(IdManager.Background2dID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaBackground2dId = xmtaBackground2dId + ":" + getXmtoAttrId();
		xmtaBackground2dIds_.put(elasId, xmtaBackground2dId);
	}

	//
	return xmtaBackground2dId;
}
/***********************************************************************
 * Get the defs id of the object.
 * @return the unique id of this object.
 **********************************************************************/
public String getXmtaDefsId() {
	if (!isInDefsTree()) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "InternalElement.getXmtaDefsId() - is being called on a none defs object", getXmtoElement());

		// This should never be called but just in case return empty string.
		return "";
	}

	//
	TimeScheduler timeScheduler = getXmto2Xmta().getTimeScheduler();
	if (null == timeScheduler)
		return "";
	Vector lifeCycles = timeScheduler.getLifeCycles(getIndex());
	if (null == lifeCycles)
		return "";
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.firstElement();
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	if (null == activeIntervals)
		return "";
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.firstElement();
	Vector elasIds = activeInterval.getELASids();
	if (null == elasIds)
		return "";
	ELASid elasId = (ELASid) elasIds.firstElement();
	String xmtaDefsId = getXmtaBackground2dId(elasId);
	return xmtaDefsId;
}
/*********************************************************************** 
 * Get the id for the MovieTexture element.
 **********************************************************************/
public String getXmtaEsDescriptorId(ELASid elasId) {
	String xmtaEsDescriptorId = (String) xmtaEsDescriptorIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaEsDescriptorId) {
		xmtaEsDescriptorId = IdManager.getNextId(IdManager.ESID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaEsDescriptorId = xmtaEsDescriptorId + ":" + getXmtoAttrId();
		xmtaEsDescriptorIds_.put(elasId, xmtaEsDescriptorId);
	}

	//
	return xmtaEsDescriptorId;
}
/*********************************************************************** 
 * Get the id for the MovieTexture element.
 **********************************************************************/
public String getXmtaObjectDescriptorId(ELASid elasId) {
	String xmtaObjectDescriptorId = (String) xmtaObjectDescriptorIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaObjectDescriptorId) {
		xmtaObjectDescriptorId = IdManager.getNextId(IdManager.ODID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaObjectDescriptorId = xmtaObjectDescriptorId + ":" + getXmtoAttrId();
		xmtaObjectDescriptorIds_.put(elasId, xmtaObjectDescriptorId);
	}

	//
	return xmtaObjectDescriptorId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrSrc() {
	return xmtoAttrSrc_;
}
/*********************************************************************** 
 * Return the canonical path of the file that the src attribute refers to.
 * The file is relative to the document path.
 **********************************************************************/
public String getXmtoAttrSrcCanonical() {
	try {
		return new java.io.File(xmtutil.DOMUtil.getDirectory(getXmto2Xmta().getXmtODocPath()), getXmtoAttrSrc()).getCanonicalPath();
	} catch (Exception e) {
		return getXmtoAttrSrc();
	}
}

/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// Determine the src
	if (DOMXMTO.src.equals(attributeName)) {
		xmtoAttrSrc_ = attributeValue;
		return true;
	}

	// Determine the color field
	if (DOMXMTO.color.equals(attributeName)) {
		Color color = DOMXMTOParameter.param2Color(attributeValue);
		if (null != color)
			xmtaAttrColor_ = DOMXMTAParameter.color2Param(color);
		else {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	//
	return success;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<backdrop>" + super.toString() + "</backdrop>";
}
}
