/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  defs.java
//
//=============================================================================
package xmto2xmta.tree;

import java.util.Hashtable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.timegraph.ELASid;

/*********************************************************************** 
 * Class to handle the conversion of the XMT-O element.  The class
 * name is XMTO instead of XMT-O since java doesn't allow a dash character 
 * in the class name.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>06 Jun 2001 - Add group support for child elements.
 * </ul>
 *********************************************************************/
public class defs extends StructureElement implements ZorderSupport {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public defs(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * Determine the field position at time of deletion for the media/group element
 * in the group.
 *
 * @param  elasId id to specify which simple duration of a child element.
 * @return 0 since the defs doesn't support z-order positioning.
 **********************************************************************/
public int getDeleteFieldPosition(ELASid elasId) {
	return 0;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 * All children go it the same ordered group id.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaOrderedGroupId(ELASid elasId) {

	// Find the XMTO root node
	TreeElement parent = getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			return ((XMTO) parent).getDefsGroupId();
		}
		parent = parent.getParent();
	}

	// This should never happen
	return "";
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<defs>" + super.toString() + "</defs>";
}
/*********************************************************************** 
 * Return whether the z-ordering positioning is used for the children 
 * or not.  The Defs section does not support z-order positioning.
 * 
 * @return false since the defs does not support z-ordering.
 **********************************************************************/
public boolean usePositions() {
	return false;
}
}
