/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  fontStyle.java
//
//=============================================================================
package xmto2xmta.tree;

import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTO;
/*********************************************************************** 
 * Class to handle the conversion of the material element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>06 Jun 2001 - Add defs support.
 * </ul>
 *********************************************************************/
public class fontStyle extends XmediaChildrenDefsElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrFamily_ = null;
	private String xmtoAttrHorizontal_ = null;
	private String xmtoAttrJustify_ = null;
	private String xmtoAttrLanguage_ = null;
	private String xmtoAttrLeftToRight_ = null;
	private String xmtoAttrSize_ = null;
	private String xmtoAttrSpacing_ = null;
	private String xmtoAttrStyle_ = null;
	private String xmtoAttrTopToBottom_ = null;

	/***********************************************************************
	 **********************************************************************/
	Hashtable xmtaFontStyleIds_ = new Hashtable(89);

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public fontStyle(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds,Hashtable convertIndexTable,  Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/***********************************************************************
 * Get the defs id of the object.
 * @return the unique id of this object.
 **********************************************************************/
protected String getXmtaDefsId(ELASid elasId) {
	return getXmtaFontStyleId(elasId);
}
/***********************************************************************
 * get the id for the FontStyle element.
 **********************************************************************/
public String getXmtaFontStyleId(ELASid elasId) {
	String xmtaFontStyleId = (String) xmtaFontStyleIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaFontStyleId) {
		xmtaFontStyleId = IdManager.getNextId(IdManager.FontStyleID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaFontStyleId = xmtaFontStyleId + ":" + getXmtoAttrId();
		xmtaFontStyleIds_.put(elasId, xmtaFontStyleId);
	}

	//
	return xmtaFontStyleId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrFamily() {
	return xmtoAttrFamily_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrHorizontal() {
	return xmtoAttrHorizontal_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrJustify() {
	return xmtoAttrJustify_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrLanguage() {
	return xmtoAttrLanguage_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrLeftToRight() {
	return xmtoAttrLeftToRight_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrSize() {
	return xmtoAttrSize_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrSpacing() {
	return xmtoAttrSpacing_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrStyle() {
	return xmtoAttrStyle_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrTopToBottom() {
	return xmtoAttrTopToBottom_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// Determine the family field
	if (DOMXMTO.family.equals(attributeName)) {
		xmtoAttrFamily_ = attributeValue;
		return true;
	}

	// Determine the horizontal field
	if (DOMXMTO.horizontal.equals(attributeName)) {
		xmtoAttrHorizontal_ = attributeValue;
		return true;
	}

	// Determine the justify field
	if (DOMXMTO.justify.equals(attributeName)) {
		xmtoAttrJustify_ = attributeValue;
		return true;
	}

	// Determine the language field
	if (DOMXMTO.language.equals(attributeName)) {
		xmtoAttrLanguage_ = attributeValue;
		return true;
	}

	// Determine the leftToRight field
	if (DOMXMTO.leftToRight.equals(attributeName)) {
		xmtoAttrLeftToRight_ = attributeValue;
		return true;
	}

	// Determine the size field
	if (DOMXMTO.size.equals(attributeName)) {
		xmtoAttrSize_ = attributeValue;
		return true;
	}

	// Determine the spacing field
	if (DOMXMTO.spacing.equals(attributeName)) {
		xmtoAttrSpacing_ = attributeValue;
		return true;
	}

	// Determine the style field
	if (DOMXMTO.style.equals(attributeName)) {
		xmtoAttrStyle_ = attributeValue;
		return true;
	}

	// Determine the topToBottom field
	if (DOMXMTO.topToBottom.equals(attributeName)) {
		xmtoAttrTopToBottom_ = attributeValue;
		return true;
	}

	//
	return success;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<FontStyle>" + super.toString() + "</FontStyle>";
}
}
