/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  group.java
//
//=============================================================================
package xmto2xmta.tree;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to handle the conversion of the body element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>19 Apr 2001 - Add simple lifecycle support.  
 * <li>06 Jun 2001 - Add defs support.
 * <li>06 Jun 2001 - Add support (ParContainable) support to distinguish
 *                   that this classes contains timing elements in a 
 *                   par manner.
 * <li>12 Jun 2001 - Add support for defs id.
 * <li>21 Jun 2001 - Add sizing support.
 * <li>30 Jun 2001 - Add animation support.
 * </ul>
 *********************************************************************/
public class group extends GroupElement implements Sensorable, Animatable, DefsObject, ParContainable {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrOrder_ = null;
	private String xmtoAttrRegion_ = null;
	private String xmtoAttrSize_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtaAttrWhichChoice_ = null;
	private String xmtaAttrCenter_ = null;
	private String xmtaAttrScale_ = null;
	private String xmtaAttrScaleOrientation_ = null;
	private String xmtaAttrTranslation_ = null;
	private String xmtaAttrRotation_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private Hashtable xmtaGroupIds_ = new Hashtable(89);
	private Hashtable xmtaOrderedGroupIds_ = new Hashtable(89);
	private Hashtable xmtaTouchSensorIds_ = new Hashtable(89);
	private Hashtable xmtaSwitchIds_ = new Hashtable(89);
	private Hashtable xmtaTransform2dIds_ = new Hashtable(89);
	private Hashtable xmtaLayer2dIds_ = new Hashtable(89);

	/*********************************************************************** 
	 **********************************************************************/
	static private Class headClass_ = null;
	static private boolean classesLoaded_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private boolean isInDefsTreeInitialized_ = false;
	private boolean isInDefsTree_ = false;
	private boolean isDefsParent_ = false;

	/***********************************************************************
	 * NOTE: order has to be handled differently.
	 * They are not just a field replacement.
	 **********************************************************************/
	private String animXmtoAttrName_[] = {
	/* 00 */ DOMXMTO.center,
	/* 01 */ DOMXMTO.scale,
	/* 02 */ DOMXMTO.translation,
	/* 03 */ DOMXMTO.scaleOrientation,
	/* 04 */ DOMXMTO.rotation,
	/* 05 */ DOMXMTO.visibility,
	/* 06 */ DOMXMTO.size,
	/*    */};
	private int animXmtoAttrFieldType_[] = {
	/* 00 */ Animatable.TYPE_SFVec2f,
	/* 01 */ Animatable.TYPE_SFVec2f,
	/* 02 */ Animatable.TYPE_SFVec2f,
	/* 03 */ Animatable.TYPE_SFFloat,
	/* 04 */ Animatable.TYPE_SFFloat,
	/* 05 */ Animatable.TYPE_SFBool,
	/* 06 */ Animatable.TYPE_SFVec2f,
	/*    */};
	private String animXmtaAttrName_[] = {
	/* 00 */ DOMXMTA.center,
	/* 01 */ DOMXMTA.scale,
	/* 02 */ DOMXMTA.translation,
	/* 03 */ DOMXMTA.scaleOrientation,
	/* 04 */ DOMXMTA.rotationAngle,
	/* 05 */ DOMXMTA.whichChoice,
	/* 06 */ DOMXMTA.size,
	/*    */};
	private int animXmtaAttrFieldType_[] = {
	/* 00 */ Animatable.TYPE_SFVec2f,
	/* 01 */ Animatable.TYPE_SFVec2f,
	/* 02 */ Animatable.TYPE_SFVec2f,
	/* 03 */ Animatable.TYPE_SFFloat,
	/* 04 */ Animatable.TYPE_SFFloat,
	/* 05 */ Animatable.TYPE_SFInt32,
	/* 06 */ Animatable.TYPE_SFVec2f,
	/*    */};
	private String animXmtaAttrFieldDefaultValue_[] = {
	/* 00 */ "0.0 0.0",
	/* 01 */ "1.0 1.0",
	/* 02 */ "0.0 0.0",
	/* 03 */ "0.0",
	/* 04 */ "0.0",
	/* 05 */ "0",
	/* 06 */ "-1.0 -1.0",
	/*    */};
	
	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public group(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * This function converts a xmto attribute value to its xmta equivalent.
 * It then returns the converted value.  
 **********************************************************************/
public String getAnimConvertValueXmto2Xmta(String xmtoAttrName, String xmtoValue, int xmtoFieldType, int xmtaFieldType) {

	//
	if (null == xmtoValue)
		return "";

	//
	if (DOMXMTO.visibility.equals(xmtoAttrName)) {
		if (Animatable.TYPE_SFBool == xmtoFieldType && Animatable.TYPE_SFInt32 == xmtaFieldType) {
			if (xmtoValue.equals(DOMXMTO.string_true))
				return "0";
			else
				if (xmtoValue.equals(DOMXMTO.string_false))
					return "-1";
				else {
					// use the default, but still send the error message.
					return "0";
				}
		}
		if (Animatable.TYPE_MFBool == xmtoFieldType && Animatable.TYPE_MFInt32 == xmtaFieldType) {
			Boolean[] mfBool = DOMXMTOParameter.param2MFBool(xmtoValue);
			StringBuffer xmtaValue = new StringBuffer();
			for (int i = 0, icount = mfBool.length; i < icount; i++) {
				if (0 != i)
					xmtaValue.append("; ");
				xmtaValue.append(mfBool[i].booleanValue() ? "0" : "-1");
			}
			return xmtaValue.toString();
		}
	}

	//
	if (xmtoFieldType == xmtaFieldType)
		return xmtoValue;
	else
		return "";
}
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public int getAnimXmtaAttrFieldType(String xmtoAttrName) {
	// Loop through the attributes looking for the field type
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrFieldType_[i];
	}

	// can't find value so return null;
	return Animatable.TYPE_UNSUPPORTED;
}
/*********************************************************************** 
 * This function returns the initial value of the field begin animated.  This
 * is used in case the attribute is not specified in the animaiton.  Generally
 * this is the original value so that after the animation is finished
 * the node field can be returned to its original state.  If this value
 * isn't specified then the default value will be used.
 **********************************************************************/
public String getAnimXmtaAttrInitialValue(String xmtoAttrName) {
	// Return the correct attribute	
	if (DOMXMTO.visibility.equals(xmtoAttrName) && null != xmtaAttrWhichChoice_)
		return xmtaAttrWhichChoice_;
	if (DOMXMTO.center.equals(xmtoAttrName) && null != xmtaAttrCenter_)
		return xmtaAttrCenter_;
	if (DOMXMTO.scale.equals(xmtoAttrName) && null != xmtaAttrScale_)
		return xmtaAttrScale_;
	if (DOMXMTO.scaleOrientation.equals(xmtoAttrName) && null != xmtaAttrScaleOrientation_)
		return xmtaAttrScaleOrientation_;
	if (DOMXMTO.translation.equals(xmtoAttrName) && null != xmtaAttrTranslation_)
		return xmtaAttrTranslation_;
	if (DOMXMTO.rotation.equals(xmtoAttrName) && null != xmtaAttrRotation_)
		return xmtaAttrRotation_;
	if (DOMXMTO.size.equals(xmtoAttrName) && null != xmtoAttrSize_)
		return xmtoAttrSize_;

	// This value isn't filled in so we need to return the default value
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrFieldDefaultValue_[i];
	}

	// can't find value so return null;
	return null;
}
/***********************************************************************
 * This function returns the MPEG-4 field being animated. 
 **********************************************************************/
public String getAnimXmtaAttrName(String xmtoAttrName) {
	// Loop through the attributes looking for the field
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrName_[i];
	}

	// can't find value so return null;
	return null;
}
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated.
 * <p>
 * Normally: This method is for when there is no elasId for this instantiation of the
 * object.  This can occur for xMediaChildrenObject that are defs objects,
 * but the parent isn't a TimingElement.
 * <p>
 * In this case where we are a subclass of TimingElement it is not possible 
 * not have an elasId since we are a timing element.  
 * <ul>
 **********************************************************************/
public String getAnimXmtaDefsNodeId(String attributeName) {
	return "";
}
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated. 
 **********************************************************************/
public String getAnimXmtaNodeId(String xmtoAttrName, ELASid elasId) {
	// The attributes center, scale, scaleOrientation, translation and rotation 
	// all go to the transform2D node of the parent.
	if (DOMXMTO.center.equals(xmtoAttrName) || DOMXMTO.scale.equals(xmtoAttrName) || DOMXMTO.scaleOrientation.equals(xmtoAttrName) || DOMXMTO.translation.equals(xmtoAttrName) || DOMXMTO.rotation.equals(xmtoAttrName)) {
		return getXmtaTransform2dId(elasId);
	}

	// Visibility goes to the switch
	if (DOMXMTO.visibility.equals(xmtoAttrName)) {
		return getXmtaSwitchId(elasId);
	}


	// size goes to the layer2D
	if (DOMXMTO.size.equals(xmtoAttrName)) {
		return getXmtaLayer2dId(elasId);
	}

	// NOTE: order has to be handled differently.
	// They are not just a field replacement.
	return "";
}
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public int getAnimXmtoAttrFieldType(String xmtoAttrName) {
	// Loop through the attributes looking for the field type
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtoAttrFieldType_[i];
	}

	// can't find value so return null;
	return Animatable.TYPE_UNSUPPORTED;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrCenter() {
	return xmtaAttrCenter_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrRotation() {
	return xmtaAttrRotation_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrScale() {
	return xmtaAttrScale_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrScaleOrientation() {
	return xmtaAttrScaleOrientation_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrTranslation() {
	return xmtaAttrTranslation_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrWhichChoice() {
	return xmtaAttrWhichChoice_;
}
/***********************************************************************
 * Get the defs id of the object.
 * @return the unique id of this object.
 **********************************************************************/
public String getXmtaDefsId() {
	if (!isInDefsTree()) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "InternalElement.getXmtaDefsId() - is being called on a none defs object", getXmtoElement());

		// This should never be called but just in case return empty string.
		return "";
	}

	//
	TimeScheduler timeScheduler = getXmto2Xmta().getTimeScheduler();
	if (null == timeScheduler)
		return "";
	Vector lifeCycles = timeScheduler.getLifeCycles(getIndex());
	if (null == lifeCycles)
		return "";
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.firstElement();
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	if (null == activeIntervals)
		return "";
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.firstElement();
	Vector elasIds = activeInterval.getELASids();
	if (null == elasIds)
		return "";
	ELASid elasId = (ELASid) elasIds.firstElement();
	String xmtaDefsId = getXmtaGroupId(elasId);
	return xmtaDefsId;
}
/*********************************************************************** 
 * Get the id for the group element.
 **********************************************************************/
public String getXmtaGroupId(ELASid elasId) {
	String xmtaGroupId = (String) xmtaGroupIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaGroupId) {
		xmtaGroupId = IdManager.getNextId(IdManager.GroupID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaGroupId = xmtaGroupId + ":" + getXmtoAttrId();
		xmtaGroupIds_.put(elasId, xmtaGroupId);
	}

	//
	return xmtaGroupId;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaLayer2dId(ELASid elasId) {
	String xmtaLayer2dId = (String) xmtaLayer2dIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaLayer2dId) {
		xmtaLayer2dId = IdManager.getNextId(IdManager.Layer2dID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaLayer2dId = xmtaLayer2dId + getXmtoAttrId();
		xmtaLayer2dIds_.put(elasId, xmtaLayer2dId);
	}

	//
	return xmtaLayer2dId;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaOrderedGroupId(ELASid elasId) {
	String xmtaOrderedGroupId = (String) xmtaOrderedGroupIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaOrderedGroupId) {
		xmtaOrderedGroupId = IdManager.getNextId(IdManager.OrderedGroupID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaOrderedGroupId = xmtaOrderedGroupId + getXmtoAttrId();
		xmtaOrderedGroupIds_.put(elasId, xmtaOrderedGroupId);
	}

	//
	return xmtaOrderedGroupId;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaSwitchId(ELASid elasId) {
	String xmtaSwitchId = (String) xmtaSwitchIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaSwitchId) {
		xmtaSwitchId = IdManager.getNextId(IdManager.SwitchID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaSwitchId = xmtaSwitchId + getXmtoAttrId();
		xmtaSwitchIds_.put(elasId, xmtaSwitchId);
	}

	//
	return xmtaSwitchId;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaTouchSensorId(ELASid elasId) {
	String xmtaTouchSensorId = (String) xmtaTouchSensorIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTouchSensorId) {
		xmtaTouchSensorId = IdManager.getNextId(IdManager.TouchSensorID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaTouchSensorId = xmtaTouchSensorId + getXmtoAttrId();
		xmtaTouchSensorIds_.put(elasId, xmtaTouchSensorId);
	}

	//
	return xmtaTouchSensorId;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaTransform2dId(ELASid elasId) {
	String xmtaTransform2dId = (String) xmtaTransform2dIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTransform2dId) {
		xmtaTransform2dId = IdManager.getNextId(IdManager.Transform2dID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaTransform2dId = xmtaTransform2dId + getXmtoAttrId();
		xmtaTransform2dIds_.put(elasId, xmtaTransform2dId);
	}

	//
	return xmtaTransform2dId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaTranslation() {
	return xmtaAttrTranslation_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrOrder() {
	return xmtoAttrOrder_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRegion() {
	return xmtoAttrRegion_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrSize() {
	return xmtoAttrSize_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
public boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// Determine the visibility.  If visibility is true (default)
	// then set whichChoice field to "0" (first and only element)
	// otherwise use the default of "-1" to have it hidden.
	if (DOMXMTO.visibility.equals(attributeName)) {
		if (null != attributeName && (!(attributeValue.equals("")))) {
			if (attributeValue.equals(DOMXMTO.string_true))
				xmtaAttrWhichChoice_ = "0";
			else
				if (attributeValue.equals(DOMXMTO.string_false))
					xmtaAttrWhichChoice_ = "-1";
				else {
					// use the default, but still send the error message.
					xmtaAttrWhichChoice_ = "0";

					// 
					MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
					Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
					String message = msgFormat.format(msgArgs);
					fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
				}
		}
		return true;
	}

	// order
	if (DOMXMTO.order.equals(attributeName)) {
		Integer order = DOMXMTOParameter.param2Integer(attributeValue);
		if (null != order)
			xmtoAttrOrder_ = order.toString();
		else {
			// 
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// size
	if (DOMXMTO.size.equals(attributeName)) {
		xmtoAttrSize_ = attributeValue;
		return true;
	}

	// center
	if (DOMXMTO.center.equals(attributeName)) {
		Vector floats = DOMXMTOParameter.param2TwoFloats(attributeValue);
		if (null != floats)
			xmtaAttrCenter_ = DOMXMTAParameter.twoFloats2Param(((Float) floats.elementAt(0)).floatValue(), ((Float) floats.elementAt(1)).floatValue());
		else {
			// 
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// scale
	if (DOMXMTO.scale.equals(attributeName)) {
		Vector floats = DOMXMTOParameter.param2TwoFloats(attributeValue);
		if (null != floats)
			xmtaAttrScale_ = DOMXMTAParameter.twoFloats2Param(((Float) floats.elementAt(0)).floatValue(), ((Float) floats.elementAt(1)).floatValue());
		else {
			// 
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// scaleOrientation
	if (DOMXMTO.scaleOrientation.equals(attributeName)) {
		Vector floats = DOMXMTOParameter.param2TwoFloats(attributeValue);
		if (null != floats)
			xmtaAttrScaleOrientation_ = DOMXMTAParameter.float2Param(((Float) floats.elementAt(0)).floatValue());
		else {
			// 
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// translatation
	if (DOMXMTO.translation.equals(attributeName)) {
		Vector floats = DOMXMTOParameter.param2TwoFloats(attributeValue);
		if (null != floats)
			xmtaAttrTranslation_ = DOMXMTAParameter.twoFloats2Param(((Float) floats.elementAt(0)).floatValue(), ((Float) floats.elementAt(1)).floatValue());
		else {
			// 
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// rotation
	if (DOMXMTO.rotation.equals(attributeName)) {
		Vector floats = DOMXMTOParameter.param2TwoFloats(attributeValue);
		if (null != floats)
			xmtaAttrRotation_ = DOMXMTAParameter.float2Param(((Float) floats.elementAt(0)).floatValue());
		else {
			// 
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// region
	if (DOMXMTO.region.equals(attributeName)) {
		if (!classesLoaded_)
			loadClasses();
		boolean regionValid = true;
		// In order to use the region tag the object 
		// - has to be in the body section (!head section)
		// - not be under a group element
		TreeElement parent = getParent();
		while (null != parent) {
			if (parent instanceof group) {
				regionValid = false;
				break;
			} else {
				if (null != headClass_ && headClass_.isInstance(parent)) {
					regionValid = false;
					break;
				}
			}
			parent = parent.getParent();
		}
		if (regionValid)
			xmtoAttrRegion_ = attributeValue;
		else {
			//
			StringBuffer regionString = new StringBuffer();
			regionString.append(attributeName).append("=\"").append(attributeValue).append("\"");
			//
			StringBuffer nameString = new StringBuffer();
			nameString.append(getXmtoElement().getNodeName());
			String id = getXmtoAttrId();
			if (null != id && !id.equals(""))
				nameString.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");

			//
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_WARNING_ATTRIBUTE_NOT_SUPPORTED_IN_LOCATION));
			Object[] msgArgs = {nameString, regionString};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	//
	return success;
}
/***********************************************************************
 * Routine to initialize the defs query flags.  This is added to provide
 * consistency for all the classes that implement the DefsObject interface.
 **********************************************************************/
public void initDefsFlags() {
	if (isInDefsTreeInitialized_)
		return;

	//
	isInDefsTreeInitialized_ = true;
	isInDefsTree_ = false;
	isDefsParent_ = false;

	// First check if parent is defs
	TreeElement parent = getParent();
	if (null != parent && parent instanceof defs) {
		isInDefsTree_ = true;
		isDefsParent_ = true;
		return;
	}


	// Next check if we are in the defs tree	
	while (null != parent) {
		if (parent instanceof defs) {
			isInDefsTree_ = true;
			return;
		}
		parent = parent.getParent();
	}
}
/***********************************************************************
 * Determine whether the object is a child of the defs element or not.  
 * This means that the object is a direct child of the defs element.
 * It isn't a child if it isn't in the defs tree or is a descendent 
 * (starting at grandchild) of the defs element.
 * @return whether the object is a child of the defs element.  True if 
 * it is a child of the defs element, false otherwise.
 **********************************************************************/
public boolean isDefsParent() {
	if (!isInDefsTreeInitialized_)
		initDefsFlags();

	//
	return isDefsParent_;
}
/***********************************************************************
 * Determine whether the object is in the defs tree or not.  This means
 * that the object is defined somewhere in the Defs tree.  It may be a
 * child or descendent of the defs element.
 * @return whether the object is in the defs tree.  True if a descendent
 * of the defs element, false otherwise.
 **********************************************************************/
public boolean isInDefsTree() {
	if (!isInDefsTreeInitialized_)
		initDefsFlags();

	//
	return isInDefsTree_;
}
/*********************************************************************** 
 **********************************************************************/
static private void loadClasses() {
	//
	if (classesLoaded_)
		return;

	//
	classesLoaded_ = true;

	//
	try {
		headClass_ = Class.forName("xmto2xmta.tree.head");
	} catch (Exception e) {
	}
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<group>" + super.toString() + "</group>";
}
}
