/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  outline.java
//
//=============================================================================
package xmto2xmta.tree;

import java.awt.Color;
import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
/*********************************************************************** 
 * Class to handle the conversion of the material element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>06 Jun 2001 - Add defs support.
 * </ul>
 *********************************************************************/
public class outline extends XmediaChildrenDefsElement implements Animatable {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrColor_ = null;
	private String xmtoAttrWidth_ = null;
	private String xmtoAttrStyle_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtaAttrLineColor_ = null;
	private String xmtaAttrLineStyle_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	Hashtable xmtaLinePropertiesIds_ = new Hashtable(89);

	/*********************************************************************** 
	 **********************************************************************/
	private String animXmtoAttrName_[] = {
	/* 00 */ DOMXMTO.color,
	/* 01 */ DOMXMTO.width,
	/* 02 */ DOMXMTO.style,
	/*    */};
	private int animXmtoAttrFieldType_[] = {
	/* 00 */ Animatable.TYPE_SFColor,
	/* 01 */ Animatable.TYPE_SFFloat,
	/* 02 */ Animatable.TYPE_SFInt32,
	/*    */};
	private String animXmtaAttrName_[] = {
	/* 00 */ DOMXMTA.lineColor,
	/* 01 */ DOMXMTA.width,
	/* 02 */ DOMXMTA.lineStyle,
	/*    */};
	private int animXmtaAttrFieldType_[] = {
	/* 00 */ Animatable.TYPE_SFColor,
	/* 01 */ Animatable.TYPE_SFFloat,
	/* 02 */ Animatable.TYPE_SFInt32,
	/*    */};
	private String animXmtaAttrFieldDefaultValue_[] = {
	/* 00 */ "0.0 0.0 0.0",
	/* 01 */ "1.0",
	/* 02 */ "0",
	/*    */};
	
	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public outline(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds,Hashtable convertIndexTable,  Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * This function converts a xmto attribute value to its xmta equivalent.
 * It then returns the converted value.  
 **********************************************************************/
public String getAnimConvertValueXmto2Xmta(String xmtoAttrName, String xmtoValue, int xmtoFieldType, int xmtaFieldType) {
	//
	if (null == xmtoValue)
		return "";

	//
	if (DOMXMTO.style.equals(xmtoAttrName)) {
		if (Animatable.TYPE_SFInt32 == xmtoFieldType && Animatable.TYPE_SFInt32 == xmtaFieldType) {
			Integer style = DOMXMTOParameter.param2OutlineStyle(xmtoValue);
			if (null != style)
				return style.toString();
			else
				return "";
		}
		if (Animatable.TYPE_MFInt32 == xmtoFieldType && Animatable.TYPE_MFInt32 == xmtaFieldType) {
			Integer[] mfInt32 = DOMXMTOParameter.param2ArrayOfOutlineStyles(xmtoValue);
			String xmtaValue = DOMXMTAParameter.integers2Param(mfInt32);
			return xmtaValue;
		}
	}

	//
	if (xmtoFieldType == xmtaFieldType)
		return xmtoValue;
	else
		return "";
}
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public int getAnimXmtaAttrFieldType(String xmtoAttrName) {
	// Loop through the attributes looking for the field type
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrFieldType_[i];
	}

	// can't find value so return null;
	return Animatable.TYPE_UNSUPPORTED;
}
/*********************************************************************** 
 * This function returns the initial value of the field begin animated.  This
 * is used in case the attribute is not specified in the animaiton.  Generally
 * this is the original value so that after the animation is finished
 * the node field can be returned to its original state.  If this value
 * isn't specified then the default value will be used.
 **********************************************************************/
public String getAnimXmtaAttrInitialValue(String xmtoAttrName) {
	// Return the correct attribute	
	if (DOMXMTO.color.equals(xmtoAttrName) && null != xmtaAttrLineColor_)
		return xmtaAttrLineColor_;
	if (DOMXMTO.width.equals(xmtoAttrName) && null != xmtoAttrWidth_)
		return xmtoAttrWidth_;
	if (DOMXMTO.style.equals(xmtoAttrName) && null != xmtaAttrLineStyle_)
		return xmtaAttrLineStyle_;

	// This value isn't filled in so we need to return the default value
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrFieldDefaultValue_[i];
	}

	// can't find value so return null;
	return null;
}
/***********************************************************************
 * This function returns the MPEG-4 field being animated. 
 **********************************************************************/
public String getAnimXmtaAttrName(String xmtoAttrName) {
	// Loop through the attributes looking for the field
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrName_[i];
	}

	// can't find value so return null;
	return null;
}
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated.
 * This method is for when there is no elasId for this instantiation of the
 * object.  This can occur for xMediaChildrenObject that are defs objects,
 * but the parent isn't a TimingElement.
 **********************************************************************/
public String getAnimXmtaDefsNodeId(String xmtoAttrName) {
	// All the attributes go to the material node so we don't
	// need to check each attribute name.  Just return the
	// appropriate material2D id.
	return getXmtaDefsId();
}
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated. 
 **********************************************************************/
public String getAnimXmtaNodeId(String xmtoAttrName, ELASid elasId) {
	// All the attributes go to the line properties node so we don't
	// need to check each attribute name.  Just return the
	// appropriate material2D id.
	return getXmtaLinePropertiesId(elasId);
}
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public int getAnimXmtoAttrFieldType(String xmtoAttrName) {
	// Loop through the attributes looking for the field type
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtoAttrFieldType_[i];
	}

	// can't find value so return null;
	return Animatable.TYPE_UNSUPPORTED;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrLineColor() {
	return xmtaAttrLineColor_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrLineStyle() {
	return xmtaAttrLineStyle_;
}
/***********************************************************************
 * Get the defs id of the object.
 * @return the unique id of this object.
 **********************************************************************/
protected String getXmtaDefsId(ELASid elasId) {
	return getXmtaLinePropertiesId(elasId);
}
/***********************************************************************
 * get the id for the LineProperties element.
 **********************************************************************/
public String getXmtaLinePropertiesId(ELASid elasId) {
	String xmtaLinePropertiesId = (String) xmtaLinePropertiesIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaLinePropertiesId) {
		xmtaLinePropertiesId = IdManager.getNextId(IdManager.LinePropertiesID);
		// make the group ids based on object id or make them unique.
		String id = getXmtoAttrId();
		if (null != id && !id.equals(""))
			xmtaLinePropertiesId = xmtaLinePropertiesId + ":" + id;
		xmtaLinePropertiesIds_.put(elasId, xmtaLinePropertiesId);
	}

	//
	return xmtaLinePropertiesId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrWidth() {
	return xmtoAttrWidth_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// Determine the color field
	if (DOMXMTO.color.equals(attributeName)) {
		Color color = DOMXMTOParameter.param2Color(attributeValue);
		if (null != color) {
			xmtoAttrColor_ = attributeName;
			xmtaAttrLineColor_ = DOMXMTAParameter.color2Param(color);
		} else {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// Determine the width field
	if (DOMXMTO.width.equals(attributeName)) {
		xmtoAttrWidth_ = attributeValue;
		return true;
	}

	// Determine the transparency field
	if (DOMXMTO.style.equals(attributeName)) {
		Integer style = DOMXMTOParameter.param2OutlineStyle(attributeValue);
		if (null != style) {
			xmtoAttrStyle_ = attributeName;
			xmtaAttrLineStyle_ = style.toString();
		} else {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// 
	return success;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<outline>" + super.toString() + "</outline>";
}
}
