/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  region.java
//
//=============================================================================
package xmto2xmta.tree;

import java.lang.reflect.Method;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;

/*********************************************************************** 
 * Class to handle the conversion of the XMT-O element.  The class
 * name is XMTO instead of XMT-O since java doesn't allow a dash character 
 * in the class name.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * </ul>
 *********************************************************************/
public class region extends RegionContainerElement implements ZorderSupport {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrTranslation_ = null;
	private String xmtoAttrSize_ = null;
	private String xmtoAttrZindex_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtaOrderedGroupId_ = null;
	private String xmtaLayoutOrderedGroupId_ = null;
	private String xmtaLayer2dId_ = null;
	private String xmtaTransform2dId_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private Hashtable deletePositions_ = null;
	private boolean positionsDetermined_ = false;
	private boolean usePositions_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	static private Class groupClass_ = null;
	static private Method getXmtoAttrRegionMethodGroup_ = null;
	static private boolean classesLoaded_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public region(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * Create a list of the timing element children of this region node.
 * This includes all media elements and group elements.  If a Time container
 * element is encountered then recurse through the time container element adding
 * its elements.  There can be par elements because the &lt.body&gt. elements is
 * a GroupElement.  The list should be in document order.
 *
 * @return list of the children elements.
 **********************************************************************/
private Vector determineChildrenElements(ConvertElement parent) {
	// empty children list.
	Vector deleteChildrenElements = new Vector();

	// get all the children
	Vector children = parent.getChildren();
	String regionId = getXmtoAttrId();

	// loop through the children adding the appropraiate children to the list
	for (Enumeration enum = children.elements(); enum.hasMoreElements();) {
		Object child = enum.nextElement();
		if (child instanceof MediaElement) {
			String xmtoAttrRegion = ((MediaElement) child).getXmtoAttrRegion();
			// only add it if it is not slated for a region 
			if (null != xmtoAttrRegion && xmtoAttrRegion.equals(regionId))
				deleteChildrenElements.add(child);
		} else
			if (child instanceof TimeContainerElement) {
				// If a Time container element is encountered then recurse through the 
				// time container element adding its elements.  There can be par 
				// elements because the &lt.body&gt. elements is a GroupElement.
				Vector newChildren = determineChildrenElements((ConvertElement) child);
				for (Enumeration newEnum = newChildren.elements(); newEnum.hasMoreElements();)
					deleteChildrenElements.add(newEnum.nextElement());
			} else {
				if (!classesLoaded_)
					loadClasses();
				if (groupClass_.isInstance(child)) {
					try {
						String xmtoAttrRegion = (String) getXmtoAttrRegionMethodGroup_.invoke(child, null);
						// only add it if it is not slated for a region 
						if (null != xmtoAttrRegion || xmtoAttrRegion.equals(regionId))
							deleteChildrenElements.add(child);
					} catch (Exception e) {
					}
				}
			}
			// else we don't do add the element.
	}

	//
	return deleteChildrenElements;
}
/*********************************************************************** 
 * Add a media element into the ordered group.
 *
 * @param mediaElement XMT-A media object to be added into ordered group.
 **********************************************************************/
private void determineDeleteFieldPositions() {
	positionsDetermined_ = true;

	// Get the body element.  We will pass this for getting the elements off the body
	// to see if they are in the region.
	// First get the XMTO element
	XMTO treeXmto = null;
	TreeElement parent = getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			treeXmto = (XMTO) parent;
			break;
		}
		parent = parent.getParent();
	}
	body treeBody = null;
	if (null != treeXmto) {
		Vector children = treeXmto.getChildren();
		for (Enumeration enum = children.elements(); enum.hasMoreElements();) {
			Object obj = enum.nextElement();
			if (obj instanceof body) {
				treeBody = (body) obj;
				break;
			}
		}
	}

	// big problems if we don't have the correct structure
	if (null == treeBody) {
		// Should be able to find the body 
		// If we get here the user has previously ignored error messages.
		// Thus send another error message and continue on.
		fireErrorEvent(ErrorEvent.ERROR, 0, xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_INVALID_DOCUMENT_STRUCTURE), getXmtoElement());
		remove();
		return;
	}


	// First need to create a list of the timing element children of this GroupElement node.
	// This includes all media elements and group elements.
	// the list is in document order.
	Vector children = determineChildrenElements(treeBody);

	//
	if (debug_) {
		StringBuffer elementString = new StringBuffer();
		elementString.append(" Element=<").append(getXmtoElement().getNodeName());
		String id = getXmtoAttrId();
		if (null != id && !id.equals(""))
			elementString.append(" id=").append(id);
		elementString.append(">");
		for (int i = 0; i < children.size(); i++) {
			ConvertElement child = (ConvertElement) children.elementAt(i);
			StringBuffer buffer = new StringBuffer();
			buffer.append("region.determineDeleteFieldPositions() -").append(elementString).append(" child[").append(i).append("]=<");
			buffer.append(child.getXmtoElement().getNodeName());
			if (null != child.getXmtoAttrId() && !child.getXmtoAttrId().equals(""))
				buffer.append(" id=").append(child.getXmtoAttrId());
			buffer.append(">");
			fireErrorEvent(ErrorEvent.TRACE, 0, buffer.toString(), getXmtoElement());
		}
	}

	// Check whether we use the z-order positions or not.
	usePositions_ = GroupElement.determineUsePosition(this, children);
	if (debug_)
		fireErrorEvent(ErrorEvent.TRACE, 0, ("region.determineDeleteFieldPositions() - usePositions=" + usePositions_), getXmtoElement());

	// If we're not using positions then there is no need to continue.
	if (!usePositions_)
		return;

	// Next go through the list of children enumerating all their simple durations
	// This should be a list of all the ELASid ids in the order in which they are added to the
	// list.
	// Next go through the list of simple durations determining their position at deletion time.
	deletePositions_ = GroupElement.determineDeletePositions(this, children);

	//
	return;
}
/*********************************************************************** 
 * Determine the field position at time of deletion for the media/group element
 * in the group.
 *
 * @param  elasId id to specify which simple duration of a child element.
 * @return the field position at time of deletion for the media/group element.
 **********************************************************************/
public int getDeleteFieldPosition(ELASid elasId) {
	if (!positionsDetermined_)
		determineDeleteFieldPositions();

	//
	Object position = deletePositions_.get(elasId);

	//
	if (null == position)
		return 0;
	else
		return ((Integer) position).intValue();
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which region objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaLayer2dId() {
	// If couldn't find one then create it.
	if (null == xmtaLayer2dId_) {
		xmtaLayer2dId_ = IdManager.getNextId(IdManager.Layer2dID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaLayer2dId_ = xmtaLayer2dId_ + ":" + getXmtoAttrId();
	}

	//
	return xmtaLayer2dId_;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which region objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaLayoutOrderedGroupId() {
	// If couldn't find one then create it.
	if (null == xmtaLayoutOrderedGroupId_) {
		xmtaLayoutOrderedGroupId_ = IdManager.getNextId(IdManager.RegionLayoutGroupID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaLayoutOrderedGroupId_ = xmtaLayoutOrderedGroupId_ + ":" + getXmtoAttrId();
	}

	//
	return xmtaLayoutOrderedGroupId_;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaOrderedGroupId() {
	// If couldn't find one then create it.
	if (null == xmtaOrderedGroupId_) {
		xmtaOrderedGroupId_ = IdManager.getNextId(IdManager.RegionOrderedGroupID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaOrderedGroupId_ = xmtaOrderedGroupId_ + ":" + getXmtoAttrId();
	}

	//
	return xmtaOrderedGroupId_;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which media objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaOrderedGroupId(ELASid elasId) {
	return getXmtaOrderedGroupId();
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which region objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaTransform2dId() {
	// If couldn't find one then create it.
	if (null == xmtaTransform2dId_) {
		xmtaTransform2dId_ = IdManager.getNextId(IdManager.Transform2dID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaTransform2dId_ = xmtaTransform2dId_ + ":" + getXmtoAttrId();
	}

	//
	return xmtaTransform2dId_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrSize() {
	//
	return xmtoAttrSize_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrTranslation() {
	//
	return xmtoAttrTranslation_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrZindex() {
	//
	return xmtoAttrZindex_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	//
	if (DOMXMTO.translation.equals(attributeName)) {
		xmtoAttrTranslation_ = attributeValue;
		return true;
	}

	//
	if (DOMXMTO.size.equals(attributeName)) {
		xmtoAttrSize_ = attributeValue;
		return true;
	}

	// 
	if (DOMXMTO.z_dash_index.equals(attributeName)) {
		xmtoAttrZindex_ = attributeValue;
		Integer intval = DOMXMTOParameter.param2Integer(attributeValue);
		if (null == intval) {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	//
	return success;
}
/*********************************************************************** 
 **********************************************************************/
static private void loadClasses() {
	//
	if (classesLoaded_)
		return;

	//
	classesLoaded_ = true;

	//
	try {
		groupClass_ = Class.forName("xmto2xmta.tree.group");
		getXmtoAttrRegionMethodGroup_ = groupClass_.getDeclaredMethod("getXmtoAttrRegion", null);
	} catch (Exception e) {
	}
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<region>" + super.toString() + "</region>";
}
/*********************************************************************** 
 * Return whether the z-ordering positioning is used for the children 
 * or not.
 * 
 * @return whether z-ordering is used for this GroupElement or not.
 **********************************************************************/
public boolean usePositions() {
	if (!positionsDetermined_)
		determineDeleteFieldPositions();

	//
	return usePositions_;
}
}
