/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  string.java
//
//=============================================================================
package xmto2xmta.tree;

import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
/*********************************************************************** 
 * Class to handle the conversion of the rectangle element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>30 Jun 2001 - Add animation support.
 * </ul>
 *********************************************************************/
public class string extends InternalElement implements Animatable {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrTextLines_ = null;

	/***********************************************************************
	 **********************************************************************/
	Hashtable xmtaTextIds_ = new Hashtable(89);

	/*********************************************************************** 
	 **********************************************************************/
	Element text_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private String animXmtoAttrName_[] = {
	/* 00 */ DOMXMTO.textLines,
	/*    */};
	private int animXmtoAttrFieldType_[] = {
	/* 00 */ Animatable.TYPE_MFString,
	/*    */};
	private String animXmtaAttrName_[] = {
	/* 00 */ DOMXMTA.string,
	/*    */};
	private int animXmtaAttrFieldType_[] = {
	/* 00 */ Animatable.TYPE_MFString,
	/*    */};
	private String animXmtaAttrFieldDefaultValue_[] = {
	/* 00 */ "&quot;&quot;",
	/*    */};

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public string(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * This function converts a xmto attribute value to its xmta equivalent.
 * It then returns the converted value.  
 **********************************************************************/
public String getAnimConvertValueXmto2Xmta(String xmtoAttrName, String xmtoValue, int xmtoFieldType, int xmtaFieldType) {
	//
	if (null == xmtoValue)
		return "";


	//
	if (DOMXMTO.textLines.equals(xmtoAttrName)) {
		if (Animatable.TYPE_MFString == xmtoFieldType && Animatable.TYPE_MFString == xmtaFieldType) {
			String [] mfString = DOMXMTOParameter.param2TextLines(xmtoValue);
			String param = DOMXMTAParameter.textLines2Param(mfString);
			return param;
		}
	}


	//
	if (xmtoFieldType == xmtaFieldType)
		return xmtoValue;
	else
		return "";
}
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public int getAnimXmtaAttrFieldType(String xmtoAttrName) {
	// Loop through the attributes looking for the field type
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrFieldType_[i];
	}

	// can't find value so return null;
	return Animatable.TYPE_UNSUPPORTED;
}
/*********************************************************************** 
 * This function returns the initial value of the field begin animated.  This
 * is used in case the attribute is not specified in the animaiton.  Generally
 * this is the original value so that after the animation is finished
 * the node field can be returned to its original state.  If this value
 * isn't specified then the default value will be used.
 **********************************************************************/
public String getAnimXmtaAttrInitialValue(String xmtoAttrName) {
	// Return the correct attribute	
	if (DOMXMTO.textLines.equals(xmtoAttrName) && null != xmtoAttrTextLines_)
		return xmtoAttrTextLines_;

	// This value isn't filled in so we need to return the default value
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrFieldDefaultValue_[i];
	}

	// can't find value so return null;
	return null;
}
/***********************************************************************
 * This function returns the MPEG-4 field being animated. 
 **********************************************************************/
public String getAnimXmtaAttrName(String xmtoAttrName) {
	// Loop through the attributes looking for the field
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrName_[i];
	}

	// can't find value so return null;
	return null;
}
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated.
 * <p>
 * Normally: This method is for when there is no elasId for this instantiation of the
 * object.  This can occur for xMediaChildrenObject that are defs objects,
 * but the parent isn't a TimingElement.
 * <p>
 * In this case where we are a subclass of TimingElement it is not possible 
 * not have an elasId since we are a timing element.  
 * <ul>
 **********************************************************************/
public String getAnimXmtaDefsNodeId(String attributeName) {
	return "";
}
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated. 
 **********************************************************************/
public String getAnimXmtaNodeId(String xmtoAttrName, ELASid elasId) {
	// All the attributes go to the string node so we don't
	// need to check each attribute name.  Just return the
	// appropriate text id.
	return getXmtaTextId(elasId);
}
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public int getAnimXmtoAttrFieldType(String xmtoAttrName) {
	// Loop through the attributes looking for the field type
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtoAttrFieldType_[i];
	}

	// can't find value so return null;
	return Animatable.TYPE_UNSUPPORTED;
}
/*********************************************************************** 
 * Get the id for the string element.
 **********************************************************************/
public String getXmtaTextId(ELASid elasId) {
	String xmtaTextId = (String) xmtaTextIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTextId) {
		xmtaTextId = IdManager.getNextId(IdManager.TextID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaTextId = xmtaTextId + ":" + getXmtoAttrId();
		xmtaTextIds_.put(elasId, xmtaTextId);
	}

	//
	return xmtaTextId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrTextLines() {
	return xmtoAttrTextLines_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// Determine the textLines
	if (DOMXMTO.textLines.equals(attributeName)) {
		xmtoAttrTextLines_ = attributeValue;
		return true;
	} // end node==textLines

	//
	return success;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<string>" + super.toString() + "</string>";
}
}
