/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  texture.java
//
//=============================================================================
package xmto2xmta.tree;

import java.awt.Color;
import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
/*********************************************************************** 
 * Class to handle the conversion of the texture element.
 * The texture element is different than the other XmediaChildrenDefsElements
 * in that it create two different XMT-A elements rather than the standard
 * one element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>06 Jun 2001 - Add defs support.
 * <li>02 Jul 2001 - Add animation support.
 * </ul><p>                                                                         
 *********************************************************************/
public class texture extends XmediaChildrenExternalElement implements DefsObject, Animatable {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	static private final String ODID_URL_PREFIX = "od://";
	//static private final String FILE_URL_PREFIX = "file:///";
	static private final String FILE_URL_PREFIX = "";

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrRepeatS_ = null;
	private String xmtoAttrRepeatT_ = null;
	private String xmtoAttrCenter_ = null;
	private String xmtoAttrRotation_ = null;
	private String xmtoAttrScale_ = null;
	private String xmtoAttrTranslation_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	Hashtable xmtaTextureIds_ = new Hashtable(89);
	Hashtable xmtaTextureTransformIds_ = new Hashtable(89);
	private String xmtaTextureId_ = null;
	private String xmtaTextureTransformId_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private boolean isInDefsTreeInitialized_ = false;
	private boolean isInDefsTree_ = false;
	private boolean isDefsParent_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String animXmtoAttrName_[] = {
	/* 00 */ DOMXMTO.repeatS,
	/* 01 */ DOMXMTO.repeatT,
	/* 02 */ DOMXMTO.center,
	/* 03 */ DOMXMTO.rotation,
	/* 04 */ DOMXMTO.scale,
	/* 05 */ DOMXMTO.translation,
	/*    */};
	private int animXmtoAttrFieldType_[] = {
	/* 00 */ Animatable.TYPE_SFBool,
	/* 01 */ Animatable.TYPE_SFBool,
	/* 02 */ Animatable.TYPE_SFVec2f,
	/* 03 */ Animatable.TYPE_SFFloat,
	/* 04 */ Animatable.TYPE_SFVec2f,
	/* 05 */ Animatable.TYPE_SFVec2f,
	/*    */};
	private String animXmtaAttrName_[] = {
	/* 00 */ DOMXMTO.repeatS,
	/* 01 */ DOMXMTO.repeatT,
	/* 02 */ DOMXMTO.center,
	/* 03 */ DOMXMTO.rotation,
	/* 04 */ DOMXMTO.scale,
	/* 05 */ DOMXMTO.translation,
	/*    */};
	private int animXmtaAttrFieldType_[] = {
	/* 00 */ Animatable.TYPE_SFBool,
	/* 01 */ Animatable.TYPE_SFBool,
	/* 02 */ Animatable.TYPE_SFVec2f,
	/* 03 */ Animatable.TYPE_SFFloat,
	/* 04 */ Animatable.TYPE_SFVec2f,
	/* 05 */ Animatable.TYPE_SFVec2f,
	/*    */};
	private String animXmtaAttrFieldDefaultValue_[] = {
	/* 00 */ "true",
	/* 01 */ "true",
	/* 02 */ "0.0 0.0",
	/* 03 */ "0.0",
	/* 04 */ "1.0 1.0",
	/* 05 */ "0.0 0.0",
	/*    */};

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public texture(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds,Hashtable convertIndexTable,  Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * This function converts a xmto attribute value to its xmta equivalent.
 * It then returns the converted value.  
 **********************************************************************/
public String getAnimConvertValueXmto2Xmta(String xmtoAttrName, String xmtoValue, int xmtoFieldType, int xmtaFieldType) {
	//
	if (null == xmtoValue)
		return "";

	//
	if (xmtoFieldType == xmtaFieldType)
		return xmtoValue;
	else
		return "";
}
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public int getAnimXmtaAttrFieldType(String xmtoAttrName) {
	// Loop through the attributes looking for the field type
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrFieldType_[i];
	}

	// can't find value so return null;
	return Animatable.TYPE_UNSUPPORTED;
}
/*********************************************************************** 
 * This function returns the initial value of the field begin animated.  This
 * is used in case the attribute is not specified in the animaiton.  Generally
 * this is the original value so that after the animation is finished
 * the node field can be returned to its original state.  If this value
 * isn't specified then the default value will be used.
 **********************************************************************/
public String getAnimXmtaAttrInitialValue(String xmtoAttrName) {
	// Return the correct attribute	
	if (DOMXMTO.repeatS.equals(xmtoAttrName) && null != xmtoAttrRepeatS_)
		return xmtoAttrRepeatS_;
	if (DOMXMTO.repeatT.equals(xmtoAttrName) && null != xmtoAttrRepeatT_)
		return xmtoAttrRepeatT_;
	if (DOMXMTO.center.equals(xmtoAttrName) && null != xmtoAttrCenter_)
		return xmtoAttrCenter_;
	if (DOMXMTO.scale.equals(xmtoAttrName) && null != xmtoAttrScale_)
		return xmtoAttrScale_;
	if (DOMXMTO.translation.equals(xmtoAttrName) && null != xmtoAttrTranslation_)
		return xmtoAttrTranslation_;
	if (DOMXMTO.rotation.equals(xmtoAttrName) && null != xmtoAttrRotation_)
		return xmtoAttrRotation_;

	// This value isn't filled in so we need to return the default value
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrFieldDefaultValue_[i];
	}

	// can't find value so return null;
	return null;
}
/***********************************************************************
 * This function returns the MPEG-4 field being animated. 
 **********************************************************************/
public String getAnimXmtaAttrName(String xmtoAttrName) {
	// Loop through the attributes looking for the field
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtaAttrName_[i];
	}

	// can't find value so return null;
	return null;
}
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated.
 * This method is for when there is no elasId for this instantiation of the
 * object.  This can occur for xMediaChildrenObject that are defs objects,
 * but the parent isn't a TimingElement.
 **********************************************************************/
public String getAnimXmtaDefsNodeId(String xmtoAttrName) {
	// The attributes repeatS and repeatT
	// all go to the Texture node. 
	if (DOMXMTO.repeatS.equals(xmtoAttrName) || DOMXMTO.repeatT.equals(xmtoAttrName)) {
		return getXmtaTextureId();
	}

	// The attributes center, rotation, scale and translation
	// all go to the TextureTransform node. 
	if (DOMXMTO.center.equals(xmtoAttrName) || DOMXMTO.scale.equals(xmtoAttrName) || DOMXMTO.translation.equals(xmtoAttrName) || DOMXMTO.rotation.equals(xmtoAttrName)) {
		return getXmtaTextureTransformId();
	}

	// 
	return "";
}
/*********************************************************************** 
 * This function returns the MPEG-4 node whose field is being animated. 
 **********************************************************************/
public String getAnimXmtaNodeId(String xmtoAttrName, ELASid elasId) {
	// The attributes repeatS and repeatT
	// all go to the Texture node. 
	if (DOMXMTO.repeatS.equals(xmtoAttrName) || DOMXMTO.repeatT.equals(xmtoAttrName)) {
		return getXmtaTextureId(elasId);
	}

	// The attributes center, rotation, scale and translation
	// all go to the TextureTransform node. 
	if (DOMXMTO.center.equals(xmtoAttrName) || DOMXMTO.scale.equals(xmtoAttrName) || DOMXMTO.translation.equals(xmtoAttrName) || DOMXMTO.rotation.equals(xmtoAttrName)) {
		return getXmtaTextureTransformId(elasId);
	}

	// 
	return "";
}
/***********************************************************************
 * This function returns the MPEG-4 field type being animated. e.g.
 * SFVec2f, MFVec3f, SFColor, MFColor, ...
 **********************************************************************/
public int getAnimXmtoAttrFieldType(String xmtoAttrName) {
	// Loop through the attributes looking for the field type
	for (int i = 0; i < animXmtoAttrName_.length; i++) {
		if ((animXmtoAttrName_[i]).equals(xmtoAttrName))
			return animXmtoAttrFieldType_[i];
	}

	// can't find value so return null;
	return Animatable.TYPE_UNSUPPORTED;
}
/***********************************************************************
 * Normally this returns the defs id of the object.  For the Texture
 * element there are actually 2 defs ids of the object and thus
 * anyone who would want to query this object must instead get both 
 * the texture and textureTransform ids.
 * @return the empty string.
 **********************************************************************/
public String getXmtaDefsId() {
	return "";
}
/***********************************************************************
 * Get the defs id of the object.
 * @return the unique id of this object.
 **********************************************************************/
public String getXmtaTextureId() {
	if (null != xmtaTextureId_)
		return xmtaTextureId_;

	// Not a valid defs object
	if (!isInDefsTree()) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "texture.getXmtaTextureId() - is being called on a none defs object", getXmtoElement());

		// This should never be called but just in case return empty string.
		return "";
	}

	// Check whether the defs for the FontStyle is a child of the
	// <defs> element or a descendent of a TimingElement.  If it is a child of the
	// defs then it doesn't have a elasid.  If it is a descendent of a timingelement then
	// use the elasId of the timingElement.
	// First check if it is a child of the <defs> element.
	if (isDefsParent()) {
		// If couldn't find one then create it.
		if (null == xmtaTextureId_) {
			xmtaTextureId_ = IdManager.getNextId(IdManager.TextureID);
			// make the group ids based on object id or make them unique.
			String id = getXmtoAttrId();
			if (null != id && !id.equals(""))
				xmtaTextureId_ = xmtaTextureId_ + ":" + id;
		}
		//
		return xmtaTextureId_;
	}

	// See if the element is a descendent of defs but not through a timing element.
	TreeElement parent = getParent();
	while (null != parent && !(parent instanceof TimingElement))
		parent = parent.getParent();

	// This is a defs element under another defs element.  Thus look for its
	// If we couldn't find the parent then this must be a descendent of the defs
	// through other non timing elements, e.g. outline.
	// Treat it the same as a child of the defs.
	if (null == parent) {
		// If couldn't find one then create it.
		if (null == xmtaTextureId_) {
			xmtaTextureId_ = IdManager.getNextId(IdManager.TextureID);
			// make the group ids based on object id or make them unique.
			String id = getXmtoAttrId();
			if (null != id && !id.equals(""))
				xmtaTextureId_ = xmtaTextureId_ + ":" + id;
		}
		//
		return xmtaTextureId_;
	}

	// Now we must be a descendent in the defs tree and a descendent of a timing
	// element.  Thus we need the elasid to distinguish this id.
	TimeScheduler timeScheduler = getXmto2Xmta().getTimeScheduler();
	if (null == timeScheduler)
		return "";
	Vector lifeCycles = timeScheduler.getLifeCycles(((ConvertElement) parent).getIndex());
	if (null == lifeCycles)
		return "";
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.firstElement();
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	if (null == activeIntervals)
		return "";
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.firstElement();
	Vector elasIds = activeInterval.getELASids();
	if (null == elasIds)
		return "";
	ELASid elasId = (ELASid) elasIds.firstElement();
	xmtaTextureId_ = getXmtaTextureId(elasId);
	return xmtaTextureId_;
}
/***********************************************************************
 * get the id for the MovieTexture or ImageTexture element.
 **********************************************************************/
public String getXmtaTextureId(ELASid elasId) {
	String xmtaTextureId = (String) xmtaTextureIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTextureId) {
		xmtaTextureId = IdManager.getNextId(IdManager.TextureID);
		// make the group ids based on object id or make them unique.
		String id = getXmtoAttrId();
		if (null != id && !id.equals(""))
			xmtaTextureId = xmtaTextureId + ":" + id;
		xmtaTextureIds_.put(elasId, xmtaTextureId);
	}

	//
	return xmtaTextureId;
}
/***********************************************************************
 * Get the defs id of the object.
 * @return the unique id of this object.
 **********************************************************************/
public String getXmtaTextureTransformId() {
	if (null != xmtaTextureTransformId_)
		return xmtaTextureTransformId_;

	// Not a valid defs object
	if (!isInDefsTree()) {
		if (debug_)
			fireErrorEvent(ErrorEvent.TRACE, 0, "texture.getXmtaTextureTransformId() - is being called on a none defs object", getXmtoElement());

		// This should never be called but just in case return empty string.
		return "";
	}

	// Check whether the defs for the FontStyle is a child of the
	// <defs> element or a descendent of a TimingElement.  If it is a child of the
	// defs then it doesn't have a elasid.  If it is a descendent of a timingelement then
	// use the elasId of the timingElement.
	// First check if it is a child of the <defs> element.
	if (isDefsParent()) {
		// If couldn't find one then create it.
		if (null == xmtaTextureTransformId_) {
			xmtaTextureTransformId_ = IdManager.getNextId(IdManager.TextureTransformID);
			// make the group ids based on object id or make them unique.
			String id = getXmtoAttrId();
			if (null != id && !id.equals(""))
				xmtaTextureTransformId_ = xmtaTextureTransformId_ + ":" + id;
		}
		//
		return xmtaTextureTransformId_;
	}

	// See if the element is a descendent of defs but not through a timing element.
	TreeElement parent = getParent();
	while (null != parent && !(parent instanceof TimingElement))
		parent = parent.getParent();

	// This is a defs element under another defs element.  Thus look for its
	// If we couldn't find the parent then this must be a descendent of the defs
	// through other non timing elements, e.g. outline.
	// Treat it the same as a child of the defs.
	if (null == parent) {
		// If couldn't find one then create it.
		if (null == xmtaTextureTransformId_) {
			xmtaTextureTransformId_ = IdManager.getNextId(IdManager.TextureTransformID);
			// make the group ids based on object id or make them unique.
			String id = getXmtoAttrId();
			if (null != id && !id.equals(""))
				xmtaTextureTransformId_ = xmtaTextureTransformId_ + ":" + id;
		}
		//
		return xmtaTextureTransformId_;
	}

	// Now we must be a descendent in the defs tree and a descendent of a timing
	// element.  Thus we need the elasid to distinguish this id.
	TimeScheduler timeScheduler = getXmto2Xmta().getTimeScheduler();
	if (null == timeScheduler)
		return "";
	Vector lifeCycles = timeScheduler.getLifeCycles(((ConvertElement) parent).getIndex());
	if (null == lifeCycles)
		return "";
	LifeCycle lifeCycle = (LifeCycle) lifeCycles.firstElement();
	Vector activeIntervals = lifeCycle.getActiveIntervals();
	if (null == activeIntervals)
		return "";
	ActiveInterval activeInterval = (ActiveInterval) activeIntervals.firstElement();
	Vector elasIds = activeInterval.getELASids();
	if (null == elasIds)
		return "";
	ELASid elasId = (ELASid) elasIds.firstElement();
	xmtaTextureTransformId_ = getXmtaTextureTransformId(elasId);
	return xmtaTextureTransformId_;
}
/***********************************************************************
 * get the id for the TextureTransform element.
 **********************************************************************/
public String getXmtaTextureTransformId(ELASid elasId) {
	String xmtaTextureTransformId = (String) xmtaTextureTransformIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTextureTransformId) {
		xmtaTextureTransformId = IdManager.getNextId(IdManager.TextureTransformID);
		// make the group ids based on object id or make them unique.
		String id = getXmtoAttrId();
		if (null != id && !id.equals(""))
			xmtaTextureTransformId = xmtaTextureTransformId + ":" + id;
		xmtaTextureTransformIds_.put(elasId, xmtaTextureTransformId);
	}

	//
	return xmtaTextureTransformId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrCenter() {
	return xmtoAttrCenter_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRepeatS() {
	return xmtoAttrRepeatS_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRepeatT() {
	return xmtoAttrRepeatT_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRotation() {
	return xmtoAttrRotation_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrScale() {
	return xmtoAttrScale_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrTranslation() {
	return xmtoAttrTranslation_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// Determine the repeatS field   
	if (DOMXMTO.repeatS.equals(attributeName)) {
		xmtoAttrRepeatS_ = attributeValue;
		return true;
	}
	// Determine the repeatT field   
	if (DOMXMTO.repeatT.equals(attributeName)) {
		xmtoAttrRepeatT_ = attributeValue;
		return true;
	}
	// Determine the center field   
	if (DOMXMTO.center.equals(attributeName)) {
		xmtoAttrCenter_ = attributeValue;
		return true;
	}
	// Determine the rotation field   
	if (DOMXMTO.rotation.equals(attributeName)) {
		xmtoAttrRotation_ = attributeValue;
		return true;
	}
	// Determine the scale field   
	if (DOMXMTO.scale.equals(attributeName)) {
		xmtoAttrScale_ = attributeValue;
		return true;
	}
	// Determine the translation field   
	if (DOMXMTO.translation.equals(attributeName)) {
		xmtoAttrTranslation_ = attributeValue;
		return true;
	}

	// 
	return success;
}
/***********************************************************************
 * Routine to initialize the defs query flags.  This is added to provide
 * consistency for all the classes that implement the DefsObject interface.
 **********************************************************************/
public void initDefsFlags() {
	if (isInDefsTreeInitialized_)
		return;

	//
	isInDefsTreeInitialized_ = true;
	isInDefsTree_ = false;
	isDefsParent_ = false;

	// First check if parent is defs
	TreeElement parent = getParent();
	if (null != parent && parent instanceof defs) {
		isInDefsTree_ = true;
		isDefsParent_ = true;
		return;
	}


	// Next check if we are in the defs tree	
	while (null != parent) {
		if (parent instanceof defs) {
			isInDefsTree_ = true;
			return;
		}
		parent = parent.getParent();
	}
}
/***********************************************************************
 * Determine whether the object is a child of the defs element or not.  
 * This means that the object is a direct child of the defs element.
 * It isn't a child if it isn't in the defs tree or is a descendent 
 * (starting at grandchild) of the defs element.
 * @return whether the object is a child of the defs element.  True if 
 * it is a child of the defs element, false otherwise.
 **********************************************************************/
public boolean isDefsParent() {
	if (!isInDefsTreeInitialized_)
		initDefsFlags();

	//
	return isDefsParent_;
}
/***********************************************************************
 * Determine whether the object is in the defs tree or not.  This means
 * that the object is defined somewhere in the Defs tree.  It may be a
 * child or descendent of the defs element.
 * @return whether the object is in the defs tree.  True if a descendent
 * of the defs element, false otherwise.
 **********************************************************************/
public boolean isInDefsTree() {
	if (!isInDefsTreeInitialized_)
		initDefsFlags();

	//
	return isInDefsTree_;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<outline>" + super.toString() + "</outline>";
}
}
