/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  topLayout.java
//
//=============================================================================
package xmto2xmta.tree;

import java.awt.Color;
import java.text.MessageFormat;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.DOMXMTOParameter;
import xmto2xmta.ErrorEvent;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.TopLayoutRes;
import xmto2xmta.resources.Xmto2XmtaRes;

/*********************************************************************** 
 * Class to handle the conversion of the XMT-O element.  The class
 * name is XMTO instead of XMT-O since java doesn't allow a dash character 
 * in the class name.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>30 Jun 2001 - Fix bug, on validating syntax if problem with parent then
 *                   return directly the problem.
 * </ul><p>                                                                         
 *********************************************************************/
public class topLayout extends RegionContainerElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrWidth_ = null;
	private String xmtoAttrHeight_ = null;
	private String xmtoAttrBackgroundColor_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtaAttrBackgroundColor_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtaBackground2dId_ = null;

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle topLayoutRes_ = ResourceBundle.getBundle("xmto2xmta.resources.TopLayoutRes", Locale.getDefault());
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public topLayout(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds, Hashtable convertIndexTable, Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaAttrBackgroundColor() {
	return xmtaAttrBackgroundColor_;
}
/*********************************************************************** 
 * Get the id for the Background2d element.
 **********************************************************************/
public String getXmtaBackground2dId() {

	// If couldn't find one then create it.
	if (null == xmtaBackground2dId_) {
		xmtaBackground2dId_ = IdManager.getNextId(IdManager.Background2dID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaBackground2dId_ = xmtaBackground2dId_ + ":" + getXmtoAttrId();
	}

	//
	return xmtaBackground2dId_;
}
/*********************************************************************** 
 * Get the id for the XMT-A ordered group to which region objects are added.
 *
 * @return the id (DEF) of the XMT-A ordered group.
 **********************************************************************/
public String getXmtaLayoutOrderedGroupId() {

	// Find the XMTO root node
	TreeElement parent = getParent();
	while (null != parent) {
		if (parent instanceof XMTO) {
			return ((XMTO) parent).getLayoutOrderedGroupId();
		}
		parent = parent.getParent();
	}

	// This should never happen
	return "";
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrHeight() {
	return xmtoAttrHeight_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrWidth() {
	return xmtoAttrWidth_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// determine the width used for pixel metrics 
	if (DOMXMTO.width.equals(attributeName)) {
		xmtoAttrWidth_ = attributeValue;
		return true;
	}

	// determine the height used for pixel metrics 
	if (DOMXMTO.height.equals(attributeName)) {
		xmtoAttrHeight_ = attributeValue;
		return true;
	}

	// Determine the from color field
	if (DOMXMTO.backgroundColor.equals(attributeName)) {
		xmtoAttrBackgroundColor_ = attributeValue;
		Color color = DOMXMTOParameter.param2Color(attributeValue);
		if (null != color)
			xmtaAttrBackgroundColor_ = DOMXMTAParameter.color2Param(color);
		else {
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_ERROR_ATTRIBUTE_VALUE_NOT_SUPPORTED));
			Object[] msgArgs = {getXmtoElement().getNodeName(), attributeName, attributeValue};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	//
	return success;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<topLayout>" + super.toString() + "</topLayout>";
}
/*********************************************************************** 
 * Validate the syntax of the element.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.
 * Mainly check the begin, dur and end values to see they are valid timing
 * attributes.  If this is a problem then remove this node from the tree.
 **********************************************************************/
public boolean validateSyntax() {
	boolean success = super.validateSyntax();
	if (!success)
		return success;

	//
	boolean widthFound = false;
	if (null != xmtoAttrWidth_) {
		if (xmtoAttrWidth_.equals(""))
			xmtoAttrWidth_ = null;
		else
			widthFound = true;
	}

	//
	boolean heightFound = false;
	if (null != xmtoAttrHeight_) {
		if (xmtoAttrHeight_.equals(""))
			xmtoAttrHeight_ = null;
		else
			heightFound = true;
	}

	//
	if (heightFound != widthFound) {
		MessageFormat msgFormat = new MessageFormat(topLayoutRes_.getString(TopLayoutRes.MSG_ERROR_INVALID_SIZE_ATTRIBUTES));
		String arg0 = heightFound ? DOMXMTO.height : DOMXMTO.width;
		String arg1 = heightFound ? DOMXMTO.width : DOMXMTO.height;
		Object[] msgArgs = {arg0, arg1};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), DOMXMTO.type);
		remove();
		return false;
	}


	// 
	return success;
}
}
