/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  use.java
//
//=============================================================================
package xmto2xmta.tree;

import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import xmto2xmta.ActiveInterval;
import xmto2xmta.ErrorEvent;
import xmto2xmta.EventModel;
import xmto2xmta.IdManager;
import xmto2xmta.LifeCycle;
import xmto2xmta.TimeScheduler;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.resources.Xmto2XmtaRes;
import xmto2xmta.resources.UseRes;
import xmto2xmta.timegraph.ELASid;
import xmtutil.DOMXMTA;
import xmtutil.DOMXMTAParameter;
import xmtutil.DOMXMTO;
import xmtutil.TimingAttribute;
/*********************************************************************** 
 * Class to handle the conversion of the set element.
 *
 * <p><b>History:</b><ul>
 * <li>08 Jun 2001 - Created file.
 * <li>12 Jun 2001 - Add defs support for media objects and group.
 * <li>30 Jun 2001 - Fix bug, on validating syntax if problem with parent then
 *                   return directly the problem.
 * </ul><p>
 *********************************************************************/
public class use extends TimingElement implements Sensorable {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private String xmtoAttrRegion_ = null;
	private String xmtoAttrXlinkHref_ = null;
	private DefsObject xmtoAttrXlinkHrefDefsObject_ = null;

	/***********************************************************************
	 **********************************************************************/
	private Hashtable xmtaGroupIds_ = new Hashtable(89);
	private Hashtable xmtaSwitchIds_ = new Hashtable(89);
	private Hashtable xmtaTransform2dIds_ = new Hashtable(89);
	private Hashtable xmtaTouchSensorIds_ = new Hashtable(89);

	/*********************************************************************** 
	 **********************************************************************/
	private ResourceBundle xmto2XmtaRes_ = ResourceBundle.getBundle("xmto2xmta.resources.Xmto2XmtaRes", Locale.getDefault());
	private ResourceBundle useRes_ = ResourceBundle.getBundle("xmto2xmta.resources.UseRes", Locale.getDefault());
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public use(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds,Hashtable convertIndexTable,  Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * Get the id for the group element.
 **********************************************************************/
public String getXmtaGroupId(ELASid elasId) {
	String xmtaGroupId = (String) xmtaGroupIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaGroupId) {
		xmtaGroupId = IdManager.getNextId(IdManager.GroupID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaGroupId = xmtaGroupId + ":" + getXmtoAttrId();
		xmtaGroupIds_.put(elasId, xmtaGroupId);
	}

	//
	return xmtaGroupId;
}
/***********************************************************************
 * Get the id for the switch element.
 **********************************************************************/
public String getXmtaSwitchId(ELASid elasId) {
	String xmtaSwitchId = (String) xmtaSwitchIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaSwitchId) {
		xmtaSwitchId = IdManager.getNextId(IdManager.SwitchID);
		// make the switch and transform ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaSwitchId = xmtaSwitchId + ":" + getXmtoAttrId();
		xmtaSwitchIds_.put(elasId, xmtaSwitchId);
	}

	//
	return xmtaSwitchId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtaTouchSensorId(ELASid elasId) {
	String xmtaTouchSensorId = (String) xmtaTouchSensorIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTouchSensorId) {
		xmtaTouchSensorId = IdManager.getNextId(IdManager.TouchSensorID);
		// make the switch and transform ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaTouchSensorId = xmtaTouchSensorId + ":" + getXmtoAttrId();
		xmtaTouchSensorIds_.put(elasId, xmtaTouchSensorId);
	}
	return xmtaTouchSensorId;
}
/***********************************************************************
 * Get the id for the transform2d element.
 **********************************************************************/
public String getXmtaTransform2dId(ELASid elasId) {
	String xmtaTransform2dId = (String) xmtaTransform2dIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaTransform2dId) {
		xmtaTransform2dId = IdManager.getNextId(IdManager.Transform2dID);
		// make the transform ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaTransform2dId = xmtaTransform2dId + ":" + getXmtoAttrId();
		xmtaTransform2dIds_.put(elasId, xmtaTransform2dId);
	}
	return xmtaTransform2dId;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrRegion() {
	return xmtoAttrRegion_;
}
/*********************************************************************** 
 **********************************************************************/
public String getXmtoAttrXlinkHref() {
	return xmtoAttrXlinkHref_;
}
/*********************************************************************** 
 **********************************************************************/
public DefsObject getXmtoAttrXlinkHrefDefsObject() {
	return xmtoAttrXlinkHrefDefsObject_;
}
/*********************************************************************** 
 * Handle the attribute by checking its syntax.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.  Call the parent handler to see if it can
 * handle the attribute or not.
 *
 * @param  attributeName  node name of attribute.
 * @param  attributeValue node value of attribute.
 * @return whether this attribute is handled or not.
 **********************************************************************/
protected boolean handleValidateSyntax(String attributeName, String attributeValue) {
	boolean success = super.handleValidateSyntax(attributeName, attributeValue);

	// Determine the xlink:href
	if (DOMXMTO.xlink_colon_href.equals(attributeName)) {
		xmtoAttrXlinkHref_ = attributeValue;
		return true;
	}

	// region
	if (DOMXMTO.region.equals(attributeName)) {
		Class headClass = TreeClassObjects.getClassObject(TreeClassObjects.XMTO2XMTA_TREE_head);
		Class groupClass = TreeClassObjects.getClassObject(TreeClassObjects.XMTO2XMTA_TREE_group);
		boolean regionValid = true;
		// In order to use the region tag the object 
		// - has to be in the body section (!head section)
		// - not be under a group element
		TreeElement parent = getParent();
		while (null != parent) {
			if (null != groupClass && groupClass.isInstance(parent)) {
				regionValid = false;
				break;
			} else {
				if (null != headClass && headClass.isInstance(parent)) {
					regionValid = false;
					break;
				}
			}
			parent = parent.getParent();
		}
		if (regionValid)
			xmtoAttrRegion_ = attributeValue;
		else {
			//
			StringBuffer regionString = new StringBuffer();
			regionString.append(attributeName).append("=\"").append(attributeValue).append("\"");
			//
			StringBuffer nameString = new StringBuffer();
			nameString.append(getXmtoElement().getNodeName());
			String id = getXmtoAttrId();
			if (null != id && !id.equals(""))
				nameString.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");

			//
			MessageFormat msgFormat = new MessageFormat(xmto2XmtaRes_.getString(Xmto2XmtaRes.MSG_WARNING_ATTRIBUTE_NOT_SUPPORTED_IN_LOCATION));
			Object[] msgArgs = {nameString, regionString};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement(), attributeName);
		}
		return true;
	}

	// 
	return success;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<use>" + super.toString() + "</use>";
}
/*********************************************************************** 
 * Validate the syntax of the element.  This is to catch all syntax 
 * errors which the schema can not check for, and thus the DOM parser 
 * doesn't find any problems.
 * Mainly check the begin, dur and end values to see they are valid timing
 * attributes.  If this is a problem then remove this node from the tree.
 **********************************************************************/
public boolean validateSyntax() {
	boolean success = super.validateSyntax();
	if (!success)
		return success;

	// Check the xlink:href attribute
	// first check that it exists
	if (null == xmtoAttrXlinkHref_) {
		MessageFormat msgFormat = new MessageFormat(useRes_.getString(UseRes.MSG_ERROR_HREF_ATTRIBUTE_MISSING));
		StringBuffer buffer = new StringBuffer();
		buffer.append(getXmtoElement().getNodeName());
		String id = getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		Object[] msgArgs = {buffer.toString(), DOMXMTO.xlink_colon_href};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement());
		xmtoAttrXlinkHref_ = null;
		remove();
		return false;
	}
	// Next check the the xlink:href attribute is in the supported form, i.e. not specifying an external file
	if ('#' != xmtoAttrXlinkHref_.charAt(0)) {
		MessageFormat msgFormat = new MessageFormat(useRes_.getString(UseRes.MSG_ERROR_HREF_FILE_NOT_SUPPORTED));
		StringBuffer buffer = new StringBuffer();
		buffer.append(getXmtoElement().getNodeName());
		String id = getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		StringBuffer attrBuffer = new StringBuffer();
		attrBuffer.append(DOMXMTO.xlink_colon_href).append("=\"").append(xmtoAttrXlinkHref_).append("\"");
		Object[] msgArgs = {buffer.toString(), attrBuffer.toString()};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), DOMXMTO.xlink_colon_href);
		xmtoAttrXlinkHref_ = null;
		remove();
		return false;
	}
	// Next check the the xlink:href attribute specifies a valid id found in the file.
	String xlinkHrefId = xmtoAttrXlinkHref_.substring(1);
	boolean idFound = false;
	if (null != xlinkHrefId && !xlinkHrefId.equals("")) {
		// First is it in the table xmto ids? Also is it a DefsObject class.
		Hashtable xmtoTableIds = getXmtoIds();
		Object obj = xmtoTableIds.get(xlinkHrefId);
		if (null != obj && obj instanceof DefsObject) {
			// if it is a DefsObject class then make sure that it actually is found
			// in the defs section of the document.
			if (((DefsObject) obj).isInDefsTree()) {
				xmtoAttrXlinkHrefDefsObject_ = (DefsObject) obj;
				idFound = true;
			}
		}
	}
	if (!idFound) {
		MessageFormat msgFormat = new MessageFormat(useRes_.getString(UseRes.MSG_ERROR_HREF_ID_NOT_FOUND));
		StringBuffer buffer = new StringBuffer();
		buffer.append(getXmtoElement().getNodeName());
		String id = getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		StringBuffer attrBuffer = new StringBuffer();
		attrBuffer.append(DOMXMTO.xlink_colon_href).append("=\"").append(xmtoAttrXlinkHref_).append("\"");
		Object[] msgArgs = {buffer.toString(), DOMXMTO.xlink_colon_href, xlinkHrefId};
		String message = msgFormat.format(msgArgs);
		fireErrorEvent(ErrorEvent.ERROR, 0, message, getXmtoElement(), DOMXMTO.xlink_colon_href);
		xmtoAttrXlinkHref_ = null;
		remove();
		return false;
	}

	// Print out the information on the object.
	if (debug_) {
		StringBuffer buffer = new StringBuffer();
		buffer.append("use.validateSyntax() - <");
		buffer.append(getXmtoElement().getNodeName());
		String id = getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		buffer.append("> uses <");
		buffer.append(((ConvertElement) xmtoAttrXlinkHrefDefsObject_).getXmtoElement().getNodeName());
		id = ((ConvertElement) xmtoAttrXlinkHrefDefsObject_).getXmtoAttrId();
		if (null != id && !id.equals(""))
			buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
		buffer.append(">");
		fireErrorEvent(ErrorEvent.TRACE, 0, buffer.toString(), getXmtoElement(), DOMXMTO.xlink_colon_href);
	}

	// Now check to see whether the object that we are using is a timing element.
	// If it isn't a timing element then there shouldn't be any timing attributes set.
	// This happens in the case of defining a xMedia children element (material, outline, fontStyle, ..._)
	if (!(xmtoAttrXlinkHrefDefsObject_ instanceof TimingElement)) {
		// verify no timing attributes
		boolean timingAttrFound = false;
		if (null != getXmtoAttrBegin() || null != getXmtoAttrDur() || null != getXmtoAttrEnd() || null != getXmtoAttrMin() || null != getXmtoAttrMax() || null != getXmtoAttrRepeatDur())
			timingAttrFound = true;
		if (!timingAttrFound && null != getXmtoAttrFill() && !getXmtoAttrFill().equals(""))
			timingAttrFound = true;
		if (!timingAttrFound && null != getXmtoAttrFillDefault() && !getXmtoAttrFillDefault().equals(""))
			timingAttrFound = true;
		if (!timingAttrFound && null != getXmtoAttrRepeatRestart() && !getXmtoAttrRepeatRestart().equals(""))
			timingAttrFound = true;
		if (!timingAttrFound && null != getXmtoAttrRepeatRestartDefault() && !getXmtoAttrRepeatRestartDefault().equals(""))
			timingAttrFound = true;
		if (!timingAttrFound && null != getXmtoAttrEndsync() && !getXmtoAttrEndsync().equals(""))
			timingAttrFound = true;
		if (!timingAttrFound && null != getXmtoAttrRepeatCount() && !getXmtoAttrRepeatCount().equals(""))
			timingAttrFound = true;
		// if there were timing attributes then send a warning message that they are being ignored
		if (timingAttrFound) {
			MessageFormat msgFormat = new MessageFormat(useRes_.getString(UseRes.MSG_WARNING_TIMING_ATTRIBUTE_NOT_SUPPORTED));
			StringBuffer buffer = new StringBuffer();
			buffer.append(getXmtoElement().getNodeName());
			String id = getXmtoAttrId();
			if (null != id && !id.equals(""))
				buffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
			StringBuffer refObjBuffer = new StringBuffer();
			refObjBuffer.append(((ConvertElement) xmtoAttrXlinkHrefDefsObject_).getXmtoElement().getNodeName());
			id = ((ConvertElement) xmtoAttrXlinkHrefDefsObject_).getXmtoAttrId();
			if (null != id && !id.equals(""))
				refObjBuffer.append(" ").append(DOMXMTO.id).append("=\"").append(id).append("\"");
			Object[] msgArgs = {buffer.toString(), refObjBuffer.toString()};
			String message = msgFormat.format(msgArgs);
			fireErrorEvent(ErrorEvent.WARNING, 0, message, getXmtoElement());
		}
	}

	// 
	return success;
}
}
