/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  video.java
//
//=============================================================================
package xmto2xmta.tree;

import java.util.Hashtable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import xmto2xmta.IdManager;
import xmto2xmta.Xmto2Xmta;
import xmto2xmta.timegraph.ELASid;
/*********************************************************************** 
 * Class to handle the conversion of the rectangle element.
 *
 * <p><b>History:</b><ul>
 * <li>13 Dec 2000 - Created file.
 * <li>26 Apr 2001 - Update object descriptors to Jan XMT-A spec.
 * <li>30 Apr 2001 - Add switch to generate new format for XMT-A files.
 * <li>01 May 2001 - Changed too much of the old stuff, video descriptor.
 * <li>03 May 2001 - Fix when repeat start time is before begining of presentation
 *                   ie. t<0.  Issue a warning and set start to t=0.
 *                 - Fix when video doesn't end (indefinite) don't remove
 *                   od.
 * </ul><p>                                                                                
 *********************************************************************/
public class video extends ExternalVisualElement {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;

	/*********************************************************************** 
	 **********************************************************************/
	private Hashtable xmtaMovieTextureIds_ = new Hashtable(89);
/**********************************************************************
 * constructor.
 * @param xmtoElement XMT-O element for conversion.
 * @param xmtoIds     hashtable to contain XMT-O Conversion elements 
 *                    with the key of their XMT-O id.
 * @param xmtaIds     hashtable to contain XMT-A elements with the key
 *                    of their XMT-A id.
 * @param xmtaDoc     DOM Document for the new XMT-A elements.
 * @param xmto2Xmta   Converter, used for firing error events.
 *********************************************************************/
public video(Element xmtoElement, Hashtable xmtoIds, Hashtable xmtaIds,Hashtable convertIndexTable,  Document xmtaDoc, Xmto2Xmta xmto2Xmta) {
	super(xmtoElement, xmtoIds, xmtaIds, convertIndexTable, xmtaDoc, xmto2Xmta);
}
/*********************************************************************** 
 * Get the id for the MovieTexture element.
 **********************************************************************/
public String getXmtaMovieTextureId(ELASid elasId) {
	String xmtaMovieTextureId= (String) xmtaMovieTextureIds_.get(elasId);

	// If couldn't find one then create it.
	if (null == xmtaMovieTextureId) {
		xmtaMovieTextureId = IdManager.getNextId(IdManager.MovieTextureID);
		// make the group ids based on object id or make them unique.
		if (null != getXmtoAttrId() && !getXmtoAttrId().equals(""))
			xmtaMovieTextureId = xmtaMovieTextureId + ":" + getXmtoAttrId();
		xmtaMovieTextureIds_.put(elasId, xmtaMovieTextureId);
	}

	//
	return xmtaMovieTextureId;
}
/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
/***********************************************************************
 **********************************************************************/
public String toString() {
	return "<video>" + super.toString() + "</video>";
}
}
