/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  ColorAnimationInterpolation.java
//
//  Overview  :
//    A utility to calculate key and key values pairs to map color animation
//    into a form that can be done using a ColorInterpolator
//=============================================================================
package xmtutil;

import java.awt.Color;

public class ColorAnimationInterpolation extends AnimationInterpolation
{
  public ColorAnimationInterpolation(Color initialValue, Color from, Color to, Color by,
                                     Color[] values,
                                     String  calcMode, String accumulate, String additive,
                                     Float[] keyTimes, Float[] keySplines, String path,
                                     TimingAttribute dur, TimingAttribute repeatDur, Float repeatCount)
  {
    super(initialValue, from, to, by, values, calcMode, accumulate, additive,
          keyTimes, keySplines, path, dur, repeatDur, repeatCount);
  }

  public Color[] getColorInterpolatorKeyValues() { return (Color[])getInterpolatorKeyValues(); }

  protected Object[] alloc(int size) { return new Color[size]; }

  protected Object sum(Object a, Object b)
  {
    Color x = (Color)a;
    Color y = (Color)b;

    int red   = x.getRed()   + y.getRed();
    int green = x.getGreen() + y.getGreen();
    int blue  = x.getBlue()  + y.getBlue();
    if (red   > 255) red   = 255;
    if (green > 255) green = 255;
    if (blue  > 255) blue  = 255;

    return new Color(red, green, blue);
  }
}
