/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  DOMUtil.java
//
//=============================================================================
package xmtutil;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;

/*********************************************************************** 
 * DOMParameter converts the parameter strings to java objects and back.
 *
 * <p><b>History:</b><ul>
 * <li>16 Jun 2000 - Creation.
 * <li>19 Jun 2000 - Add some minimal time parameter support.
 * <li>20 Jun 2000 - Fix converting from Color to parameter.
 * </ul>
 *********************************************************************/
public class DOMUtil {
	/*********************************************************************** 
	 * Whether debug information is to be printed to the console. 
	 * default is not have debug information printed (false).
	 **********************************************************************/
	static private boolean debug_ = false;
/*********************************************************************** 
 **********************************************************************/
public static Element getChildElementByTagName(Element element, String name) {
	if (element == null)
		return null;

	//
	NodeList nl = element.getChildNodes();
	if (null == nl)
		return null;

	//
	for (int i = 0, count = nl.getLength(); i < count; i++) {
		Node node = nl.item(i);
		if (node instanceof Element) {
			element = (Element) node;
			if (name.equals(element.getNodeName()))
				return element;
		}
	}

	//
	return null;
}

/*********************************************************************** 
 * Return the path part of a filname path. If the filename ends with the
 * separator character the same string is returned.  Otherwise, the portion
 * up to and including the last separator character is returned.
 **********************************************************************/
public static String getDirectory(String fileName) {
	try {
		int lastSeparator = 
			Math.max(fileName.lastIndexOf(java.io.File.separator), fileName.lastIndexOf("/")); 
		if (lastSeparator > -1)
			return fileName.substring(0, lastSeparator + 1);
		else if (fileName.length() >= 2 && Character.isLetter(fileName.charAt(0)) && fileName.charAt(1) == ':')
			// hack for Windows drive letter format.
			return fileName.substring(0,2);
		else
			return null;
	} catch (Exception e) {
		return null;
	}
}

/*********************************************************************** 
 * Enable debugging information to be printed to the console. 
 *
 * @param debug     sets debug information is printed
 **********************************************************************/
static public void setDebug(boolean debug) {
	debug_ = debug;
}
}
