/******************************************************************************\
** This software module was originally developed by IBM Research (T.J.Watson) **
** in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)       **
** standard.  This software module is an implementation of a part of one or   **
** more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4     **
** Systems (ISO/IEC 14496-1) standard.  ISO/IEC gives users of the MPEG-4     **
** Systems (ISO/IEC 14496-1) free license to this software module or          **
** modifications thereof for use in hardware or software products claiming    **
** conformance to the MPEG-4 Systems (ISO/IEC 14496-1).  Those intending to   **
** use this software module in hardware or software products are advised that **
** its use may infringe existing patents.  The original developer of this     **
** software module and his/her company, the subsequent editors and their      **
** companies, and ISO/IEC have no liability for use of this software module   **
** or modifications thereof in an implementation.  Copyright is not released  **
** for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  IBM retain  **
** full right to use the code for their own purpose, assign or donate the     **
** code to a third party and to inhibit third parties from using the code for **
** non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.  This copyright  **
** notice must be included in all copies or derivative works.                 **
** Copyright  2000-2001.                                                     **
** -------------------------------------------------------------------------- **
**  IBM  TJ Watson Research, USA                                              **
**  Contact :                                                                 **
**    Dr. Michelle Kim ( mykim@us.ibm.com )                                   **
\******************************************************************************/

//=============================================================================
//  TimingAttributeRes.java
//
//=============================================================================
package xmtutil.resources;

import java.util.ListResourceBundle;
import java.lang.String;

/***********************************************************************
 * This is the default language resources.
 * The language is English (United States).
 * <p>
 * There is a good explanation on the National Language Support (NLS)
 * in the book "Core Java Volume II - Advanced Features".  The chapter
 * is Internationalization.
 * <p>
 * The resources are listed on two lines so that it is easier to compare
 * the various language resource files to see what strings are missing.
 * <p>
 * The resources that use parameters (like filenames or numbers) are formated
 * into MessageFormat (java.text.MessageFormat). To put a single quote (')
 * in such a resources, you have to put it twice (''). For the others
 * resources, you just have to put it once.
 *
 *
 * <p><b>History:</b><ul>
 * <li>18 Feb 2000 - Created file to handle resources for NLS.
 * </ul>
 *********************************************************************/
public class TimingAttributeRes extends ListResourceBundle {

  /***********************************************************************
   * All these keys are public since this will keep accessing the string
   * standardized and lead to less problems.
   **********************************************************************/
  public static final String MSG_ERROR_INVALID_TIMING_ATTRIBUTE = "MsgErrorInvalidTimingAttribute";
  public static final String MSG_ERROR_SYNC_TO_PREV_TIMING_NOT_SUPPORTED = "MsgErrorSyncToPrevTimingNotSupported";
  public static final String MSG_ERROR_ACCESS_KEY_TIMING_NOT_SUPPORTED = "MsgErrorAccessKeyTimingNotSupported";
  public static final String MSG_ERROR_WALL_CLOCK_TIMING_NOT_SUPPORTED = "MsgErrorWallClockTimingNotSupported";
  public static final String MSG_ERROR_REPEAT_TIMING_NOT_SUPPORTED = "MsgErrorRepeatTimingNotSupported";
  public static final String MSG_ERROR_MARKER_TIMING_NOT_SUPPORTED = "MsgErrorMarkerTimingNotSupported";
  public static final String MSG_WARNING_MULTPLE_TIMINGS_NOT_SUPPORTED = "MsgWarningMultipleTimingsTimingNotSupported";
  public static final String MSG_ERROR_INVALID_ID_VALUE = "MsgErrorInvalidIdValue";
  public static final String MSG_ERROR_INVALID_EVENT_SYMBOL = "MsgErrorInvalidEventSymbol";
  public static final String MSG_ERROR_INVALID_SYNCBASE_TIMING = "MsgErrorInvalidSyncbaseTiming";
  public static final String MSG_ERROR_INVALID_MULTI_TIMING_ATTRIBUTE_ELEMENT = "MsgErrorInvalidMultiTimingAttributeElement";

  /***********************************************************************
   * These are the strings which should get translated.
   * The first line is the KEY and the second line is the string
   * to be translated.  These could be on the same line but having them on
   * separate lines will make it easier to compare the various language files
   * to see which items have not been translated and which lines are missing
   * from the default language file English.
   **********************************************************************/
  static final Object[][] contents = {
  /* */ {TimingAttributeRes.MSG_ERROR_INVALID_TIMING_ATTRIBUTE,
  /* */ "Invalid timing attribute."},
  /* */ {TimingAttributeRes.MSG_ERROR_SYNC_TO_PREV_TIMING_NOT_SUPPORTED,
  /* */ "Sync to previous timing not supported."},
  /* */ {TimingAttributeRes.MSG_ERROR_ACCESS_KEY_TIMING_NOT_SUPPORTED,
  /* */ "Access key timing not supported."},
  /* */ {TimingAttributeRes.MSG_ERROR_WALL_CLOCK_TIMING_NOT_SUPPORTED,
  /* */ "Wall clock timing not supported."},
  /* */ {TimingAttributeRes.MSG_ERROR_REPEAT_TIMING_NOT_SUPPORTED,
  /* */ "Repeat timing not supported."},
  /* */ {TimingAttributeRes.MSG_ERROR_MARKER_TIMING_NOT_SUPPORTED,
  /* */ "Marker timing not supported."},
  /* */ {TimingAttributeRes.MSG_WARNING_MULTPLE_TIMINGS_NOT_SUPPORTED,
  /* */ "Multiple timing values not supported, using first time value."},
  /***********************************************************************
   * The text is the error message when invalid media object id is used.
   * There is 1 parameters.
   * <ol>
   * <li>The parameter is a String for the media object id.
   * </ol>
   **********************************************************************/
  /* */ {TimingAttributeRes.MSG_ERROR_INVALID_ID_VALUE,
  /* */ "Object id of \"{0}\" does not exist as a valid id"},
  /***********************************************************************
   * The text is the error message when invalid media object id is used.
   * There is 1 parameters.
   * <ol>
   * <li>The parameter is a String for the invalid event symbol.
   * </ol>
   **********************************************************************/
  /* */ {TimingAttributeRes.MSG_ERROR_INVALID_EVENT_SYMBOL,
  /* */ "Event timing symbol of \"{0}\" is not one of the known event types."},
  /* */ {TimingAttributeRes.MSG_ERROR_INVALID_SYNCBASE_TIMING,
  /* */ "Invalid syncbase timing."},
  /***********************************************************************
   * The text is the error message when invalid invalid timing attribute element
   * is passed in to create a multi timeing attribute.
   * There is 1 parameters.
   * <ol>
   * <li>The parameter is a String for the invalid timing attribute element.
   * </ol>
   **********************************************************************/
  /* */ {TimingAttributeRes.MSG_ERROR_INVALID_MULTI_TIMING_ATTRIBUTE_ELEMENT,
  /* */ "Invalid timing attribute \"{0}\" in multi timing attribute."},
  /* */};
/***********************************************************************
 **********************************************************************/
public Object[][] getContents() {
  return contents;
}
}
